/*

  ColorCube.java

  Written by Ross Duckworth (lestat@ihug.co.nz)

  A simple demonstration a an RGB color cube.
  The ColorCube can be rotated with the mouse.

*/

package com.hermetica.magician.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class ColorCubeapp extends Applet implements demoTemplate, GLEventListener, MouseListener, MouseMotionListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 300,
                             HEIGHT = 300;

    /** ColorCube vertex data */
    private double[][] vertices = {
	{-0.5,-0.5,0.5}, {0.5,-0.5,0.5}, {0.5,0.5,0.5}, {-0.5,0.5,0.5}, 				
	{0.5,-0.5,0.5}, {0.5,-0.5,-0.5}, {0.5,0.5,-0.5}, {0.5,0.5,0.5},
	{0.5,-0.5,-0.5}, {-0.5,-0.5,-0.5}, {-0.5,0.5,-0.5}, {0.5,0.5,-0.5},
	{-0.5,-0.5,-0.5}, {-0.5,-0.5,0.5}, {-0.5,0.5,0.5}, {-0.5,0.5,-0.5},
	{-0.5,0.5,0.5}, {0.5,0.5,0.5}, {0.5,0.5,-0.5}, {-0.5,0.5,-0.5},
	{-0.5,-0.5,-0.5}, {0.5,-0.5,-0.5}, {0.5,-0.5,0.5}, {-0.5,-0.5,0.5},
        };

    /** ColorCube color data */
    private float[][] colors = {
	{0,0,1}, {1,0,1}, {1,1,1}, {0,1,1},
	{1,0,1}, {1,0,0}, {1,1,0}, {1,1,1},
	{1,0,0}, {0,0,0}, {0,1,0}, {1,1,0},
	{0,0,0}, {0,0,1}, {0,1,1}, {0,1,0},
	{0,1,1}, {1,1,1}, {1,1,0}, {0,1,0},
	{0,0,0}, {1,0,0}, {1,0,1}, {0,0,1},
    };

    /** trackball */
    private trackball tracker = new trackball();

    /** rotation quaternion */
    private float[] curquat = new float[4];

    /** mouse position */
    private float beginx, beginy;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
           curquat = tracker.buildQuaternion(0, 0, 0, 0);
            startDemo();
    }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: ColorCube.java.stub,v 1.2 1998/10/19 00:52:51 descarte Exp descarte $" );
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "RGB Colour Cube" );
      }

    /** Returns the meaningful description of what this demo does */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration program renders a RGB color cube using\n" );
        sb.append( "a vertex array and corresponding vertex color array\n\n" );
        sb.append( "The ColorCube can be rotated with the mouse\n\n" );
        sb.append( "Hit the ``Escape'' key or window close button to destroy the demo." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Ross Duckworth <lestat@ihug.co.nz>" );
    }

    public void setExit( boolean state ) {
        callExit = state;
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setRedBits( 1 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register the MouseListeners with the GLComponent */
        glc.addMouseListener( this );
        glc.addMouseMotionListener( this );

        /** Register the keyboard listener */
        KeyListener kl = new KeyAdapter() {
                    public void keyPressed( KeyEvent evt ) {
                           if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
                              /** Destroy the GLContext */
                              glc.destroy();
                              /**#dispose */
                           }
                    }
        };
        glc.addKeyListener( kl );

        /**#windowListener */

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
        gl_.glShadeModel( GL.GL_SMOOTH );
        gl_.glEnable(GL.GL_DEPTH_TEST);
    }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y,
                         int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 45.0, (float) width/height, 1.0, 20.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
    }

    /** Renders a ColorCube */
    public void display( GLDrawable component ) {
        /** Draw the cube */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
        gl_.glLoadIdentity();
        glu_.gluLookAt( 1.5f, 1.5f, 1.5f, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 );
        gl_.glMultMatrixf(tracker.buildMatrix(curquat));
        for (int i=0;i<6;i++) {
            gl_.glBegin(GL.GL_POLYGON);
            for (int j=0;j<4;j++) {
                gl_.glColor3f(colors[i*4+j][0],colors[i*4+j][1],colors[i*4+j][2]);
                gl_.glVertex3d(vertices[i*4+j][0],vertices[i*4+j][1],vertices[i*4+j][2]);
            }
            gl_.glEnd();
        }
    }

    /** methods required for MouseListener and MouseMotionListener interfaces */
    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseMoved(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}

    /** record mouse down coordinates */
    public void mousePressed(MouseEvent e) {
           beginx = e.getX();
           beginy = e.getY();
    }

    /** create a rotation quaternion that tracks the mouse movement */
    public void mouseDragged(MouseEvent e) {
           int x = e.getX();
           int y = e.getY();
           Dimension dim = e.getComponent().getSize();
           int width = dim.width;
           int height = dim.height;

           float[] lastquat = tracker.buildQuaternion( ( 2.0f * beginx - width ) / width,
                                                      ( height - 2.0f * beginy ) / height,
                                                      ( 2.0f * x - width ) / width,
                                                      ( height - 2.0f * y ) / height );

           curquat = tracker.addQuats(lastquat,curquat);

            glc.repaint();

           beginx = x;
           beginy = y;
    }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
    }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
        }
    }

    /** Applet only */
    public void start() {
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
        }
    }

    /** Applet only */
    public void destroy() {
        /** Destroy the GL component to free up the resources */
        glc.destroy();
    }
}
