/**
 * $Id: sharkFrame.java,v 1.12 1998/10/19 10:37:13 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 */

package com.hermetica.magician.demos.atlantis;

import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.vecmath.*;
import com.hermetica.util3d.constrain;

public class sharkFrame extends Frame implements GLEventListener, ActionListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    GLU glu_ = null;

    /** The shark we're drawing! */
    shark theShark = null;

    /** The OpenGL Component embedded in this frame */
    GLComponent glc = null;

    /** Dimensions of the component */
    private static final int WIDTH = 100,
                             HEIGHT = 100;

    /** Default constructor */
    public sharkFrame( GL gl_, GLU glu_ ) {

        /** Setup the rendering pipeline */
        this.gl_ = gl_;
        this.glu_ = glu_;

        /** Set the layout for this frame */
        setLayout( new GridBagLayout() );

        /** Create a new shark */
        theShark = new shark( gl_, glu_ );
        theShark.position = 
/*            new Vec3f( 70000.0f + (float)( (int)( Math.random() * atlantis.RAND_MAX ) % 6000 ),
                       (float)( (int)( Math.random() * atlantis.RAND_MAX ) % 6000 ),
                       (float)( (int)( Math.random() * atlantis.RAND_MAX ) % 6000 ) ); */
            new Vec3f( 5000.0f, 0.0f, 0.0f );
        theShark.psi = (float)( (int)( Math.random() * atlantis.RAND_MAX ) % 360 ) - 180.0f;
        theShark.v = 0.0f;

        /** Create a new OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the component to the frame */
        constrain.constrain( this, glc, 0, 0, 1, 1 );

        /** Create a new TextArea to contain shark information */
        TextArea infoField = new TextArea( 6, 65 );
        infoField.setText( "" );
        infoField.append( "The shark is one of the more misunderstood animal on this earth,\n" );
        infoField.append( "having the reputation of being a ferocious man-killer and general\n" );
        infoField.append( "nasty beastie.\n\n" );
        infoField.append( "This is not exactly the case.\n\n" );
        infoField.append( "There are around 350 different species of shark today, varying in\n" );
        infoField.append( "size from the ``Whale Shark'' ( 13.7m ) to the ``Pygmy\n" );
        infoField.append( "Ribbontail Catshark'' which is amongst the smaller sharks\n" );
        infoField.append( "( 22-25cm ). Each species of shark may not have the same dietary\n" );
        infoField.append( "requirements as other species, for example, hammerhead sharks\n" );
        infoField.append( "eat stingrays mostly, while whale sharks eat plankton. ( You have\n" );
        infoField.append( "more chance of being sucked to death by a whale shark, than be eaten by\n" );
        infoField.append( "one! )\n\n" );
        infoField.append( "Sadly, several shark species are now approaching extinction due to being\n" );
        infoField.append( "accidentally trapped in fishing nets, or just fished for for sport,\n" );
        infoField.append( "meat or teeth for jewellery.\n\n" );
        infoField.append( "A bevy of links to shark information on the WWW can be located at:\n\n" );
        infoField.append( "      http://www.ncf.carleton.edu/~bz050/HomePage.sharklinks.html" );
        constrain.constrain( this, infoField, 1, 0, 1, 1 );

        /** Add a Dismiss button */
        Button dismissButton = new Button( "Dismiss" );
        constrain.constrain( this, dismissButton, 
                          0, 1, 2, 1,
                          GridBagConstraints.NONE, 
                          GridBagConstraints.CENTER,
                          1.0, 1.0, 0, 0, 0, 0 );

        pack();
        show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 16 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register an action listener for the dismiss button */
        dismissButton.addActionListener( this );

        /** Tell the component to start processing and generating events */
        glc.initialize();
        glc.start();
      }

    /** Handles action events on the dismiss button */
    public void actionPerformed( ActionEvent evt ) {
        if ( evt.getActionCommand().equals( "Dismiss" ) ) {
            glc.destroy();
            dispose();
            return;
          }
      }

    /** Initialize the OpenGL stuff */
    public void initialize( GLDrawable component ) {

        float[] ambient = {0.1f, 0.1f, 0.1f, 1.0f};
        float[] diffuse = {1.0f, 1.0f, 1.0f, 1.0f};
        float[] position = {0.0f, 1.0f, 0.0f, 0.0f};
        float[] mat_shininess = {90.0f};
        float[] mat_specular = {0.8f, 0.8f, 0.8f, 1.0f};
        float[] mat_diffuse = {0.46f, 0.66f, 0.795f, 1.0f};
        float[] mat_ambient = {0.0f, 0.1f, 0.2f, 1.0f};
        float[] lmodel_ambient = {0.4f, 0.4f, 0.4f, 1.0f};
        float[] lmodel_localviewer = {0.0f};

        try {
            gl_.glFrontFace(GL.GL_CW);
            gl_.glDepthFunc(GL.GL_LEQUAL);
            gl_.glEnable(GL.GL_DEPTH_TEST);
        
            gl_.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, ambient);
            gl_.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse);
            gl_.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, position);
            gl_.glLightModelfv(GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
            gl_.glLightModelfv(GL.GL_LIGHT_MODEL_LOCAL_VIEWER, lmodel_localviewer);
            gl_.glEnable(GL.GL_LIGHTING);
            gl_.glEnable(GL.GL_LIGHT0);
        
            gl_.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_SHININESS, mat_shininess);
            gl_.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_SPECULAR, mat_specular);
            gl_.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE, mat_diffuse);
            gl_.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT, mat_ambient);
    
            gl_.glClearColor(0.0f, 0.5f, 0.9f, 0.0f);
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 400.0f, 2.0f, 1000.0f, 400000.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
      }

    /** Redraws the component */
    public void display( GLDrawable component ) {
        try {

            /** Move the shark */
            theShark.psi -= 5;

            /** Clear the frame and depth buffers */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

            /** Draw the shark */
            gl_.glPushMatrix();
            theShark.transform();
            theShark.draw();
            gl_.glPopMatrix();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline to GLEventListener */
    public GL getGL() {
        return gl_;
      }
  }
