/**
 * $Id: whaleFrame.java,v 1.9 1998/10/19 10:37:32 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * $Log: whaleFrame.java,v $
 * Revision 1.9  1998/10/19 10:37:32  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.8  1998/10/02 12:10:21  descarte
 * Changed GLComponentListener to GLEventListener
 * Altered arguments to reshape() method
 *
 * Revision 1.7  1998/06/07 13:53:55  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy
 *
 * Revision 1.6  1998/03/28 14:44:26  descarte
 * Upgraded demo to Java 1.1 listeners.
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.5  1998/03/24 15:59:44  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.4  1998/03/19 21:59:13  descarte
 * Added getGL() method.
 *
 * Revision 1.3  1998/02/23 21:53:43  descarte
 * Converted demo to new beta 4 GLComponent structure.
 *
 * Revision 1.2  1998/01/04 17:32:13  descarte
 * Added constructor for default pipeline propagation.
 * Converted class across to GLEventListener interface.
 *
 * Revision 1.1  1997/09/02 22:32:26  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.atlantis;

import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.vecmath.*;
import com.hermetica.util3d.constrain;

public class whaleFrame extends Frame implements GLEventListener, ActionListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    GLU glu_ = null;

    /** The whale we're drawing! */
    whale theWhale = null;

    /** The OpenGL Component embedded in this frame */
    GLComponent glc = null;

    /** Dimensions of the component */
    private static final int WIDTH = 100,
                             HEIGHT = 100;

    /** Default constructor */
    public whaleFrame( GL gl_, GLU glu_ ) {

        /** Initialize the OpenGL pipelines */
        this.gl_ = gl_;
        this.glu_ = glu_;

        /** Set the layout for this frame */
        setLayout( new GridBagLayout() );

        /** Create a new whale */
        theWhale = new whale( gl_, glu_ );
        theWhale.position = 
/*            new Vec3f( 70000.0f + (float)( (int)( Math.random() * atlantis.RAND_MAX ) % 6000 ),
                       (float)( (int)( Math.random() * atlantis.RAND_MAX ) % 6000 ),
                       (float)( (int)( Math.random() * atlantis.RAND_MAX ) % 6000 ) ); */
            new Vec3f( 30000.0f, 0.0f, 0.0f );
        theWhale.psi = (float)( (int)( Math.random() * atlantis.RAND_MAX ) % 360 ) - 180.0f;
        theWhale.v = 0.0f;

        /** Create a new OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the component to the frame */
        constrain.constrain( this, glc, 0, 0, 1, 1 );

        /** Create a new TextArea to contain whale information */
        TextArea infoField = new TextArea( 6, 65 );
        infoField.setText( "" );
        infoField.append( "Whales are evolutionary wonders, originally being land-based mammals\n" );
        infoField.append( "( admittedly many million years ago ) which began to search and hunt\n" );
        infoField.append( "food in the seas. Over a prolonged period of time, these mammals evolved\n" );
        infoField.append( "into today's whales.\n\n" );
        infoField.append( "There are two broad categories of whales, both belonging to the Cetacea\n" );
        infoField.append( "family, which are the ``Baleen'', or whale-bone, whales, and ``Toothed''\n" );
        infoField.append( "whales. Baleen whales are primarily ``filter-feeders'' and strain small\n" );
        infoField.append( "shrimp-like animals called ``Krill'' through the baleen plates in their\n" );
        infoField.append( "mouths. Toothed whales eat primarily fish.\n" );
        constrain.constrain( this, infoField, 1, 0, 1, 1 );

        /** Add a Dismiss button */
        Button dismissButton = new Button( "Dismiss" );
        constrain.constrain( this, dismissButton, 
                          0, 1, 2, 1,
                          GridBagConstraints.NONE, 
                          GridBagConstraints.CENTER,
                          1.0, 1.0, 0, 0, 0, 0 );

        pack();
        show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 16 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register the action listener for the dismiss button */
        dismissButton.addActionListener( this );

        /** Tell the component to process events */
        glc.initialize();
        glc.start();
      }

    /** Handles action events for the dismissButton */
    public void actionPerformed( ActionEvent evt ) {
        if ( evt.getActionCommand().equals( "Dismiss" ) ) {
            glc.destroy();
            dispose();
            return;
          }
      }

    /** Initialize the OpenGL stuff */
    public void initialize( GLDrawable component ) {

        float[] ambient = {0.1f, 0.1f, 0.1f, 1.0f};
        float[] diffuse = {1.0f, 1.0f, 1.0f, 1.0f};
        float[] position = {0.0f, 1.0f, 0.0f, 0.0f};
        float[] mat_shininess = {90.0f};
        float[] mat_specular = {0.8f, 0.8f, 0.8f, 1.0f};
        float[] mat_diffuse = {0.46f, 0.66f, 0.795f, 1.0f};
        float[] mat_ambient = {0.0f, 0.1f, 0.2f, 1.0f};
        float[] lmodel_ambient = {0.4f, 0.4f, 0.4f, 1.0f};
        float[] lmodel_localviewer = {0.0f};

        try {
            gl_.glFrontFace(GL.GL_CW);
            gl_.glDepthFunc(GL.GL_LEQUAL);
            gl_.glEnable(GL.GL_DEPTH_TEST);
        
            gl_.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, ambient);
            gl_.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse);
            gl_.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, position);
            gl_.glLightModelfv(GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
            gl_.glLightModelfv(GL.GL_LIGHT_MODEL_LOCAL_VIEWER, lmodel_localviewer);
            gl_.glEnable(GL.GL_LIGHTING);
            gl_.glEnable(GL.GL_LIGHT0);
    
            gl_.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_SHININESS, mat_shininess);
            gl_.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_SPECULAR, mat_specular);
            gl_.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE, mat_diffuse);
            gl_.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT, mat_ambient);
    
//            initializeFish();
        
            gl_.glClearColor(0.0f, 0.5f, 0.9f, 0.0f);
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 400.0f, 2.0f, 1000.0f, 400000.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
      }

    /** Redraws the component */
    public void display( GLDrawable component ) {
        try {
            /** Move the whale */
            theWhale.psi -= 5;

            /** Clear the frame and depth buffers */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

            /** Draw the whale */
            gl_.glPushMatrix();
            theWhale.transform();
            theWhale.draw();
            gl_.glPopMatrix();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Return a GL pipeline to GLEventListener */
    public GL getGL() {
        return gl_;
      }
  }
