/**
 * $Id: bootFrame.java,v 1.18 1998/07/09 09:27:27 descarte Exp descarte $
 *
 * Copyright (c)1997 Alligator Descartes <descarte@hermetica.com>
 *
 * Frame that drives the selection and running of demo programs
 *
 * $Log: bootFrame.java,v $
 * Revision 1.18  1998/07/09 09:27:27  descarte
 * Added ``tentacle'' demo.
 *
 * Revision 1.17  1998/06/08 22:11:24  descarte
 * Added Microsoft VM hack to avoid List.clear() bug which throws bogus
 * exceptions.
 *
 * Revision 1.16  1998/06/01 11:21:05  descarte
 * Commented out some non-functional demos.
 *
 * Revision 1.15  1998/05/22 22:57:55  descarte
 * Explicitly specified java.awt.List for the listboxes as JavaSoft appeared
 * to have been dumb enough to define two identically named classes in
 * different core packages. Duh.
 *
 * Revision 1.14  1998/04/23 14:19:00  descarte
 * Added ``Throwing Shapes!'' and ``USS Enterpoop'' to the standard demos.
 *
 * Revision 1.13  1998/03/05 20:47:10  descarte
 * Renamed ``DinoBall'' to ``Dinosaur''
 *
 * Revision 1.12  1998/03/05 14:07:17  descarte
 * Added ``Shadowy Leapin' Lizards'' demo!
 *
 * Revision 1.11  1998/02/23 17:04:42  descarte
 * Added GLUT fog demo.
 *
 * Revision 1.10  1998/02/22 20:08:17  descarte
 * Added ``GLUT Demos'' handling.
 *
 * Revision 1.9  1998/01/14 23:27:52  descarte
 * Added redbook ``picksquare'' demo back in.
 *
 * Revision 1.8  1997/11/11 22:50:02  descarte
 * Fixed LIST_SELECT Event bug. It now selects the demo when you select the
 * list. No double-clicking required.
 * The ``Run This Demo!'' button is greyed out until you select a demo to run.
 * Keyboard events, except ESC, are disabled.
 * Added new Mesa demos and removed old demos that are broken.
 *
 * Revision 1.7  1997/11/10 22:40:42  descarte
 * Added ``Mesa'' group of demos.
 *
 * Revision 1.6  1997/09/09 09:23:12  descarte
 * Added a host of new demos.
 * Added the OpenGL information frame and menubar
 *
 * Revision 1.5  1997/08/29 21:43:01  descarte
 * Fixed up layout problems on the GUI
 *
 * Revision 1.4  1997/08/29 21:11:56  descarte
 * Added logo
 *
 * Revision 1.3  1997/08/26 18:48:02  descarte
 * Added checkbox handling to split the demos into logical groupings,
 * which makes for shorter lists!
 *
 * Revision 1.2  1997/08/26 00:54:10  descarte
 * Added all the demos from the red book and the two big demos as well.
 *
 *
 */

package com.hermetica.magician.demos;

import java.awt.*;
import java.util.*;

import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class bootFrame extends Frame {

    /** A hashtable of all the demos we have, keyed on classname */
    private Hashtable stdClassHash = new Hashtable();
    private Hashtable redbookClassHash = new Hashtable();
    private Hashtable bigClassHash = new Hashtable();
    private Hashtable mesaClassHash = new Hashtable();
    private Hashtable glutClassHash = new Hashtable();

    /** The list containing the demo names */
    private java.awt.List demoList = null;

    /** The text area containing the description of the demo */
    private TextArea descriptionField = null;

    /** The classname that we have currently selected */
    private String selectedClassName = null;

    /** The group of radio buttons signifying which demo list we want to see */
    private CheckboxGroup demoListCheckboxGroup = new CheckboxGroup();

    /** Checkboxes to select the various lists of demos */
    Checkbox stdCheckbox = null;
    Checkbox redbookCheckbox = null;
    Checkbox bigCheckbox = null;
    Checkbox mesaCheckbox = null;
    Checkbox glutCheckbox = null;

    /** The dimensions of the texture ( auto-generated ) */
    private int arcaneLogoWidth = 82;
    private int arcaneLogoHeight = 128;

    /** Component to render the Arcane Technologies logo onto */
/*    TextureComponent logoComponent =
        new TextureComponent( arcaneLogoData, arcaneLogoWidth, arcaneLogoHeight );
*/
    /** The button component that you click to run the demo */
    Button runButton = null;

    /** Create a new boot frame */
    public bootFrame() {

        /** Set the LayoutManager for this frame */
        setLayout( new GridBagLayout() );
        setBackground( Color.white );
        setTitle( "Magician Demo Harness" );

        /** Create the menu bar */
        MenuBar mb = new MenuBar();
        Menu fileMenu = new Menu( "File" );
        MenuItem exitItem = new MenuItem( "Exit" );
        fileMenu.add( exitItem );
        mb.add( fileMenu );

        Menu aboutMenu = new Menu( "Help" );
        MenuItem magicianItem = new MenuItem( "About Magician" );
        magicianItem.disable();
        aboutMenu.add( magicianItem );
        aboutMenu.add( new MenuItem( "OpenGL Information" ) );
        mb.setHelpMenu( aboutMenu );
        setMenuBar( mb );

        /** Create a title bar */
        Font times18 = new Font( "TimesRoman", Font.BOLD, 18 );
        Label titleLabel = new Label( "Magician Demo Harness" );
        titleLabel.setFont( times18 );

        /** Create a side panel containing the company bumph */
        Panel sidePanel = new Panel();
        sidePanel.setLayout( new GridBagLayout() );

        /** Create the checkboxes for demo group selection */
        stdCheckbox = 
            new Checkbox( "Standard Demos", demoListCheckboxGroup, true );
        redbookCheckbox =
            new Checkbox( "``Red Book'' Demos", demoListCheckboxGroup, false );
        bigCheckbox =
            new Checkbox( "``Big'' Demos", demoListCheckboxGroup, false );
        mesaCheckbox =
            new Checkbox( "``Mesa'' Demos", demoListCheckboxGroup, false );
        glutCheckbox =
            new Checkbox( "GLUT Demos", demoListCheckboxGroup, false );

        /** Add the side panel widgets in */
/*        constrain.constrain( this, logoComponent, 0, 1, 1, 1,
                             GridBagConstraints.NONE,
                             GridBagConstraints.CENTER,
                             0, 0, 0, 0, 0, 0 ); */
        constrain.constrain( sidePanel, stdCheckbox, 0, 0, 1, 1 );
        constrain.constrain( sidePanel, redbookCheckbox, 0, 1, 1, 1 );
        constrain.constrain( sidePanel, bigCheckbox, 0, 2, 1, 1 );
        constrain.constrain( sidePanel, mesaCheckbox, 0, 3, 1, 1 );
        constrain.constrain( sidePanel, glutCheckbox, 0, 4, 1, 1 );

        /** Create a panel that will contain the list of demos */
        Panel demoPanel = new Panel();
        demoList = new java.awt.List( 20, false );
        demoPanel.add( demoList );

        /** Create a panel that will contain the textual description info on the demo */
        Panel descriptionPanel = new Panel();
        descriptionPanel.setLayout( new GridBagLayout() );
        descriptionField = new TextArea( 15, 60 );
        descriptionField.setEditable( false );
        constrain.constrain( descriptionPanel, descriptionField,
                             0, 0, 1, 2 );
        runButton = new Button( "Run This Demo!" );
        runButton.setBackground( Color.white );
        runButton.disable();
        constrain.constrain( descriptionPanel, runButton, 0, 2, 1, 1,
                             GridBagConstraints.HORIZONTAL,
                             GridBagConstraints.CENTER,
                             1.0, 1.0, 0, 0, 0, 0 );

        /** Create a panel containing the copyright blurb */
        Panel copyrightPanel = new Panel();
        copyrightPanel.setLayout( new GridBagLayout() );
        Font times6 = new Font( "TimesRoman", Font.BOLD, 6 );
        Label copyrightLabel = new Label( "Copyright (c)1997-1998 Arcane Technologies Ltd." );
        copyrightLabel.setBackground( Color.white );
        constrain.constrain( copyrightPanel, copyrightLabel, 0, 0, 1, 1 );

        /** Layout each panel in the frame */
        constrain.constrain( this, titleLabel, 0, 0, 3, 1,
                             GridBagConstraints.NONE,
                             GridBagConstraints.CENTER,
                             0, 0, 0, 0, 0, 0 );
        constrain.constrain( this, sidePanel, 0, 2, 1, 2 );
        constrain.constrain( this, demoPanel, 1, 1, 1, 3 );
        constrain.constrain( this, descriptionPanel, 2, 1, 1, 2 );
        constrain.constrain( this, copyrightPanel, 0, 4, 3, 1,
                             GridBagConstraints.NONE,
                             GridBagConstraints.CENTER,
                             1.0, 1.0, 0, 0, 0, 0 );

        pack();
        show();

        /** Create the hashtable of classes */
        createHashes();

        /** Display all the classes */
        for ( Enumeration classEnum = stdClassHash.keys() ;
              classEnum.hasMoreElements() ; ) {
            String key = (String)classEnum.nextElement();
            demoList.addItem( key );
          }
      }

    /** Handles AWT Events on this frame and its subcomponents */
    public boolean handleEvent( Event evt ) {
        switch ( evt.id ) {
            case Event.KEY_PRESS:
                switch ( evt.key ) {
                    case 27:
                        System.exit( 0 );
                  }
                break;
            case Event.LIST_SELECT:
            case Event.ACTION_EVENT:
                /** Check to see if we've selected a list item */
                if ( evt.target instanceof java.awt.List ) {
                    String className = null;

                    if ( runButton.isEnabled() == false ) {
                        runButton.enable();
                      }

                    /** Get the classname for this demo */
                    if ( demoListCheckboxGroup.getCurrent() == 
                            stdCheckbox ) {
                        className = 
                            (String)stdClassHash.get( demoList.getSelectedItem() );
                      } else { 
                        if ( demoListCheckboxGroup.getCurrent() ==
                                redbookCheckbox ) {
                            className =
                                (String)redbookClassHash.get( demoList.getSelectedItem() );
                          } else {
                            if ( demoListCheckboxGroup.getCurrent() ==
                                    bigCheckbox ) {
                                className =
                                    (String)bigClassHash.get( demoList.getSelectedItem() );
                              } else {
                                if ( demoListCheckboxGroup.getCurrent() ==
                                        mesaCheckbox ) {
                                    className =
                                        (String)mesaClassHash.get( demoList.getSelectedItem() );
                                  } else {
                                    if ( demoListCheckboxGroup.getCurrent() ==
                                            glutCheckbox ) {
                                        className =
                                            (String)glutClassHash.get( demoList.getSelectedItem() );
                                      }
                                  }
                              }
                          }
                      }

                    if ( className == null ) {
                        return false;
                      }

                    /** Instantiate a new class of the given name */
                    try {
                        demoTemplate tmpObject = 
                            (demoTemplate)( Class.forName( className ) ).newInstance();
                        selectedClassName = new String( className );

                        /** Copy the details of this demo into the appropriate windoze */
                        StringBuffer sb = new StringBuffer();
                        sb.append( demoList.getSelectedItem() + " - " +
                                   tmpObject.getOriginalAuthor() + "\n\n" );
                        sb.append( tmpObject.getDescription() + "\n\n" );
                        sb.append( tmpObject.getVersion() + "\n" );
                        descriptionField.setText( sb.toString() );
                        runButton.setLabel( "Run ``" + demoList.getSelectedItem() + "''" );
                      } catch ( ClassNotFoundException e ) {
                        e.printStackTrace();
                      } catch ( InstantiationException e2 ) {
                        e2.printStackTrace();
                      } catch ( IllegalAccessException e3 ) {
                        e3.printStackTrace();
                      }

                    return true;
                  }

                /** Check to see if we've hit the ``Run'' button */
                if ( evt.arg.toString().startsWith( "Run" ) ) {

                    /** Instantiate a new class of the given name */
                    try {
                        demoTemplate tmpObject = 
                            (demoTemplate)( Class.forName( selectedClassName ) ).newInstance();
                        tmpObject.setExit( false );
                        tmpObject.startDemo();

                        /** Set the title of the window */
                        ((Frame)tmpObject).setTitle( demoList.getSelectedItem() );
                      } catch ( ClassNotFoundException e ) {
                        e.printStackTrace();
                      } catch ( InstantiationException e2 ) {
                        e2.printStackTrace();
                      } catch ( IllegalAccessException e3 ) {
                        e3.printStackTrace();
                      }
                    return true;
                  }

                /**
                 * Check to see whether it is one of the ``Help'' menu options
                 */
                if ( evt.arg.toString().equals( "OpenGL Information" ) ) {
                    bootOpenGLFrame of = new bootOpenGLFrame();
                    return true;
                  }

                if ( evt.arg.toString().equals( "Exit" ) ) {
                    System.exit( 0 );
                  }

                /** 
                 * Check to see whether its a Checkbox, and, if it is,
                 * clear the list and add the entries for that selected
                 * section of demos
                 */
                if ( evt.target instanceof Checkbox ) {

                    /** Clear the demo list */
                    /** 
                     * This is a monster hack because list.clear() generally
                     * throws exceptions under MSVM. Therefore, we need to
                     * continually catch exceptions and call list.clear()
                     * until no exceptions are thrown. At this point, the
                     * list is probably clear.....*sheesh*!
                     */
                    boolean exceptionCaught = true;
                    while ( exceptionCaught ) {
                        try {
                            demoList.clear();
                            exceptionCaught = false;
                          } catch ( Exception e ) {
                            exceptionCaught = true;
                          }
                      }

                    /** Add the demos from the selected hash */
                    if ( evt.target == stdCheckbox ) {
                        for ( Enumeration classEnum = stdClassHash.keys() ;
                                classEnum.hasMoreElements() ; ) {
                            String key = (String)classEnum.nextElement();
                            demoList.addItem( key );
                          }
                      } else {
                        if ( evt.target == bigCheckbox ) {
                            for ( Enumeration classEnum = bigClassHash.keys() ;
                                    classEnum.hasMoreElements() ; ) {
                                String key = (String)classEnum.nextElement();
                                demoList.addItem( key );
                              }
                          } else {
                            if ( evt.target == redbookCheckbox ) {
                                for ( Enumeration classEnum = redbookClassHash.keys() ;
                                        classEnum.hasMoreElements() ; ) {
                                    String key = (String)classEnum.nextElement();
                                    demoList.addItem( key );
                                  }
                              } else {
                                if ( evt.target == mesaCheckbox ) {
                                    for ( Enumeration classEnum = mesaClassHash.keys() ;
                                            classEnum.hasMoreElements() ; ) {
                                        String key = (String)classEnum.nextElement();
                                        demoList.addItem( key );
                                      }
                                  } else {
                                    if ( evt.target == glutCheckbox ) {
                                        for ( Enumeration classEnum = glutClassHash.keys() ;
                                                classEnum.hasMoreElements() ; ) {
                                            String key = (String)classEnum.nextElement();
                                            demoList.addItem( key );
                                          }
                                      }
                                  }
                              }
                          }
                      }

                    return true;
                  }

                return false;
          }
        return false;
      }

    /** Creates a hashtable of classnames and meaningful names */
    private void createHashes() {
        /** Standard top-level demos */
        stdClassHash.put( new String( "Simple OpenGL Test" ),
                          new String( "com.hermetica.magician.demos.testGL" ) );
        stdClassHash.put( new String( "Shared Display Lists" ),
                          new String( "com.hermetica.magician.demos.sharedContext" ) );
        stdClassHash.put( new String( "MTPaperplane" ),
                          new String( "com.hermetica.magician.demos.mtpaperplane" ) );
        stdClassHash.put( new String( "Multiple Views" ),
                          new String( "com.hermetica.magician.demos.multiView" ) );
        stdClassHash.put( new String( "Solar System" ),
                          new String( "com.hermetica.magician.demos.solarSystem" ) );
        stdClassHash.put( new String( "Throwing Shapes!" ),
                          new String( "com.hermetica.magician.demos.throwShapes" ) );
        stdClassHash.put( new String( "USS Enterpoop" ),
                          new String( "com.hermetica.magician.demos.enterprise" ) );
/*        stdClassHash.put( new String( "Image Copying" ),
                          new String( "com.hermetica.magician.demos.imageCopier" ) ); */
        stdClassHash.put( new String( "Triangle File Object Viewer" ),
                          new String( "com.hermetica.magician.demos.objViewer" ) );
        stdClassHash.put( new String( "RGB Colour Cube" ),
                          new String( "com.hermetica.magician.demos.ColorCube" ) );
        stdClassHash.put( new String( "Tentacle" ),
                          new String( "com.hermetica.magician.demos.tentacle" ) );
/*        stdClassHash.put( new String( "Maze Game" ),
                          new String( "com.hermetica.magician.demos.quest" ) ); */

        /** Big demos! */
        bigClassHash.put( new String( "Atlantis" ),
                          new String( "com.hermetica.magician.demos.atlantis.atlantis" ) );
        bigClassHash.put( new String( "OpenGL Logo" ),
                          new String( "com.hermetica.magician.demos.openGL.logo" ) );
/*        bigClassHash.put( new String( "Stonehenge" ),
                          new String( "com.hermetica.magician.demos.stonehenge.stonehenge" ) ); */
        bigClassHash.put( new String( "Ideas in Motion" ),
                          new String( "com.hermetica.magician.demos.ideas.track" ) );
        bigClassHash.put( new String( "Lorenz Attractors" ),
                          new String( "com.hermetica.magician.demos.lorenz.lorenz" ) );
        bigClassHash.put( new String( "Molecule Viewer" ),
                          new String( "com.hermetica.magician.demos.molview.molview" ) );
        bigClassHash.put( new String( "Colour Editor" ),
                          new String( "com.hermetica.magician.demos.colEdit.colEdit" ) );
        bigClassHash.put( new String( "Heads-Up Display" ),
                          new String( "com.hermetica.magician.demos.hud.hud" ) );

        /** Redbook demos */
        redbookClassHash.put( new String( "Double-Buffer(I)" ),
                              new String( "com.hermetica.magician.demos.redbook.doubleb" ) );
        redbookClassHash.put( new String( "Cube" ),
                              new String( "com.hermetica.magician.demos.redbook.cube" ) );
        redbookClassHash.put( new String( "Polygon Stipples" ),
                              new String( "com.hermetica.magician.demos.redbook.polys" ) );
        redbookClassHash.put( new String( "Line Effects" ),
                              new String( "com.hermetica.magician.demos.redbook.lines" ) );
        redbookClassHash.put( new String( "Simple Transformations" ),
                              new String( "com.hermetica.magician.demos.redbook.model" ) );
        redbookClassHash.put( new String( "Clipping planes" ),
                              new String( "com.hermetica.magician.demos.redbook.clip" ) );
        redbookClassHash.put( new String( "Solar System" ),
                              new String( "com.hermetica.magician.demos.redbook.planet" ) );
        redbookClassHash.put( new String( "Robot Arm" ),
                              new String( "com.hermetica.magician.demos.redbook.robot" ) );
        redbookClassHash.put( new String( "Viewport Unprojection!" ),
                              new String( "com.hermetica.magician.demos.redbook.unproject" ) );
        redbookClassHash.put( new String( "Colour Interpolation" ),
                              new String( "com.hermetica.magician.demos.redbook.smooth" ) );
/**        redbookClassHash.put( new String( "Lit Sphere" ),
                              new String( "com.hermetica.magician.demos.redbook.light" ) ); */
        redbookClassHash.put( new String( "Moving Light" ),
                              new String( "com.hermetica.magician.demos.redbook.movelight" ) );
        redbookClassHash.put( new String( "Materials" ),
                              new String( "com.hermetica.magician.demos.redbook.material" ) );
/**        redbookClassHash.put( new String( "Changing Materials" ),
                              new String( "com.hermetica.magician.demos.redbook.colormat" ) ); */
        redbookClassHash.put( new String( "Alpha Blending" ),
                              new String( "com.hermetica.magician.demos.redbook.alpha" ) );
        redbookClassHash.put( new String( "3D Alpha Blending" ),
                              new String( "com.hermetica.magician.demos.redbook.alpha3D" ) );
        redbookClassHash.put( new String( "Fog" ),
                              new String( "com.hermetica.magician.demos.redbook.fog" ) );
        redbookClassHash.put( new String( "Polygon Offset" ),
                              new String( "com.hermetica.magician.demos.redbook.polyoff" ) );
        redbookClassHash.put( new String( "Torus Display List" ),
                              new String( "com.hermetica.magician.demos.redbook.torus" ) );
        redbookClassHash.put( new String( "Display Lists" ),
                              new String( "com.hermetica.magician.demos.redbook.list" ) );
        redbookClassHash.put( new String( "Stroked Fonts" ),
                              new String( "com.hermetica.magician.demos.redbook.stroke" ) );
        redbookClassHash.put( new String( "Bitmap Character Rendering" ),
                              new String( "com.hermetica.magician.demos.redbook.drawf" ) );
/*        redbookClassHash.put( new String( "Image Manipulation" ),
                              new String( "com.hermetica.magician.demos.redbook.image" ) ); */
        redbookClassHash.put( new String( "Checkerboards!" ),
                              new String( "com.hermetica.magician.demos.redbook.checker" ) );
        redbookClassHash.put( new String( "Texture Sub-Images" ),
                              new String( "com.hermetica.magician.demos.redbook.texsub" ) );
        redbookClassHash.put( new String( "Mipmapping" ),
                              new String( "com.hermetica.magician.demos.redbook.mipmap" ) );
        redbookClassHash.put( new String( "Texture Binding" ),
                              new String( "com.hermetica.magician.demos.redbook.texbind" ) );
        redbookClassHash.put( new String( "Stencilling" ),
                              new String( "com.hermetica.magician.demos.redbook.stencil" ) );
        redbookClassHash.put( new String( "Selection Buffer" ),
                              new String( "com.hermetica.magician.demos.redbook.select" ) );
        redbookClassHash.put( new String( "Picking" ),
                              new String( "com.hermetica.magician.demos.redbook.picksquare" ) );
        redbookClassHash.put( new String( "Feedback Buffer" ),
                              new String( "com.hermetica.magician.demos.redbook.feedback" ) );
        redbookClassHash.put( new String( "GLU Quadrics" ),
                              new String( "com.hermetica.magician.demos.redbook.quadric" ) );
        redbookClassHash.put( new String( "Bezier Curves ( 1D )" ),
                              new String( "com.hermetica.magician.demos.redbook.bezcurve" ) );
        redbookClassHash.put( new String( "Bezier Surfaces ( 2D )" ),
                              new String( "com.hermetica.magician.demos.redbook.bezsurf" ) );
        redbookClassHash.put( new String( "Vertex Arrays" ),
                              new String( "com.hermetica.magician.demos.redbook.varray" ) );
        redbookClassHash.put( new String( "Contour Mapping" ),
                              new String( "com.hermetica.magician.demos.redbook.texgen" ) );
        redbookClassHash.put( new String( "Bezier Mesh" ),
                              new String( "com.hermetica.magician.demos.redbook.bezmesh" ) );
        redbookClassHash.put( new String( "Textured Bezier Surface" ),
                              new String( "com.hermetica.magician.demos.redbook.texturesurf" ) );
        redbookClassHash.put( new String( "Teapots!" ),
                              new String( "com.hermetica.magician.demos.redbook.teapots" ) );
        redbookClassHash.put( new String( "Accumulation Perspective" ),
                              new String( "com.hermetica.magician.demos.redbook.accpersp" ) );
        redbookClassHash.put( new String( "Accumulation Orthographic" ),
                              new String( "com.hermetica.magician.demos.redbook.accanti" ) );
        redbookClassHash.put( new String( "Depth of Field" ),
                              new String( "com.hermetica.magician.demos.redbook.dof" ) );
        redbookClassHash.put( new String( "Ambiently Lit Scene" ),
                              new String( "com.hermetica.magician.demos.redbook.scene" ) );
        redbookClassHash.put( new String( "NURBS Surface" ),
                              new String( "com.hermetica.magician.demos.redbook.surface" ) );

        /** The demos bundled with Mesa */
        mesaClassHash.put( new String( "Reflections" ),
                           new String( "com.hermetica.magician.demos.mesa.reflect" ) );
        mesaClassHash.put( new String( "Isosurf" ),
                           new String( "com.hermetica.magician.demos.mesa.isosurf" ) );
        mesaClassHash.put( new String( "Texture Objects" ),
                           new String( "com.hermetica.magician.demos.mesa.texobj" ) );
        mesaClassHash.put( new String( "Hidden-Lines with Polygon Offset" ),
                           new String( "com.hermetica.magician.demos.mesa.offset" ) );
        mesaClassHash.put( new String( "Depth Buffer Test" ),
                           new String( "com.hermetica.magician.demos.mesa.depth" ) );
        mesaClassHash.put( new String( "Olympic Rings" ),
                           new String( "com.hermetica.magician.demos.mesa.olympic" ) );
        mesaClassHash.put( new String( "Gears" ),
                           new String( "com.hermetica.magician.demos.mesa.gears" ) );

        /** GLUT demos */
        glutClassHash.put( new String( "Spotlights" ),
                           new String( "com.hermetica.magician.demos.glut.spots" ) );
        glutClassHash.put( new String( "Fog" ),
                           new String( "com.hermetica.magician.demos.glut.fog" ) );
        glutClassHash.put( new String( "Dinosaur" ),
                           new String( "com.hermetica.magician.demos.glut.dinoball" ) );
        glutClassHash.put( new String( "Shadowy Leapin' Lizards!" ),
                           new String( "com.hermetica.magician.demos.glut.dinoshade" ) );
        glutClassHash.put( new String( "Tesselator Demo" ),
                           new String( "com.hermetica.magician.demos.glut.tessdemo" ) );
        glutClassHash.put( new String( "Bitmapped Fonts" ),
                           new String( "com.hermetica.magician.demos.glut.bitfont" ) );
        glutClassHash.put( new String( "Font Demo" ),
                           new String( "com.hermetica.magician.demos.glut.fontdemo" ) );
        glutClassHash.put( new String( "Traffic Signal" ),
                           new String( "com.hermetica.magician.demos.glut.signal" ) );
      }

    public static void main( String argv[] ) {
        bootFrame bf = new bootFrame();
      }
  }
