/**
 * $Id: bootOpenGLFrame.java,v 1.9 1998/11/02 22:39:54 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Displays information about the version of OpenGL currently running
 *
 * $Log: bootOpenGLFrame.java,v $
 * Revision 1.9  1998/11/02 22:39:54  descarte
 * Added ActionListener to the ``Dismiss'' button....
 *
 * Revision 1.8  1998/10/19 00:38:29  descarte
 * Made compliant with JDK-1.1. Removed all deprecated API methods.
 *
 * Revision 1.7  1998/03/28 12:35:53  descarte
 * Upgraded demo to GLComponentFactory and Java 1.1
 *
 * Revision 1.6  1998/03/22 20:44:16  descarte
 * Converted to new GLComponentFactory syntax.
 *
 * Revision 1.5  1998/02/23 17:26:05  descarte
 * Converted across to new GLComponent syntax.
 *
 * Revision 1.3  1997/11/20 21:03:26  descarte
 * Fixed layout problem on extensions box.
 *
 * Revision 1.2  1997/11/07 18:59:58  descarte
 * Altered demo to correctly unlock the GL context.
 *
 * Revision 1.1  1997/09/09 09:23:57  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos;

import com.hermetica.magician.*;
import com.hermetica.util3d.*;
import java.awt.*;
import java.awt.event.*;

public class bootOpenGLFrame extends Frame implements GLEventListener, WindowListener, ActionListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** A ``hidden'' GLComponent */
    private GLComponent glc = null;

    /** The information on the OpenGL implementation */
    private String vendor = null;
    private String renderer = null;
    private String version = null;
    private String extensions = null;

    /** Text area for extensions */
    TextArea resultsArea = null;

    /** The labels containing the results of the context query */
    Label[] results = new Label[3];

    /** Creates a default OpenGL info frame */
    public bootOpenGLFrame() {

        /** Sets up the basic layout rules */
        setLayout( new GridBagLayout() );
        setBackground( Color.white );
        setTitle( "OpenGL Information" );

        /** Creates the ``hidden'' GLComponent */
        glc = GLComponentFactory.createGLComponent( 2, 2 );

        /** Setup the capabilities of the OpenGL context */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setRedBits( 1 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Big title */
        Font times18 = new Font( "TimesRoman", Font.BOLD, 18 );
        Label titleLabel = new Label( "OpenGL Information" );
        titleLabel.setFont( times18 );
        titleLabel.setBackground( Color.white );

        /** The information panel containing 4 items of information, 2 labels each */
        Panel infoPanel = new Panel();
        infoPanel.setLayout( new GridBagLayout() );

        Label[] titles = new Label[4];
        titles[0] = new Label( "Vendor:" );
        titles[1] = new Label( "Renderer:" );
        titles[2] = new Label( "Version:" );
        titles[3] = new Label( "Extensions:" );

        String space = new String( "                                " );

        results[0] = new Label( "Unknown" + space );
        results[1] = new Label( "Unknown" + space );
        results[2] = new Label( "Unknown" + space );

        resultsArea = new TextArea( 5, 20 );
        resultsArea.setEditable( false );

        constrain.constrain( infoPanel, titles[0], 0, 0, 1, 1 );
        constrain.constrain( infoPanel, titles[1], 0, 1, 1, 1 );
        constrain.constrain( infoPanel, titles[2], 0, 2, 1, 1 );
        constrain.constrain( infoPanel, titles[3], 0, 3, 1, 1 );

        constrain.constrain( infoPanel, results[0], 1, 0, 1, 1 );
        constrain.constrain( infoPanel, results[1], 1, 1, 1, 1 );
        constrain.constrain( infoPanel, results[2], 1, 2, 1, 1 );
        constrain.constrain( infoPanel, resultsArea, 1, 3, 1, 1 );

        /** Big dismiss button */
        Button dismissButton = new Button( "Dismiss" );
        dismissButton.addActionListener( this );

        /** Layout the window */
        constrain.constrain( this, titleLabel, 0, 0, 2, 1,
                             GridBagConstraints.NONE,
                             GridBagConstraints.CENTER,
                             0, 0, 0, 0, 0, 0 );
//        constrain.constrain( this, logoComponent, 0, 1, 1, 1 );
        constrain.constrain( this, glc, 0, 2, 1, 1 );
        constrain.constrain( this, infoPanel, 1, 1, 1, 1 );
        constrain.constrain( this, dismissButton, 0, 2, 2, 1,
                             GridBagConstraints.HORIZONTAL,
                             GridBagConstraints.CENTER,
                             0, 0, 0, 0, 0, 0 );

        /** Pack and display */
        pack();
        show();

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Add the window and action event listeners */
        this.addWindowListener( this );
//        this.addActionListener( this );

        /** Tell the component to start processing events */
        glc.initialize();
      }

    public void initialize( GLDrawable component ) {
        /** Grab the information from the context */
        try {
            vendor = gl_.glGetString( GL.GL_VENDOR );
            renderer = gl_.glGetString( GL.GL_RENDERER );
            version = gl_.glGetString( GL.GL_VERSION );
            extensions = gl_.glGetString( GL.GL_EXTENSIONS );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }

        if ( vendor != null ) {
            results[0].setText( vendor );
          }
        if ( renderer != null ) {
            results[1].setText( renderer );
          }
        if ( version != null ) {
            results[2].setText( version );
          }
        if ( extensions != null ) {

            /** Split each extension into a different line.... */
            char[] tmpArray = extensions.toCharArray();

            for ( int i = 0 ; i < tmpArray.length ; i++ ) {
                if ( tmpArray[i] == ' ' ) {
                    tmpArray[i] = '\n';
                  }
              }
            String tmpExtensions = new String( tmpArray );
            resultsArea.setText( tmpExtensions );
          }

        /** Re-pack the Components, since the sizes of the labels may have changed */
        pack();
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
      }

    /** Handles refreshing */
    public void display( GLDrawable component ) {
      }

    /** Returns a valid GL pipeline */
    public GL getGL() {
        return gl_;
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
      }

    public void windowIconified( WindowEvent evt ) {
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** AWT Event Handler */
    public void actionPerformed( ActionEvent evt ) {
        if ( evt.getActionCommand().equals( "Dismiss" ) ) {
            glc.destroy();
            dispose();
          }
      }
  }
