/**
 * $Id: colEdit.java.stub,v 1.10 1998/10/19 10:45:34 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Test program to show off glReadPixels() and stuff
 *
 * $Log: colEdit.java.stub,v $
 * Revision 1.10  1998/10/19 10:45:34  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.9  1998/10/02 12:12:05  descarte
 * Changed GLComponentListener to GLEventListener
 * Altered arguments to reshape() method
 *
 * Revision 1.8  1998/04/21 14:06:37  descarte
 * Added new demoTemplate methods
 * Added WindowListener
 * Added standalone boot
 *
 * Revision 1.7  1998/03/30 12:29:58  descarte
 * Removed private swatchListener class for applet support.
 *
 * Revision 1.6  1998/03/28 15:08:08  descarte
 * Upgraded demo to Java 1.1 listeners.
 *
 * Revision 1.5  1998/03/24 16:03:56  descarte
 * Converted to new GLDrawable interface.
 * Made GLContext method changes.
 *
 * Revision 1.4  1998/03/19 22:07:07  descarte
 * Added getGL() methods to both listeners.
 *
 * Revision 1.3  1998/03/16 13:50:32  descarte
 * Converted demo from old beta 4 GLComponent syntax.
 *
 * Revision 1.2  1998/03/03 21:02:34  descarte
 * A bit more fiddling to test out byte arrays. Changed back to float arrays
 * for this checkin though.
 *
 *
 */

package com.hermetica.magician.demos.colEdit;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class colEdit extends Frame implements demoTemplate, GLEventListener, ActionListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent swatch = null;
    swatchListener sListener = new swatchListener();
    GLComponent torusView = null;

    /** A button to dismiss the demo */
    Button dismissButton = new Button( "Dismiss" );

    /** The dimensions of the demo */
    private int WIDTH = 200,
                HEIGHT = 200;

    /** The currently picked pixel colour */
    float[] pixels = new float[3];

    /** Rotation value */
    private float spin = 0.0f;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        colEdit d = new colEdit();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public colEdit() {
        gl_ = coregl_;
        glu_ = coreglu_;
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Colour Editor" );
      }

    public String getVersion() {
        return new String( "$Id: colEdit.java.stub,v 1.10 1998/10/19 10:45:34 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of multiple Magician\n" );
        sb.append( "components being used in a GUI to affect each others'\n" );
        sb.append( "rendering style. The colour swatch on the right-hand\n" );
        sb.append( "can be clicked on to select the colour that the spinning\n" );
        sb.append( "torus will be painted with.\n\n" );
        sb.append( "To quit this demo, click the ``Dismiss'' button." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH * 3;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT * 2;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Alligator Descartes <descarte@arcana.co.uk>" );
      }

    public void startDemo() {
        setLayout( new GridBagLayout() );
        setBackground( Color.black );
        dismissButton.setForeground( Color.white );

        /** Create the OpenGL Component for the swatch */
        swatch = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Create the OpenGL Component for the cone view */
        torusView = GLComponentFactory.createGLComponent( WIDTH * 2, HEIGHT * 2 );

        constrain.constrain( this, torusView, 0, 0, 1, 2,
                             GridBagConstraints.BOTH, 
                             GridBagConstraints.CENTER,
                             1.0f, 1.0f, 0, 0, 0, 0 );

        constrain.constrain( this, swatch, 1, 0, 1, 1,
                             GridBagConstraints.NONE,
                             GridBagConstraints.CENTER,
                             1.0f, 1.0f, 0, 0, 0, 0 );
        constrain.constrain( this, dismissButton, 0, 2, 2, 1,
                             GridBagConstraints.HORIZONTAL,
                             GridBagConstraints.CENTER,
                             1, 1, 0, 0, 0, 0 );
        pack(); show();

        /** Setup the context capabilities for the swatch */
        GLCapabilities cap = swatch.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Setup the context capabilities for the torus view */
        torusView.getContext().setCapabilities( cap );

        /** Register a listener with the component */
        swatch.addGLEventListener( sListener );
        torusView.addGLEventListener( this );

        this.addWindowListener( this );

        /** Register mouse events with the swatch */
        swatch.addMouseListener( sListener );

        /** Register an action listener with the dismiss button */
        dismissButton.addActionListener( this );

        /** Get the component to start processing events */
        swatch.initialize();

        /** Start the torus view up and tell it to continuously refresh */
        torusView.initialize();
        torusView.start();
      }

    public void initialize( GLDrawable component ) {

        /** Setup the lighting */
        float[] position = { 0.0f, 0.0f, 5.0f };
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position );

        /** Enable z-buffering */
        gl_.glEnable( GL.GL_DEPTH_TEST );
      }

    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0, 1.0, -1.0, 1.0, 1.0, 1000 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }
        
    public void display( GLDrawable component ) {
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        gl_.glPushMatrix();
            /** Move and rotate the torus */
            gl_.glTranslatef( 0.0f, 0.0f, -15 );
            gl_.glRotatef( spin, 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( spin, 0.0f, 1.0f, 0.0f );
            gl_.glRotatef( spin, 0.0f, 0.0f, 1.0f );

            /** Get the colour from the swatch and draw the torus */
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, 
                              sListener.getPixels() );
            shapes.solidTorus( 2.0f, 5.0f, 16, 16 );
        gl_.glPopMatrix();

        /** Increase the spin */
        spin++;
        if ( spin > 360.0f ) {
            spin -= 360.0f;
          }
      }

    /** Returns a GL pipeline to the GLEventListener */
    public GL getGL() {
        return gl_;
      }

    /** Handles action events */
    public void actionPerformed( ActionEvent evt ) {
        if ( evt.getActionCommand().equals( "Dismiss" ) ) {
            torusView.destroy();
            swatch.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

            return;
          }
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        torusView.destroy();
        swatch.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( torusView.isRunning() ) {
            torusView.resume();
          }
        if ( swatch.isRunning() ) {
            swatch.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( torusView.isRunning() ) {
            torusView.suspend();
          }
        if ( swatch.isRunning() ) {
            swatch.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( torusView.isRunning() ) {
            torusView.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( torusView.isRunning() ) {
            torusView.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        torusView.destroy();
        swatch.destroy();
      }
  }
