/**
 * $Id: enterprise.java.stub,v 1.6 1998/10/31 16:47:50 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Program that shows the USS Enterpoop chasing a bad guy about. This is
 * a conversion from Stan Melax's ``fun'' example bundled with the original
 * perl OpenGL module.
 *
 * $Log: enterprise.java.stub,v $
 * Revision 1.6  1998/10/31 16:47:50  descarte
 * Removed component and context tracing.
 *
 * Revision 1.5  1998/10/31 16:46:48  descarte
 * Made default pipeline CoreGL again...
 *
 * Revision 1.4  1998/10/31 16:45:16  descarte
 * Made some alterations since enabling texturing on a polygon with no
 * valid texture causes a crash with the latest version of Mesa.
 *
 * Revision 1.3  1998/10/19 00:53:03  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.2  1998/10/02 11:30:31  descarte
 * Changed GLComponentListener to GLEventListener
 * Altered arguments to reshape() method
 *
 * Revision 1.1  1998/04/23 14:17:09  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class enterprise extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();
    TraceGL tracegl_ = new TraceGL( coregl_ );

    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();
    TraceGLU traceglu_ = new TraceGLU( coreglu_ );

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** Display list identifiers */
    private int borgList = -1,
                planeList = -1,
                floorList = -1,
                enterpriseList = -1;

    /** Angle of spin of the scene */
    private float angle = 0.0f;

    /** Variables for the world */
    private final float WORLD_BOUNDARY = 20.0f;
    private final float WORLD_CEILING = 10.0f;
    private final float WORLD_FLOOR = 0.0f;

    /** Variables for the target */
    private float tDamping = 0.99f;
    private float tX = 10.0f;
    private float tY = 5.0f;
    private float tZ = 0.0f;
    private float tDx = 0.4f;
    private float tDy = 0.0f;
    private float tDz = 0.0f;

    /** Variables for the enterpoop */
    private final float DAMPING = 0.98f;
    private final float ACCELERATION = 0.01f;
    private final float BANK_DAMPING = 0.8f;
    private float eDamping = 0.98f;
    private float eAcceleration = 0.01f;
    private float eBankDamping = 0.8f;
    private float eX = -10.0f;
    private float eY = 0.0f;
    private float eZ = 5.0f;
    private float eDx = 0.0f;
    private float eDy = 0.0f;
    private float eDz = 0.0f;
    private float eH = 0.0f;
    private float eDh = 0.0f;
    private float eSpeed = 0.0f;

    /** Vertex data for the enterprise */
    double[][] enterpriseData = {
        { -60.000000, 50.000004, 10.508476 },
        { -20.000000, 50.000000, -9.830511 },
        { -20.000000, 60.000000, -9.830509 },
        { -60.000000, 60.000004, 10.508476 },
        { -60.000000, 50.000004, 0.508473 },
        { -20.000000, 50.000000, -19.830513 },
        { -20.000000, 60.000000, -19.830509 },
        { -60.000000, 60.000004, 0.508474 },
        { 20.000000, 50.000000, -9.836067 },
        { 60.000000, 50.000004, 10.491802 },
        { 60.000000, 60.000004, 10.491802 },
        { 20.000000, 60.000000, -9.836065 },
        { 20.000000, 50.000000, -19.836069 },
        { 60.000000, 50.000004, 0.491801 },
        { 60.000000, 60.000004, 0.491802 },
        { 20.000000, 60.000000, -19.836065 },
        { -70.000000, 20.000000, 10.000005 },
        { -60.000000, 20.000000, 10.000005 },
        { -60.000000, 120.000000, 9.999995 },
        { -70.000000, 120.000000, 9.999995 },
        { -70.000000, 20.000000, 0.000005 },
        { -60.000000, 20.000000, 0.000005 },
        { -60.000000, 120.000000, -0.000005 },
        { -70.000000, 120.000000, -0.000005 },
        { 60.000000, 20.000000, 10.000005 },
        { 70.000000, 20.000000, 10.000005 },
        { 70.000000, 120.000000, 9.999995 },
        { 60.000000, 120.000000, 9.999995 },
        { 60.000000, 20.000000, 0.000005 },
        { 70.000000, 20.000000, 0.000005 },
        { 70.000000, 120.000000, -0.000005 },
        { 60.000000, 120.000000, -0.000005 },
        { -20.000000, 0.000001, -9.999996 },
        { 20.000000, 0.000001, -9.999996 },
        { 20.000000, 90.000000, -10.000004 },
        { -20.000000, 90.000000, -10.000004 },
        { -20.000000, -0.000001, -39.999996 },
        { 20.000000, -0.000001, -39.999996 },
        { 20.000000, 90.000000, -40.000004 },
        { -20.000000, 90.000000, -40.000004 },
        { 0.000000, -50.000000, 0.000005 },
        { 0.000000, -50.000000, 10.000005 },
        { -15.450733, -2.447136, 0.000001 },
        { -15.450733, -2.447135, 10.000001 },
        { -29.389170, -9.549080, 0.000001 },
        { -29.389170, -9.549080, 10.000001 },
        { -40.450787, -20.610649, 0.000002 },
        { -40.450787, -20.610649, 10.000002 },
        { -47.552795, -34.549053, 0.000004 },
        { -47.552795, -34.549053, 10.000004 },
        { -50.000000, -49.999901, 0.000005 },
        { -50.000000, -49.999901, 10.000005 },
        { -47.552853, -65.450768, 0.000006 },
        { -47.552853, -65.450768, 10.000006 },
        { -40.450897, -79.389198, 0.000007 },
        { -40.450897, -79.389198, 10.000008 },
        { -29.389326, -90.450806, 0.000008 },
        { -29.389326, -90.450806, 10.000009 },
        { -15.450917, -97.552803, 0.000008 },
        { -15.450917, -97.552803, 10.000009 },
        { -0.000065, -100.000000, 0.000010 },
        { -0.000065, -100.000000, 10.000010 },
        { 15.450794, -97.552841, 0.000008 },
        { 15.450794, -97.552841, 10.000009 },
        { 29.389221, -90.450882, 0.000008 },
        { 29.389221, -90.450882, 10.000009 },
        { 40.450825, -79.389297, 0.000007 },
        { 40.450825, -79.389297, 10.000008 },
        { 47.552814, -65.450890, 0.000006 },
        { 47.552814, -65.450890, 10.000006 },
        { 50.000000, -50.000031, 0.000005 },
        { 50.000000, -50.000031, 10.000005 },
        { 47.552834, -34.549175, 0.000004 },
        { 47.552834, -34.549175, 10.000004 },
        { 40.450863, -20.610752, 0.000002 },
        { 40.450863, -20.610752, 10.000002 },
        { 29.389273, -9.549156, 0.000001 },
        { 29.389273, -9.549156, 10.000001 },
        { 0.000001, 0.000000, 0.000000 },
        { 15.450856, -2.447175, 0.000001 },
        { 15.450856, -2.447174, 10.000001 },
        { 0.000001, 0.000000, 10.000000 }
      };

    /** Enterprise polygon data */
    int[][] enterprisePolygons = {
        { 81, 80, 79 },
        { 81, 79, 78 },
        { 80, 77, 76 },
        { 80, 76, 79 },
        { 77, 75, 74 },
        { 77, 74, 76 },
        { 75, 73, 72 },
        { 75, 72, 74 },
        { 73, 71, 70 },
        { 73, 70, 72 },
        { 71, 69, 68 },
        { 71, 68, 70 },
        { 69, 67, 66 },
        { 69, 66, 68 },
        { 67, 65, 64 },
        { 67, 64, 66 },
        { 65, 63, 62 },
        { 65, 62, 64 },
        { 63, 61, 60 },
        { 63, 60, 62 },
        { 61, 59, 58 },
        { 61, 58, 60 },
        { 59, 57, 56 },
        { 59, 56, 58 },
        { 57, 55, 54 },
        { 57, 54, 56 },
        { 55, 53, 52 },
        { 55, 52, 54 },
        { 53, 51, 50 },
        { 53, 50, 52 },
        { 51, 49, 48 },
        { 51, 48, 50 },
        { 49, 47, 46 },
        { 49, 46, 48 },
        { 47, 45, 44 },
        { 47, 44, 46 },
        { 45, 43, 42 },
        { 45, 42, 44 },
        { 43, 81, 78 },
        { 43, 78, 42 },
        { 80, 81, 41 },
        { 77, 80, 41 },
        { 75, 77, 41 },
        { 73, 75, 41 },
        { 71, 73, 41 },
        { 69, 71, 41 },
        { 67, 69, 41 },
        { 65, 67, 41 },
        { 63, 65, 41 },
        { 61, 63, 41 },
        { 59, 61, 41 },
        { 57, 59, 41 },
        { 55, 57, 41 },
        { 53, 55, 41 },
        { 51, 53, 41 },
        { 49, 51, 41 },
        { 47, 49, 41 },
        { 45, 47, 41 },
        { 43, 45, 41 },
        { 81, 43, 41 },
        { 78, 79, 40 },
        { 79, 76, 40 },
        { 76, 74, 40 },
        { 74, 72, 40 },
        { 72, 70, 40 },
        { 70, 68, 40 },
        { 68, 66, 40 },
        { 66, 64, 40 },
        { 64, 62, 40 },
        { 62, 60, 40 },
        { 60, 58, 40 },
        { 58, 56, 40 },
        { 56, 54, 40 },
        { 54, 52, 40 },
        { 52, 50, 40 },
        { 50, 48, 40 },
        { 48, 46, 40 },
        { 46, 44, 40 },
        { 44, 42, 40 },
        { 42, 78, 40 },
        { 39, 38, 37 },
        { 39, 37, 36 },
        { 39, 35, 34 },
        { 39, 34, 38 },
        { 38, 34, 33 },
        { 38, 33, 37 },
        { 37, 33, 32 },
        { 37, 32, 36 },
        { 36, 32, 35 },
        { 36, 35, 39 },
        { 35, 32, 33 },
        { 35, 33, 34 },
        { 31, 30, 29 },
        { 31, 29, 28 },
        { 31, 27, 26 },
        { 31, 26, 30 },
        { 30, 26, 25 },
        { 30, 25, 29 },
        { 29, 25, 24 },
        { 29, 24, 28 },
        { 28, 24, 27 },
        { 28, 27, 31 },
        { 27, 24, 25 },
        { 27, 25, 26 },
        { 23, 22, 21 },
        { 23, 21, 20 },
        { 23, 19, 18 },
        { 23, 18, 22 },
        { 22, 18, 17 },
        { 22, 17, 21 },
        { 21, 17, 16 },
        { 21, 16, 20 },
        { 20, 16, 19 },
        { 20, 19, 23 },
        { 19, 16, 17 },
        { 19, 17, 18 },
        { 15, 14, 13 },
        { 15, 13, 12 },
        { 15, 11, 10 },
        { 15, 10, 14 },
        { 14, 10, 9 },
        { 14, 9, 13 },
        { 13, 9, 8 },
        { 13, 8, 12 },
        { 12, 8, 11 },
        { 12, 11, 15 },
        { 11, 8, 9 },
        { 11, 9, 10 },
        { 7, 6, 5 },
        { 7, 5, 4 },
        { 7, 3, 2 },
        { 7, 2, 6 },
        { 6, 2, 1 },
        { 6, 1, 5 },
        { 5, 1, 0 },
        { 5, 0, 4 },
        { 4, 0, 3 },
        { 4, 3, 7 },
        { 3, 0, 1 },
        { 3, 1, 2 }
      };

    /** Names of the textures */
    private int[] textureNames = new int[2];

    /** Moon surface Texture */
    private Texture moonTexture = null;
    private boolean moonTextureInitialized = false;

    /** Borg texture */
    private Texture borgTexture = null;
    private boolean borgTextureInitialized = false;

    /** Specifies whether or not the planet surface should be drawn */
    private boolean planetFlag = false;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        enterprise d = new enterprise();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public enterprise() {
        gl_ = coregl_;
        glu_ = coreglu_;
/*        gl_ = tracegl_;
        glu_ = traceglu_;
        tracegl_.setMode( TraceGL.VERBOSE ); */
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "USS Enterpoop" );
      }

    public String getVersion() {
        return new String( "$Id: enterprise.java.stub,v 1.6 1998/10/31 16:47:50 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration shows the USS Enterpoop chasing a Borg\n" );
        sb.append( "cube over a planet's surface. The Enterpoop banks and turns\n" );
        sb.append( "as it chases the cube about and the cube will run away when\n" );
        sb.append( "the Enterpoop gets too close ( not exactly Borg behaviour ).\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    p,P - Toggle drawing planet surface\n" );
        sb.append( "    Escape - Exit demo" );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Stan Melax <melax@bioware.com>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the key listener to the component */
        glc.addKeyListener( this );

        /** Add the window listener */
        this.addWindowListener( this );

        /** Add the listener and initialize the component */
        glc.addGLEventListener( this );
        glc.initialize();
        glc.start();
      }

    public void initialize( GLDrawable component ) {
        /** Initialize the display lists */
        borgList = gl_.glGenLists( 1 );
        gl_.glNewList( borgList, GL.GL_COMPILE );
            gl_.glEnable( GL.GL_CULL_FACE );
            gl_.glColor3f( 1.0f, 1.0f, 1.0f );
            shapes.solidCube( 5 );
            gl_.glDisable( GL.GL_CULL_FACE );
        gl_.glEndList();

        planeList = gl_.glGenLists( 1 );
        gl_.glNewList( planeList, GL.GL_COMPILE );
            gl_.glColor3f( 0f, 0f, 0f );
            gl_.glBegin( GL.GL_POLYGON );
                gl_.glVertex3f( 1 , 2, 1.0f );
                gl_.glVertex3f( 2 , 1, 1.0f );
                gl_.glVertex3f( 2 , -1, 1.0f );
                gl_.glVertex3f( 1 , -2, 1.0f );
                gl_.glVertex3f( -1 , -2, 1.0f );
                gl_.glVertex3f( -2 , -1, 1.0f );
                gl_.glVertex3f( -2 , 1, 1.0f );
                gl_.glVertex3f( -1 , 2, 1.0f );
            gl_.glEnd();

            gl_.glColor3f( 1.0f, 0.0f, 1.0f );

            gl_.glBegin( GL.GL_POLYGON );
                gl_.glVertex3f( 1, 2, 3.0f );
                gl_.glVertex3f( 2, 1, 3.0f );
                gl_.glVertex3f( 2, -1, 3.0f );
                gl_.glVertex3f( 1, -2, 3.0f );
                gl_.glVertex3f( -1, -2, 3.0f );
                gl_.glVertex3f( -2, -1, 3.0f );
                gl_.glVertex3f( -2, 1, 3.0f );
                gl_.glVertex3f( -1, 2, 3.0f );
            gl_.glEnd();
        gl_.glEndList();

        floorList = gl_.glGenLists( 1 );
        gl_.glNewList( floorList, GL.GL_COMPILE );
            gl_.glDisable( GL.GL_DEPTH_TEST );
            gl_.glColor3f( 0.2f, 0.2f, 0.2f );
            gl_.glBegin( GL.GL_POLYGON );
                gl_.glTexCoord2f( 0.0f, 0.0f );
                gl_.glVertex3f( -50, -50, 0 );
                gl_.glTexCoord2f( 1.0f, 0.0f );
                gl_.glVertex3f( 50, -50, 0 );
                gl_.glTexCoord2f( 1.0f, 1.0f );
                gl_.glVertex3f( 50, 50, 0 );
                gl_.glTexCoord2f( 0.0f, 1.0f );
                gl_.glVertex3f( -50, 50, 0 );
            gl_.glEnd();
            gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glEndList();

        /** Create the display list for the Enterpoop */
        enterpriseList = gl_.glGenLists( 1 );
        gl_.glNewList( enterpriseList, GL.GL_COMPILE );
            gl_.glScalef( 0.02f, 0.02f, 0.02f );
            gl_.glRotatef( 90.0f, 0.0f, 0.0f, 1.0f );
            gl_.glColor3f( 0.0f, 1.0f, 1.0f );
            for ( int i = 0 ; i < enterprisePolygons.length ; i++ ) {
                gl_.glBegin( GL.GL_TRIANGLES );
                    gl_.glVertex3dv( enterpriseData[enterprisePolygons[i][0]] );
                    gl_.glVertex3dv( enterpriseData[enterprisePolygons[i][1]] );
                    gl_.glVertex3dv( enterpriseData[enterprisePolygons[i][2]] );
                gl_.glEnd();
              }
        gl_.glEndList();

        /** Setup the colours */
        gl_.glColor3f( 1.0f, 0.0f, 0.0f );
        gl_.glClearColor( 0.0f, 0.0f, 0.3f, 1.0f );

        /** Disable stuff */
        gl_.glShadeModel( GL.GL_FLAT );

        /** Setup the texture maps */
        gl_.glGenTextures( 2, textureNames );

        /** Fetch the texture for the moon's surface */
        if ( moonTexture == null ) {
            try {
                Image moonImage =
                    Toolkit.getDefaultToolkit().getImage( new URL( "http://aberlemno/img/clem_nir.jpg" ) );
                moonTexture =
                    new Texture( moonImage, glc, Texture.SCALE_NEAREST );
              } catch ( Exception e ) {
                e.printStackTrace();
              }
          }

        /** Fetch the texture for the Borg surface */
        if ( borgTexture == null ) {
            try {
                Image borgImage =
                    Toolkit.getDefaultToolkit().getImage( new URL( "http://aberlemno/img/plasma2.gif" ) );
                borgTexture =
                    new Texture( borgImage, glc, Texture.SCALE_NEAREST );
              } catch ( Exception e ) {
                e.printStackTrace();
              }
          }
      }

    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0, 1.0, -1.0, 1.0, 1.5, 500 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }
        
    public void display( GLDrawable component ) {
        /** Clear the buffers */
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        /** Store the modelview matrix */
        gl_.glPushMatrix();
            /** Transform the viewpoint */
            gl_.glTranslatef( 0.0f, 0.0f, -30.0f );
            gl_.glRotatef( -45.0f, 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( angle, 0.0f, 0.0f, 1.0f );

            /** Make the moon texture if it's not already done */
            if ( moonTextureInitialized == false ) {
                if ( moonTexture.isValid() ) {
                    gl_.glBindTexture( GL.GL_TEXTURE_2D, textureNames[0] );
                    glu_.gluBuild2DMipmaps( GL.GL_TEXTURE_2D, 4,
                                            moonTexture.getWidth(),
                                            moonTexture.getHeight(),
                                            GL.GL_RGBA, GL.GL_UNSIGNED_BYTE,
                                            moonTexture.getTexture() );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST );
                    gl_.glTexEnvf( GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE,
                                   GL.GL_DECAL );
    
                    moonTextureInitialized = true;
                    planetFlag = !planetFlag;
                  }
              }

            /** Draw the floor */
            if ( planetFlag ) {
                gl_.glPushMatrix();
                    gl_.glEnable( GL.GL_TEXTURE_2D );
                    gl_.glBindTexture( GL.GL_TEXTURE_2D, textureNames[0] );
                    gl_.glCallList( floorList );
                    gl_.glDisable( GL.GL_TEXTURE_2D );
                gl_.glPopMatrix();
              }

            /** Make the Borg texture if it's not already done */
            if ( borgTextureInitialized == false ) {
                if ( borgTexture.isValid() ) {
                    gl_.glBindTexture( GL.GL_TEXTURE_2D, textureNames[1] );
                    glu_.gluBuild2DMipmaps( GL.GL_TEXTURE_2D, 4,
                                            borgTexture.getWidth(),
                                            borgTexture.getHeight(),
                                            GL.GL_RGBA, GL.GL_UNSIGNED_BYTE,
                                            borgTexture.getTexture() );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST );
                    gl_.glTexEnvf( GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE,
                                   GL.GL_DECAL );
    
                    borgTextureInitialized = true;
                  }
              }

            /** Move the bad guy */
            tDz *= tDamping;
            tDy *= tDamping;
            tX += tDx;
            tY += tDy;
            tZ += tDz;
            /** Bounce off walls */
            if ( tX > WORLD_BOUNDARY ) { tDx = (float)-Math.abs( tDx ); }
            if ( tX < -WORLD_BOUNDARY ) { tDx = (float)Math.abs( tDx ); }
            if ( tY > WORLD_BOUNDARY ) { tDy = (float)-Math.abs( tDy ); }
            if ( tY < -WORLD_BOUNDARY ) { tDy = (float)Math.abs( tDy ); }
            if ( tZ > WORLD_CEILING ) { tDz = (float)-Math.abs( tDz ); }
            if ( tZ < -WORLD_FLOOR ) { tDz = (float)Math.abs( tDz ); }

            /** Draw the bad guy */
            gl_.glPushMatrix();
                gl_.glTranslatef( tX, tY, tZ );
                boolean borgTextureValid = borgTexture.isValid();
                if ( borgTextureValid ) {
                    gl_.glBindTexture( GL.GL_TEXTURE_2D, textureNames[1] );
                    gl_.glEnable( GL.GL_TEXTURE_2D );
                  }
                gl_.glPushMatrix();
                    gl_.glMatrixMode( GL.GL_TEXTURE );
                    gl_.glPushMatrix();
                        gl_.glRotatef( angle, 1, 0, 0 );
                        gl_.glRotatef( angle, 0, 1, 0 );
                    gl_.glMatrixMode( GL.GL_MODELVIEW );
                gl_.glPopMatrix();
                gl_.glCallList( borgList );
                if ( borgTextureValid ) {
                    gl_.glDisable( GL.GL_TEXTURE_2D );
                  }
                gl_.glPushMatrix();
                gl_.glMatrixMode( GL.GL_TEXTURE );
                gl_.glPopMatrix();
                gl_.glMatrixMode( GL.GL_MODELVIEW );
                gl_.glPopMatrix();
            gl_.glPopMatrix();

            /** Move the enterpoop */
            float tHeading = (float)Math.atan2( tY - eY, tX - eX );
            float speed = (float)Math.sqrt( eDx * eDx + eDy * eDy );
            eH = ( eDy == 0.0f && eDx == 0.0f ) ? eH : (float)Math.atan2( eDy, eDx );
            float tdHeading = tHeading - eH;
            while ( tdHeading > Math.PI ) {
                tdHeading -= (float)( 2 * Math.PI );
              }
            while ( tdHeading < -Math.PI ) {
                tdHeading += (float)( 2 * Math.PI );
              }
            eDh *= BANK_DAMPING;
            eDh += tdHeading * 0.01f;
            eH += eDh;
            if ( ( tdHeading < Math.PI / 6 ) && ( tdHeading > -Math.PI / 6 ) ) {
                eSpeed += ACCELERATION;
              } else {
                if ( ( tdHeading < Math.PI / 3 ) && ( tdHeading > -Math.PI / 3 ) ) {
                    eSpeed += ACCELERATION / 4;
                  }
              }

            eDx = (float)Math.cos( eH ) * eSpeed;
            eDy = (float)Math.sin( eH ) * eSpeed;
            eDx *= DAMPING;
            eDy *= DAMPING;
            eDz *= DAMPING;

            eX += eDx;
            eY += eDy;
            eZ += eDz;

            /** Check to see if we need to make the target run away! */
            if ( ( eX > tX - 5 ) && ( eX < tX + 5 ) &&
                 ( eY > tY - 5 ) && ( eY < tY + 5 ) ) {
                tDx = (float)( ( Math.random() * 3 ) - 1.0f );
                tDy = (float)( ( Math.random() * 3 ) - 1.0f );
              }

            /** Draw the enterpoop */
            gl_.glPushMatrix();
                gl_.glTranslatef( eX, eY, eZ );
                gl_.glRotatef( eH * ( 180 / (float)Math.PI ), 0.0f, 0.0f, 1.0f );
                gl_.glRotatef( -60 * eDh / ( 1.0f / ( 1.0f - 0.8f ) * 0.01f * 3.14f ),
                               1.0f, 0.0f, 0.0f );
                gl_.glCallList( enterpriseList );
            gl_.glPopMatrix();
        /** Restore the matrix stack */
        gl_.glPopMatrix();

        /** Update the spin angle */
        angle += 5;
        if ( angle >= 360.0f ) {
            angle -= 360.0f;
          }
      }

    public GL getGL() {
        return coregl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
        if ( evt.getKeyChar() == 'p' || evt.getKeyChar() == 'P' ) {
            planetFlag = !planetFlag;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
