
/* Copyright (c) Mark J. Kilgard, 1994. */

/* This program is freely distributable without licensing fees 
   and is provided without guarantee or warrantee expressed or 
   implied. This program is -not- in the public domain. */

/**
 * $Id: bitfont.java,v 1.9 1998/10/19 10:54:01 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Demonstrates bitmapped fonts using Magician's funky font handling classes
 *
 * $Log: bitfont.java,v $
 * Revision 1.9  1998/10/19 10:54:01  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.8  1998/10/02 12:23:32  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.7  1998/06/07 14:08:36  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.6  1998/04/21 14:55:53  descarte
 * Added WindowListener. Added new demoTemplate methods. Added standalone code.
 *
 * Revision 1.5  1998/03/28 16:14:09  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.4  1998/03/24 17:01:32  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.3  1998/03/19 22:18:33  descarte
 * Added getGL() method.
 *
 * Revision 1.2  1998/03/16 14:35:39  descarte
 * Added frame dispose()
 * Converted to new beta 4 GLComponent syntax.
 *
 * Revision 1.1  1998/03/13 22:37:06  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.glut;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.magician.fonts.GLBitmapFont;
import com.hermetica.magician.fonts.*;
import com.hermetica.util3d.*;

public class bitfont extends Frame implements demoTemplate, GLEventListener, KeyListener, ActionListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();
    TraceGL tracegl_ = new TraceGL( coregl_ );

    /** The GLU pipeline */
    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** An array of fonts */
    GLBitmapFont[] fonts = new GLBitmapFont[3];

    /** The current font */
    GLBitmapFont currentFont = null;

    /** The default message to display */
    String defaultMessage = new String( "Magician means OpenGL" );

    /** The current message */
    String currentMessage = defaultMessage;

    /** The dimensions of the demo */
    private int WIDTH = 500,
                HEIGHT = 150;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        bitfont d = new bitfont();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    /** Constructor for the class */
    public bitfont() {
        gl_ = coregl_;
        glu_ = coreglu_;
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Bitmapped Fonts" );
      }

    public String getVersion() {
        return new String( "$Id: bitfont.java,v 1.9 1998/10/19 10:54:01 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demo illustrates the use of the dynamic bitmapped font\n" );
        sb.append( "capabilities present with Magician to render text portably.\n" );
        sb.append( "Several fonts are bundled with Magician by default and you\n" );
        sb.append( "can also generate additional fonts to suit your own\n" );
        sb.append( "applications.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mark J. Kilgard <mjk@sgi.com>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add a menubar to the window */
        MenuBar mb = new MenuBar();
        Menu optionsMenu = new Menu( "Options" );
        MenuItem[] optionsMenuItems = new MenuItem[10];
        String[] optionsMenuItemsLabels = {
            "9x15", "TimesRoman10", "TimesRoman24", "separator", 
            "OpenGL", "abc", "ABC", "separator",
            "Green", "Red", "White", "separator",
            "Exit"
          };

        int index = 0;
        for ( int i = 0 ; i < optionsMenuItemsLabels.length ; i++ ) {
            if ( optionsMenuItemsLabels[i].equals( "separator" ) == false ) {
                optionsMenuItems[index] = 
                    new MenuItem( optionsMenuItemsLabels[i] );
                optionsMenu.add( optionsMenuItems[index] );
                optionsMenuItems[index].addActionListener( this );
                index++;
              } else {
                optionsMenu.addSeparator();
              }
          }
        mb.add( optionsMenu );
        setMenuBar( mb );

        /** Do the layout of the GUI */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setStencilBits( 2 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the listener with the component */
        glc.addGLEventListener( this );

        /** Register a window listener */
        this.addWindowListener( this );

        /** Register a key event listener */
        glc.addKeyListener( this );

        /** Start the component processing events */
        glc.initialize();
      }

    /** Perform some basic setup of the scene */
    public void initialize( GLDrawable component ) {

        /** Create the three fonts and stick them in the array */
        fonts[0] = new GL9x15BitmapFont();
        fonts[1] = new GLTimesRoman10BitmapFont();
        fonts[2] = new GLTimesRoman24BitmapFont();

        currentFont = fonts[2];
      }

    /** Resizes the viewport when the window is stretched */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluOrtho2D( 0, width, height, 0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
      }
        
    /** Renders the scene */
    public void display( GLDrawable component ) {

        gl_.glClear( GL.GL_COLOR_BUFFER_BIT );
        outputString( 0, 24, "This is written in a Magician bitmap font." );
        outputString( 100, 100, currentMessage );
        outputString( 50, 145, "( positioned in pixels with upper-left origin )" );
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Draws the string */
    private void outputString( int x, int y, String str ) {
        gl_.glRasterPos2f( (float)x, (float)y );
        currentFont.drawString( str );
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** Handles action events */
    /** 
     * @@This assumes that events pertain to the global GLComponent, so
     * @@it doesn't use ((GLComponent)evt.getSource())....
     */
    public void actionPerformed( ActionEvent evt ) {
        if ( evt.getActionCommand().equals( "9x15" ) ) {
            currentFont = fonts[0];
            glc.repaint();
            return;
          }
        if ( evt.getActionCommand().equals( "TimesRoman10" ) ) {
            currentFont = fonts[1];
            glc.repaint();
            return;
          }
        if ( evt.getActionCommand().equals( "TimesRoman24" ) ) {
            currentFont = fonts[2];
            glc.repaint();
            return;
          }
        if ( evt.getActionCommand().equals( "abc" ) ) {
            currentMessage = new String( "abcdefghijklmnop" );
            glc.repaint();
            return;
          }
        if ( evt.getActionCommand().equals( "ABC" ) ) {
            currentMessage = new String( "ABCDEFGHIJKLMNOP" );
            glc.repaint();
            return;
          }
        if ( evt.getActionCommand().equals( "OpenGL" ) ) {
            currentMessage = defaultMessage;
            glc.repaint();
            return;
          }
        if ( evt.getActionCommand().equals( "Green" ) ) {
            glc.getContext().makeCurrent();
            gl_.glColor3f( 0.0f, 1.0f, 0.0f );
            glc.getContext().unlock();
            glc.repaint();
            return;
          }
        if ( evt.getActionCommand().equals( "Red" ) ) {
            glc.getContext().makeCurrent();
            gl_.glColor3f( 1.0f, 0.0f, 0.0f );
            glc.getContext().unlock();
            glc.repaint();
            return;
          }
        if ( evt.getActionCommand().equals( "White" ) ) {
            glc.getContext().makeCurrent();
            gl_.glColor3f( 1.0f, 1.0f, 1.0f );
            glc.getContext().unlock();
            glc.repaint();
            return;
          }
        if ( evt.getActionCommand().equals( "Exit" ) ) {
            glc.destroy();
            dispose();
            return;
          }
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
  }
