
/* Copyright (c) Mark J. Kilgard, 1994. */

/* This program is freely distributable without licensing fees 
   and is provided without guarantee or warrantee expressed or 
   implied. This program is -not- in the public domain. */

/**
 * $Id: dinoball.java.stub,v 1.13 1998/10/19 10:49:08 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Demonstrates OpenGL GLU tesselators in action in tesselating a dinosaur.
 * Jurassic Park, eat yer heart out!
 *
 * $Log: dinoball.java.stub,v $
 * Revision 1.13  1998/10/19 10:49:08  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.12  1998/10/02 12:23:23  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/06/07 14:07:51  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.10  1998/05/14 15:15:03  descarte
 * Removed GLComponenta argument from tesselator constructor.
 *
 * Revision 1.9  1998/04/21 14:55:14  descarte
 * Added WindowListener. Added new demoTemplate methods. Added standalone code.
 *
 * Revision 1.8  1998/03/30 12:10:30  descarte
 * Split tesselator off into public class.
 *
 * Revision 1.7  1998/03/28 15:42:23  descarte
 * Upgraded to GLComponentFactory.
 * Upgraded to Java 1.1 listeners.
 *
 * Revision 1.6  1998/03/24 16:29:18  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.5  1998/03/19 22:11:44  descarte
 * Added getGL() method to listener.
 *
 * Revision 1.4  1998/03/16 17:31:35  descarte
 * Removed verbose messages.
 * Added edgeFlag() method to private tesselator.
 *
 * Revision 1.3  1998/03/05 20:44:38  descarte
 * Added description.
 *
 * Revision 1.2  1998/03/05 13:28:57  descarte
 * Added private tesselator class overriding GLUTesselator.
 * Converted vertex() tesselator callback to use glVertex2fv() as per the
 * original demo. That works fine as well.
 *
 * Revision 1.1  1998/03/05 10:46:54  descarte
 * Initial revision
 *
 * Revision 1.1  1998/02/22 20:57:32  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.glut;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class dinoballapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();
    TraceGL tracegl_ = new TraceGL( coregl_ );
    ErrorGL errorgl_ = new ErrorGL( coregl_ );
    ProfileGL profiler_ = new ProfileGL();

    /** The GLU pipeline */
    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();
//    TraceGLU traceglu_ = new TraceGLU( coreglu_ );
//    ProfileGLU profileglu_ = new ProfileGLU();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** The angle of rotation */
    private float angle = -150;

    /** The width of the dinosaur body */
    private float bodyWidth = 3.0f;

    /** The body data */
    private float[][] body = {
        { 0, 3 }, { 1, 1 }, { 5, 1 }, { 8, 4 }, { 10, 4 }, { 11, 5 },
        { 11, 11.5f }, { 13, 12 }, { 13, 13 }, { 10, 13.5f }, { 13, 14 }, { 13, 15 }, { 11, 16 },
        { 8, 16 }, { 7, 15 }, { 7, 13 }, { 8, 12 }, { 7, 11 }, { 6, 6 }, { 4, 3 }, { 3, 2 },
        { 1, 2 }
      };

    /** The arm data */
    private float[][] arm = {
        { 8, 10 }, { 9, 9 }, { 10, 9 }, { 13, 8 }, { 14, 9 }, { 16, 9 },
        { 15, 9.5f }, { 16, 10 }, { 15, 10 }, { 15.5f, 11 }, { 14.5f, 10 }, 
        { 14, 11 }, { 14, 10 }, { 13, 9 }, { 11, 11 }, { 9, 11 }
      };

    /** The leg data */
    private float[][] leg = {
        { 8, 6 }, { 8, 4 }, { 9, 3 }, { 9, 2 }, { 8, 1 }, { 8, 0.5f }, { 9, 0 },
        { 12, 0 }, { 10, 1 }, { 10, 2 }, { 12, 4 }, { 11, 6 }, { 10, 7 }, { 9, 7 }
      };

    /** The eye data */
    private float[][] eye = {
        { 8.75f, 15 }, { 9, 14.7f }, { 9.6f, 14.7f }, { 10.1f, 15 },
        { 9.6f, 15.25f }, { 9, 15.25f }
      };

    /** The display list ids */
    private int BODY_SIDE = 1,
                BODY_EDGE = 2,
                BODY_WHOLE = 3,
                ARM_SIDE = 4,
                ARM_EDGE = 5,
                ARM_WHOLE = 6,
                LEG_SIDE = 7,
                LEG_EDGE = 8,
                LEG_WHOLE = 9,
                EYE_SIDE = 10,
                EYE_EDGE = 11,
                EYE_WHOLE = 12,
                DINOSAUR = 13;

    /** Has the dinosaur been created yet? */
    private boolean newModel = false;

    /** Lighting information */
    float[] lightZeroPosition = { 10.0f, 4.0f, 10.0f, 1.0f };
    float[] lightZeroColour = { 0.8f, 1.0f, 0.8f, 1.0f };
    float[] lightOnePosition = { -1.0f, -2.0f, 1.0f, 0.0f };
    float[] lightOneColour = { 0.6f, 0.3f, 0.2f, 1.0f };
    float[] skinColour = { 0.1f, 1.0f, 0.1f, 1.0f };
    float[] eyeColour = { 1.0f, 0.2f, 0.2f, 1.0f };

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        gl_ = coregl_;
        glu_ = coreglu_;
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Dinosaur!" );
      }

    public String getVersion() {
        return new String( "$Id: dinoball.java.stub,v 1.13 1998/10/19 10:49:08 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demo illustrates the use of the GLUTesselator class\n" );
        sb.append( "to triangulate complex polygons which are then extruded\n" );
        sb.append( "to form the body, arms, legs and eye of the dinosaur.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mark J. Kilgard <mjk@sgi.com>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
//        glc.getContext().setTraceStatus( true );

        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the listener with the component */
        glc.addGLEventListener( this );

        /** Register the keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Start the component processing events */
        glc.initialize();
        glc.start();
      }

    /** Perform some basic setup of the scene */
    public void initialize( GLDrawable component ) {

        /** Setup OpenGL */
        gl_.glEnable( GL.GL_CULL_FACE );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_DEPTH_TEST );

        gl_.glLightModeli( GL.GL_LIGHT_MODEL_LOCAL_VIEWER, 1 );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightZeroPosition );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, lightZeroColour );
        gl_.glLightf( GL.GL_LIGHT0, GL.GL_CONSTANT_ATTENUATION, 0.1f );
        gl_.glLightf( GL.GL_LIGHT0, GL.GL_LINEAR_ATTENUATION, 0.05f );

        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, lightOnePosition );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, lightOneColour );

        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_LIGHT1 );

        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );

        /** Create the dinosaur */
        makeDinosaur();
      }

    /** Resizes the viewport when the window is stretched */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 40.0f, 1.0f, 1.0f, 100.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        glu_.gluLookAt( 0.0f, 0.0f, 30.0f,
                        0.0f, 0.0f, 0.0f,
                        0.0f, 1.0f, 0.0f );
      }
        
    /** Renders the scene */
    public void display( GLDrawable component ) {

        /** Clear the frame buffer and depth buffer */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        /** Draw the scene */
        gl_.glPushMatrix();
            gl_.glRotatef( angle, 0.0f, 1.0f, 0.0f );
            gl_.glTranslatef( -8, -8, -( bodyWidth / 2 ) );
            gl_.glCallList( DINOSAUR );
            angle++;
            if ( angle > 360.0f ) {
                angle -= 360.0f;
              }
        gl_.glPopMatrix();
      }

    /** Returns a GL pipeline to the listener */
    public GL getGL() {
        return gl_;
      }

    /** Create the dinosaur */
    private void makeDinosaur() {

        /** Perform the extrusions for 2d -> 3d */
        extrudeSolidFromPolygon( body, body.length, bodyWidth,
                                 BODY_SIDE, BODY_EDGE, BODY_WHOLE );

        extrudeSolidFromPolygon( arm, arm.length, bodyWidth / 4,
                                 ARM_SIDE, ARM_EDGE, ARM_WHOLE );

        extrudeSolidFromPolygon( leg, leg.length, bodyWidth / 2,
                                 LEG_SIDE, LEG_EDGE, LEG_WHOLE );

        extrudeSolidFromPolygon( eye, eye.length, bodyWidth + 0.2f,
                                 EYE_SIDE, EYE_EDGE, EYE_WHOLE );

        gl_.glNewList( DINOSAUR, GL.GL_COMPILE );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, skinColour );
            gl_.glCallList( BODY_WHOLE );
            gl_.glPushMatrix();
                gl_.glTranslatef( 0.0f, 0.0f, bodyWidth );
                gl_.glCallList( ARM_WHOLE );
                gl_.glCallList( LEG_WHOLE );
                gl_.glTranslatef( 0.0f, 0.0f, -bodyWidth - bodyWidth / 4 );
                gl_.glCallList( ARM_WHOLE );
                gl_.glTranslatef( 0.0f, 0.0f, -bodyWidth / 4 );
                gl_.glCallList( LEG_WHOLE );
                gl_.glTranslatef( 0.0f, 0.0f, ( bodyWidth / 2 ) - 0.1f );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, eyeColour );
                gl_.glCallList( EYE_WHOLE );
            gl_.glPopMatrix();
        gl_.glEndList();
      }

    /** Extrudes a flat polygon into an extruded section */
    private void extrudeSolidFromPolygon( float[][] data, int dataSize, float thickness,
                                          int side, int edge, int whole ) {

        /** 
         * Tesselator object used to tesselate complex polygons into 
         * triangles
         */
        dinoballTesselator tesselator = new dinoballTesselator();

        double[] vertex = new double[3];

        gl_.glNewList( side, GL.GL_COMPILE );
            gl_.glShadeModel( GL.GL_SMOOTH );
            tesselator.beginPolygon();
            for ( int i = 0 ; i < dataSize ; i++ ) {
                vertex[0] = data[i][0];
                vertex[1] = data[i][1];
                vertex[2] = 0;
                tesselator.tessVertex( vertex, data[i] );
              }
            tesselator.endPolygon();
        gl_.glEndList();

        gl_.glNewList( edge, GL.GL_COMPILE );
            gl_.glShadeModel( GL.GL_FLAT );
            gl_.glBegin( GL.GL_QUAD_STRIP );
                for ( int i = 0 ; i <= dataSize ; i++ ) {
                    gl_.glVertex3f( data[i % dataSize][0], data[i % dataSize][1], 0.0f );
                    gl_.glVertex3f( data[i % dataSize][0], data[i % dataSize][1], thickness );

                    /** Calculate the unit normal */
                    float dx, dy, len;
                    dx = data[( i + 1 ) % dataSize][1] - data[i % dataSize][1];
                    dy = data[i % dataSize][0] - data[(i + 1 ) % dataSize][0];
                    len = (float)Math.sqrt( dx * dx + dy * dy );
                    gl_.glNormal3f( dx / len, dy / len, 0.0f );
                  }
            gl_.glEnd();
        gl_.glEndList();

        gl_.glNewList( whole, GL.GL_COMPILE );
            gl_.glFrontFace( GL.GL_CW );
            gl_.glCallList( edge );
            gl_.glNormal3f( 0.0f, 0.0f, -1.0f );
            gl_.glCallList( side );
            gl_.glPushMatrix();
                gl_.glTranslatef( 0.0f, 0.0f, thickness );
                gl_.glFrontFace( GL.GL_CCW );
                gl_.glNormal3f( 0.0f, 0.0f, 1.0f );
                gl_.glCallList( side );
            gl_.glPopMatrix();
        gl_.glEndList();
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
