
/* Copyright (c) Mark J. Kilgard, 1994. */

/* This program is freely distributable without licensing fees 
   and is provided without guarantee or warrantee expressed or 
   implied. This program is -not- in the public domain. */

/* Example for PC game developers to show how to *combine* texturing,
   reflections, and projected shadows all in real-time with OpenGL.
   Robust reflections use stenciling.  Robust projected shadows
   use both stenciling and polygon offset.  PC game programmers
   should realize that neither stenciling nor polygon offset are 
   supported by Direct3D, so these real-time rendering algorithms
   are only really viable with OpenGL. 
   
   The program has modes for disabling the stenciling and polygon
   offset uses.  It is worth running this example with these features
   toggled off so you can see the sort of artifacts that result.
   
   Notice that the floor texturing, reflections, and shadowing
   all co-exist properly. */

/* When you run this program:  Left mouse button controls the
   view.  Middle mouse button controls light position (left &
   right rotates light around dino; up & down moves light
   position up and down).  Right mouse button pops up menu. */

/* Check out the comments in the "redraw" routine to see how the
   reflection blending and surface stenciling is done.  You can
   also see in "redraw" how the projected shadows are rendered,
   including the use of stenciling and polygon offset. */

/**
 * $Id: dinoshade.java,v 1.8 1998/10/19 10:54:05 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Demonstrates OpenGL GLU tesselators in action in tesselating a dinosaur.
 * Jurassic Park, eat yer heart out!
 *
 * $Log: dinoshade.java,v $
 * Revision 1.8  1998/10/19 10:54:05  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.7  1998/10/02 12:23:31  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.6  1998/06/07 14:08:13  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.5  1998/05/14 15:16:30  descarte
 * Removed GLComponent argument from tesselator constructor.
 *
 * Revision 1.4  1998/04/21 14:56:01  descarte
 * Added WindowListener. Added new demoTemplate methods. Added standalone code.
 *
 * Revision 1.3  1998/03/28 16:13:53  descarte
 * Upgraded to Java-1.1 listeners.
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.2  1998/03/24 16:59:49  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.1  1998/03/18 21:51:43  descarte
 * Initial revision
 *
 * Revision 1.3  1998/03/05 20:44:48  descarte
 * Added description.
 *
 * Revision 1.2  1998/03/05 20:30:04  descarte
 * Removed some stale debugging messages.
 *
 * Revision 1.1  1998/03/05 20:19:04  descarte
 * Initial revision
 *
 * Revision 1.2  1998/03/05 13:28:57  descarte
 * Added private tesselator class overriding GLUTesselator.
 * Converted vertex() tesselator callback to use glVertex2fv() as per the
 * original demo. That works fine as well.
 *
 * Revision 1.1  1998/03/05 10:46:54  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.glut;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class dinoshade extends Frame implements demoTemplate, GLEventListener, KeyListener, ActionListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();
    TraceGL tracegl_ = new TraceGL( coregl_ );

    /** The GLU pipeline */
    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** Frame rate measurement device */
    FrameRateComponent frc = new FrameRateComponent();

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** Variables controlling various rendering modes */
    private boolean stencilReflection = true,
                    stencilShadow = true,
                    offsetShadow = true;
    private boolean renderShadow = true,
                    renderDinosaur = true,
                    renderReflection = true;
    private boolean linearFiltering = false,
                    useMipmaps = false,
                    useTexture = true;
    private boolean reportSpeed = false;
    private boolean animation = true;
    private boolean lightSwitch = true;
    private boolean directionalLight = true;
    private boolean forceExtension = false;
    private boolean moving = true;
    private boolean lightMoving = true;

    /** Time-varying or user-controlled variables */
    private boolean jumpUp = true;
    private float jump = 0.0f;
    private float lightAngle = 0.0f,
                  lightHeight = 20.0f;
    private float angle = -150;
    private float angle2 = 30;

    /** The width of the dinosaur body */
    private float bodyWidth = 3.0f;

    /** The body data */
    private float[][] body = {
        { 0, 3 }, { 1, 1 }, { 5, 1 }, { 8, 4 }, { 10, 4 }, { 11, 5 },
        { 11, 11.5f }, { 13, 12 }, { 13, 13 }, { 10, 13.5f }, { 13, 14 }, { 13, 15 }, { 11, 16 },
        { 8, 16 }, { 7, 15 }, { 7, 13 }, { 8, 12 }, { 7, 11 }, { 6, 6 }, { 4, 3 }, { 3, 2 },
        { 1, 2 }
      };

    /** The arm data */
    private float[][] arm = {
        { 8, 10 }, { 9, 9 }, { 10, 9 }, { 13, 8 }, { 14, 9 }, { 16, 9 },
        { 15, 9.5f }, { 16, 10 }, { 15, 10 }, { 15.5f, 11 }, { 14.5f, 10 }, 
        { 14, 11 }, { 14, 10 }, { 13, 9 }, { 11, 11 }, { 9, 11 }
      };

    /** The leg data */
    private float[][] leg = {
        { 8, 6 }, { 8, 4 }, { 9, 3 }, { 9, 2 }, { 8, 1 }, { 8, 0.5f }, { 9, 0 },
        { 12, 0 }, { 10, 1 }, { 10, 2 }, { 12, 4 }, { 11, 6 }, { 10, 7 }, { 9, 7 }
      };

    /** The eye data */
    private float[][] eye = {
        { 8.75f, 15 }, { 9, 14.7f }, { 9.6f, 14.7f }, { 10.1f, 15 },
        { 9.6f, 15.25f }, { 9, 15.25f }
      };

    /** The display list ids */
    private int BODY_SIDE = 1,
                BODY_EDGE = 2,
                BODY_WHOLE = 3,
                ARM_SIDE = 4,
                ARM_EDGE = 5,
                ARM_WHOLE = 6,
                LEG_SIDE = 7,
                LEG_EDGE = 8,
                LEG_WHOLE = 9,
                EYE_SIDE = 10,
                EYE_EDGE = 11,
                EYE_WHOLE = 12,
                DINOSAUR = 13;

    /** Has the dinosaur been created yet? */
    private boolean newModel = false;

    /** Lighting information */
    float[] lightPosition = { 10.0f, 4.0f, 10.0f, 1.0f };
    float[] lightColour = { 0.8f, 1.0f, 0.8f, 1.0f };
    float[] skinColour = { 0.1f, 1.0f, 0.1f, 1.0f };
    float[] eyeColour = { 1.0f, 0.2f, 0.2f, 1.0f };

    /** Nice floor texture tiling pattern */
    private final byte[] circles = {
        0x2e, 0x2e, 0x2e, 0x2e, 0x78, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x2e, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e,
        0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e,
        0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e,
        0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x2e, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x2e, 0x2e, 0x2e, 0x78, 0x78, 0x78, 0x78, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
        0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
      };

    /** Vertices for the flooring */
    private float[][] floorVertices = {
        { -20.0f, 0.0f, 20.0f },
        { 20.0f, 0.0f, 20.0f },
        { 20.0f, 0.0f, -20.0f },
        { -20.0f, 0.0f, -20.0f }
      };

    /** Plane equation describing the floor */
    private float[] floorPlane = new float[4];

    /** Matrix for projecting shadows onto the floor */
    private float[][] floorShadow = new float[4][4];

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        dinoshade d = new dinoshade();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    /** Constructor for the class */
    public dinoshade() {
        gl_ = coregl_;
        glu_ = coreglu_;
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Shadowy Leapin' Lizards!" );
      }

    public String getVersion() {
        return new String( "$Id: dinoshade.java,v 1.8 1998/10/19 10:54:05 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demo, as per the ``Dinosaur'' demo, tesselates complex\n" );
        sb.append( "polygons to form the dinosaur. Furthermore, this demo uses\n" );
        sb.append( "some advanced features of OpenGL to render projected shadows\n" );
        sb.append( "and reflections.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mark J. Kilgard <mjk@sgi.com>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add a menubar to the window */
        MenuBar mb = new MenuBar();
        Menu optionsMenu = new Menu( "Options" );
        MenuItem[] optionsMenuItems = new MenuItem[13];
        String[] optionsMenuItemsLabels = {
            "Toggle Motion", "separator",
            "Toggle Light", "Toggle Texture", "Toggle Shadow",
                "Toggle Reflection", "Toggle Dinosaur", "separator",
            "Toggle Reflection Stencilling", "Toggle Shadow Stencilling",
                "Toggle Shadow Offset", "separator",
            "Positional Light", "Directional Light", "separator",
            "Toggle Performance", "separator",
            "Exit"
          };

        int index = 0;
        for ( int i = 0 ; i < optionsMenuItemsLabels.length ; i++ ) {
            if ( optionsMenuItemsLabels[i].equals( "separator" ) ) {
                optionsMenu.addSeparator();
              } else {
                optionsMenuItems[index] = 
                    new MenuItem( optionsMenuItemsLabels[i] );
                optionsMenu.add( optionsMenuItems[index] );
                optionsMenuItems[index].addActionListener( this );
                index++;
              }
          }
        mb.add( optionsMenu );
        setMenuBar( mb );

        /** Do the layout of the GUI */
        add( "Center", glc );
        pack(); show();

        /** Make the FrameRateComponent act in summary mode */
        frc.setSummaryOnly( true );

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setStencilBits( 2 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the listener with the component */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /** Register a window listener on the Frame */
        this.addWindowListener( this );

        /** Start the component processing events */
        glc.initialize();
        glc.start();
      }

    /** Perform some basic setup of the scene */
    public void initialize( GLDrawable component ) {

        /** Setup OpenGL */
        gl_.glEnable( GL.GL_CULL_FACE );
        gl_.glEnable( GL.GL_TEXTURE_2D );
        gl_.glEnable( GL.GL_DEPTH_TEST );

        gl_.glLineWidth( 3.0f );

        gl_.glLightModeli( GL.GL_LIGHT_MODEL_LOCAL_VIEWER, 1 );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, lightColour );
        gl_.glLightf( GL.GL_LIGHT0, GL.GL_CONSTANT_ATTENUATION, 0.1f );
        gl_.glLightf( GL.GL_LIGHT0, GL.GL_LINEAR_ATTENUATION, 0.05f );

        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );

        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );

        /** Make the dinosaur */
        makeDinosaur();

        /** Make the floor texture */
        makeFloorTexture();

        /** Calculate the floor's plane equation */
        floorPlane = 
            findPlane( floorVertices[1], floorVertices[2], floorVertices[3] );


        /** Setup the polygon offset */
        gl_.glPolygonOffset( -2.0f, -1.0f );
      }

    /** Resizes the viewport when the window is stretched */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 40.0f, 1.0f, 20.0f, 100.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        glu_.gluLookAt( 0.0f, 8.0f, 60.0f,
                        0.0f, 8.0f, 0.0f,
                        0.0f, 1.0f, 0.0f );
      }
        
    /** Renders the scene */
    public void display( GLDrawable component ) {

        /** Take a timing sample */
        if ( reportSpeed ) {
            frc.startSample();
          }

        /** 
         * Clear out some buffers. Test to see if we're using stencilling 
         * to avoid unnecessary stencil buffer clears
         */
        if ( ( stencilReflection && renderReflection ) ||
             ( stencilShadow && renderShadow ) ) {
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT |
                         GL.GL_STENCIL_BUFFER_BIT );
          } else {
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
          }

        /** Reposition the light source */
        lightPosition[0] = (float)( 12 * Math.cos( lightAngle ) );
        lightPosition[1] = lightHeight;
        lightPosition[2] = (float)( 12 * Math.sin( lightAngle ) );
        if ( directionalLight ) {
            lightPosition[3] = 0.0f;
          } else {
            lightPosition[3] = 1.0f;
          }

        /** Calculate the shadow matrix */
        floorShadow = shadowMatrix( floorPlane, lightPosition );

        /** Draw the scene */
        gl_.glPushMatrix();
            /** Rotate the scene according to user input */
            gl_.glRotatef( angle2, 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( angle, 0.0f, 1.0f, 0.0f );

            /** Move the light position */
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightPosition );

            /** Render the reflection and stencil if necessary */
            if ( renderReflection ) {
                if ( stencilReflection ) {
                    /** 
                     * We can eliminate the visual "artifact" of seeing 
                     * the "flipped" dinosaur underneath the floor by 
                     * using stencil.  The idea is draw the floor without 
                     * color or depth update but so that a stencil value 
                     * of one is where the floor will be.  Later when rendering 
                     * the dinosaur reflection, we will only update pixels
                     * with a stencil value of 1 to make sure the reflection 
                     * only lives on the floor, not below the floor. 
                     */

                    /** Don't update colour or depth */
                    gl_.glDisable( GL.GL_DEPTH_TEST );
                    gl_.glColorMask( (byte)GL.GL_FALSE, (byte)GL.GL_FALSE, 
                                     (byte)GL.GL_FALSE, (byte)GL.GL_FALSE );

                    /** Draw 1 into the stencil buffer */
                    gl_.glEnable( GL.GL_STENCIL_TEST );
                    gl_.glStencilOp( GL.GL_REPLACE, GL.GL_REPLACE, GL.GL_REPLACE );
                    gl_.glStencilFunc( GL.GL_ALWAYS, 1, 0xffffffff );

                    /** Now render the floor. Floor pixels get their stencil set to 1 */
                    drawFloor();

                    /** Re-enable update of colour and depth */
                    gl_.glColorMask( (byte)GL.GL_TRUE, (byte)GL.GL_TRUE, 
                                     (byte)GL.GL_TRUE, (byte)GL.GL_TRUE );
                    gl_.glEnable( GL.GL_DEPTH_TEST );

                    /** Now, only render where the stencil is set to 1 */
                    gl_.glStencilFunc( GL.GL_EQUAL, 1, 0xffffffff );
                    gl_.glStencilOp( GL.GL_KEEP, GL.GL_KEEP, GL.GL_KEEP );
                  }

                gl_.glPushMatrix();

                    /** 
                     * The critical reflection step. Reflect the dinosaur
                     * through the floor ( the Y=0 plane ) to make a
                     * reflection
                     */
                    gl_.glScalef( 1.0f, -1.0f, 1.0f );

                    /** Reflect the light position */
                    gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightPosition );

                    /** 
                     * To avoid our normals getting reversed and hence botched
                     * lighting on the reflection, turn on normalize
                     */
                    gl_.glEnable( GL.GL_NORMALIZE );
                    gl_.glCullFace( GL.GL_FRONT );

                    /** Draw the reflected dinosaur */
                    drawDinosaur();

                    /** Disable normalize again and re-enable back-face culling */
                    gl_.glDisable( GL.GL_NORMALIZE );
                    gl_.glCullFace( GL.GL_BACK );
                gl_.glPopMatrix();

                /** Switch back to the unreflected light position */
                gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightPosition );

                if ( stencilReflection ) {
                    gl_.glDisable( GL.GL_STENCIL_TEST );
                  }
              }

            /** 
             * Back-face culling will get used to only draw either the 
             * top or the bottom floor. This lets us get a floor with 
             * two distinct appearances. The top floor surface is 
             * reflective and kind of red. The bottom floor surface is 
             * not reflective and blue. 
             */
            gl_.glFrontFace( GL.GL_CW );
            gl_.glColor4f( 0.1f, 0.1f, 0.7f, 1.0f );
            drawFloor();
            gl_.glFrontFace( GL.GL_CCW );

            /** Render shadow onto floor */
            if ( renderShadow ) {
                if ( stencilShadow ) {
                    /** 
                     * Draw the floor with stencil value 3. This helps us 
                     * only draw the shadow once per floor pixel ( and only 
                     * on the floor pixels ). 
                     */
                    gl_.glEnable( GL.GL_STENCIL_TEST );
                    gl_.glStencilFunc( GL.GL_ALWAYS, 3, 0xffffffff );
                    gl_.glStencilOp( GL.GL_KEEP, GL.GL_KEEP, GL.GL_REPLACE );
                  }
              }

            /** Draw ``top'' of floor. Use blending to blend in reflection */
            gl_.glEnable( GL.GL_BLEND );
            gl_.glBlendFunc( GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA );
            gl_.glColor4f( 0.7f, 0.0f, 0.0f, 0.3f );
            gl_.glColor4f( 1.0f, 1.0f, 1.0f, 0.3f );
            drawFloor();
            gl_.glDisable( GL.GL_BLEND );

            /** Draw the dinosaur */
            if ( renderDinosaur ) {
                drawDinosaur();
              }

            /** Render the projected shadow */
            if ( renderShadow ) {
                if ( stencilShadow ) {
                    /** 
                     * Now, only render where stencil is set above 2 ( ie, 
                     * 3 where the top floor is).  Update stencil with 2 
                     * where the shadow gets drawn so we don't redraw (
                     * and accidently reblend ) the shadow ). 
                     */
                    gl_.glStencilFunc( GL.GL_LESS, 2, 0xffffffff );
                    gl_.glStencilOp( GL.GL_REPLACE, GL.GL_REPLACE, GL.GL_REPLACE );
                  }

                /** Polygon offset stuff ... */
                if ( offsetShadow ) {
                    gl_.glEnable( GL.GL_POLYGON_OFFSET_FILL );
                  }

                /** Render 50% black shadow colour on top of the floor */
                gl_.glEnable( GL.GL_BLEND );
                gl_.glBlendFunc( GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA );
                gl_.glDisable( GL.GL_LIGHTING );
                gl_.glColor4f( 0.0f, 0.0f, 0.0f, 0.5f );

                /** Project the shadow */
                gl_.glPushMatrix();
                    gl_.glMultMatrixf( floorShadow );
                    drawDinosaur();
                gl_.glPopMatrix();

                gl_.glDisable( GL.GL_BLEND );
                gl_.glEnable( GL.GL_LIGHTING );

                /** More polygon offset... */
                if ( offsetShadow ) {
                    gl_.glDisable( GL.GL_POLYGON_OFFSET_FILL );
                  }

                if ( stencilShadow ) {
                    gl_.glDisable( GL.GL_STENCIL_TEST );
                  }
              }

            /** Draw the light source */
            gl_.glPushMatrix();
                gl_.glDisable( GL.GL_LIGHTING );
                gl_.glColor3f( 1.0f, 1.0f, 0.0f );
                if ( directionalLight ) {
                    /** Draw an arrowhead */
                    gl_.glDisable( GL.GL_CULL_FACE );
                    gl_.glTranslatef( lightPosition[0], lightPosition[1], lightPosition[2] );
                    gl_.glRotatef( lightAngle * -180.0f / (float)Math.PI, 0, 1, 0 );
                    gl_.glRotatef( (float)( Math.atan( lightHeight / 12 ) * 180.0f ) / (float)Math.PI,
                                   0, 0, 1 );
                    gl_.glBegin( GL.GL_TRIANGLE_FAN );
                        gl_.glVertex3f( 0, 0, 0 );
                        gl_.glVertex3f( 2, 1, 1 );
                        gl_.glVertex3f( 2, -1, 1 );
                        gl_.glVertex3f( 2, -1, -1 );
                        gl_.glVertex3f( 2, 1, -1 );
                        gl_.glVertex3f( 2, 1, 1 );
                    gl_.glEnd();

                    /** Draw a white line from light direction */
                    gl_.glColor3f( 1.0f, 1.0f, 1.0f );
                    gl_.glBegin( GL.GL_LINES );
                        gl_.glVertex3f( 0, 0, 0 );
                        gl_.glVertex3f( 5, 0, 0 );
                    gl_.glEnd();

                    gl_.glEnable( GL.GL_CULL_FACE );
                  } else {
                    gl_.glTranslatef( lightPosition[0], lightPosition[1], lightPosition[2] );
                    shapes.solidSphere( 1.0f, 5, 5 );
                  }
                gl_.glEnable( GL.GL_LIGHTING );
            gl_.glPopMatrix();

        gl_.glPopMatrix();

        /** Stop the timing sample */
        if ( reportSpeed ) {
            frc.stopSample();
          }

        /** Move either the scene or the light or both about */
        if ( moving ) {
            /** Spin the scene */
            angle++;
            if ( angle > 360.0f ) {
                angle -= 360.0f;
              }

            /** Make the dinosaur jump */
            if ( jumpUp ) {
                jump++;
                if ( jump >= 10.0f ) {
                    jumpUp = false;
                  }
              } else {
                jump--;
                if ( jump <= 0.0f ) {
                    jumpUp = true;
                  }
              }
          }

        /** Move the light */
        if ( lightMoving ) {
            lightAngle += 0.2f;
            if ( lightAngle > 360.0f ) {
                lightAngle -= 360.0f;
              }
          }
      }

    /** Returns a valid GL pipeline */
    public GL getGL() {
        return gl_;
      }

    /** Draws the dinosaur */
    private void drawDinosaur() {
        gl_.glPushMatrix();
            gl_.glTranslatef( -8, 0, -( bodyWidth / 2 ) );
            gl_.glTranslatef( 0.0f, jump, 0.0f );
            gl_.glCallList( DINOSAUR );
        gl_.glPopMatrix();
      }

    /** Draws the flooring */
    private void drawFloor() {
        gl_.glDisable( GL.GL_LIGHTING );

        if ( useTexture ) {
            gl_.glEnable( GL.GL_TEXTURE_2D );
          }

        gl_.glBegin( GL.GL_QUADS );
            gl_.glTexCoord2f( 0.0f, 0.0f );
            gl_.glVertex3fv( floorVertices[0] );
            gl_.glTexCoord2f( 0.0f, 16.0f );
            gl_.glVertex3fv( floorVertices[1] );
            gl_.glTexCoord2f( 16.0f, 16.0f );
            gl_.glVertex3fv( floorVertices[2] );
            gl_.glTexCoord2f( 16.0f, 0.0f );
            gl_.glVertex3fv( floorVertices[3] );
        gl_.glEnd();

        if ( useTexture ) {
            gl_.glDisable( GL.GL_TEXTURE_2D );
          }

        gl_.glEnable( GL.GL_LIGHTING );
      }

    /** Create the dinosaur */
    private void makeDinosaur() {

        extrudeSolidFromPolygon( body, body.length, bodyWidth,
                                 BODY_SIDE, BODY_EDGE, BODY_WHOLE );
        extrudeSolidFromPolygon( arm, arm.length, bodyWidth / 4,
                                 ARM_SIDE, ARM_EDGE, ARM_WHOLE );
        extrudeSolidFromPolygon( leg, leg.length, bodyWidth / 2,
                                 LEG_SIDE, LEG_EDGE, LEG_WHOLE );
        extrudeSolidFromPolygon( eye, eye.length, bodyWidth + 0.2f,
                                 EYE_SIDE, EYE_EDGE, EYE_WHOLE );

        gl_.glNewList( DINOSAUR, GL.GL_COMPILE );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, skinColour );
            gl_.glCallList( BODY_WHOLE );
            gl_.glPushMatrix();
                gl_.glTranslatef( 0.0f, 0.0f, bodyWidth );
                gl_.glCallList( ARM_WHOLE );
                gl_.glCallList( LEG_WHOLE );
                gl_.glTranslatef( 0.0f, 0.0f, -bodyWidth - bodyWidth / 4 );
                gl_.glCallList( ARM_WHOLE );
                gl_.glTranslatef( 0.0f, 0.0f, -bodyWidth / 4 );
                gl_.glCallList( LEG_WHOLE );
                gl_.glTranslatef( 0.0f, 0.0f, ( bodyWidth / 2 ) - 0.1f );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, eyeColour );
                gl_.glCallList( EYE_WHOLE );
            gl_.glPopMatrix();
        gl_.glEndList();
      }

    /** Extrudes a flat polygon into an extruded section */
    private void extrudeSolidFromPolygon( float[][] data, int dataSize, float thickness,
                                          int side, int edge, int whole ) {

        /** 
         * Tesselator object used to tesselate complex polygons into 
         * triangles
         */
        dinoshadeTesselator tesselator = new dinoshadeTesselator();

        double[] vertex = new double[3];

        gl_.glNewList( side, GL.GL_COMPILE );
            gl_.glShadeModel( GL.GL_SMOOTH );
            tesselator.beginPolygon();
            for ( int i = 0 ; i < dataSize ; i++ ) {
                vertex[0] = data[i][0];
                vertex[1] = data[i][1];
                vertex[2] = 0;
                tesselator.tessVertex( vertex, data[i] );
              }
            tesselator.endPolygon();
        gl_.glEndList();

        gl_.glNewList( edge, GL.GL_COMPILE );
            gl_.glShadeModel( GL.GL_FLAT );
            gl_.glBegin( GL.GL_QUAD_STRIP );
                for ( int i = 0 ; i <= dataSize ; i++ ) {
                    gl_.glVertex3f( data[i % dataSize][0], data[i % dataSize][1], 0.0f );
                    gl_.glVertex3f( data[i % dataSize][0], data[i % dataSize][1], thickness );

                    /** Calculate the unit normal */
                    float dx, dy, len;
                    dx = data[( i + 1 ) % dataSize][1] - data[i % dataSize][1];
                    dy = data[i % dataSize][0] - data[(i + 1 ) % dataSize][0];
                    len = (float)Math.sqrt( dx * dx + dy * dy );
                    gl_.glNormal3f( dx / len, dy / len, 0.0f );
                  }
            gl_.glEnd();
        gl_.glEndList();

        gl_.glNewList( whole, GL.GL_COMPILE );
            gl_.glFrontFace( GL.GL_CW );
            gl_.glCallList( edge );
            gl_.glNormal3f( 0.0f, 0.0f, -1.0f );
            gl_.glCallList( side );
            gl_.glPushMatrix();
                gl_.glTranslatef( 0.0f, 0.0f, thickness );
                gl_.glFrontFace( GL.GL_CCW );
                gl_.glNormal3f( 0.0f, 0.0f, 1.0f );
                gl_.glCallList( side );
            gl_.glPopMatrix();
        gl_.glEndList();
      }

    /** Makes the floor texture-map from the tile data */
    private void makeFloorTexture() {
        byte[][][] floorTexture = new byte[16][16][3];
        int index = 0;

        /** Scan through the byte array and set the colours accordingly */
        for ( int i = 0 ; i < 16 ; i++ ) {
            for ( int j = 0 ; j < 16 ; j++ ) {
                if ( circles[index] == 'x' ) {
                    floorTexture[i][j][0] = (byte)0x1f;
                    floorTexture[i][j][1] = (byte)0x8f;
                    floorTexture[i][j][2] = (byte)0x1f;
                  } else {
                    floorTexture[i][j][0] = (byte)0xaa;
                    floorTexture[i][j][1] = (byte)0xaa;
                    floorTexture[i][j][2] = (byte)0xaa;
/*                    floorTexture[i][j][0] = (byte)0xff;
                    floorTexture[i][j][1] = (byte)0xff;
                    floorTexture[i][j][2] = (byte)0xff; */
                  }
                index++;
              }
          }

        /** Setup the texture map */
        gl_.glPixelStorei( GL.GL_UNPACK_ALIGNMENT, 1 );

        /** Create the texture */
        if ( useMipmaps ) {
          } else {
            if ( linearFiltering ) {
              } else {
                gl_.glTexParameteri( GL.GL_TEXTURE_2D,
                                     GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST );
              }
            gl_.glTexImage2D( GL.GL_TEXTURE_2D, 0, 3, 16, 16, 0, GL.GL_RGB,
                              GL.GL_UNSIGNED_BYTE, floorTexture );
          }
      }

    /** Finds the plane equation given three points */
    private float[] findPlane( float[] v0, float[] v1, float[] v2 ) {

        float[] plane = new float[4];

        float[] vec0 = new float[3];
        float[] vec1 = new float[3];

        /** Need two vectors to compute cross-product */
        vec0[0] = v1[0] - v0[0];
        vec0[1] = v1[1] - v0[1];
        vec0[2] = v1[2] - v0[2];

        vec1[0] = v2[0] - v0[0];
        vec1[1] = v2[1] - v0[1];
        vec1[2] = v2[2] - v0[2];

        /** Find cross-product to get A, B and C of plane equation */
        plane[0] = vec0[1] * vec1[2] - vec0[2] * vec1[1];
        plane[1] = -( vec0[0] * vec1[2] - vec0[2] * vec1[0] );
        plane[2] = vec0[0] * vec1[1] - vec0[1] * vec1[0];
        plane[3] = -( plane[0] * v0[0] + plane[1] * v0[1] + plane[2] * v0[2] );

        return plane;
      }

    /** Calculates a matrix that will project the dinosaur shadow onto the floor */
    private float[][] shadowMatrix( float[] groundPlane, float[] lightPos ) {

        float[][] shadowMat = new float[4][4];

        float dot = 
            groundPlane[0] * lightPos[0] +
            groundPlane[1] * lightPos[1] +
            groundPlane[2] * lightPos[2] +
            groundPlane[3] * lightPos[3];

        shadowMat[0][0] = dot - lightPos[0] * groundPlane[0];
        shadowMat[1][0] = 0.f - lightPos[0] * groundPlane[1];
        shadowMat[2][0] = 0.f - lightPos[0] * groundPlane[2];
        shadowMat[3][0] = 0.f - lightPos[0] * groundPlane[3];

        shadowMat[0][1] = 0.f - lightPos[1] * groundPlane[0];
        shadowMat[1][1] = dot - lightPos[1] * groundPlane[1];
        shadowMat[2][1] = 0.f - lightPos[1] * groundPlane[2];
        shadowMat[3][1] = 0.f - lightPos[1] * groundPlane[3];

        shadowMat[0][2] = 0.f - lightPos[2] * groundPlane[0];
        shadowMat[1][2] = 0.f - lightPos[2] * groundPlane[1];
        shadowMat[2][2] = dot - lightPos[2] * groundPlane[2];
        shadowMat[3][2] = 0.f - lightPos[2] * groundPlane[3];

        shadowMat[0][3] = 0.f - lightPos[3] * groundPlane[0];
        shadowMat[1][3] = 0.f - lightPos[3] * groundPlane[1];
        shadowMat[2][3] = 0.f - lightPos[3] * groundPlane[2];
        shadowMat[3][3] = dot - lightPos[3] * groundPlane[3];

        return shadowMat;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
            if ( callExit ) {
                System.exit( 0 );
              }
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** Handles AWT Events */
    public void actionPerformed( ActionEvent evt ) {
        if ( evt.getActionCommand().equals( "Toggle Motion" ) ) {
            if ( moving ) {
                moving = false;
                lightMoving = false;
                glc.suspend();
              } else {
                moving = true;
                lightMoving = true;
                glc.resume();
              }
            return;
          }
        if ( evt.getActionCommand().equals( "Toggle Light" ) ) {
            glc.getContext().makeCurrent();
            if ( lightSwitch ) {
                gl_.glDisable( GL.GL_LIGHT0 );
                lightSwitch = false;
              } else {
                gl_.glEnable( GL.GL_LIGHT0 );
                lightSwitch = true;
              }
            glc.getContext().unlock();
            return;
          }
        if ( evt.getActionCommand().equals( "Toggle Texture" ) ) {
            if ( useTexture ) {
                useTexture = false;
              } else {
                useTexture = true;
              }
            return;
          }
        if ( evt.getActionCommand().equals( "Toggle Shadows" ) ) {
            if ( renderShadow ) {
                renderShadow = false;
              } else {
                renderShadow = true;
              }
            return;
          }
        if ( evt.getActionCommand().equals( "Toggle Reflection" ) ) {
            if ( renderReflection ) {
                renderReflection = false;
              } else {
                renderReflection = true;
              }
            return;
          }
        if ( evt.getActionCommand().equals( "Toggle Dinosaur" ) ) {
            if ( renderDinosaur ) {
                renderDinosaur = false;
              } else {
                renderDinosaur = true;
              }
            return;
          }
        if ( evt.getActionCommand().equals( "Toggle Reflection Stencilling" ) ) {
            if ( stencilReflection ) {
                stencilReflection = false;
              } else {
                stencilReflection = true;
              }
            return;
          }
        if ( evt.getActionCommand().equals( "Toggle Shadow Stencilling" ) ) {
            if ( stencilShadow ) {
                stencilShadow = false;
              } else {
                stencilShadow = true;
              }
            return;
          }
        if ( evt.getActionCommand().equals( "Toggle Shadow Offset" ) ) {
            if ( offsetShadow ) {
                offsetShadow = false;
              } else {
                offsetShadow = true;
              }
            return;
          }
        if ( evt.getActionCommand().equals( "Positional Light" ) ) {
            directionalLight = false;
            return;
          }
        if ( evt.getActionCommand().equals( "Directional Light" ) ) {
            directionalLight = true;
            return;
          }
        if ( evt.getActionCommand().equals( "Toggle Performance" ) ) {
            reportSpeed = true;
            return;
          }
        if ( evt.getActionCommand().equals( "Exit" ) ) {
            glc.destroy();
            dispose();
            if ( callExit ) {
                System.exit( 0 );
              }
            return;
          }
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
  }

class dinoshadeTesselator extends GLUTesselator {

    /** OpenGL pipeline */
    CoreGL coregl_ = new CoreGL();

    /** Begins tesselation of a polygon */
    public void begin( int mode ) {
        coregl_.glBegin( mode );
      }

    /** Ends tesselation of a polygon */
    public void end() {
        coregl_.glEnd();
      }

    /** Handles a single vertex from the tesselator */
    public void vertex( float[] vertex ) {
        coregl_.glVertex2fv( vertex );
      }

    /** Handles the error callback */
    public void error( int errorNum ) {
        System.err.println( "GLUTesselator error: " + errorNum );
      }

    /** Handles the edge flag callback */
    public void edgeFlag( boolean flag ) {
        coregl_.glEdgeFlag( flag ? (byte)GL.GL_TRUE : (byte)GL.GL_FALSE );
      }
  }
