/**
 * $Id: fog.java.stub,v 1.8 1998/10/19 10:49:10 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Silicon Graphics test program that demonstrates the use of Fog
 *
 * $Log: fog.java.stub,v $
 * Revision 1.8  1998/10/19 10:49:10  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.7  1998/10/02 12:23:18  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.6  1998/06/07 14:07:51  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.5  1998/04/21 14:55:35  descarte
 * Added WindowListener. Added new demoTemplate methods. Added standalone code.
 *
 * Revision 1.4  1998/03/28 15:36:07  descarte
 * Upgraded to JDK-1.1 listeners.
 * Updated to GLComponentFactory.
 *
 * Revision 1.3  1998/03/24 16:27:12  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.2  1998/03/19 22:10:27  descarte
 * Added getGL() method to listener.
 *
 * Revision 1.1  1998/02/23 17:08:17  descarte
 * Initial revision
 *
 * Revision 1.4  1997/08/11 20:33:02  descarte
 * Converted to latest Magician API, replete with funky context and
 * component code plus Exception handling! Oo!
 *
 * Revision 1.3  1997/06/07 06:17:04  descarte
 * Added class to correct package.
 *
 * Revision 1.2  1997/06/04 18:57:20  descarte
 * Added versioning.
 *
 * Revision 1.1  1997/06/04 18:56:49  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.glut;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class fogapp extends Applet implements GLEventListener, demoTemplate, KeyListener, WindowListener {

    /** OpenGL pipeline */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    /** The OpenGL-aware AWT Component we render onto */
    GLComponent glc = null;

    /** Dimensions of the window */
    private static final int WIDTH = 300;
    private static final int HEIGHT = 300;

    float rotX = 5.0f,
          rotY = -5.0f,
          zTranslate = -65.0f;
    float fogDensity = 0.02f;
    int tubeList = 1;

    float[][] scp = {
        { 1.000000f, 0.000000f, 0.000000f },
        { 1.000000f, 0.000000f, 5.000000f },
        { 0.707107f, 0.707107f, 0.000000f },
        { 0.707107f, 0.707107f, 5.000000f },
        { 0.000000f, 1.000000f, 0.000000f },
        { 0.000000f, 1.000000f, 5.000000f },
        { -0.707107f, 0.707107f, 0.000000f },
        { -0.707107f, 0.707107f, 5.000000f },
        { -1.000000f, 0.000000f, 0.000000f },
        { -1.000000f, 0.000000f, 5.000000f },
        { -0.707107f, -0.707107f, 0.000000f },
        { -0.707107f, -0.707107f, 5.000000f },
        { 0.000000f, -1.000000f, 0.000000f },
        { 0.000000f, -1.000000f, 5.000000f },
        { 0.707107f, -0.707107f, 0.000000f },
        { 0.707107f, -0.707107f, 5.000000f },
        { 1.000000f, 0.000000f, 0.000000f },
        { 1.000000f, 0.000000f, 5.000000f },
      };

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        gl_ = coregl_;
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Fog" );
      }

    public String getVersion() {
        return new String( "$Id: fog.java.stub,v 1.8 1998/10/19 10:49:10 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of OpenGL's\n" );
        sb.append( "fog effects. The fog density can be altered.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    h - Rotate Left\n" );
        sb.append( "    l - Rotate Right\n" );
        sb.append( "    j - Rotate Up\n" );
        sb.append( "    k - Rotate Down\n" );
        sb.append( "    d - Decrease Fog Density\n" );
        sb.append( "    D - Increase Fog Density\n" );
        sb.append( "    ESC - Quit Demo" );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Brian Paul <brianp@ra.avid.com>" );
      }

    /** Creates the new program and initializes the GL state */
    public void startDemo() {

        /** Set the LayoutManager to allow reshaping */
        setLayout( new BorderLayout() );

        /** Create the GL Component and add it to the Frame */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        add( "Center", glc );

        /** Pack the components into the Frame and display it */
        /**#layout */

        /** Setup the capabilities of the OpenGL context */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the listener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Tell the component to start processing events */
        glc.initialize();
      }

    /** Builds the display lists */
    private void buildLists() throws OpenGLException {
        gl_.glNewList( tubeList, GL.GL_COMPILE );
            gl_.glBegin( GL.GL_TRIANGLE_STRIP );
                gl_.glNormal3fv(scp[0]); gl_.glVertex3fv(scp[0]);
                gl_.glNormal3fv(scp[0]); gl_.glVertex3fv(scp[1]);
                gl_.glNormal3fv(scp[2]); gl_.glVertex3fv(scp[2]);
                gl_.glNormal3fv(scp[2]); gl_.glVertex3fv(scp[3]);
                gl_.glNormal3fv(scp[4]); gl_.glVertex3fv(scp[4]);
                gl_.glNormal3fv(scp[4]); gl_.glVertex3fv(scp[5]);
                gl_.glNormal3fv(scp[6]); gl_.glVertex3fv(scp[6]);
                gl_.glNormal3fv(scp[6]); gl_.glVertex3fv(scp[7]);
                gl_.glNormal3fv(scp[8]); gl_.glVertex3fv(scp[8]);
                gl_.glNormal3fv(scp[8]); gl_.glVertex3fv(scp[9]);
                gl_.glNormal3fv(scp[10]); gl_.glVertex3fv(scp[10]);
                gl_.glNormal3fv(scp[10]); gl_.glVertex3fv(scp[11]);
                gl_.glNormal3fv(scp[12]); gl_.glVertex3fv(scp[12]);
                gl_.glNormal3fv(scp[12]); gl_.glVertex3fv(scp[13]);
                gl_.glNormal3fv(scp[14]); gl_.glVertex3fv(scp[14]);
                gl_.glNormal3fv(scp[14]); gl_.glVertex3fv(scp[15]);
                gl_.glNormal3fv(scp[16]); gl_.glVertex3fv(scp[16]);
                gl_.glNormal3fv(scp[16]); gl_.glVertex3fv(scp[17]);
            gl_.glEnd();
        gl_.glEndList();
      }

    /** General initialization routines */
    public void initialize( GLDrawable component ) {
        float[] ambient = { 0.1f, 0.1f, 0.1f, 1.0f };
        float[] diffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] position = { 90.0f, 90.0f, 0.0f, 0.0f };
        float[] front_mat_shininess = { 30.0f };
        float[] front_mat_specular = { 0.0f, 0.0f, 0.0f, 1.0f };
        float[] front_mat_diffuse = { 0.0f, 1.0f, 0.0f, 1.0f };
        float[] back_mat_shininess = { 50.0f };
        float[] back_mat_specular = { 0.0f, 0.0f, 1.0f, 1.0f };
        float[] back_mat_diffuse = { 1.0f, 0.0f, 0.0f, 1.0f };
        float[] lmodel_ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
        float[] fog_color = { 0.8f, 0.8f, 0.8f, 1.0f };

        gl_.glFrontFace( GL.GL_CW );

        gl_.glEnable( GL.GL_DEPTH_TEST );

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, ambient );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position );
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );
        gl_.glLightModeli( GL.GL_LIGHT_MODEL_TWO_SIDE, GL.GL_TRUE );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
    
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, front_mat_shininess );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, front_mat_specular );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, front_mat_diffuse );
        gl_.glMaterialfv( GL.GL_BACK, GL.GL_SHININESS, back_mat_shininess );
        gl_.glMaterialfv( GL.GL_BACK, GL.GL_SPECULAR, back_mat_specular );
        gl_.glMaterialfv( GL.GL_BACK, GL.GL_DIFFUSE, back_mat_diffuse );

        gl_.glEnable( GL.GL_FOG );
        gl_.glFogi( GL.GL_FOG_MODE, GL.GL_EXP );
        gl_.glFogf( GL.GL_FOG_DENSITY, fogDensity );
        gl_.glFogfv( GL.GL_FOG_COLOR, fog_color );
        gl_.glClearColor( 0.8f, 0.8f, 0.8f, 1.0f );
    
        buildLists();
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {

        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0, 1.0, -1.0, 1.0, 1.0, 200 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Renders the scene */
    public void display( GLDrawable component ) {
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        gl_.glPushMatrix();
            gl_.glTranslatef( 0, 0, zTranslate );
            gl_.glRotatef( rotY, 0, 1, 0 );
            gl_.glRotatef( rotX, 1, 0, 0 );
            gl_.glScalef( 1.0f, 1.0f, 10.0f );
    
            gl_.glCallList( tubeList );
        gl_.glPopMatrix();
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** Handles input events */
    /** We use:
     *  
     *      o d - Decrease fog density
     *      o D - Increase fog density
     *      o h - Rotate to the left
     *      o l - Rotate to the right
     *      o j - Rotate upwards
     *      o k - Rotate downwards
     *      o ESC - Exit
     */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            glc.destroy();
            /**#dispose */
            return;
          }
        if ( evt.getKeyChar() == 'd' ) {
            fogDensity /= 1.10;
            glc.getContext().makeCurrent();
            gl_.glFogf( GL.GL_FOG_DENSITY, fogDensity );
            glc.getContext().unlock();
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'D' ) {
            fogDensity *= 1.10;
            glc.getContext().makeCurrent();
            gl_.glFogf( GL.GL_FOG_DENSITY, fogDensity );
            glc.getContext().unlock();
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'h' ) {
            rotY -= 5;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'l' ) {
            rotY += 5;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'j' ) {
            rotX -= 5;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'k' ) {
            rotX += 5;
            glc.repaint();
            return;
          }
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
