/* Copyright (c) Mark J. Kilgard, 1994. */

/* This program is freely distributable without licensing fees 
   and is provided without guarantee or warrantee expressed or 
   implied. This program is -not- in the public domain. */

/**
 * $Id: fontdemo.java.stub,v 1.9 1998/10/19 10:49:14 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Demonstrates the use of stroked and bitmapped fonts
 *
 * $Log: fontdemo.java.stub,v $
 * Revision 1.9  1998/10/19 10:49:14  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.8  1998/10/02 12:23:28  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.7  1998/06/07 14:07:52  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.6  1998/04/21 14:55:38  descarte
 * Added WindowListener. Added new demoTemplate methods. Added standalone code.
 *
 * Revision 1.5  1998/03/28 15:51:47  descarte
 * Upgraded to Java 1.1 listeners.
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.4  1998/03/24 16:52:52  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.3  1998/03/19 22:14:48  descarte
 * Added getGL() method to listener.
 *
 * Revision 1.2  1998/03/14 00:05:35  descarte
 * Corrected description.
 *
 * Revision 1.1  1998/03/14 00:00:00  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.glut;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.magician.fonts.*;
import com.hermetica.magician.fonts.GLBitmapFont;
import com.hermetica.util3d.*;

public class fontdemoapp extends Applet implements GLEventListener, demoTemplate, KeyListener, WindowListener {

    /** OpenGL pipeline */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    /** The OpenGL-aware AWT Component we render onto */
    GLComponent glc = null;

    /** Dimensions of the window */
    private static final int WIDTH = 465;
    private static final int HEIGHT = 250;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        gl_ = coregl_;
        glu_ = coreglu_;
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Font Demo" );
      }

    public String getVersion() {
        return new String( "$Id: fontdemo.java.stub,v 1.9 1998/10/19 10:49:14 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demo illustrates the use of both bitmapped and\n" );
        sb.append( "stroke fonts within Magician.\n\n" );
        sb.append( "Press ESCAPE to terminate this demo." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mark Kilgard <mjk@sgi.com>" );
      }

    /** Creates the new program and initializes the GL state */
    public void startDemo() {

        /** Set the LayoutManager to allow reshaping */
        setLayout( new BorderLayout() );

        /** Create the GL Component and add it to the Frame */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        add( "Center", glc );

        /** Pack the components into the Frame and display it */
        /**#layout */

        /** Setup the capabilities of the OpenGL context */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the listener with the GLComponent */
        glc.addGLEventListener( this );

        /** Add a key listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Tell the component to start processing events */
        glc.initialize();
      }

    /** General initialization routines */
    public void initialize( GLDrawable component ) {
        gl_.glClearColor( 1.0f, 1.0f, 1.0f, 1.0f );
        gl_.glColor3f( 0, 0, 0 );
        gl_.glLineWidth( 3.0f );
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluOrtho2D( 0, width, 0, height );
        gl_.glScalef( 1, -1, 1 );
        gl_.glTranslatef( 0, -(float)height, 0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
      }

    /** Renders the scene */
    public void display( GLDrawable component ) {
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT );

        /** Draw the bitmapped fonts */
        bitmapOutput( 40, 35, "This is written in a Magician bitmap font.",
                      new GLTimesRoman24BitmapFont() );
        bitmapOutput( 30, 210, "More bitmap text is a fixed 9 by 15 font.",
                      new GL9x15BitmapFont() );
        bitmapOutput( 70, 240, "                Helvetica is yet another bitmap font.",
                      new GLHelvetica18BitmapFont() );
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Draws a bitmapped string */
    private void bitmapOutput( int x, int y, String str, GLBitmapFont font ) {
        gl_.glRasterPos2f( x, y );
        font.drawString( str );
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
