/* 
 *  Demonstration of picking and rendering luminous objects.  Drag the
 *  middle mouse button to spin the object.  Move the mouse over the
 *  bulbs to light them.
 * 
 *  author: Nate Robins
 *  email: ndr@pobox.com
 *  www: http://www.pobox.com/~ndr 
 */

/**
 * $Id: signal.java.stub,v 1.7 1998/10/19 10:49:16 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Demonstrates picking luminous objects. Middle mouse button rotation hasn't
 * been implemented.
 *
 * $Log: signal.java.stub,v $
 * Revision 1.7  1998/10/19 10:49:16  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.6  1998/10/02 12:23:30  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.5  1998/06/07 14:07:52  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.4  1998/04/21 14:55:39  descarte
 * Added WindowListener. Added new demoTemplate methods. Added standalone code.
 *
 * Revision 1.3  1998/03/28 16:00:04  descarte
 * Upgraded to Java-1.1 listeners.
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.2  1998/03/27 00:11:05  descarte
 * Fixed demo. glSelectBuffer() was not being called in the correct place
 * or often enough.
 * Added glFlush() to ensure all OpenGL rendering commands have been called
 * before hit processing begins.
 *
 * Revision 1.1  1998/03/24 16:55:21  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.glut;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class signal extends Frame implements demoTemplate, GLEventListener, KeyListener, MouseMotionListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    /** The GLU pipeline */
    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** The display list for the traffic signal */
    private int signalList = 0;

    /** Current light that is currentLight */
    public int currentLight = 0;
    public int lastLight = 0;

    /** The select buffer */
    private int SELECT_BUFFER_SIZE = 32;
    private int[] selectBuffer = new int[SELECT_BUFFER_SIZE];

    /** A quadric creation object */
    private GLUQuadric quadric;

    /** Lighting information */
    float[] lightZeroPosition = { 10.0f, 4.0f, 10.0f, 1.0f };
    float[] lightZeroColour = { 0.8f, 1.0f, 0.8f, 1.0f };
    float[] lightOnePosition = { -1.0f, -2.0f, 1.0f, 0.0f };
    float[] lightOneColour = { 0.6f, 0.3f, 0.2f, 1.0f };
    float[] skinColour = { 0.1f, 1.0f, 0.1f, 1.0f };
    float[] eyeColour = { 1.0f, 0.2f, 0.2f, 1.0f };

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        signal d = new signal();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public signal() {
        gl_ = coregl_;
        glu_ = coreglu_;
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Traffic Signals" );
      }

    public String getVersion() {
        return new String( "$Id: signal.java.stub,v 1.7 1998/10/19 10:49:16 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demo uses the OpenGL selection mechanisms to\n" );
        sb.append( "light up the three different lights in the traffic\n" );
        sb.append( "light model. Moving the mouse over each light will\n" );
        sb.append( "cause it to be illuminated.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Nate Robins <ndr@pobox.com>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the listener with the component */
        glc.addGLEventListener( this );

        /** Register the keyboard and mouse listeners */
        glc.addMouseMotionListener( this );
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Start the component processing events */
        glc.initialize();
      }

    /** Perform some basic setup of the scene */
    public void initialize( GLDrawable component ) {

        /** Create the display lists */
        quadric = glu_.gluNewQuadric();
        glu_.gluQuadricDrawStyle( quadric, GLU.GLU_FILL );
        glu_.gluQuadricNormals( quadric, GLU.GLU_SMOOTH );
        glu_.gluQuadricOrientation( quadric, GLU.GLU_OUTSIDE );

        /** Create the display list */
        if ( signalList != 0 ) {
            gl_.glDeleteLists( signalList, 1 );
          }

        signalList = gl_.glGenLists( 1 );
        gl_.glNewList( signalList, GL.GL_COMPILE );
            gl_.glBegin( GL.GL_QUADS );
                gl_.glColor3f( 0.4f, 0.4f, 0.4f );
                gl_.glNormal3f( 0.0f, 0.0f, 1.0f );
                gl_.glVertex3f( -1.0f, 3.0f, 1.0f );
                gl_.glVertex3f( -1.0f, -3.0f,  1.0f );
                gl_.glVertex3f( 1.0f, -3.0f,  1.0f );
                gl_.glVertex3f( 1.0f,  3.0f,  1.0f );
                gl_.glNormal3f( 1.0f, 0.0f, 0.0f );
                gl_.glVertex3f( 1.0f,  3.0f,  1.0f );
                gl_.glVertex3f( 1.0f, -3.0f,  1.0f );
                gl_.glVertex3f( 1.0f, -3.0f, -1.0f );
                gl_.glVertex3f( 1.0f,  3.0f, -1.0f );
                gl_.glNormal3f( 0.0f, 0.0f, -1.0f );
                gl_.glVertex3f( 1.0f,  3.0f, -1.0f );
                gl_.glVertex3f( 1.0f, -3.0f, -1.0f );
                gl_.glVertex3f( -1.0f, -3.0f, -1.0f );
                gl_.glVertex3f( -1.0f,  3.0f, -1.0f );
                gl_.glNormal3f( -1.0f, 0.0f, 0.0f );
                gl_.glVertex3f( -1.0f,  3.0f, -1.0f );
                gl_.glVertex3f( -1.0f, -3.0f, -1.0f );
                gl_.glVertex3f( -1.0f, -3.0f,  1.0f );
                gl_.glVertex3f( -1.0f,  3.0f,  1.0f );
                gl_.glNormal3f( 0.0f, 1.0f, 0.0f );
                gl_.glVertex3f( 1.0f,  3.0f, -1.0f );
                gl_.glVertex3f( -1.0f,  3.0f, -1.0f );
                gl_.glVertex3f( -1.0f,  3.0f,  1.0f );
                gl_.glVertex3f( 1.0f,  3.0f,  1.0f );
                gl_.glNormal3f( 0.0f, -1.0f, 0.0f);
                gl_.glVertex3f( 1.0f, -3.0f,  1.0f );
                gl_.glVertex3f( -1.0f, -3.0f,  1.0f );
                gl_.glVertex3f( -1.0f, -3.0f, -1.0f );
                gl_.glVertex3f( 1.0f, -3.0f, -1.0f );
            gl_.glEnd();

            gl_.glDisable( GL.GL_CULL_FACE );
            gl_.glTranslatef( 0.0f, 1.75f, 1.0f );
            glu_.gluCylinder( quadric, 0.75f, 0.75f, 0.75f, 32, 32 );
            gl_.glTranslatef( 0.0f, -1.75f, 0.0f );
            glu_.gluCylinder( quadric, 0.75f, 0.75f, 0.75f, 32, 32 );
            gl_.glTranslatef( 0.0f, -1.75f, 0.0f );
            glu_.gluCylinder( quadric, 0.75f, 0.75f, 0.75f, 32, 32 );
            gl_.glTranslatef( 0.0f, 1.75f, -1.0f );
            gl_.glEnable( GL.GL_CULL_FACE );
        gl_.glEndList();

        glu_.gluQuadricOrientation( quadric, GLU.GLU_INSIDE );

        /** Setup OpenGL lighting */
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_COLOR_MATERIAL );

        /** Depth functions */
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glDepthFunc( GL.GL_LEQUAL );

        /** Turn on culling */
        gl_.glEnable( GL.GL_CULL_FACE );

        /** Set the clear colour for the framebuffer */
        gl_.glClearColor( 0.3f, 0.3f, 1.0f, 0.0f );

        /** Set the lighting mode */
        gl_.glLightModeli( GL.GL_LIGHT_MODEL_TWO_SIDE, GL.GL_FALSE );
      }

    /** Resizes the viewport when the window is stretched */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {

        float[] lightPosition = { 1.0f, 1.0f, 1.0f, 0.0f };

        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 60.0f, (float)height / (float)width, 1.0f, 128.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightPosition );

        gl_.glTranslatef( 0.0f, 0.0f, -8.0f );
        gl_.glRotatef( 15.0f, 0.0f, 1.0f, 0.0f );
      }
        
    /** Renders the scene */
    public void display( GLDrawable component ) {

        /** Clear the frame buffer and depth buffer */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        /** Draw the scene */
        gl_.glPushMatrix();
            gl_.glDisable( GL.GL_LIGHT1 );
            /* draw the lights */
            gl_.glTranslatef( 0.0f, 1.75f, 1.0f );

            if ( currentLight == 1 ) {
                gl_.glDisable( GL.GL_LIGHTING );
                gl_.glColor3f( 0.3f, 0.0f, 0.0f );
                glu_.gluCylinder( quadric, 0.74f, 0.74f, 0.75f, 32, 32 );
                gl_.glColor3f( 1.0f, 0.0f, 0.0f );
                shapes.solidSphere( 0.75f, 32, 32 );
                gl_.glEnable( GL.GL_LIGHTING );
              } else {
                gl_.glColor3f( 0.4f, 0.4f, 0.4f );
                glu_.gluCylinder( quadric, 0.74f, 0.74f, 0.75f, 32, 32 );
                gl_.glColor3f( 0.3f, 0.0f, 0.0f );
                shapes.solidSphere( 0.75f, 32, 32 );
              }
            gl_.glTranslatef( 0.0f, -1.75f, 0.0f );
            if ( currentLight == 2 ) {
                gl_.glDisable( GL.GL_LIGHTING );
                gl_.glColor3f( 0.3f, 0.3f, 0.0f );
                glu_.gluCylinder( quadric, 0.74f, 0.74f, 0.75f, 32, 32 );
                gl_.glColor3f( 1.0f, 1.0f, 0.0f );
                shapes.solidSphere( 0.75f, 32, 32 );
                gl_.glEnable( GL.GL_LIGHTING );
              } else {
                gl_.glColor3f( 0.4f, 0.4f, 0.4f );
                glu_.gluCylinder( quadric, 0.74f, 0.74f, 0.75f, 32, 32 );
                gl_.glColor3f( 0.3f, 0.3f, 0.0f );
                shapes.solidSphere( 0.75f, 32, 32 );
              }
            gl_.glTranslatef( 0.0f, -1.75f, 0.0f );
            if ( currentLight == 3 ) {
                gl_.glDisable( GL.GL_LIGHTING );
                gl_.glColor3f( 0.0f, 0.3f, 0.0f );
                glu_.gluCylinder( quadric, 0.74f, 0.74f, 0.75f, 32, 32 );
                gl_.glColor3f( 0.0f, 1.0f, 0.0f );
                shapes.solidSphere( 0.75f, 32, 32 );
                gl_.glEnable( GL.GL_LIGHTING );
              } else {
                gl_.glColor3f( 0.4f, 0.4f, 0.4f );
                glu_.gluCylinder( quadric, 0.74f, 0.74f, 0.75f, 32, 32 );
                gl_.glColor3f( 0.0f, 0.3f, 0.0f );
                shapes.solidSphere( 0.75f, 32, 32 );
              }
            gl_.glTranslatef( 0.0f, 1.75f, -1.0f );
                
            gl_.glCallList( signalList );
              
        gl_.glPopMatrix();
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }


    /** Handles mouse drag events */
    public void mouseDragged( MouseEvent evt ) {
      }

    /** Handles mouse motion events */
    public void mouseMoved( MouseEvent evt ) {

        /** Enable GLContext tracing */
//        glc.getContext().setTraceStatus( true );

        /** Make the rendering context current */
        glc.getContext().makeCurrent();

        currentLight = pick( evt.getX(), evt.getY() );

        /** Unlock the rendering context */
        glc.getContext().unlock();

        /** If the picking status has changed, repaint... */
        if ( ( currentLight != lastLight ) ) {
            glc.repaint();
          }
        lastLight = currentLight;

        /** Turn off tracing */
//        glc.getContext().setTraceStatus( false );
      }

    /** Handles picking */
    private int pick( int x, int y ) {
        int picked = 0;
        int hits = 0;

        /** The viewport dimensions */
        int[] viewport = new int[4];

        /** Setup the select buffer */
        gl_.glSelectBuffer( SELECT_BUFFER_SIZE, selectBuffer );

        /** Get the current viewport parameters */
        gl_.glGetIntegerv( GL.GL_VIEWPORT, viewport );

        /** Set the render mode to selection */
        gl_.glRenderMode( GL.GL_SELECT );
        gl_.glInitNames();
        gl_.glPushName( 0 );

        /** Setup a picking matrix and render into selection buffer */
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glPushMatrix();
            gl_.glLoadIdentity();
            glu_.gluPickMatrix( x, viewport[3] - y, 5.0f, 5.0f, viewport );
            glu_.gluPerspective( 60.0f, 
                                 (float)viewport[3] / (float)viewport[2],
                                 1.0f, 128.0f );
            gl_.glMatrixMode( GL.GL_MODELVIEW );
            gl_.glLoadIdentity();
            gl_.glTranslatef( 0.0f, 0.0f, -8.0f );
            gl_.glRotatef( 15.0f, 0.0f, 1.0f, 0.0f );

            gl_.glPushMatrix();
                gl_.glTranslatef( 0.0f, 1.75f, 1.0f );
                gl_.glLoadName( 1 );
                shapes.solidSphere( 0.75f, 8, 8 );
                gl_.glTranslatef( 0.0f, -1.75f, 0.0f );
                gl_.glLoadName( 2 );
                shapes.solidSphere( 0.75f, 8, 8 );
                gl_.glTranslatef( 0.0f, -1.75f, 0.0f );
                gl_.glLoadName( 3 );
                shapes.solidSphere( 0.75f, 8, 8 );
            gl_.glPopMatrix();

            gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glPopMatrix();
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glFlush();

        hits = gl_.glRenderMode( GL.GL_RENDER );
//        System.err.println( "Hits: " + hits );

        picked = 0;
        int index = 0;
        int depth = -1;
        for ( int i = 0 ; i < hits ; i++ ) {
            int save = GL.GL_FALSE;
            int numNames = selectBuffer[index];
//            System.err.println( "numNames: " + numNames );
            index++;

            if ( selectBuffer[index] <= depth ) {
                depth = selectBuffer[index];
                save = GL.GL_TRUE;
              }
            index++;

            if ( selectBuffer[index] / (float)0x7fffffffl <= depth ) {
                depth = selectBuffer[index];
                save = GL.GL_TRUE;
              }
            index++;

            if ( save == GL.GL_TRUE ) {
                picked = selectBuffer[index];
              }

//            System.err.println( "name: " + selectBuffer[index] );
            index += numNames;
          }
        return picked;
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
