
/* Copyright (c) Mark J. Kilgard, 1994. */

/**
 * (c) Copyright 1993, 1994, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics,

/**
 * $Id: spots.java.stub,v 1.10 1998/10/19 10:49:17 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Demonstrates OpenGL lighting in a ``spotlight'' fashion.
 *
 * $Log: spots.java.stub,v $
 * Revision 1.10  1998/10/19 10:49:17  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.9  1998/10/02 12:22:57  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.8  1998/06/07 14:07:52  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.7  1998/04/21 14:55:41  descarte
 * Added WindowListener. Added new demoTemplate methods. Added standalone code.
 *
 * Revision 1.6  1998/03/30 12:03:10  descarte
 * Removed Light class and made it public.
 *
 * Revision 1.5  1998/03/28 15:23:38  descarte
 * Upgraded to GLComponentFactory.
 * Upgraded to Java 1.1 listeners.
 *
 * Revision 1.4  1998/03/24 16:18:01  descarte
 * Converted to GLDrawable interface.
 *
 * Revision 1.3  1998/03/19 22:09:00  descarte
 * Added getGL() method to listener.
 *
 * Revision 1.2  1998/03/16 12:41:00  descarte
 * Converted from old beta4 to new beta4 component syntax.
 *
 * Revision 1.1  1998/02/22 20:57:32  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.glut;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class spots extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();
    ProfileGL profiler_ = new ProfileGL();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 200,
                HEIGHT = 200;

    /** The number of lights in the demo */
    private final int NUM_LIGHTS = 3;

    /** An array of lights */
    private final Light[] spots = new Light[NUM_LIGHTS];

    /** Specifies whether specular and ambient are treated identically */
    private boolean useSAME_AMB_SPEC = true;

    /** Ambient setting for the model */
    float[] modelAmb = { 0.2f, 0.2f, 0.2f, 1.0f };
    float[] matAmb = { 0.2f, 0.2f, 0.2f, 1.0f };
    float[] matDiff = { 0.8f, 0.8f, 0.8f, 1.0f };
    float[] matSpec = { 0.4f, 0.4f, 0.4f, 1.0f };
    float[] matEmission = { 0.0f, 0.0f, 0.0f, 1.0f };

    /** The angle of spin of the scene */
    private int spin = 0;

    /** Convenient variable for 2 * PI */
    private float TWO_PI = (float)Math.PI * 2;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        spots d = new spots();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public spots() {
        gl_ = coregl_;
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Spotlights" );
      }

    public String getVersion() {
        return new String( "$Id: spots.java.stub,v 1.10 1998/10/19 10:49:17 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of OpenGL's\n" );
        sb.append( "sophisticated lighting model to render spotlight\n" );
        sb.append( "effects playing on a surface.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mark J. Kilgard <mjk@sgi.com>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the listener with the component */
        glc.addGLEventListener( this );

        /** Register the keyboard listener with the GLComponent */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Start the component processing events with continuous refresh */
        glc.initialize();
        glc.start();
      }

    public void initialize( GLDrawable component ) {

        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_NORMALIZE );

        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, modelAmb );
        gl_.glLightModelf( GL.GL_LIGHT_MODEL_LOCAL_VIEWER, GL.GL_TRUE );
        gl_.glLightModelf( GL.GL_LIGHT_MODEL_TWO_SIDE, GL.GL_FALSE );

        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, matAmb );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, matDiff );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, matSpec );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, matEmission );
        gl_.glMaterialf( GL.GL_FRONT, GL.GL_SHININESS, 10.0f );

        /** Create the lights */
        spots[0] = 
            new Light( new float[]{ 0.2f, 0.0f, 0.0f, 1.0f },
                       new float[]{ 0.8f, 0.0f, 0.0f, 1.0f },
                       new float[]{ 0.4f, 0.0f, 0.0f, 1.0f },
                       new float[]{ 0.0f, 0.0f, 0.0f, 1.0f },
                       new float[]{ 0.0f, -1.0f, 0.0f },
                       20.0f, 60.0f,
                       new float[]{ 1.0f, 0.0f, 0.0f },
                       new float[]{ 0.0f, 1.25f, 0.0f },
                       new float[]{ 0.0f, 0.0f, 0.0f },
                       new float[]{ 20.0f, 0.0f, 40.0f },
                       new float[]{ 0.0f, 0.0f, 0.0f },
                       new float[]{ TWO_PI / 70.0f, 0.0f, TWO_PI / 140.0f } );

        spots[1] =
            new Light( new float[]{ 0.0f, 0.2f, 0.0f, 1.0f },
                       new float[]{ 0.0f, 0.8f, 0.0f, 1.0f },
                       new float[]{ 0.0f, 0.4f, 0.0f, 1.0f },
                       new float[]{ 0.0f, 0.0f, 0.0f, 1.0f },
                       new float[]{ 0.0f, -1.0f, 0.0f },
                       20.0f, 60.0f,
                       new float[]{ 1.0f, 0.0f, 0.0f },
                       new float[]{ 0.0f, 1.25f, 0.0f },
                       new float[]{ 0.0f, 0.0f, 0.0f },
                       new float[]{ 20.0f, 0.0f, 40.0f },
                       new float[]{ 0.0f, 0.0f, 0.0f },
                       new float[]{ TWO_PI / 120.0f, 0.0f, TWO_PI / 60.0f } );

        spots[2] =
            new Light( new float[]{ 0.0f, 0.0f, 0.2f, 1.0f },
                       new float[]{ 0.0f, 0.0f, 0.8f, 1.0f },
                       new float[]{ 0.0f, 0.0f, 0.4f, 1.0f },
                       new float[]{ 0.0f, 0.0f, 0.0f, 1.0f },
                       new float[]{ 0.0f, -1.0f, 0.0f },
                       20.0f, 60.0f,
                       new float[]{ 1.0f, 0.0f, 0.0f },
                       new float[]{ 0.0f, 1.25f, 0.0f },
                       new float[]{ 0.0f, 0.0f, 0.0f },
                       new float[]{ 20.0f, 0.0f, 40.0f },
                       new float[]{ 0.0f, 0.0f, 0.0f },
                       new float[]{ TWO_PI / 50.0f, 0.0f, TWO_PI / 100.0f } );

        /** Initialize the lights */
        initLights();
      }

    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0, 1.0, -1.0, 1.0, 2, 6 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glTranslatef( 0.0f, 0.0f, -3.0f );
        gl_.glRotatef( 45.0f, 1.0f, 0.0f, 0.0f );
      }
        
    public void display( GLDrawable component ) {
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT );

        gl_.glPushMatrix();
        gl_.glRotatef( spin, 0.0f, 1.0f, 0.0f );

        aimLights();
        setLights();

        gl_.glPushMatrix();
        gl_.glRotatef( -90.0f, 1.0f, 0.0f, 0.0f );
        gl_.glScalef( 1.9f, 1.9f, 1.0f );
        gl_.glTranslatef( -0.5f, -0.5f, 0.0f );
        drawPlane( 16, 16 );
        gl_.glPopMatrix();

        drawLights();
        gl_.glPopMatrix();

        spin += 1;
        if ( spin > 360 ) {
            spin -= 360;
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Initializes the lights */
    private void initLights() {

        for ( int i = 0 ; i < NUM_LIGHTS ; ++i ) {
            int lt = GL.GL_LIGHT0 + i;

            gl_.glEnable( lt );
            gl_.glLightfv( lt, GL.GL_AMBIENT, spots[i].amb );
            gl_.glLightfv( lt, GL.GL_DIFFUSE, spots[i].diff );

            if ( useSAME_AMB_SPEC) {
                gl_.glLightfv( lt, GL.GL_SPECULAR, spots[i].amb );
              } else {
                gl_.glLightfv( lt, GL.GL_SPECULAR, spots[i].spec );
              }
      
            gl_.glLightf( lt, GL.GL_SPOT_EXPONENT, spots[i].spotExp );
            gl_.glLightf( lt, GL.GL_SPOT_CUTOFF, spots[i].spotCutoff );
            gl_.glLightf( lt, GL.GL_CONSTANT_ATTENUATION, spots[i].atten[0] );
            gl_.glLightf( lt, GL.GL_LINEAR_ATTENUATION, spots[i].atten[1] );
            gl_.glLightf( lt, GL.GL_QUADRATIC_ATTENUATION, spots[i].atten[2] );
          }
      }

    /** Sets the lights */
    private void setLights() {
        for ( int i = 0 ; i < NUM_LIGHTS ; ++i ) {
            int lt = GL.GL_LIGHT0 + i;

            gl_.glPushMatrix();
                gl_.glTranslatef( spots[i].trans[0], spots[i].trans[1], spots[i].trans[2] );
                gl_.glRotatef( spots[i].rot[0], 1.0f, 0.0f, 0.0f );
                gl_.glRotatef( spots[i].rot[1], 0.0f, 1.0f, 0.0f );
                gl_.glRotatef( spots[i].rot[2], 0.0f, 0.0f, 1.0f );
                gl_.glLightfv( lt, GL.GL_POSITION, spots[i].pos );
                gl_.glLightfv( lt, GL.GL_SPOT_DIRECTION, spots[i].spotDir );
            gl_.glPopMatrix();
          }
      }

    /** Aims the lights */
    private void aimLights() {
        for ( int i = 0 ; i < NUM_LIGHTS ; ++i ) {
            spots[i].rot[0] = spots[i].swing[0] * (float)Math.sin( spots[i].arc[0] );
            spots[i].arc[0] += spots[i].arcIncr[0];
            if ( spots[i].arc[0] > TWO_PI ) {
                spots[i].arc[0] -= TWO_PI;
              }
    
            spots[i].rot[1] = spots[i].swing[1] * (float)Math.sin( spots[i].arc[1] );
            spots[i].arc[1] += spots[i].arcIncr[1];
            if ( spots[i].arc[1] > TWO_PI ) {
                spots[i].arc[1] -= TWO_PI;
              }

            spots[i].rot[2] = spots[i].swing[2] * (float)Math.sin( spots[i].arc[2] );
            spots[i].arc[2] += spots[i].arcIncr[2];
            if ( spots[i].arc[2] > TWO_PI ) {
                spots[i].arc[2] -= TWO_PI;
              }
          }
      }

    /** Draws the lights */
    private void drawLights() {

        gl_.glDisable( GL.GL_LIGHTING );

        for ( int i = 0 ; i < NUM_LIGHTS ; ++i ) {

            gl_.glColor4fv( spots[i].diff );

            gl_.glPushMatrix();
                gl_.glTranslatef( spots[i].trans[0], 
                                  spots[i].trans[1], 
                                  spots[i].trans[2] );
                gl_.glRotatef( spots[i].rot[0], 1, 0, 0 );
                gl_.glRotatef( spots[i].rot[1], 0, 1, 0 );
                gl_.glRotatef( spots[i].rot[2], 0, 0, 1 );
                gl_.glBegin( GL.GL_LINES );
                    gl_.glVertex3f( spots[i].pos[0], 
                                    spots[i].pos[1], 
                                    spots[i].pos[2] );
                    gl_.glVertex3f( spots[i].spotDir[0], 
                                    spots[i].spotDir[1], 
                                    spots[i].spotDir[2] );
                gl_.glEnd();
            gl_.glPopMatrix();
          }
        gl_.glEnable( GL.GL_LIGHTING );
      }

    /** Draws the plane the lights play onto */
    private void drawPlane( int width, int height ) {
        float dw = 1.0f / (float)width;
        float dh = 1.0f / (float)height;

        gl_.glNormal3f( 0.0f, 0.0f, 1.0f );
        for ( int j = 0 ; j < height ; ++j ) {
            gl_.glBegin( GL.GL_TRIANGLE_STRIP );
                for ( int i = 0 ; i <= width ; ++i ) {
                    gl_.glVertex2f( dw * i, dh * ( j + 1 ) );
                    gl_.glVertex2f( dw * i, dh * j );
                  }
            gl_.glEnd();
          }
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
