
/* A demo of the GLU polygon tesselation functions written by Bogdan Sikorski */

/**
 * $Id: tessdemo.java.stub,v 1.12 1998/10/19 10:49:19 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Demonstrates the GLU tesselator functions
 *
 * $Log: tessdemo.java.stub,v $
 * Revision 1.12  1998/10/19 10:49:19  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.11  1998/10/02 12:23:27  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.10  1998/06/07 14:07:52  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.9  1998/05/14 15:14:49  descarte
 * Removed GLComponent argument from tesselator constructor.
 *
 * Revision 1.8  1998/04/21 14:55:43  descarte
 * Added WindowListener. Added new demoTemplate methods. Added standalone code.
 *
 * Revision 1.7  1998/03/30 12:19:01  descarte
 * Removed private classes and moved to being public in their own files.
 *
 * Revision 1.6  1998/03/28 16:38:45  descarte
 * Upgraded to Java-1.1 listeners.
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.5  1998/03/24 16:50:55  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.4  1998/03/19 22:13:28  descarte
 * Added getGL() method to listener.
 *
 * Revision 1.3  1998/03/18 21:52:00  descarte
 * Added edgeFlag() callback.
 *
 * Revision 1.2  1998/03/15 20:45:02  descarte
 * Slightly twiddled the error callback for the private tesselator class.
 *
 * Revision 1.1  1998/03/14 14:12:35  descarte
 * Initial revision
 *
 * Revision 1.1  1998/02/22 20:57:32  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.glut;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.magician.fonts.*;
import com.hermetica.util3d.*;

public class tessdemo extends Frame implements demoTemplate, GLEventListener, ActionListener, MouseListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();
    TraceGLU traceglu_ = new TraceGLU( coreglu_ );

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** A dismiss button */
    Button dismissButton = new Button( "Dismiss" );

    /** Typedef for actions */
    private final int QUIT = 0,
                      TESSELATE = 1,
                      CLEAR = 2;

    /** Typedef for mode */
    public final int DEFINE = 0,
                     TESSELATED = 1;

    /** The mode tracking variable */
    protected int mode = -1;

    /** The number of contours and an array of contours */
    public final int MAX_CONTOURS = 50;
    public contour[] contours = new contour[MAX_CONTOURS];
    public int contourCount = 0;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        tessdemo d = new tessdemo();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public tessdemo() {
        gl_ = coregl_;
        glu_ = coreglu_;
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Tesselator Demo" );
      }

    public String getVersion() {
        return new String( "$Id: tessdemo.java.stub,v 1.12 1998/10/19 10:49:19 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of GLU\n" );
        sb.append( "tesselators to tesselate complex polygons drawn by\n" );
        sb.append( "the user into triangles that can be rendered by\n" );
        sb.append( "OpenGL.\n\n" );
        sb.append( "Use the mouse to place points on the grid then click\n" );
        sb.append( "``Close'' to close the polygon. To tesselate this\n" );
        sb.append( "complex polygon, click the ``Tesselate'' button.\n" );
        sb.append( "If all goes well, the polygon will be tesselator and\n" );
        sb.append( "drawn, otherwise the error will be printed. To draw\n" );
        sb.append( "another polygon, click ``Clear''.\n\n" );
        sb.append( "Click ``Dismiss'' to quit the demo." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return glc.WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return glc.HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Bogdan Sikorski <???@???.???>" );
      }

    public void startDemo() {
        setLayout( new GridBagLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( 200, 200 );

        constrain.constrain( this, glc, 0, 0, 1, 1 );

        /** Add a panel containing some buttons */
        Button closeButton = new Button( "Close" );
        Button tesselateButton = new Button( "Tesselate" );
        Button clearButton = new Button( "Clear" );
        Panel buttonPanel = new Panel();
        buttonPanel.setLayout( new GridBagLayout() );
        constrain.constrain( buttonPanel, closeButton, 0, 0, 1, 1,
                             GridBagConstraints.HORIZONTAL,
                             GridBagConstraints.CENTER,
                             1, 1, 0, 0, 0, 0 );
        constrain.constrain( buttonPanel, tesselateButton, 1, 0, 1, 1,
                             GridBagConstraints.HORIZONTAL,
                             GridBagConstraints.CENTER,
                             1, 1, 0, 0, 0, 0 );
        constrain.constrain( buttonPanel, clearButton, 2, 0, 1, 1,
                             GridBagConstraints.HORIZONTAL,
                             GridBagConstraints.CENTER,
                             1, 1, 0, 0, 0, 0  );
        constrain.constrain( buttonPanel, dismissButton, 0, 1, 3, 1,
                             GridBagConstraints.HORIZONTAL,
                             GridBagConstraints.CENTER,
                             1, 1, 0, 0, 0, 0 );

        constrain.constrain( this, buttonPanel, 0, 1, 1, 1,
                             GridBagConstraints.HORIZONTAL,
                             GridBagConstraints.CENTER,
                             1, 1, 0, 0, 0, 0 );

        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the listener with the component */
        glc.addGLEventListener( this );

        /** Add the mouse listener to the GLComponent */
        glc.addMouseListener( this );

        this.addWindowListener( this );

        /** Add the action listeners to the buttons */
        closeButton.addActionListener( this );
        tesselateButton.addActionListener( this );
        clearButton.addActionListener( this );
        dismissButton.addActionListener( this );

        /** Start the component processing events with continuous refresh */
        glc.initialize();
      }

    public void initialize( GLDrawable component ) {

        /** Clear background to grey */
        gl_.glClearColor( 0.4f, 0.4f, 0.4f, 0.0f );

        /** Set the shade model */
        gl_.glShadeModel( GL.GL_FLAT );

        contourCount = 0;

        /** Populate the contour array */
        for ( int i = 0 ; i < MAX_CONTOURS ; i++ ) {
            contours[i] = new contour();
          }

        gl_.glPolygonMode( GL.GL_FRONT, GL.GL_FILL );
        mode = DEFINE;
      }

    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glOrtho( 0.0f, (float)width, 0.0f, (float)height, -1.0f, 1.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }
        
    public void display( GLDrawable component ) {

        int pointCounter;

        /** Flush the framebuffer */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT );

        /** Render accordingly */
        switch ( mode ) {
            case DEFINE: {
                /** Draw the background grid */
                gl_.glColor3f( 0.6f, 0.5f, 0.5f );
                gl_.glBegin( GL.GL_LINES );
                    for ( int i = 0 ; i < glc.getSize().width ; i += 10 ) {

                        for ( int j = 0 ; j < glc.getSize().height ; j += 10 ) {
                            gl_.glVertex2i( 0, j );
                            gl_.glVertex2i( glc.getSize().width, j );
                            gl_.glVertex2i( i, glc.getSize().height );
                            gl_.glVertex2i( i, 0 );
                          }
                      }
                gl_.glEnd();

                /** Draw the contour that has been drawn so far */
                gl_.glColor3f( 1.0f, 1.0f, 0.0f );
                for ( int i = 0 ; i <= contourCount ; i++ ) {
                    pointCounter = contours[i].pointCount;
                    gl_.glBegin( GL.GL_LINES );
                        switch ( pointCounter ) {
                            case 0: {
                                break;
                              }
                            case 1: {
                                gl_.glVertex2iv( contours[i].p[0] );
                                gl_.glVertex2iv( contours[i].p[0] );
                                break;
                              }
                            case 2: {
                                gl_.glVertex2iv( contours[i].p[0] );
                                gl_.glVertex2iv( contours[i].p[1] );
                                break;
                              }
                            default: {
                                int j;
                                --pointCounter;
                                for ( j = 0 ; j < pointCounter ; j++ ) {
                                    gl_.glVertex2iv( contours[i].p[j] );
                                    gl_.glVertex2iv( contours[i].p[j+1] );
                                  }
                                if ( contours[i].p[j+1][0] == -1 ) {
                                    gl_.glVertex2iv( contours[i].p[0] );
                                    gl_.glVertex2iv( contours[i].p[j] );
                                  }
                                break;
                              }
                          }
                    gl_.glEnd();
                  }
                break;
              }
            case TESSELATED: {
                /** Draw the lines */
                tesselate();
                break;
              }
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Tesselates the polygon */
    private void tesselate() {
        tessdemoTesselator tess = new tessdemoTesselator();

        gl_.glClear( GL.GL_COLOR_BUFFER_BIT );
        gl_.glColor3f( 0.7f, 0.7f, 0.0f );

        glu_.gluBeginPolygon( tess );
            for ( int j = 0 ; j < contourCount ; j++ ) {
                int pointCount =
                    contours[j].pointCount;
                glu_.gluNextContour( tess, GLU.GLU_UNKNOWN );
                for ( int i = 0 ; i < pointCount ; i++ ) {
                    double[] vertex = new double[3];
                    vertex[0] = 
                        (double)( contours[j].p[i][0] );
                    vertex[1] = 
                        (double)( contours[j].p[i][1] );
                    vertex[2] = 0.0;
                    glu_.gluTessVertex( tess, vertex,
                                        contours[j].p[i] );
                  }
              }
        glu_.gluEndPolygon( tess );
        mode = TESSELATED;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** Handles action events */
    public void actionPerformed( ActionEvent evt ) {

        /** Make the context current */
        glc.getContext().makeCurrent();

        if ( evt.getActionCommand().equals( "Close" ) ) {
            int pointCount = contours[contourCount].pointCount;
            if ( pointCount > 2 ) {
                gl_.glBegin( GL.GL_LINES );
                    gl_.glVertex2iv( contours[contourCount].p[0] );
                    gl_.glVertex2iv( contours[contourCount].p[pointCount - 1] );
                    contours[contourCount].p[pointCount][0] = -1;
                gl_.glEnd();

                contourCount++;
                if ( contourCount >= MAX_CONTOURS ) {
                    System.err.println( "Too many contours specified!" );
                   }
                contours[contourCount].pointCount = 0;
              }
          }
        if ( evt.getActionCommand().equals( "Tesselate" ) ) {
            tesselate();
          }
        if ( evt.getActionCommand().equals( "Clear" ) ) {
            contourCount = 0;
            mode = DEFINE;
            for ( int i = 0 ; i < MAX_CONTOURS ; i++ ) {
                contours[i] = new contour();
              }
          }
        if ( evt.getActionCommand().equals( "Dismiss" ) ) {
            glc.getContext().unlock();
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

            return;
          }

        /** Unlock the context and repaint */
        glc.getContext().unlock();
        glc.repaint();
      }

    /** Handles mouse events */
    /** Handles mouse clicked events */
    public void mouseClicked( MouseEvent evt ) {
        /** Switch in a rendering context */
        glc.getContext().makeCurrent();

        /** Test to see whether shift is down. If so, end contour */
        /** Snap to grid */
//        x += x % 10;
//        y += y % 10;

        /** Translate coordinates from AWT space to OpenGL space */
        int glx = evt.getX();
        int gly = glc.getSize().height - evt.getY();

        /** Set up the contour segment */
        int pointCount = contours[contourCount].pointCount;
        contours[contourCount].p[pointCount][0] = glx;
        contours[contourCount].p[pointCount][1] = gly;
    
        gl_.glBegin( GL.GL_LINES );
            if ( pointCount != 0 ) {
                gl_.glVertex2iv( contours[contourCount].p[pointCount - 1] );
                gl_.glVertex2i( glx, gly );
              } else {
                gl_.glVertex2i( glx, gly );
                gl_.glVertex2i( glx, gly );
              }
        gl_.glEnd();
    
        /** Increment the contour counts */
        ++contours[contourCount].pointCount;
        if ( contours[contourCount].pointCount >= contour.MAX_POINTS ) {
            System.err.println( "Too many points specified!" );
          }
    
        /** Switch out the rendering context */
        glc.getContext().unlock();
    
        /** Repaint the component to show any changes */
        glc.repaint();
      }

    /** Handles mouse entered events */
    public void mouseEntered( MouseEvent evt ) {
      }

    /** Handles mouse exited events */
    public void mouseExited( MouseEvent evt ) {
      }

    /** Handles mouse pressed events */
    public void mousePressed( MouseEvent evt ) {
      }

    /** Handles mouse released events */
    public void mouseReleased( MouseEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
