/**
 * $Id: hud.java.stub,v 1.6 1998/10/19 10:57:29 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Demo that tests out multiple listeners attached to a single component
 * that draws a HUD
 *
 * $Log: hud.java.stub,v $
 * Revision 1.6  1998/10/19 10:57:29  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.5  1998/10/02 12:29:57  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.4  1998/04/21 15:04:02  descarte
 * Added new demoTemplate methods.
 * Added WindowListener
 * Added standalone capability
 *
 * Revision 1.3  1998/03/30 11:31:54  descarte
 * Added applet resource handling.
 *
 * Revision 1.2  1998/03/28 16:46:36  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.1  1998/03/17 20:41:37  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.hud;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class hudapp extends Applet implements demoTemplate, KeyListener, ItemListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The three listeners that will be attached to the component */
    GLEventListener topListener = new hudTopLayer();
    GLEventListener middleListener = new hudMiddleLayer();
    GLEventListener bottomListener = new hudBottomLayer();

    /** Checkboxes regulating the displaying of the various layers */
    Checkbox topCheckbox = null;
    Checkbox middleCheckbox = null;
    Checkbox bottomCheckbox = null;

    /** The dimensions of the demo */
    private int WIDTH = 200,
                HEIGHT = 200;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        gl_ = coregl_;
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Heads-Up Display" );
      }

    public String getVersion() {
        return new String( "$Id: hud.java.stub,v 1.6 1998/10/19 10:57:29 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demo shows the possibilities of adding multiple\n" );
        sb.append( "GLEventListeners to a single GLComponent object in\n" );
        sb.append( "Magician programs. The scene in this demo is composed of\n" );
        sb.append( "three different ``layers'', the background, a spinning cube\n" );
        sb.append( "and some text. A listener responsible for drawing each layer\n" );
        sb.append( "is added to the GLComponent and the top two can be switched\n" );
        sb.append( "by clicking on the checkboxes. In reality, the listeners\n" );
        sb.append( "are unregistered from the component temporarily. They can\n" );
        sb.append( "be easily re-added at a later date.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Alligator Descartes <descarte@arcana.co.uk>" );
      }

    public void startDemo() {
        setLayout( new GridBagLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        constrain.constrain( this, glc, 0, 0, 2, 1,
                             GridBagConstraints.BOTH,
                             GridBagConstraints.CENTER,
                             1.0f, 1.0f, 0, 0, 0, 0 );

        /** Add the checkboxen */
        middleCheckbox = new Checkbox( "Show Middle" );
        middleCheckbox.setState( true );
        topCheckbox = new Checkbox( "Show Top" );
        topCheckbox.setState( true );

        constrain.constrain( this, topCheckbox, 0, 1, 1, 1,
                             GridBagConstraints.HORIZONTAL,
                             GridBagConstraints.CENTER,
                             1, 1, 0, 0, 0, 0 );
        constrain.constrain( this, middleCheckbox, 1, 1, 1, 1,
                             GridBagConstraints.HORIZONTAL,
                             GridBagConstraints.CENTER,
                             1, 1, 0, 0, 0, 0 );

        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the keyboard listener */
        glc.addKeyListener( this );

        /** Add the action event listener to the checkboxen */
        topCheckbox.addItemListener( this );
        middleCheckbox.addItemListener( this );

        /**#windowListener */

        /** Setup the listeners */
        glc.addGLEventListener( bottomListener );
        glc.addGLEventListener( middleListener );
        glc.addGLEventListener( topListener );
        glc.initialize();
        glc.start();
      }

    /** Handles AWT Events */
    public void itemStateChanged( ItemEvent evt ) {
        if ( evt.getSource() == topCheckbox ) {
            if ( topCheckbox.getState() ) {
                glc.addGLEventListener( topListener );
              } else {
                glc.removeGLEventListener( topListener );
              }
          }
        if ( evt.getSource() == middleCheckbox ) {
            if ( middleCheckbox.getState() ) {
                glc.addGLEventListener( middleListener );
              } else {
                glc.removeGLEventListener( middleListener );
              }
          }
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
