/* Copyright (c) Mark J. Kilgard, 1995. */

/*
 * (c) Copyright 1993, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */
/**
 * $Id: track.java.stub,v 1.16 1998/10/19 11:03:27 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * $Log: track.java.stub,v $
 * Revision 1.16  1998/10/19 11:03:27  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.15  1998/10/02 12:31:44  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.14  1998/04/21 15:07:24  descarte
 * Added new demoTemplate methods
 * Added WindowListener
 * Added standalone capability
 *
 * Revision 1.13  1998/03/30 11:33:17  descarte
 * Added applet resource handling.
 *
 * Revision 1.12  1998/03/28 16:51:27  descarte
 * Upgraded to JDK-1.1 listeners.
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.11  1998/03/24 17:11:21  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.10  1998/03/19 22:27:18  descarte
 * Added getGL() method.
 *
 * Revision 1.9  1998/03/17 21:02:54  descarte
 * Converted demo from old beta 4 to new beta 4 syntax.
 *
 * Revision 1.8  1998/03/05 00:05:08  descarte
 * Moved pipeline initializers to constructor from initialize() as invocation
 * order of reshape() and initialize() has been reversed to support correct
 * gluLookAt().
 *
 * Revision 1.7  1998/02/23 19:55:27  descarte
 * Converted to new GLComponent structure for beta 4.
 *
 * Revision 1.6  1998/01/07 20:45:07  descarte
 * Added better thread handling for applets although it's still wrong.
 *
 * Revision 1.5  1998/01/03 15:24:32  descarte
 * Corrected argument ordering in 'add()' method.
 *
 * Revision 1.4  1998/01/01 16:28:45  descarte
 * Corrected a few small problems.
 * Added more comments.
 * Added GLComponent arguments as required by GLEventListener interface.
 *
 * Revision 1.3  1997/12/31 22:28:48  descarte
 * Converted demo across to using the GLEventListener interface.
 *
 */

package com.hermetica.magician.demos.ideas;

import com.hermetica.magician.*;
import com.hermetica.magician.demos.*;
import com.hermetica.util3d.*;
import com.hermetica.vecmath.*;
import java.applet.*;
import java.awt.event.*;
import java.awt.*;

public class trackapp extends Applet implements demoTemplate, GLEventListener, MouseListener, KeyListener, WindowListener {

    private static final int X = 0;
    private static final int Y = 1;
    private static final int Z = 2;

    private static final double DEG = Math.PI / 180.0;
    private static final double RAD = 180.0 / Math.PI;

    float move_speed;		/* Spline distance per second */

    boolean needsRestart = false;

    /** Dimensions of Component */
    private int WIDTH = 300,
                HEIGHT = 240;

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    CoreGLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    /** The GLComponent to render into */
    private GLComponent glc = null;

    /** Object display list definitions */
    private static final int MAT_LOGO = 1;

    private static final int MAT_PENCIL_ERASER = 2;
    private static final int MAT_PENCIL_END = 3;
    private static final int MAT_PENCIL_PAINT = 4;
    private static final int MAT_PENCIL_WOOD = 5;
    private static final int MAT_PENCIL_TIP = 6;

    private static final int MAT_HOLDER_BASE = 7;
    private static final int MAT_HOLDER_RINGS = 8;

    private static final int MAT_HEMISPHERE = 9;
    
    private static final int LIGHT_TMP = 10;
    
    private static final int HALFTONE = 11;
    
    private static final int TEX_LOGO = 12;

    /** Spline distances per second */
    private static final float SPEED_SLOW = 0.2f;	
    private static final float SPEED_MEDIUM	= 0.4f;
    private static final float SPEED_FAST = 0.7f;
    private static final float SPEED_SUPER_FAST = 1.0f;

    private static final int O_NOMS	= 7;
    private static final int O_4MS = 8;
    private static final int O_8MS = 9;
    private static final int O_16MS = 10;

    float[] light1_ambient = { 0.0f, 0.0f, 0.0f, 1.0f  };
    float[] light1_lcolor = { 1.0f, 1.0f, 1.0f, 1.0f };
    float[] light1_position = { 0.0f, 1.0f, 0.0f, 0.0f };

    float[] light2_ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
    float[] light2_lcolor = { 0.3f, 0.3f, 0.5f, 1.0f };
    float[] light2_position = { -1.0f, 0.0f, 0.0f, 0.0f };
    
    float[] light3_ambient = { 0.2f, 0.2f, 0.2f, 1.0f };
    float[] light3_lcolor = { 0.2f, 0.2f, 0.2f, 1.0f };
    float[] light3_position = { 0.0f, -1.0f, 0.0f, 0.0f };

    float[] lmodel_LVW = { 0.0f };
    float[] lmodel_ambient = { 0.3f, 0.3f, 0.3f, 1.0f };
    float[] lmodel_TWO = { (float)GL.GL_TRUE };

    float[] mat_logo_ambient = { 0.1f, 0.1f, 0.1f, 1.0f };
    float[] mat_logo_diffuse = { 0.5f, 0.4f, 0.7f, 1.0f };
    float[] mat_logo_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
    float[] mat_logo_shininess = { 30.0f };

    float[] mat_holder_base_ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
    float[] mat_holder_base_diffuse = { 0.6f, 0.6f, 0.6f, 1.0f };
    float[] mat_holder_base_specular = { 0.8f, 0.8f, 0.8f, 1.0f };
    float[] mat_holder_base_shininess = { 30.0f };

    float[] mat_holder_rings_ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
    float[] mat_holder_rings_diffuse = { 0.9f, 0.8f, 0.0f, 1.0f };
    float[] mat_holder_rings_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
    float[] mat_holder_rings_shininess = { 30.0f };

    float[] mat_hemisphere_ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
    float[] mat_hemisphere_diffuse = { 1.0f, 0.2f, 0.2f, 1.0f };
    float[] mat_hemisphere_specular = { 0.5f, 0.5f, 0.5f, 1.0f };
    float[] mat_hemisphere_shininess = { 20.0f };

    byte[] stipple = new byte[32 * 32];

    float[][] tv = {
        { 1.0f, 0.0f, 0.0f, 0.0f },
        { 0.0f, 1.0f, 0.0f, -1.0f },
        { 0.0f, 0.0f, 1.0f, 0.0f },
        { 0.0f, 0.0f, 0.0f, 0.0f },
      };

    short dev, val;

    float current_time = 0.0f;
    float hold_time = 0.0f;     /** Used when auto-running */

    float[] tmplight = { (float)GL.GL_POSITION, 0.0f, 0.0f, 0.0f, 0.0f };

    float[][] paper_points = {
        { -0.8f, 0.0f, 0.4f },
        { -0.2f, 0.0f, -1.4f },
        { 1.0f, 0.0f, -1.0f },
        { 0.4f, 0.0f, 0.8f },
      };

    private static final int TABLERES = 12;

    float pcr = 0.0f,
          pcg = 0.0f, 
          pcb = 0.0f, 
          pca = 0.0f;

    float[][][] table_points = new float[TABLERES+1][TABLERES+1][3];
    int[][] tablecolors = new int[TABLERES+1][TABLERES+1];

    private static final long TIME = 15;
    private static final float START_TIME = 0.6f;

    float[][] light_pos_ctl = {

        { 0.0f, 1.8f, 0.0f },
        { 0.0f, 1.8f, 0.0f },
        { 0.0f, 1.6f, 0.0f },

        { 0.0f, 1.6f, 0.0f },
        { 0.0f, 1.6f, 0.0f },
        { 0.0f, 1.6f, 0.0f },
        { 0.0f, 1.4f, 0.0f },

        { 0.0f, 1.3f, 0.0f },
        { -0.2f, 1.5f, 2.0f },
        { 0.8f, 1.5f, -0.4f },
        { -0.8f, 1.5f, -0.4f },

        { 0.8f, 2.0f, 1.0f },
        { 1.8f, 5.0f, -1.8f },
        { 8.0f, 10.0f, -4.0f },
        { 8.0f, 10.0f, -4.0f },
        { 8.0f, 10.0f, -4.0f },
      };

    float[][] logo_pos_ctl = {

        { 0.0f, -0.5f, 0.0f },

        { 0.0f, -0.5f, 0.0f },
        { 0.0f, -0.5f, 0.0f },

        { 0.0f, -0.5f, 0.0f },
        { 0.0f, -0.5f, 0.0f },
        { 0.0f, -0.5f, 0.0f },
        { 0.0f, 0.0f, 0.0f },

        { 0.0f, 0.6f, 0.0f },
        { 0.0f, 0.75f, 0.0f },
        { 0.0f, 0.8f, 0.0f },
        { 0.0f, 0.8f, 0.0f },
    
        { 0.0f, 0.5f, 0.0f },
        { 0.0f, 0.5f, 0.0f },
        { 0.0f, 0.5f, 0.0f },
        { 0.0f, 0.5f, 0.0f },
        { 0.0f, 0.5f, 0.0f },
      };


    float[][] logo_rot_ctl = {

        { 0.0f, 0.0f, -18.4f },

        { 0.0f, 0.0f, -18.4f },
        { 0.0f, 0.0f, -18.4f },
    
        { 0.0f, 0.0f, -18.4f },
        { 0.0f, 0.0f, -18.4f },
        { 0.0f, 0.0f, -18.4f },
        { 0.0f, 0.0f, -18.4f },
        { 0.0f, 0.0f, -18.4f },

/*        { 90.0f, 0.0f, -90.0f },
        { 180.0f, 180.0f, 90.0f }, */
        { 240.0f, 360.0f, 180.0f },
        { 90.0f, 180.0f, 90.0f },
 
        { 11.9f, 0.0f, -18.4f },
        { 11.9f, 0.0f, -18.4f },
        { 11.9f, 0.0f, -18.4f },
        { 11.9f, 0.0f, -18.4f },
        { 11.9f, 0.0f, -18.4f },
      };


    float[][] view_from_ctl = {

        { -1.0f, 1.0f, -4.0f },

        { -1.0f, -3.0f, -4.0f },	/* 0 */
        { -3.0f, 1.0f, -3.0f },	/* 1 */

        { -1.8f, 2.0f, 5.4f },	/* 2 */
        { -0.4f, 2.0f, 1.2f },	/* 3 */
        { -0.2f, 1.5f, 0.6f },	/* 4 */
        { -0.2f, 1.2f, 0.6f },	/* 5 */

        { -0.8f, 1.0f, 2.4f },	/* 6 */
        { -1.0f, 2.0f, 3.0f },	/* 7 */
        { 0.0f, 4.0f, 3.6f },	/* 8 */
        { -0.8f, 4.0f, 1.2f },	/* 9 */

        { -0.2f, 3.0f, 0.6f },	/* 10 */
        { -0.1f, 2.0f, 0.3f },	/* 11 */
        { -0.1f, 2.0f, 0.3f },	/* 12 */
        { -0.1f, 2.0f, 0.3f },	/* 13 */
        { -0.1f, 2.0f, 0.3f },	/* 13 */
      };

    float[][] view_to_ctl = {

        { -1.0f, 1.0f, 0.0f },

        { -1.0f, -3.0f, 0.0f },
        { -1.0f, 1.0f, 0.0f },

        { 0.1f, 0.0f, -0.3f },
        { 0.1f, 0.0f, -0.3f },
        { 0.1f, 0.0f, -0.3f },
        { 0.0f, 0.2f, 0.0f },

        { 0.0f, 0.6f, 0.0f },
        { 0.0f, 0.8f, 0.0f },
        { 0.0f, 0.8f, 0.0f },
        { 0.0f, 0.8f, 0.0f },

        { 0.0f, 0.8f, 0.0f },
        { 0.0f, 0.8f, 0.0f },
        { 0.0f, 0.8f, 0.0f },
        { 0.0f, 0.8f, 0.0f },
        { 0.0f, 0.8f, 0.0f },
      };

    private float[] view_from = new float[3], 
                    view_to = new float[3], 
                    logo_pos = new float[3], 
                    logo_rot = new float[3];
    private float[] light_pos = new float[4];    //@@Should be Vec4f

    private float[][][] view_from_spline = null,
                        view_to_spline = null,
	                    light_pos_spline = null,
	                    logo_pos_spline = null,
	                    logo_rot_spline = null;

    double a3, a4;

    int post_idle = 0;

    boolean auto_run = true;    /* If set, then automatically run forever */
    int timejerk = 0;		/* Set to indicate time jerked! (menu pulled down) */
    int paused = 0;	/* Paused? */
    int right = 0;	/* Draw right eye? */
    int resetclock = 0;	/* Reset the clock? */
    float timeoffset;	/* Used to compute timing */
    long start = System.currentTimeMillis();

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    /** Initializer */
    public void init() {
        gl_ = coregl_;
        glu_ = coreglu_;

        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Ideas In Motion" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: track.java.stub,v 1.16 1998/10/19 11:03:27 descarte Exp descarte $" );
      }

    /** Returns a description for this demonstration */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration program illustrates several of the more\n" );
        sb.append( "advanced features of OpenGL, including the ability to \n" );
        sb.append( "simulate real-time shadow casting and accurate spot-lighting.\n" );
        sb.append( "The font is also formed by two-dimensional drawing.\n\n" );
        sb.append( "Hit the ``Escape'' key to terminate this demo or click\n" );
        sb.append( "your mouse buttom to restart the demo.\n" );
        return new String( sb );
      }

    public int getWidth() {
        return WIDTH;
      }

    public int getHeight() {
        return HEIGHT;
      }

    public String getOriginalAuthor() {
        return new String( "Mark J. Kilgard <mjk@sgi.com>" );
      }

    public void startDemo() {

        /* .4 spline distance per second by default */
        move_speed = SPEED_MEDIUM;
        timeoffset = START_TIME;

        /** Set the layout manager */
        setLayout( new BorderLayout() );

        /** Create a new OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Stick it in the frame */
        add( "Center", glc );
        /**#layout */

        /** Setup the OpenGL rendering context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register a new GLEventListener */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /** Register a mouse listener */
        glc.addMouseListener( this );

        /**#windowListener */

        /** Set some timer parameters */
        current_time = timeoffset;
        resetclock = 1;
        timejerk = 0;

        /** Tell the component to start processing events in refresh mode */
        glc.initialize();
        glc.start();
      }

    private void initMaterials() {

        /* Stipple pattern */
        for ( int y = 0 ;  y < 32 ; y++ ) { 
            for ( int x = 0 ; x < 4 ; x++ ) { 
                stipple[y * 4 + x] = (byte)( ( y % 2 != 0 ) ? 0xaa : 0x55 );
              }
          }

        gl_.glNewList( MAT_LOGO, GL.GL_COMPILE ); 
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_logo_ambient ); 
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_logo_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_logo_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, mat_logo_shininess );
        gl_.glEndList(); 

        gl_.glNewList( MAT_HOLDER_BASE, GL.GL_COMPILE );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_holder_base_ambient ); 
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_holder_base_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_holder_base_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, mat_holder_base_shininess );
        gl_.glEndList();

        gl_.glNewList( MAT_HOLDER_RINGS, GL.GL_COMPILE ); 
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_holder_rings_ambient ); 
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_holder_rings_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_holder_rings_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, mat_holder_rings_shininess );
        gl_.glEndList();

        gl_.glNewList( MAT_HEMISPHERE, GL.GL_COMPILE ); 
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_hemisphere_ambient ); 
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_hemisphere_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_hemisphere_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, mat_hemisphere_shininess );
        gl_.glEndList();
      }

    private void initLights() {

        float[] ambient = { 0.1f, 0.1f, 0.1f, 1.0f };
        float[] diffuse = { 0.5f, 1.0f, 1.0f, 1.0f };
        float[] position = { 90.0f, 90.0f, 150.0f, 0.0f };
        
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, ambient );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position );

        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_AMBIENT, light1_ambient );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_SPECULAR, light1_lcolor );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, light1_lcolor );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, light1_position );
    
        gl_.glLightfv( GL.GL_LIGHT2, GL.GL_AMBIENT, light2_ambient );
        gl_.glLightfv( GL.GL_LIGHT2, GL.GL_SPECULAR, light2_lcolor );
        gl_.glLightfv( GL.GL_LIGHT2, GL.GL_DIFFUSE, light2_lcolor );
        gl_.glLightfv( GL.GL_LIGHT2, GL.GL_POSITION, light2_position );

        gl_.glLightfv( GL.GL_LIGHT3, GL.GL_AMBIENT, light3_ambient );
        gl_.glLightfv( GL.GL_LIGHT3, GL.GL_SPECULAR, light3_lcolor );
        gl_.glLightfv( GL.GL_LIGHT3, GL.GL_DIFFUSE, light3_lcolor );
        gl_.glLightfv( GL.GL_LIGHT3, GL.GL_POSITION, light3_position );
  
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_LOCAL_VIEWER, lmodel_LVW );
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );
      }

    public void display( GLDrawable component ) {

        float x, y, z, c;

        if ( needsRestart ) {
            resetclock = 1;
            needsRestart = false;
          }
        float timediff = 0;
        long current;

        if ( timejerk != 0) {
            timejerk = 0;
            timeoffset = current_time;
            start = System.currentTimeMillis();
          }
    
        if ( resetclock != 0 ) {
            resetclock = 0;
            paused = 0;
            timeoffset = START_TIME;
            start = System.currentTimeMillis();
          }
    
        current = System.currentTimeMillis();
        timediff = (float)( current - start );
        timediff /= 1200;
        current_time = timediff * move_speed + timeoffset;

        /** Calculate the various splines for viewing and navigating the scene */
        view_from = calc_spline( view_from_spline, current_time );
        view_to = calc_spline( view_to_spline, current_time );
        light_pos = calc_spline( light_pos_spline, current_time );
        logo_pos = calc_spline( logo_pos_spline, current_time );
        logo_rot = calc_spline( logo_rot_spline, current_time );

        /** If any are null, it's time to restart... */
        if ( view_from == null || view_to == null || light_pos == null ||
             logo_pos == null || logo_rot == null || needsRestart == true ) {
            needsRestart = true;
            return;
          }
        light_pos[3] = 0.0f;
    
        tmplight[1] = light_pos[X] - logo_pos[X];
        tmplight[2] = light_pos[Y] - logo_pos[Y];
        tmplight[3] = light_pos[Z] - logo_pos[Z];

        gl_.glNewList(LIGHT_TMP, GL.GL_COMPILE); 
            gl_.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE, tmplight ); 
        gl_.glEndList();
    
        tv[0][0] = tv[1][1] = tv[2][2] = light_pos[Y];
    
        gl_.glColor3ub( (byte)0,  (byte)0,  (byte)0 );
        gl_.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT); 
    
        /*
         * SHADOW
         */
        gl_.glMatrixMode(GL.GL_MODELVIEW);
        gl_.glLoadIdentity();
        glu_.gluLookAt( (double)view_from[X], 
                        (double)view_from[Y], 
                        (double)view_from[Z], 
      	                (double)view_to[X], 
     	                (double)view_to[Y], 
     	                (double)view_to[Z],
     	                0.0, 1.0, 0.0 );
    
        if ( view_from[Y] > 0.0 ) {
            draw_table();
          }
    
        gl_.glEnable(GL.GL_CULL_FACE); 
        gl_.glDisable(GL.GL_DEPTH_TEST); 
    
        if (logo_pos[Y] < 0.0) {
      
            if (logo_pos[Y]>-0.33) {
        	    /* We're emerging from the table */
        	    c = (float)( 1.0 - ( logo_pos[Y] ) / -0.33 );
        	    pca /= 4.0;
        	    gl_.glColor3ub( (byte)( (int)(128.0*(1.0-c)*0.5 + 255.0*pca*c) ),
        	        	       (byte)( (int)(102.0*(1.0-c)*0.5 + 255.0*pca*c) ),
        	        	       (byte)( (int)(179.0*(1.0-c)*0.5 + 200.0*pca*c)) );
              } else {
        	    /* Still under table */
        	    gl_.glColor3ub( (byte)( 128 / 2 ),
        	                   (byte)( 102 / 2 ),
        	                   (byte)( 179 / 2 ) );
              }
      
            gl_.glPushMatrix();
            gl_.glScalef( 0.04f,  0.0f,  0.04f );
            gl_.glRotatef( (float)( 0.1 * (-900) ), 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * ((int)(10.0*logo_rot[Z])) ), 
                           0.0f, 0.0f, 1.0f );
            gl_.glRotatef( (float)( 0.1 * ((int)(10.0*logo_rot[Y])) ), 
                           0.0f, 1.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * ((int)(10.0*logo_rot[X])) ), 
                           1.0f, 0.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * (353) ), 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * (450) ), 0.0f, 1.0f, 0.0f );
            shadow.draw( gl_ );
            gl_.glPopMatrix();
          }
        
        if ( logo_pos[Y] > 0.0f ) {
            gl_.glPushMatrix();
            if ( logo_pos[Y] < 0.33f ) {
                pca /= 4.0;
                c = (float)( 1.0 - ( logo_pos[Y] )/0.33 );
                gl_.glColor3ub( (byte)( (int)(255.0*pca*c) ),
     	                        (byte)( (int)(255.0*pca*c) ),
                                (byte)( (int)(200.0*pca*c)) );
              } else {
                gl_.glColor3ub( (byte)0, (byte)0, (byte)0 );
              }
          
            gl_.glTranslatef(light_pos[X],  light_pos[Y],  light_pos[Z]);

            /** Create a temporary flat matrix for glMultMatrixf() */
            float tmp[] = new float[16];
            int index = 0;
            for ( int i = 0 ; i < 4 ; i++ ) {
                for ( int j = 0 ; j < 4 ; j++ ) {
                    tmp[index] = tv[i][j];
                    index++;
                  }
              }
            gl_.glMultMatrixf( tmp );

            gl_.glTranslatef( -light_pos[X]+logo_pos[X],
    	       	              -light_pos[Y]+logo_pos[Y],
    		                  -light_pos[Z]+logo_pos[Z]);
            gl_.glScalef( 0.04f,  0.04f,  0.04f );
            gl_.glRotatef( (float)( 0.1 * (-900) ), 
                           1.0f, 0.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * ((int)(10.0*logo_rot[Z])) ), 
                           0.0f, 0.0f, 1.0f );
            gl_.glRotatef( (float)( 0.1 * ((int)(10.0*logo_rot[Y])) ), 
                           0.0f, 1.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * ((int)(10.0*logo_rot[X])) ), 
                          1.0f, 0.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * (353) ), 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * (450) ), 0.0f, 1.0f, 0.0f );
    
        
            gl_.glEnable(GL.GL_POLYGON_STIPPLE);
            gl_.glPolygonStipple(stipple);
            shadow.draw( gl_ );
            gl_.glDisable(GL.GL_POLYGON_STIPPLE);
            gl_.glPopMatrix();
          }
    
        /*
         * DONE SHADOW 
         */
        gl_.glEnable(GL.GL_DEPTH_TEST);
        gl_.glDisable(GL.GL_CULL_FACE);
        gl_.glEnable(GL.GL_LIGHTING);

        gl_.glMatrixMode(GL.GL_PROJECTION);
        gl_.glLoadIdentity();
        glu_.gluPerspective( .1*(450),  5.0/4.0,  0.5,  20.0 );
        gl_.glMatrixMode(GL.GL_MODELVIEW);
        gl_.glLoadIdentity();

        glu_.gluLookAt( view_from[X],  view_from[Y],  view_from[Z],
     	                view_to[X],  view_to[Y],  view_to[Z], 
     	                0.0, 1.0, 0.0 );
    
        gl_.glCallList( MAT_HOLDER_RINGS); 
    
        gl_.glPushMatrix();
        gl_.glTranslatef(light_pos[X],  light_pos[Y],  light_pos[Z]);
        gl_.glScalef( 0.1f,  0.1f,  0.1f );

        x = light_pos[X] - logo_pos[X];
        y = light_pos[Y] - logo_pos[Y];
        z = light_pos[Z] - logo_pos[Z];

        if (x!=0.0) {
            a3 = -Math.atan2(z, x) * 10.0 * RAD;
          } else {
            a3 = 0.0;
          }
    
        a4 = -Math.atan2( Math.sqrt( x*x + z*z ), y ) * 10.0 * RAD;
    
        gl_.glRotatef( (float)( 0.1 * ((int)a3) ), 0.0f, 1.0f, 0.0f );
        gl_.glRotatef( (float)( 0.1 * ((int)a4) ), 0.0f, 0.0f, 1.0f );
        gl_.glRotatef( (float)( 0.1 * (-900) ), 1.0f, 0.0f, 0.0f );
    
        gl_.glEnable(GL.GL_LIGHT2);
        gl_.glEnable(GL.GL_LIGHT3);
        gl_.glCallList(MAT_HEMISPHERE);
        gl_.glEnable(GL.GL_NORMALIZE);
        lamp.draw( gl_ );
        gl_.glDisable(GL.GL_NORMALIZE);
        gl_.glPopMatrix();

        gl_.glDisable(GL.GL_LIGHT2);
        gl_.glDisable(GL.GL_LIGHT3); 
        gl_.glEnable(GL.GL_LIGHT1);
        gl_.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, light_pos);

        if (logo_pos[Y] > -0.33) {

            gl_.glCallList(MAT_LOGO);
    
            gl_.glPushMatrix();
            gl_.glTranslatef( logo_pos[X],  logo_pos[Y],  logo_pos[Z] );
            gl_.glScalef( 0.04f,  0.04f,  0.04f );
            gl_.glRotatef( (float)( 0.1 * (-900) ), 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * ((int)(10.0*logo_rot[Z])) ), 
                           0.0f, 0.0f, 1.0f );
            gl_.glRotatef( (float)( 0.1 * ((int)(10.0*logo_rot[Y])) ), 
                           0.0f, 1.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * ((int)(10.0*logo_rot[X])) ), 
                           1.0f, 0.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * (353) ), 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( (float)( 0.1 * (450) ), 0.0f, 1.0f, 0.0f );
            gl_.glEnable(GL.GL_LIGHTING);
            logo.draw( gl_ );
            gl_.glPopMatrix();
          }

        if ( view_from[Y] < 0.0 ) {
            draw_under_table();
          }
      }

    /** Perform the OpenGL initialization */
    public void initialize( GLDrawable component ) {

        initLights();
        initMaterials();

        buildTable();

        view_from_spline = calc_spline_params( view_from_ctl, TIME );
        view_to_spline = calc_spline_params( view_to_ctl, TIME );
        light_pos_spline = calc_spline_params( light_pos_ctl, TIME );
        logo_pos_spline = calc_spline_params( logo_pos_ctl, TIME );
        logo_rot_spline = calc_spline_params( logo_rot_ctl, TIME );
      }

    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 45.0, 5.0 / 4.0, 0.5, 20.0 ); 
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Creates the table the logo is pasted onto */
    private void buildTable() { 

        float i, j;

        for ( j = 0.0f ; j <= TABLERES * 1.0f ; j+=1.0f ) {
            for ( i=0.0f ; i <= TABLERES * 1.0 ; i+=1.0f ) {
                table_points[(int)j][(int)i][Z] = 
                    (float)( (i-TABLERES*1.0/2.0)/2.0 );
                table_points[(int)j][(int)i][X] = 
                    (float)( (j-TABLERES*1.0/2.0)/2.0 );
                table_points[(int)j][(int)i][Y] = 0.0f;
	          }
          }
      }

    /** Renders the table */
    private void draw_table() {

        float c;
        int i2, j;
        int k, l;
        float[] ov = new float[3], 
                lv = new float[3];

        gl_.glDisable(GL.GL_DEPTH_TEST);
        gl_.glDisable(GL.GL_LIGHTING);

        ov[X] = light_pos[X]-logo_pos[X];
        ov[Y] = light_pos[Y]-logo_pos[Y];
        ov[Z] = light_pos[Z]-logo_pos[Z];

        normalize(ov);

        for (j=0; j<=TABLERES; j++) {
            for (i2 = 0 ; i2 <= TABLERES ; i2++ ) {
	            lv[X] = light_pos[X] - table_points[j][i2][X];
	            lv[Y] = light_pos[Y] - table_points[j][i2][Y];
	            lv[Z] = light_pos[Z] - table_points[j][i2][Z];
	            normalize(lv);
	            if ((c = dot(lv, ov))<0.0) {
	                c = 0.0f;
	              }
	            c = (float)( c * c * c * lv[Y] * 255.0 );
	            /* fade */
	            if ((current_time>TIME-5.0) && (current_time<TIME-3.0)) {
	                c *= (float)( 1.0 - (current_time-(TIME-5.0)) * 0.5 );
                  }
	
	            tablecolors[j][i2] = (int)c;
              }
          }
    
        for (l=0; l<TABLERES; l++) {
            gl_.glBegin(GL.GL_TRIANGLE_STRIP);
            for (k=0; k<=TABLERES; k++) {
                gl_.glColor3ub( (byte)tablecolors[l][k],
                	           (byte)tablecolors[l][k],
                	           (byte)tablecolors[l][k]);
	            gl_.glVertex3fv(table_points[l][k]);

	            gl_.glColor3ub( (byte)tablecolors[l+1][k],
	            	           (byte)tablecolors[l+1][k], 
	            	           (byte)tablecolors[l+1][k]);
	            gl_.glVertex3fv(table_points[l+1][k]);
              }
	        gl_.glEnd();
          }

        if (logo_pos[Y]>-0.33 && logo_pos[Y]<0.33) {
	        gl_.glEnable(GL.GL_DEPTH_TEST);
          }

        pca = 0.0f;
        gl_.glBegin(GL.GL_POLYGON);
        for (i2=0; i2<4; i2++) {
            lv[X] = light_pos[X] - paper_points[i2][X];
            lv[Y] = light_pos[Y] - paper_points[i2][Y];
            lv[Z] = light_pos[Z] - paper_points[i2][Z];
            normalize(lv);
            if ((c = dot(lv, ov))<0.0) {
                c = 0.0f;
              }
            c = (float)( c * c * c * lv[Y] );
            /* fade */
            if ((current_time>TIME-5.0) && (current_time<TIME-3.0)) {
                c *= (float)( 1.0 - (current_time-(TIME-5.0)) * 0.5 );
              } 

            pcr = c * 255; pcg = c * 255; pcb = c * 200;
            pca += c;
            gl_.glColor3ub( (byte)( (int)pcr ),
                           (byte)( (int)pcg ),
                           (byte)( (int)pcb ) );
            gl_.glVertex3fv(paper_points[i2]);
          }
        gl_.glEnd();

        gl_.glPushMatrix();
        gl_.glRotatef( (float)( 0.1 * (-184) ), 0.0f, 1.0f, 0.0f );
        gl_.glTranslatef( -0.3f, 0.0f, -0.8f );
        gl_.glRotatef( (float)( 0.1 * (-900) ), 1.0f, 0.0f, 0.0f );
        gl_.glScalef( 0.015f, 0.015f, 0.015f );


        if (current_time>TIME*1.0-5.0) {
        	c = (float)( (current_time-(TIME*1.0-5.0))/2.0 );
        	gl_.glColor3ub( (byte)( (int)(c*255.0) ),
        	               (byte)( (int)(c*255.0) ),
        	               (byte)( (int)(c*255.0)) );
          } else {
            gl_.glColor3ub( (byte)0,  (byte)0,  (byte)0 );
          }

        gl_.glDisable(GL.GL_DEPTH_TEST);

        i.draw( gl_ );
        gl_.glTranslatef( 3.0f,  0.0f,  0.0f );

        d.draw( gl_ );
        gl_.glTranslatef( 6.0f,  0.0f,  0.0f );

        e.draw( gl_ );
        gl_.glTranslatef( 5.0f,  0.0f,  0.0f );

        a.draw( gl_ );
        gl_.glTranslatef( 6.0f,  0.0f,  0.0f );

        s.draw( gl_ );
        gl_.glTranslatef( 10.0f,  0.0f,  0.0f );

        i.draw( gl_ );
        gl_.glTranslatef( 3.0f,  0.0f,  0.0f );

        n.draw( gl_ );
        gl_.glTranslatef( -31.0f,  -13.0f,  0.0f );

        m.draw( gl_ );
        gl_.glTranslatef( 10.0f,  0.0f,  0.0f );

        o.draw( gl_ );
        gl_.glTranslatef( 5.0f,  0.0f,  0.0f );

        t.draw( gl_ );
        gl_.glTranslatef( 4.0f,  0.0f,  0.0f );

        i.draw( gl_ );
        gl_.glTranslatef( 3.5f,  0.0f,  0.0f );

        o.draw( gl_ );
        gl_.glTranslatef( 5.0f,  0.0f,  0.0f );

        n.draw( gl_ );
    
        gl_.glPopMatrix();
      }

    private void draw_under_table() {

        int k, l;

        gl_.glDisable(GL.GL_DEPTH_TEST);

        gl_.glColor3ub( (byte)0,  (byte)0,  (byte)0 );

        for (l=0; l<TABLERES; l++) {
        	gl_.glBegin(GL.GL_TRIANGLE_STRIP);
        	for (k=0; k<=TABLERES; k++) {
        	    gl_.glVertex3fv(table_points[l][k]);
        	    gl_.glVertex3fv(table_points[l+1][k]);
          	  }
        	gl_.glEnd();
          }
        gl_.glEnable(GL.GL_DEPTH_TEST); 
      }

    /** Normalizes a vector */
    private void normalize( float[] v ) {
        float r;

        r = (float)( Math.sqrt( v[X] * v[X] + v[Y] * v[Y] + v[Z] * v[Z] ) );

        v[X] /= r;
        v[Y] /= r;
        v[Z] /= r;
      }

    /** Calculates the dot product of a vector */
    private float dot( float[] v1, float[] v2 ) {
        return (float)( v1[X] * v2[X] + v1[Y] * v2[Y] + v1[Z] * v2[Z] );
      }

    private float[] calc_spline( float[][][] params, float current_time ) {

        float t;
        int i;
        float[] tmpArray = new float[4];

        t = current_time - (float)((int)current_time);

        if ( current_time >= params.length ) {
            return null;
          }

        for (i=0; i<3; i++) {
	        tmpArray[i] = params[(int)current_time][3][i] +
	                      params[(int)current_time][2][i] * t +
	                      params[(int)current_time][1][i] * t * t +
	                      params[(int)current_time][0][i] * t * t * t;
          }
        return tmpArray;
      }

    private float[][][] calc_spline_params( float[][] ctl_pts, long n ) {

        int i, j;
        float[][][] params = new float[(int)(n-3)][4][3];

        if ( n < 4 ) {
            System.err.println( "calc_spline_params: not enough control points" );
            return null;
          }

        for ( i = 0 ; i < n - 3 ; i++ ) {
            for ( j = 0 ; j < 3 ; j++ ) {
                params[i][3][j] = ctl_pts[i+1][j];
                params[i][2][j] = ctl_pts[i+2][j] - ctl_pts[i][j];
                params[i][1][j] = 2.0f * ctl_pts[i][j] +
                                  -2.0f * ctl_pts[i+1][j] +
                                  1.0f * ctl_pts[i+2][j] +
                                  -1.0f * ctl_pts[i+3][j];
                params[i][0][j] = -1.0f * ctl_pts[i][j] +
                                  1.0f * ctl_pts[i+1][j] +
                                  -1.0f * ctl_pts[i+2][j] +
                                  1.0f * ctl_pts[i+3][j];
              }
          }
        return params;
      }

    /** Returns a GL pipeline for this listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** Handles mouse events */
    /** Handles mouse clicked events */
    public void mouseClicked( MouseEvent evt ) {
        needsRestart = true;
      }

    /** Handles mouse entered events */
    public void mouseEntered( MouseEvent evt ) {
      }

    /** Handles mouse exited events */
    public void mouseExited( MouseEvent evt ) {
      }

    /** Handles mouse pressed events */
    public void mousePressed( MouseEvent evt ) {
      }

    /** Handles mouse released events */
    public void mouseReleased( MouseEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
