/**
 * $Id: lorenz.java.stub,v 1.14 1998/10/19 11:05:19 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Test program that simulates Lorenz attractors
 *
 * $Log: lorenz.java.stub,v $
 * Revision 1.14  1998/10/19 11:05:19  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.13  1998/10/02 12:32:41  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.12  1998/04/21 15:13:10  descarte
 * Added WindowListener
 * Added standalone capability
 * Added new demoTemplate methods
 * Fixed bug in which the modelview matrix stack was being corrupted if 'w'
 * was being hit during rendering. The addition of explicit context locks
 * around the flag resetting ensures that the rendering will finish before any
 * state changes are made.
 *
 * Revision 1.11  1998/03/30 11:38:04  descarte
 * Added applet resource handling.
 *
 * Revision 1.10  1998/03/28 16:56:05  descarte
 * Upgraded to Java 1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.9  1998/03/24 17:13:39  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.8  1998/03/19 22:29:58  descarte
 * Added getGL() method.
 *
 * Revision 1.7  1998/03/17 21:03:11  descarte
 * Converted demo from old beta 4 to new beta 4 syntax.
 *
 * Revision 1.6  1998/02/22 14:39:07  descarte
 * Updated demo to new GLComponent internal threading structure. This sees
 * to have ironed out the Win32 bug with this demo and bizarre viewpoints.
 *
 * Revision 1.5  1998/01/07 20:58:36  descarte
 * Added applet thread handling. Doesn't work very well.
 *
 * Revision 1.4  1998/01/04 17:50:08  descarte
 * Converted demo across to GLEventListener interface.
 * Corrected a couple of glitches.
 *
 * Revision 1.3  1997/11/30 15:25:00  descarte
 * Some tidy-ups.
 *
 * Revision 1.2  1997/11/30 15:22:38  descarte
 * Completed.
 *
 * Revision 1.1  1997/11/30 14:41:01  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.lorenz;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class lorenzapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    CoreGL gl_ = new CoreGL();
    CoreGLU glu_ = new CoreGLU();

    /** OpenGL-aware AWT Component */
    GLComponent glc = null;

    /** Dimensions of the OpenGL component */
    private static final int WIDTH = 300;
    private static final int HEIGHT = 300;

    /** Variable to track thread status in applets */
    private boolean appletThreadStopped = false;

    /** Some variables */
    private static final int POINTMASK = 511;
    private static final float G = 0.002f;
    private static final float LG = 0.3f;
    private static final int CUBESIDE = 120;
    private static final int CUBESCALE = 23;
    private static final int CUBEOFFX = -4;
    private static final int CUBEOFFY = 0;
    private static final int CUBEOFFZ = 57;
    private static final float sigma = 10f,
                               r = 28,
                               b = (float)( 8 / 3 ),
                               dt = 0.003f;
    private float[][] rv = new float[POINTMASK + 1][3];
    private float[][] gv = new float[POINTMASK + 1][3];
    private float[][] bv = new float[POINTMASK + 1][3];
    private float[][] yv = new float[POINTMASK + 1][3];
    private float[][] mv = new float[POINTMASK + 1][3];

    /** Eye position */
    private float[] eyex = new float[3];
    private float[] eyev = new float[3];
    private float[] eyel = new float[3];

    /** Field of view */
    private float fovy = 600;

    /** Tracking variables */
    private int rp = 0,
                bp = 0,
                gp = 0,
                yp = 0,
                mp = 0;

    /** Quadric object */
    private GLUQuadric quadObj = null;

    /** Display list ID for the sphere */
    private int asphere = -1;

    /** The gravitation pull */
    private float gravity = G;

    /** Speed of animation */
    private int speed = 3;

    /** Maximum x, y, z values */
    private int xmax = 0,
                ymax = 0,
                zmax = 0,
                zmin = 0;

    /** Deltas */
    private float dx = 0,
                  dy = 0,
                  dz = 0;

    /** The distance to the far clipping plane */
    private float farplane = 300.0f;

    /** Draw the hexcube? */
    private boolean hexflag = false;

    /** Rotate the hexcube? */
    private boolean wflag = false;

    /** Hexagon rotational velocity ( .1 degree / redraw ) */
    private float a = 0,
                  da = 1;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Lorenz Attractors" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: lorenz.java.stub,v 1.14 1998/10/19 11:05:19 descarte Exp descarte $" );
      }

    /** Returns a description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This program shows some particles stuck in a Lorenz attractor\n" );
        sb.append( "( the parameters used are r=28, b=8/3, sigma=10 ). The eye is\n" );
        sb.append( "attracted to the red particle, with a force directly proportional\n" );
        sb.append( "to distance.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    x - Toggles display of a hexagonal box\n" );
        sb.append( "    w - Toggles rotation of the hexagons ( very trippy )\n" );
        sb.append( "    ESC - Quit demo" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Aaron T. Ferruci <aaronf@cse.ucsc.edu>" );
      }

    public void startDemo() {

        /** Set the layout manager */
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        add( "Center", glc );
        /**#layout */

        /** Setup the capabilities of the OpenGL rendering context */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setColourBits( 24 );
        cap.setDepthBits( 12 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener with the GLComponent */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Start the component up in animation mode */
        glc.initialize();
        glc.start();
      }

    /** Initializes the application */
    public void initialize( GLDrawable component ) {
        /** Initialize global vectors */
        rv[0][0] = (float)( Math.random() * 10 );
        rv[0][1] = (float)( Math.random() * 10 );
        rv[0][2] = (float)( Math.random() * 10 ) - 10;

        bv[0][0] = rv[0][0] + (float)( Math.random()*5. );
        bv[0][1] = rv[0][1] + (float)( Math.random()*5. );
        bv[0][0] = rv[0][2] + (float)( Math.random()*5. );

        gv[0][0] = rv[0][0] + (float)( Math.random()*5. );
        gv[0][1] = rv[0][1] + (float)( Math.random()*5. );
        gv[0][0] = rv[0][2] + (float)( Math.random()*5. );

        yv[0][0] = rv[0][0] + (float)( Math.random()*5. );
        yv[0][1] = rv[0][1] + (float)( Math.random()*5. );
        yv[0][0] = rv[0][2] + (float)( Math.random()*5. );

        mv[0][0] = rv[0][0] + (float)( Math.random()*5. );
        mv[0][1] = rv[0][1] + (float)( Math.random()*5. );
        mv[0][0] = rv[0][2] + (float)( Math.random()*5. );

        /* initialize eye velocity */
        eyev[0] = eyev[1] = eyev[2] = 0;

        /** Initialize the graphics */
        xmax = WIDTH;
        ymax = HEIGHT;

        gl_.glDrawBuffer( GL.GL_BACK );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
        gl_.glClearDepth( 1.0f );

        gl_.glEnable( GL.GL_CULL_FACE );
        gl_.glCullFace( GL.GL_BACK );

        quadObj = glu_.gluNewQuadric();
        glu_.gluQuadricNormals( quadObj, GLU.GLU_NONE );
        asphere = gl_.glGenLists( 1 );
        gl_.glNewList( asphere, GL.GL_COMPILE );
            glu_.gluSphere( quadObj, 0.3f, 12, 8 );
        gl_.glEndList();

        /** Calculate the next points on the lines */
        while ( rp < POINTMASK ) {
            rv = nextLine( rv, rp );
            rp = ( rp + 1 ) & POINTMASK;
            bv = nextLine( bv, bp );
            bp = ( bp + 1 ) & POINTMASK;
            gv = nextLine( gv, gp );
            gp = ( gp + 1 ) & POINTMASK;
            yv = nextLine( yv, yp );
            yp = ( yp + 1 ) & POINTMASK;
            mv = nextLine( mv, mp );
            mp = ( mp + 1 ) & POINTMASK;
          }

        /** Move the eye */
        eyex[0] = eyex[1] = eyex[2] = 0;
        eyel[0] = rv[rp][0];
        eyel[1] = rv[rp][1];
        eyel[2] = rv[rp][2];

        gl_.glPushMatrix();
            moveEye();
        gl_.glPopMatrix();
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        xmax = width;
        ymax = height;
        setPerspective( fovy, (float)width/ (float)height, 0.01f, farplane );
      }

    /** Sets the perspective of the scene */
    private void setPerspective( float angle, float aspect, 
                                 float zNear, float zFar ) {
//        gl_.glPushAttrib( GL.GL_TRANSFORM_BIT );
            gl_.glMatrixMode( GL.GL_PROJECTION );
            gl_.glLoadIdentity();
            glu_.gluPerspective( angle * 0.1f, aspect, zNear, zFar );
          gl_.glMatrixMode( GL.GL_MODELVIEW );
          gl_.glLoadIdentity();
//        gl_.glPopAttrib();
      }

    /** Renders the scene */
    public void display( GLDrawable component ) {

        int i = speed;

        while ( i-- != 0 ) {
            rv = nextLine( rv, rp );
            rp = ( rp + 1 ) & POINTMASK;
            bv = nextLine( bv, bp );
            bp = ( bp + 1 ) & POINTMASK;
            gv = nextLine( gv, gp );
            gp = ( gp + 1 ) & POINTMASK;
            yv = nextLine( yv, yp );
            yp = ( yp + 1 ) & POINTMASK;
            mv = nextLine( mv, mp );
            mp = ( mp + 1 ) & POINTMASK;
          }
        gl_.glPushMatrix();
            moveEye();

            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
    
            if ( hexflag ) {
                drawHexcube();
              }
            gl_.glColor3f( 1.0f, 0.0f, 0.0f );
            drawLines( rp, rv );
            drawSphere( rv[rp] );
    
            gl_.glColor3f( 0.0f, 0.0f, 1.0f );
            drawLines( bp, bv );
            drawSphere( bv[bp] );

            gl_.glColor3f( 0.0f, 1.0f, 0.0f );
            drawLines( gp, gv );
            drawSphere( gv[gp] );

            gl_.glColor3f( 1.0f, 0.0f, 1.0f );
            drawLines( yp, yv );
            drawSphere( yv[yp] );

            gl_.glColor3f( 0.0f, 1.0f, 1.0f );
            drawLines( mp, mv );
            drawSphere( mv[mp] );
        gl_.glPopMatrix();
      }

    /** Returns a GL pipeline from this listener */
    public GL getGL() {
        return gl_;
      }

    /** Move the eye position */
    private void moveEye() {

        /* first move the eye */
        eyev[0] += gravity * ( rv[rp][0] - eyex[0] );
        eyev[1] += gravity * ( rv[rp][1] - eyex[1] );
        eyev[2] += gravity * ( rv[rp][2] - eyex[2] );

        /* adjust position using new velocity */
        eyex[0] += eyev[0] * dt;
        eyex[1] += eyev[1] * dt;
        eyex[2] += eyev[2] * dt;

        /* move the lookat point */
        /* it catches up to the red point if it's moving slowly enough */
        eyel[0] += LG * ( rv[rp][0] - eyel[0] );
        eyel[1] += LG * ( rv[rp][1] - eyel[1] );
        eyel[2] += LG * ( rv[rp][2] - eyel[2] );

        /* change view */
        glu_.gluLookAt( eyex[0], eyex[1], eyex[2], eyel[0], eyel[1], eyel[2],
                        0, 1, 0);
      }

    /** Computes the next point on the path */
    private float[][] nextLine( float[][] v, int p ) {

        dx = sigma * ( v[p][1] - v[p][0] ) * dt;
        dy = ( r * v[p][0] - v[p][1] + v[p][0] * v[p][2] ) * dt;
        dz = ( v[p][0] * v[p][1] + b * v[p][2] ) * dt;

        v[( p + 1 ) & POINTMASK][0] = v[p][0] + dx;
        v[( p + 1 ) & POINTMASK][1] = v[p][1] + dy;
        v[( p + 1 ) & POINTMASK][2] = v[p][2] - dz;

        return v;
      }

    /** Renders a sphere */
    private void drawSphere( float[] args ) {
        gl_.glPushMatrix();
            gl_.glTranslatef( args[0], args[1], args[2] );
            gl_.glCallList( asphere );
        gl_.glPopMatrix();
      }

    /** Draws the lines */
    private void drawLines( int index, float[][] array ) {
        int p;
        int i;
        int LINE_STEP = 4;

        p = ( index + 1 ) & POINTMASK;
        i = LINE_STEP - ( p % LINE_STEP );
        if ( i == LINE_STEP ) {
            i = 0;
          }

        gl_.glBegin( GL.GL_LINE_STRIP );
            while ( p != index ) {
                if ( i == 0 ) {
                    gl_.glVertex3fv( array[p] );
                    i = LINE_STEP;
                  }
                i--;
                p = ( p + 1 ) & POINTMASK;
              }
        gl_.glVertex3fv( array[index] );
        gl_.glEnd();
      }

    /** Draws the hexcube */
    private void drawHexcube() {
        a += da;
        if ( a >= 720 ) {
            a = 0;
          }

        /* draw hexplanes, without changing z-values */
        gl_.glDepthMask( (byte)GL.GL_FALSE );
        gl_.glDisable( GL.GL_DEPTH_TEST );
        gl_.glDisable( GL.GL_CULL_FACE );

        /* x-y plane */
        gl_.glColor3f( 0.2f, 0.2f, 0.6f );
        gl_.glPushMatrix();
            gl_.glTranslatef( CUBEOFFX, CUBEOFFY, CUBEOFFZ );
            gl_.glScalef( CUBESCALE, CUBESCALE, CUBESCALE );
            drawHexplane();
        gl_.glPopMatrix();

        /* x-y plane, translated */
        gl_.glPushMatrix();
            gl_.glTranslatef( CUBEOFFX, CUBEOFFY, CUBEOFFZ - 2*CUBESIDE );
            gl_.glScalef( CUBESCALE, CUBESCALE, CUBESCALE );
            drawHexplane();
            gl_.glPopMatrix();
        gl_.glColor3f( 0.6f, 0.2f, 0.2f );

        /* x-z plane, translate low */
        gl_.glPushMatrix();
            gl_.glRotatef( 90, 1.0f, 0.0f, 0.0f );
            gl_.glTranslatef( CUBEOFFX, CUBEOFFZ - CUBESIDE, -CUBEOFFY + CUBESIDE );
            gl_.glScalef( CUBESCALE, CUBESCALE, CUBESCALE );
            drawHexplane();
        gl_.glPopMatrix();

        /* x-z plane, translate high */
        gl_.glPushMatrix();
            gl_.glRotatef( 90, 1.0f, 0.0f, 0.0f );
            gl_.glTranslatef( CUBEOFFX, CUBEOFFZ - CUBESIDE, -CUBEOFFY - CUBESIDE );
            gl_.glScalef( CUBESCALE, CUBESCALE, CUBESCALE );
            drawHexplane();
        gl_.glPopMatrix();
        gl_.glColor3f( 0.2f, 0.6f, 0.2f );

        /* y-z plane, translate low */
        gl_.glPushMatrix();
            gl_.glRotatef( 90, 0.0f, 1.0f, 0.0f );
            gl_.glTranslatef( -CUBEOFFZ + CUBESIDE, CUBEOFFY, CUBEOFFX + CUBESIDE );
            gl_.glScalef( CUBESCALE, CUBESCALE, CUBESCALE );
            drawHexplane();
        gl_.glPopMatrix();

        /* y-z plane, translate high */
        gl_.glPushMatrix();
            gl_.glRotatef ( 90, 0.0f, 1.0f, 0.0f );
            gl_.glTranslatef( -CUBEOFFZ + CUBESIDE, CUBEOFFY, CUBEOFFX - CUBESIDE );
            gl_.glScalef( CUBESCALE, CUBESCALE, CUBESCALE );
            drawHexplane();
        gl_.glPopMatrix();

        gl_.glDepthMask( (byte)GL.GL_TRUE );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glEnable( GL.GL_CULL_FACE );
      }

    /** Draws a hexplane, ie, one side of the hexagonal box */
    private void drawHexplane() {

        if ( wflag ) {
            gl_.glPushMatrix();
            gl_.glRotatef( -0.5f * a, 0.0f, 0.0f, 1.0f );
          }

       /* center , <0, 0, 0> */
       drawHexagon();

       /* 12 o'clock, <0, 4, 0> */
       gl_.glTranslatef( 0f, 4f, 0f );
       drawHexagon();

       /* 10 o'clock, <-3.464, 2, 0> */
       gl_.glTranslatef( -3.464f, -2f, 0f );
       drawHexagon();

       /* 8 o'clock, <-3.464, -2, 0> */
       gl_.glTranslatef( 0f, -4f, 0f );
       drawHexagon();

       /* 6 o'clock, <0, -4, 0> */
       gl_.glTranslatef( 3.464f, -2f, 0.f );
       drawHexagon();

       /* 4 o'clock, <3.464, -2, 0> */
       gl_.glTranslatef( 3.464f, 2f, 0f );
       drawHexagon();

       /* 2 o'clock, <3.464, 2, 0> */
       gl_.glTranslatef( 0f, 4f, 0f );
       drawHexagon();

       if( wflag ) {
           gl_.glPopMatrix();
         }
      }

    private float[][] hex_data =  {
        { 0.f, 0.f, 0.f },
        { 1.155f, 0.f, 0.f },
        { 0.577f, 1.f, 0.f },
        { -0.577f, 1.f, 0.f },
        { -1.155f, 0.f, 0.f },
        { -0.577f, -1.f, 0.f },
        { 0.577f, -1.f, 0.f },
        { 1.155f, 0.f, 0.f },
      };


    /** Draws a single hexagon */
    private void drawHexagon() {

        if ( wflag ) {
            gl_.glPushMatrix();
            gl_.glRotatef( a, 0.0f, 0.0f, 1.0f );
          }

        gl_.glBegin( GL.GL_TRIANGLE_FAN );
            gl_.glVertex3fv( hex_data[0] );
            gl_.glVertex3fv( hex_data[1] );
            gl_.glVertex3fv( hex_data[2] );
            gl_.glVertex3fv( hex_data[3] );
            gl_.glVertex3fv( hex_data[4] );
            gl_.glVertex3fv( hex_data[5] );
            gl_.glVertex3fv( hex_data[6] );
            gl_.glVertex3fv( hex_data[7] );
        gl_.glEnd();

        if ( wflag ) {
            gl_.glPopMatrix();
          }
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
            return;
          }
        if ( evt.getKeyChar() == 'x' ) {
            /**
             * Lock the context to ensure that the resetting of this flag
             * doesn't interfere with rendering
             */
            glc.getContext().lock();
            if ( hexflag ) {
                hexflag = false;
                farplane = 80;
              } else {
                farplane = 300;
                hexflag = true;
              }
            /*setPerspective( fovy, (float)( xmax / ymax ),
                            0.01f, farplane );*/
            glc.getContext().unlock();
            return;
          }
        if ( evt.getKeyChar() == 'w' ) {
            /** 
             * Lock the context to ensure the resetting of this flag doesn't
             * interfere with rendering
             */
            glc.getContext().lock();
            if ( wflag ) {
                wflag = false;
              } else {
                wflag = true;
              }
            glc.getContext().unlock();
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
