/**
 * $Id: gears.java.stub,v 1.13 1998/10/19 11:09:46 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Test program that draws some gears
 *
 * $Log: gears.java.stub,v $
 * Revision 1.13  1998/10/19 11:09:46  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.12  1998/10/02 12:37:11  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/04/21 15:29:57  descarte
 * Added new demoTemplate methods. Added standalone code. Added WindowListener.
 *
 * Revision 1.10  1998/03/30 11:42:46  descarte
 * Added applet resource handling.
 *
 * Revision 1.9  1998/03/28 17:35:37  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.8  1998/03/24 17:25:24  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.7  1998/03/19 22:36:28  descarte
 * Added getGL() method.
 *
 * Revision 1.6  1998/02/23 21:59:51  descarte
 * Converted demo to beta 4 GLComponent structure.
 *
 * Revision 1.5  1998/01/04 19:58:01  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.4  1998/01/03 15:43:10  descarte
 * Fixed add() method for Microsoft VM. The "Center" argument was missing.
 *
 * Revision 1.3  1998/01/02 17:23:50  descarte
 * Converted demo across to GLEventListener interface.
 *
 * Revision 1.2  1997/12/23 17:22:44  descarte
 * Added layout manager to make window resize correctly under MSVM.
 *
 * Revision 1.1  1997/11/29 17:13:20  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.mesa;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class gearsapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    CoreGL gl_ = new CoreGL();
    CoreGLU glu_ = new CoreGLU();

    /** OpenGL-aware AWT Component */
    GLComponent glc = null;

    /** Dimensions of the OpenGL component */
    private static final int WIDTH = 300;
    private static final int HEIGHT = 300;

    /** Display list ids for the three gears */
    int[] gear = new int[3];

    /** The angle of rotation of the gears */
    float angle = 0.0f;

    /** The viewpoint rotation */
    private float view_rotx = 20.0f,
                  view_roty = 30.0f,
                  view_rotz = 0.0f;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Gears" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: gears.java.stub,v 1.13 1998/10/19 11:09:46 descarte Exp descarte $" );
      }

    /** Returns a description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration generates 3 gears or cogs and rotates\n" );
        sb.append( "in a way to simulate a basic gearing mechanism.\n\n" );
        sb.append( "    o,O - Rotate positively around the x-axis\n" );
        sb.append( "    p,P - Rotate negatively around the x-axis\n" );
        sb.append( "    q,Q - Rotate positively around the y-axis\n" );
        sb.append( "    a,A - Rotate negatively around the y-axis\n" );
        sb.append( "    z - Rotate positively around the z-axis\n" );
        sb.append( "    Z - Rotate negatively around the z-axis\n" );
        sb.append( "    ESC - Quit demo" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Brian Paul <brianp@ra.avid.com>" );
      }

    public void startDemo() {

        /** Set the layout manager */
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        add( "Center", glc );
        /**#layout */

        /** Setup the capabilities of the OpenGL rendering context */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setColourBits( 24 );
        cap.setDepthBits( 12 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Tell the component to process and generate events */
        glc.initialize();
        glc.start();
      }

    /** Initializes the application */
    public void initialize( GLDrawable component ) {
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        float[] pos = { 5.0f, 5.0f, 10.0f, 0.0f };
        float[] red = { 0.8f, 0.1f, 0.0f, 1.0f };
        float[] green = { 0.0f, 0.8f, 0.2f, 1.0f };
        float[] blue = { 0.2f, 0.2f, 1.0f, 1.0f };

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, pos );
        gl_.glEnable( GL.GL_CULL_FACE );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_DEPTH_TEST );

        /* make the gears */
        gear[0] = gl_.glGenLists( 1 );
        gl_.glNewList( gear[0], GL.GL_COMPILE );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, red );
            gear( 1.0f, 4.0f, 1.0f, 20, 0.7f );
        gl_.glEndList();

        gear[1] = gl_.glGenLists( 1 );
        gl_.glNewList( gear[1], GL.GL_COMPILE );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, green );
            gear( 0.5f, 2.0f, 2.0f, 10, 0.7f );
        gl_.glEndList();

        gear[2] = gl_.glGenLists( 1 );
        gl_.glNewList( gear[2], GL.GL_COMPILE );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, blue );
            gear( 1.3f, 2.0f, 0.5f, 10, 0.7f );
        gl_.glEndList();

        gl_.glEnable( GL.GL_NORMALIZE );
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the frustum */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0f, 1.0f, -( (float)height / (float)width ),
                       ( (float)height / (float)width ), 5.0f, 60.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glTranslatef( 0.0f, 0.0f, -40.0f );
      }

    /** Renders the scene */
    public void display( GLDrawable component ) {
        /** Rotate the gears */
        angle += 2.0f;

        /** Clear the framebuffer */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        gl_.glPushMatrix();
        gl_.glRotatef( view_rotx, 1.0f, 0.0f, 0.0f );
        gl_.glRotatef( view_roty, 0.0f, 1.0f, 0.0f );
        gl_.glRotatef( view_rotz, 0.0f, 0.0f, 1.0f );

        gl_.glPushMatrix();
            gl_.glTranslatef( -3.0f, -2.0f, 0.0f );
            gl_.glRotatef( angle, 0.0f, 0.0f, 1.0f );
            gl_.glCallList( gear[0] );
        gl_.glPopMatrix();

        gl_.glPushMatrix();
            gl_.glTranslatef( 3.1f, -2.0f, 0.0f );
            gl_.glRotatef( (float)( -2.0*angle-9.0 ), 0.0f, 0.0f, 1.0f );
            gl_.glCallList( gear[1] );
        gl_.glPopMatrix();

        gl_.glPushMatrix();
            gl_.glTranslatef( -3.1f, 4.2f, 0.0f );
            gl_.glRotatef( (float)( -2.0*angle-25.0 ), 0.0f, 0.0f, 1.0f );
            gl_.glCallList( gear[2] );
        gl_.glPopMatrix();

        gl_.glPopMatrix();
      }

    /** Returns the current GL pipeline for this listener */
    public GL getGL() {
        return gl_;
      }

    /** Creates the geometry for a gear */
    private void gear( float innerRadius, float outerRadius,
                       float width, int teeth, float toothDepth ) {

        float r0, r1, r2;
        float angle, da;
        float u, v, len;

        r0 = innerRadius;
        r1 = outerRadius - toothDepth / 2.0f;
        r2 = outerRadius + toothDepth / 2.0f;

        da = (float)( 2.0 * Math.PI / teeth / 4.0 );

        gl_.glShadeModel( GL.GL_FLAT );

        gl_.glNormal3f( 0.0f, 0.0f, 1.0f );

        /* draw front face */
        gl_.glBegin( GL.GL_QUAD_STRIP );
        for ( int i = 0 ; i <= teeth ;i++ ) {
            angle = (float)( i * 2.0*Math.PI / teeth );
            gl_.glVertex3f( r0*(float)Math.cos(angle), r0*(float)Math.sin(angle), width*0.5f );
            gl_.glVertex3f( r1*(float)Math.cos(angle), r1*(float)Math.sin(angle), width*0.5f );
            gl_.glVertex3f( r0*(float)Math.cos(angle), r0*(float)Math.sin(angle), width*0.5f );
            gl_.glVertex3f( r1*(float)Math.cos(angle+3*da), r1*(float)Math.sin(angle+3*da), width*0.5f );
          }
        gl_.glEnd();
     
        /* draw front sides of teeth */
        gl_.glBegin( GL.GL_QUADS );
        da = (float)( 2.0*Math.PI / teeth / 4.0 );
        for ( int i = 0 ; i < teeth ; i++ ) {
            angle = (float)( i * 2.0*Math.PI / teeth );
     
            gl_.glVertex3f( r1*(float)Math.cos(angle),      r1*(float)Math.sin(angle),      width*0.5f );
            gl_.glVertex3f( r2*(float)Math.cos(angle+da),   r2*(float)Math.sin(angle+da),   width*0.5f );
            gl_.glVertex3f( r2*(float)Math.cos(angle+2*da), r2*(float)Math.sin(angle+2*da), width*0.5f );
            gl_.glVertex3f( r1*(float)Math.cos(angle+3*da), r1*(float)Math.sin(angle+3*da), width*0.5f );
          }
        gl_.glEnd();
     
        gl_.glNormal3f( 0.0f, 0.0f, -1.0f );
     
        /* draw back face */
        gl_.glBegin( GL.GL_QUAD_STRIP );
        for ( int i = 0 ; i <= teeth ; i++ ) {
            angle = (float)( i * 2.0*Math.PI / teeth );
            gl_.glVertex3f( r1*(float)Math.cos(angle), r1*(float)Math.sin(angle), -width*0.5f );
            gl_.glVertex3f( r0*(float)Math.cos(angle), r0*(float)Math.sin(angle), -width*0.5f );
            gl_.glVertex3f( r1*(float)Math.cos(angle+3*da), r1*(float)Math.sin(angle+3*da), -width*0.5f );
            gl_.glVertex3f( r0*(float)Math.cos(angle), r0*(float)Math.sin(angle), -width*0.5f );
          }
        gl_.glEnd();
     
        /* draw back sides of teeth */
        gl_.glBegin( GL.GL_QUADS );
        da = (float)( 2.0*Math.PI / teeth / 4.0 );
        for ( int i = 0 ; i < teeth ; i++ ) {
            angle = (float)( i * 2.0*Math.PI / teeth );
     
            gl_.glVertex3f( r1*(float)Math.cos(angle+3*da), r1*(float)Math.sin(angle+3*da), -width*0.5f );
            gl_.glVertex3f( r2*(float)Math.cos(angle+2*da), r2*(float)Math.sin(angle+2*da), -width*0.5f );
            gl_.glVertex3f( r2*(float)Math.cos(angle+da),   r2*(float)Math.sin(angle+da),   -width*0.5f );
            gl_.glVertex3f( r1*(float)Math.cos(angle),      r1*(float)Math.sin(angle),      -width*0.5f );
          }
        gl_.glEnd();
     
     
        /* draw outward faces of teeth */
        gl_.glBegin( GL.GL_QUAD_STRIP );
        for ( int i = 0 ; i < teeth ; i++ ) {
            angle = (float)( i * 2.0*Math.PI / teeth );
     
            gl_.glVertex3f( r1*(float)Math.cos(angle),      r1*(float)Math.sin(angle),       width*0.5f );
            gl_.glVertex3f( r1*(float)Math.cos(angle),      r1*(float)Math.sin(angle),      -width*0.5f );
            u = r2*(float)Math.cos(angle+da) - r1*(float)Math.cos(angle);
            v = r2*(float)Math.sin(angle+da) - r1*(float)Math.sin(angle);
            len = (float)Math.sqrt( u*u + v*v );
            u /= len;
            v /= len;
            gl_.glNormal3f( v, -u, 0.0f );
            gl_.glVertex3f( r2*(float)Math.cos(angle+da),   r2*(float)Math.sin(angle+da),    width*0.5f );
            gl_.glVertex3f( r2*(float)Math.cos(angle+da),   r2*(float)Math.sin(angle+da),   -width*0.5f );
            gl_.glNormal3f( (float)Math.cos(angle), (float)Math.sin(angle), 0.0f );
            gl_.glVertex3f( r2*(float)Math.cos(angle+2*da), r2*(float)Math.sin(angle+2*da),  width*0.5f );
            gl_.glVertex3f( r2*(float)Math.cos(angle+2*da), r2*(float)Math.sin(angle+2*da), -width*0.5f );
            u = r1*(float)Math.cos(angle+3*da) - r2*(float)Math.cos(angle+2*da);
            v = r1*(float)Math.sin(angle+3*da) - r2*(float)Math.sin(angle+2*da);
            gl_.glNormal3f( v, -u, 0.0f );
            gl_.glVertex3f( r1*(float)Math.cos(angle+3*da), r1*(float)Math.sin(angle+3*da),  width*0.5f );
            gl_.glVertex3f( r1*(float)Math.cos(angle+3*da), r1*(float)Math.sin(angle+3*da), -width*0.5f );
            gl_.glNormal3f( (float)Math.cos(angle), (float)Math.sin(angle), 0.0f );
          }
     
        gl_.glVertex3f( r1*(float)Math.cos(0), r1*(float)Math.sin(0), width*0.5f );
        gl_.glVertex3f( r1*(float)Math.cos(0), r1*(float)Math.sin(0), -width*0.5f );

        gl_.glEnd();

     
        gl_.glShadeModel( GL.GL_SMOOTH );


        /* draw inside radius cylinder */
        gl_.glBegin( GL.GL_QUAD_STRIP );
        for ( int i = 0 ; i <= teeth ; i++ ) {
            angle = (float)( i * 2.0 * Math.PI / teeth );
            gl_.glNormal3f( (float)-Math.cos(angle), -(float)Math.sin(angle), 0.0f );
            gl_.glVertex3f( r0*(float)Math.cos(angle), r0*(float)Math.sin(angle), -width*0.5f );
            gl_.glVertex3f( r0*(float)Math.cos(angle), r0*(float)Math.sin(angle), width*0.5f );
          }
        gl_.glEnd();
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
        if ( evt.getKeyChar() == 'o' ||
             evt.getKeyChar() == 'O' ) {
            view_roty += 5.0f;
            return;
          }
        if ( evt.getKeyChar() == 'p' ||
             evt.getKeyChar() == 'P' ) {
            view_roty -= 5.0f;
            return;
          }
        if ( evt.getKeyChar() == 'q' ||
             evt.getKeyChar() == 'Q' ) {
            view_rotx += 5.0f;
            return;
          }
        if ( evt.getKeyChar() == 'a' ||
             evt.getKeyChar() == 'A' ) {
            view_rotx -= 5.0f;
            return;
          }
        if ( evt.getKeyChar() == 'z' ) {
            view_rotz += 5.0f;
            return;
          }
        if ( evt.getKeyChar() == 'Z' ) {
            view_rotz -= 5.0f;
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
