/**
 * $Id: isosurf.java,v 1.15 1998/10/19 11:07:54 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Test program corresponding to 'isosurf' bundled with Mesa
 *
 * $Log: isosurf.java,v $
 * Revision 1.15  1998/10/19 11:07:54  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.14  1998/10/02 12:37:27  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.13  1998/04/21 15:30:15  descarte
 * Added new demoTemplate methods. Added standalone code. Added WindowListener.
 *
 * Revision 1.12  1998/03/28 17:05:39  descarte
 * Upgraded to Java 1.1 listeners.
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.11  1998/03/24 17:16:04  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.10  1998/03/19 22:36:38  descarte
 * Added getGL() method.
 *
 * Revision 1.9  1998/02/23 22:03:14  descarte
 * Converted demo to new beta 4 GLComponent structure.
 *
 * Revision 1.8  1998/01/04 20:01:00  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.7  1998/01/03 15:44:12  descarte
 * Fixed add() method for Microsoft VM. The "Center" argument was missing.
 *
 * Revision 1.6  1998/01/02 17:54:35  descarte
 * Converted demo across to GLEventListener interface.
 *
 * Revision 1.5  1997/11/13 13:05:52  descarte
 * Fixed some spelling mistakes in the descriptions.
 *
 * Revision 1.4  1997/11/13 13:03:06  descarte
 * Converted old demo to Magician beta 2 syntax.
 * Implemented demoTemplate
 * Added filename dialog box if the datafile cannot be located in the current
 * working directory
 *
 * Revision 1.3  1997/08/04 23:19:19  descarte
 * Fixed this up for Win32. Yet another bizarre glc.getContext().problem.
 *
 * Revision 1.2  1997/06/07 06:10:52  descarte
 * Put class in correct package.
 *
 * Revision 1.1  1997/06/05 20:37:18  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.mesa;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class isosurf extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    private static final String VERSION = new String( "$Revision: 1.15 $" );

    /** Angles of rotation of the surfaces being modelled */
    float xrot,
          yrot;

    /** The Magician OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The Magician Java components for OpenGL rendering */
    GLComponent glc = null;

    /** Dimensions of the viewport we wish to create */
    private static final int WIDTH = 300;
    private static final int HEIGHT = 300;

    /** Indicates whether smooth shading is in use */
    int smooth = GL.GL_TRUE;

    /** Indicates whether lighting is in use */
    int lighting = GL.GL_TRUE;

    /** Structures to hold surface information */
    private static final int MAXVERTS = 10000;

    float[][] verts = new float[MAXVERTS][3];
    float[][] norms = new float[MAXVERTS][3];
    int numverts;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        isosurf d = new isosurf();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public isosurf() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Isosurf" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: isosurf.java,v 1.15 1998/10/19 11:07:54 descarte Exp descarte $" );
      }

    /** Returns a description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration loads a datafile containing triangle\n" );
        sb.append( "data representing wind in a wind tunnel. The model may\n" );
        sb.append( "be rotated and the shading mode altered. If the datafile\n" );
        sb.append( "cannot be located firstly in the current directory, a\n" );
        sb.append( "file dialog box will be displayed for you to locate\n" );
        sb.append( "the file. It will be present in the ``mesa'' subdirectory\n" );
        sb.append( "of the demo classes called 'isosurf.dat'. Finally, once you\n" );
        sb.append( "have begun loading the datafile, it may take a minute to fully\n" );
        sb.append( "load.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    h - Rotate left\n" );
        sb.append( "    l - Rotate right\n" );
        sb.append( "    j - Rotate away from\n" );
        sb.append( "    k - Rotate towards\n" );
        sb.append( "    s - Toggle shade mode\n" );
        sb.append( "    i - Toggle lighting\n" );
        sb.append( "    ESC - Quits the demo" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Brian Paul <brianp@ra.avid.com>" );
      }

    public void startDemo() {
        /** Create a new frame */
        setLayout( new BorderLayout() );

        setSize( WIDTH, HEIGHT );
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        add( "Center", glc );
        pack(); show();

        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Read the datafile in */
        try {
            readDatafile( "isosurf.dat" );
          } catch ( FileNotFoundException e ) {
            try {
                FileDialog fd = new FileDialog( this, "isosurf datafile", FileDialog.LOAD );
                fd.show();

                /** Should have the filename here */
                readDatafile( fd.getDirectory() + File.separator + fd.getFile() );
              } catch ( IOException e3 ) {
                e3.printStackTrace();
              }
          } catch ( IOException e2 ) {
            System.err.println( "Exception caught in readDatafile: " +
                                e2.toString() );
          }

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /** Register a window listener */
        this.addWindowListener( this );

        /** Tell the component to start processing events */
        glc.initialize();
      }

    /** Does some basic initialization */
    public void initialize( GLDrawable component ) {

        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );

        gl_.glShadeModel( GL.GL_SMOOTH );
        gl_.glEnable( GL.GL_DEPTH_TEST );

        initializeMaterials();
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0f, 1.0f, -1.0f, 1.0f, 5f, 25f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glTranslatef( 0.0f, 0.0f, -6.0f );
      }

    /** Initializes the materials in the scene */
    private void initializeMaterials() throws OpenGLException {

        float[] ambient = { 0.1f, 0.1f, 0.1f, 1.0f };
        float[] diffuse = { 0.5f, 1.0f, 1.0f, 1.0f };
        float[] position0 = { 0.0f, 0.0f, 20.0f, 0.0f };
        float[] position1 = { 0.0f, 0.0f, -20.0f, 0.0f };
        float[] front_mat_shininess = { 60.0f };
        float[] front_mat_specular = { 0.2f, 0.2f, 0.2f, 1.0f };
        float[] front_mat_diffuse = { 0.5f, 0.28f, 0.38f, 1.0f };
        float[] lmodel_ambient = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] lmodel_twoside = { GL.GL_FALSE };

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, ambient );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position0 );
        gl_.glEnable( GL.GL_LIGHT0 );

        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_AMBIENT, ambient );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, diffuse );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, position1 );
        gl_.glEnable( GL.GL_LIGHT1 );
    
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside );
        gl_.glEnable( GL.GL_LIGHTING );

        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_SHININESS, front_mat_shininess );
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_SPECULAR, front_mat_specular );
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE, front_mat_diffuse );
      }

    /** Renders the scene */
    public void display( GLDrawable component ) {

        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
        gl_.glPushMatrix();
        gl_.glRotatef( yrot, 0, 1, 0 );
        gl_.glRotatef( xrot, 1, 0, 0 );

        gl_.glBegin( GL.GL_TRIANGLE_STRIP );
            for ( int i = 0 ; i < numverts ; i++ ) {
                gl_.glNormal3f( norms[i][0], norms[i][1], norms[i][2] );
                gl_.glVertex3f( verts[i][0], verts[i][1], verts[i][2] );
              }
        gl_.glEnd();

        gl_.glPopMatrix();
      }

    /** Returns a GL pipeline for this listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
          }

        if ( evt.getKeyChar() == 'h' ) {
            yrot -= 15.0f;
            glc.repaint();
            return;
          }

        if ( evt.getKeyChar() == 'l' ) {
            yrot += 15.0;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'j' ) {
            xrot += 15.0;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'k' ) {
            xrot -= 15.0;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 's' ) {
            /** Make the glc.getContext().current */
            glc.getContext().makeCurrent();

            if ( smooth == GL.GL_FALSE ) {
                smooth = GL.GL_TRUE;
                gl_.glShadeModel( GL.GL_SMOOTH );
              } else {
                smooth = GL.GL_FALSE;
                gl_.glShadeModel( GL.GL_FLAT );
              }
            /** Release the glc.getContext().*/
            glc.getContext().unlock();

            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'i' ) {
            /** Make the glc.getContext().current */
            glc.getContext().makeCurrent();

            if ( lighting == GL.GL_FALSE ) {
                lighting = GL.GL_TRUE;
                gl_.glEnable( GL.GL_LIGHTING );
              } else {
                lighting = GL.GL_FALSE;
                gl_.glDisable( GL.GL_LIGHTING );
              }
            /** Release the glc.getContext().*/
            glc.getContext().unlock();
            
            glc.repaint();
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Read in a datafile containing the surface information */
    private boolean readDatafile( String filename ) throws IOException {

        File inFile = new File( filename );
        DataInputStream inStream = 
            new DataInputStream( new FileInputStream( inFile ) );

        numverts = 0;
        while ( numverts < MAXVERTS && inStream.available() > 0 ) {
            verts[numverts][0] = parseFloat( inStream );
            verts[numverts][1] = parseFloat( inStream );
            verts[numverts][2] = parseFloat( inStream );

            norms[numverts][0] = parseFloat( inStream );
            norms[numverts][1] = parseFloat( inStream );
            norms[numverts][2] = parseFloat( inStream );

            numverts++;
          }
        numverts--;

        System.err.println( numverts + " vertices, " + 
                            ( numverts - 2 ) + " triangles" );
        return true;
      }

    /** Parses a float from a stream */
    private float parseFloat( DataInputStream inStream ) throws IOException,
                                                                NumberFormatException {
        StringBuffer sb = new StringBuffer();

        int tmp = inStream.read();
        while ( tmp == ' ' ) {
            tmp = inStream.read();
          }
        while ( tmp != ' ' && tmp != '\n' ) {
            sb.append( (char)tmp );
            tmp = inStream.read();
          }

        return new Float( new String( sb ) ).floatValue();
      }
  }
