/**
 * $Id: isosurf.java,v 1.5 1997/11/13 13:05:52 descarte Exp descarte $
 *
 * Copyright (c)1997 Alligator Descartes <descarte@hermetica.com>
 *
 * Test program corresponding to 'isosurf' bundled with Mesa
 *
 * $Log: isosurf.java,v $
 * Revision 1.5  1997/11/13 13:05:52  descarte
 * Fixed some spelling mistakes in the descriptions.
 *
 * Revision 1.4  1997/11/13 13:03:06  descarte
 * Converted old demo to Magician beta 2 syntax.
 * Implemented demoTemplate
 * Added filename dialog box if the datafile cannot be located in the current
 * working directory
 *
 * Revision 1.3  1997/08/04 23:19:19  descarte
 * Fixed this up for Win32. Yet another bizarre context problem.
 *
 * Revision 1.2  1997/06/07 06:10:52  descarte
 * Put class in correct package.
 *
 * Revision 1.1  1997/06/05 20:37:18  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.mesa;

import java.applet.*;
import java.awt.*;
import java.io.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class isosurfapp extends Applet implements demoTemplate {

    private static final String VERSION = new String( "$Revision: 1.5 $" );

    /** Angles of rotation of the surfaces being modelled */
    float xrot,
          yrot;

    /** The Magician OpenGL state machine */
    private GL gl_ = new GL();

    /** The Magician Java components for OpenGL rendering */
    GLContext context = new GLContext();
    GLComponent glc = null;

    /** Dimensions of the viewport we wish to create */
    private static final int WIDTH = 300;
    private static final int HEIGHT = 300;

    /** Indicates whether smooth shading is in use */
    int smooth = GL.GL_TRUE;

    /** Indicates whether lighting is in use */
    int lighting = GL.GL_TRUE;

    /** Structures to hold surface information */
    private static final int MAXVERTS = 10000;

    float[][] verts = new float[MAXVERTS][3];
    float[][] norms = new float[MAXVERTS][3];
    int numverts;

    public void init() {
        startDemo();
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: isosurf.java,v 1.5 1997/11/13 13:05:52 descarte Exp descarte $" );
      }

    /** Returns a description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration loads a datafile containing triangle\n" );
        sb.append( "data representing wind in a wind tunnel. The model may\n" );
        sb.append( "be rotated and the shading mode altered. If the datafile\n" );
        sb.append( "cannot be located firstly in the current directory, a\n" );
        sb.append( "file dialog box will be displayed for you to locate\n" );
        sb.append( "the file. It will be present in the ``mesa'' subdirectory\n" );
        sb.append( "of the demo classes called 'isosurf.dat'. Finally, once you\n" );
        sb.append( "have begun loading the datafile, it may take a minute to fully\n" );
        sb.append( "load.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    h - Rotate left\n" );
        sb.append( "    l - Rotate right\n" );
        sb.append( "    j - Rotate away from\n" );
        sb.append( "    k - Rotate towards\n" );
        sb.append( "    s - Toggle shade mode\n" );
        sb.append( "    i - Toggle lighting\n" );
        sb.append( "    ESC - Quits the demo" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Brian Paul <brianp@ra.avid.com>" );
      }

    public void startDemo() {
        /** Create a new frame */
        resize( WIDTH, HEIGHT );
        glc = 
            new GLComponent( WIDTH, HEIGHT );
        add( glc );
        /**#layout */

        GLCapabilities cap = context.getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setRedBits( 1 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        glc.setContext( context );

        /** Read the datafile in */
        try {
            readDatafile( "isosurf.dat" );
          } catch ( FileNotFoundException e ) {
            try {
                FileDialog fd = new FileDialog( this, "isosurf datafile", FileDialog.LOAD );
                fd.show();

                /** Should have the filename here */
                readDatafile( fd.getDirectory() + File.separator + fd.getFile() );
              } catch ( IOException e3 ) {
                e3.printStackTrace();
              }
          } catch ( IOException e2 ) {
            System.err.println( "Exception caught in readDatafile: " +
                                e2.toString() );
          }

        initialize();
        display();
      }

    /** Does some basic initialization */
    private void initialize() throws OpenGLException {

        /** Make the GLContext current */
        context.makeCurrent( glc );

        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );

        gl_.glShadeModel( GL.GL_SMOOTH );
        gl_.glEnable( GL.GL_DEPTH_TEST );

        initializeMaterials();

        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0f, 1.0f, -1.0f, 1.0f, 5f, 25f );

        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glTranslatef( 0.0f, 0.0f, -6.0f );

        /** Unlock the GLContext */
        context.unlock();
      }

    /** Initializes the materials in the scene */
    private void initializeMaterials() throws OpenGLException {

        float[] ambient = { 0.1f, 0.1f, 0.1f, 1.0f };
        float[] diffuse = { 0.5f, 1.0f, 1.0f, 1.0f };
        float[] position0 = { 0.0f, 0.0f, 20.0f, 0.0f };
        float[] position1 = { 0.0f, 0.0f, -20.0f, 0.0f };
        float[] front_mat_shininess = { 60.0f };
        float[] front_mat_specular = { 0.2f, 0.2f, 0.2f, 1.0f };
        float[] front_mat_diffuse = { 0.5f, 0.28f, 0.38f, 1.0f };
        float[] lmodel_ambient = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] lmodel_twoside = { GL.GL_FALSE };

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, ambient );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position0 );
        gl_.glEnable( GL.GL_LIGHT0 );

        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_AMBIENT, ambient );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, diffuse );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, position1 );
        gl_.glEnable( GL.GL_LIGHT1 );
    
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside );
        gl_.glEnable( GL.GL_LIGHTING );

        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_SHININESS, front_mat_shininess );
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_SPECULAR, front_mat_specular );
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE, front_mat_diffuse );
      }

    /** Renders the scene */
    private void display() {

        context.makeCurrent( glc );

        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
        gl_.glPushMatrix();
        gl_.glRotatef( yrot, 0, 1, 0 );
        gl_.glRotatef( xrot, 1, 0, 0 );

        gl_.glBegin( GL.GL_TRIANGLE_STRIP );
            for ( int i = 0 ; i < numverts ; i++ ) {
                gl_.glNormal3f( norms[i][0], norms[i][1], norms[i][2] );
                gl_.glVertex3f( verts[i][0], verts[i][1], verts[i][2] );
              }
        gl_.glEnd();

        gl_.glPopMatrix();

        context.swapBuffers( glc );
      }

    /** Handles AWT Events */
    /**
     *
     */
    public boolean handleEvent( Event evt ) {
        glc.handleEvent( evt );
        switch ( evt.id ) {
            case Event.KEY_PRESS: {
                switch ( evt.key ) {
                    case 27: {
                        /**#dispose */
                        return true;
                      }
                    case 'h': {
                        yrot -= 15.0f;
                        try {
                            display();
                          } catch ( OpenGLException e ) {
                            e.printStackTrace();
                          }
                        return true;
                      }
                    case 'l': {
                        yrot += 15.0;
                        try {
                            display();
                          } catch ( OpenGLException e ) {
                            e.printStackTrace();
                          }
                        return true;
                      }
                    case 'j': {
                        xrot += 15.0;
                        try {
                            display();
                          } catch ( OpenGLException e ) {
                            e.printStackTrace();
                          }
                        return true;
                      }
                    case 'k': {
                        xrot -= 15.0;
                        try {
                            display();
                          } catch ( OpenGLException e ) {
                            e.printStackTrace();
                          }
                        return true;
                      }
                    case 's': {
                        try {
                            /** Make the context current */
                            context.makeCurrent( glc );

                            if ( smooth == GL.GL_FALSE ) {
                                smooth = GL.GL_TRUE;
                                gl_.glShadeModel( GL.GL_SMOOTH );
                              } else {
                                smooth = GL.GL_FALSE;
                                gl_.glShadeModel( GL.GL_FLAT );
                              }
                            /** Release the context */
                            context.unlock();

                            display();
                          } catch ( OpenGLException e ) {
                            e.printStackTrace();
                          }
                        return true;
                      }
                    case 'i': {
                        try {
                            /** Make the context current */
                            context.makeCurrent( glc );

                            if ( lighting == GL.GL_FALSE ) {
                                lighting = GL.GL_TRUE;
                                gl_.glEnable( GL.GL_LIGHTING );
                              } else {
                                lighting = GL.GL_FALSE;
                                gl_.glDisable( GL.GL_LIGHTING );
                              }
                            /** Release the context */
                            context.unlock();
                            
                            display();
                          } catch ( OpenGLException e ) {
                            e.printStackTrace();
                          }
                        return true;
                      }
                  }
              }
          }
        return false;
      }

    /** Read in a datafile containing the surface information */
    private boolean readDatafile( String filename ) throws IOException {

        File inFile = new File( filename );
        DataInputStream inStream = 
            new DataInputStream( new FileInputStream( inFile ) );

        numverts = 0;
        while ( numverts < MAXVERTS && inStream.available() > 0 ) {
            verts[numverts][0] = parseFloat( inStream );
            verts[numverts][1] = parseFloat( inStream );
            verts[numverts][2] = parseFloat( inStream );

            norms[numverts][0] = parseFloat( inStream );
            norms[numverts][1] = parseFloat( inStream );
            norms[numverts][2] = parseFloat( inStream );

            numverts++;
          }
        numverts--;

        System.err.println( numverts + " vertices, " + 
                            ( numverts - 2 ) + " triangles" );
        return true;
      }

    /** Parses a float from a stream */
    private float parseFloat( DataInputStream inStream ) throws IOException,
                                                                NumberFormatException {
        StringBuffer sb = new StringBuffer();

        int tmp = inStream.read();
        while ( tmp == ' ' ) {
            tmp = inStream.read();
          }
        while ( tmp != ' ' && tmp != '\n' ) {
            sb.append( (char)tmp );
            tmp = inStream.read();
          }

        return new Float( new String( sb ) ).floatValue();
      }
  }
