/**
 * $Id: offset.java.stub,v 1.13 1998/10/19 11:09:47 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Test program that tests the polygon offset capabilities of OpenGL
 * for hidden-line removal
 *
 * $Log: offset.java.stub,v $
 * Revision 1.13  1998/10/19 11:09:47  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.12  1998/10/02 12:37:15  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/04/21 15:30:01  descarte
 * Added new demoTemplate methods. Added standalone code. Added WindowListener.
 *
 * Revision 1.10  1998/03/30 11:42:53  descarte
 * Added applet resource handling.
 *
 * Revision 1.9  1998/03/28 17:15:02  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.8  1998/03/24 17:21:16  descarte
 * Converted to new GLDrawable interface.
 * Corrected glPolygonOffset() call.
 *
 * Revision 1.7  1998/03/19 22:36:47  descarte
 * Added getGL() method.
 *
 * Revision 1.6  1998/02/23 22:04:52  descarte
 * Converted demo to new beta 4 GLComponent structure.
 *
 * Revision 1.5  1998/01/04 19:58:07  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.4  1998/01/03 15:43:15  descarte
 * Fixed add() method for Microsoft VM. The "Center" argument was missing.
 *
 * Revision 1.3  1998/01/02 17:32:43  descarte
 * Converted demo across to GLEventListener interface.
 *
 * Revision 1.2  1997/12/23 17:23:06  descarte
 * Added layout manager to make window resize correctly under MSVM.
 *
 * Revision 1.1  1997/11/29 15:03:19  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.mesa;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class offset extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    CoreGL gl_ = new CoreGL();
    CoreGLU glu_ = new CoreGLU();

    /** OpenGL-aware AWT Component */
    GLComponent glc = null;

    /** Dimensions of the OpenGL component */
    private static final int WIDTH = 900;
    private static final int HEIGHT = 300;

    /** Maximum number of quads */
    private static final int MAXQUAD = 6;

    /** Data to define the six faces of a unit cube */
    float[][] quads = {
        { 0,0,0, 1,0,0, 1,1,0, 0,1,0 },
        { 0,0,1, 1,0,1, 1,1,1, 0,1,1 },
        { 0,0,0, 1,0,0, 1,0,1, 0,0,1 },
        { 0,1,0, 1,1,0, 1,1,1, 0,1,1 },
        { 0,0,0, 0,0,1, 0,1,1, 0,1,0 },
        { 1,0,0, 1,0,1, 1,1,1, 1,1,0 }
      };

    /** Dimension of a cube */
    private static final int dimension = 3;

    /** Various rendering modes */
    private static final int WIREFRAME = 0;
    private static final int HIDDEN_LINE = 1;

    /** Mouse tracking variables */
    private int mx = 90,
                my = 40;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        offset d = new offset();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public offset() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Hidden-Line Removal with Polygon Offset" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: offset.java.stub,v 1.13 1998/10/19 11:09:47 descarte Exp descarte $" );
      }

    /** Returns a description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of polygon offset\n" );
        sb.append( "to perform hidden-line removal. The left-most cubes are\n" );
        sb.append( "rendered in wire-frame mode which makes discerning the\n" );
        sb.append( "structure of the objects difficult. The middle cubes are\n" );
        sb.append( "rendered using a hidden-line algorithm but the outlines\n" );
        sb.append( "of the cubes suffer from an artifact known as ``stitching''\n" );
        sb.append( "without using polygon offset. The right-most cubes are\n" );
        sb.append( "rendered using polygon offset and do not suffer from\n" );
        sb.append( "stitching.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    o - Rotate left\n" );
        sb.append( "    p - Rotate right\n" );
        sb.append( "    q - Rotate up\n" );
        sb.append( "    a - Rotate down\n" );
        sb.append( "    ESC - Quit demo" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Based on code originally by Kurt Akeley" );
      }

    public void startDemo() {

        /** Set the layout manager */
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        add( "Center", glc );
        pack(); show();

        /** Setup the capabilities of the OpenGL rendering context */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setColourBits( 24 );
        cap.setDepthBits( 12 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a new keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Tell the component to start processing events */
        glc.initialize();
      }

    /** Initializes the application */
    public void initialize( GLDrawable component ) {
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glPolygonOffset( 2.0f, 1f );

        gl_.glShadeModel( GL.GL_FLAT );
        gl_.glDisable( GL.GL_DITHER );
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, ( height - width ) >> 1, width, width );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 20, 1, 0.1f, 20 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glTranslatef( 0, 0, -15 );
      }

    /** Renders the scene */
    public void display( GLDrawable component ) {
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
        gl_.glPushMatrix();
            gl_.glTranslatef( -1.7f, 0.0f, 0.0f );
            cubes( mx, my, WIREFRAME );
        gl_.glPopMatrix();

        gl_.glPushMatrix();
            cubes( mx, my, HIDDEN_LINE );
        gl_.glPopMatrix();

        gl_.glPushMatrix();
            gl_.glTranslatef( 1.7f, 0, 0 );
            gl_.glEnable( GL.GL_POLYGON_OFFSET_FILL );
            cubes( mx, my, HIDDEN_LINE );
            gl_.glDisable( GL.GL_POLYGON_OFFSET_FILL );
        gl_.glPopMatrix();
      }

    /** Returns a GL pipeline for this listener */
    public GL getGL() {
        return gl_;
      }

    /** Draws a cube */
    private void cubes( int mx, int my, int mode ) {

        /** Track the mouse */
        gl_.glRotatef( (float)( mx / 2.0 ), 0, 1, 0 );
        gl_.glRotatef( (float)( my / 2.0 ), 1, 0, 0 );

        /** Draw the lines as hidden polygons */
        gl_.glTranslatef( -0.5f, -0.5f, -0.5f );
        gl_.glScalef( (float)( 1.0 / dimension ),
                      (float)( 1.0 / dimension ),
                      (float)( 1.0 / dimension ) );
        for ( int z = 0 ; z < dimension ; z++ ) {
            for ( int y = 0 ; y < dimension ; y++ ) {
                for ( int x = 0 ; x < dimension ; x++ ) {
                    gl_.glPushMatrix();
                        gl_.glTranslatef( (float)x, (float)y, (float)z );
                        gl_.glScalef( 0.8f, 0.8f, 0.8f );
                        for ( int i = 0 ; i < MAXQUAD ; i++ ) {
                            drawHidden( quads[i], mode );
                          }
                    gl_.glPopMatrix();
                  }
              }
          }
      }

    /** Draws a hidden quad */
    private void drawHidden( float[] quad, int mode ) {
        if ( mode == HIDDEN_LINE ) {
            gl_.glColor3f( 0, 0, 0 );
            fill( quad );
          }

        /** Draw the outline using white, optionally fill the interior with black */
        gl_.glColor3f( 1, 1, 1 );
        outline( quad );
      }

    /** Draws a filled quad */
    private void fill( float[] quad ) {
        gl_.glBegin( GL.GL_QUADS );
            gl_.glVertex3f( quad[0], quad[1], quad[2] );
            gl_.glVertex3f( quad[3], quad[4], quad[5] );
            gl_.glVertex3f( quad[6], quad[7], quad[8] );
            gl_.glVertex3f( quad[9], quad[10], quad[11] );
        gl_.glEnd();
      }

    /** Draws the outline of a quad */
    private void outline( float[] quad ) {
        gl_.glBegin( GL.GL_LINE_LOOP );
            gl_.glVertex3f( quad[0], quad[1], quad[2] );
            gl_.glVertex3f( quad[3], quad[4], quad[5] );
            gl_.glVertex3f( quad[6], quad[7], quad[8] );
            gl_.glVertex3f( quad[9], quad[10], quad[11] );
        gl_.glEnd();
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
        if ( evt.getKeyChar() == 'o' ) {
            mx -= 5;
            mx %= 360;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'p' ) {
            mx += 5;
            mx %= 360;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'q' ) {
            my += 5;
            my %= 360;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'a' ) {
            my -= 5;
            my %= 360;
            glc.repaint();
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
