/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: olympic.java.stub,v 1.13 1998/10/19 11:09:47 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Test program that draws the Olympic rings
 *
 * $Log: olympic.java.stub,v $
 * Revision 1.13  1998/10/19 11:09:47  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.12  1998/10/02 12:37:16  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/04/21 15:30:03  descarte
 * Added new demoTemplate methods. Added standalone code. Added WindowListener.
 *
 * Revision 1.10  1998/03/30 11:42:54  descarte
 * Added applet resource handling.
 *
 * Revision 1.9  1998/03/28 17:29:24  descarte
 * Upgraded to Java 1.1 listeners.
 * Upgraded to GLComponentFactory
 *
 * Revision 1.8  1998/03/24 17:24:08  descarte
 * Converted to GLDrawable interface.
 *
 * Revision 1.7  1998/03/19 22:36:56  descarte
 * Added getGL() method.
 *
 * Revision 1.6  1998/02/23 22:09:52  descarte
 * Converted demo to new beta 4 GLComponent structure.
 *
 * Revision 1.5  1998/01/04 19:58:08  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.4  1998/01/03 15:43:18  descarte
 * Fixed add() method for Microsoft VM. The "Center" argument was missing.
 *
 * Revision 1.3  1998/01/02 17:37:53  descarte
 * Converted demo across to GLEventListener interface.
 *
 * Revision 1.2  1997/12/23 17:25:45  descarte
 * Added layout manager for correct window sizing under MSVM.
 *
 * Revision 1.1  1997/11/29 16:25:32  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.mesa;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class olympicapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    CoreGL gl_ = new CoreGL();
    CoreGLU glu_ = new CoreGLU();

    /** OpenGL-aware AWT Component */
    GLComponent glc = null;

    /** Dimensions of the OpenGL component */
    private static final int WIDTH = 400;
    private static final int HEIGHT = 300;

    /** Maximum integer */
    private final int RAND_MAX = 2147483647;

    /** Some definitions */
    private static final int RINGS = 5;
    private static final int BLUERING = 0;
    private static final int BLACKRING = 1;
    private static final int REDRING = 2;
    private static final int YELLOWRING = 3;
    private static final int GREENRING = 4;
    private static final int BACKGROUND = 8;

    private static final int XSIZE = 100;
    private static final int YSIZE = 75;

    /** RGB colourmap */
    byte[][] rgbColors = new byte[RINGS][3];
    float[][] dests = new float[RINGS][3];
    float[][] offsets = new float[RINGS][3];
    float[] angs = new float[RINGS];
    float[][] rotAxis = new float[RINGS][3];
    int[] iters = new int[RINGS];
    int theTorus = -1;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Olympic Rings" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: olympic.java.stub,v 1.13 1998/10/19 11:09:47 descarte Exp descarte $" );
      }

    /** Returns a description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration program renders the Olympic rings\n" );
        sb.append( "by spinning 5 torus shapes into place.\n\n" );
        sb.append( "    Space - Restart demo\n" );
        sb.append( "    ESC - Quit demo" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Silicon Graphics Incorporated ( http://www.sgi.com )" );
      }

    public void startDemo() {

        /** Set the layout manager */
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        add( "Center", glc );
        /**#layout */

        /** Setup the capabilities of the OpenGL rendering context */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setColourBits( 24 );
        cap.setDepthBits( 12 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Tells the component to generate events and stuff */
        glc.initialize();
        glc.start();
      }

    /** Initializes the application */
    public void initialize( GLDrawable component ) {

        float x, y;

        float base, height;
        float aspect;
        float top_y = 1.0f;
        float bottom_y = 0.0f;
        float top_z = 0.15f;
        float bottom_z = 0.69f;
        float spacing = 2.5f;

        float[] lmodel_ambient = { 0.0f, 0.0f, 0.0f, 0.0f };
        float[] lmodel_twoside = { GL.GL_FALSE };
        float[] lmodel_local = { GL.GL_FALSE };
        float[] light0_ambient = { 0.1f, 0.1f, 0.1f, 1.0f };
        float[] light0_diffuse = { 1.0f, 1.0f, 1.0f, 0.0f };
        float[] light0_position = { 0.8660254f, 0.5f, 1f, 0f };
        float[] light0_specular = { 1.0f, 1.0f, 1.0f, 0.0f };
        float[] bevel_mat_ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
        float[] bevel_mat_shininess = { 40.0f };
        float[] bevel_mat_specular = { 1.0f, 1.0f, 1.0f, 0.0f };
        float[] bevel_mat_diffuse = { 1.0f, 0.0f, 0.0f, 0.0f };

        reInitialize();

        for ( int i = 0 ; i < RINGS ; i++ ) {
            rgbColors[i][0] = rgbColors[i][1] = rgbColors[i][2] = 0;
          }
        rgbColors[BLUERING][2] = (byte)255;
        rgbColors[REDRING][0] = (byte)255;
        rgbColors[GREENRING][1] = (byte)255;
        rgbColors[YELLOWRING][0] = (byte)255;
        rgbColors[YELLOWRING][1] = (byte)255;

        dests[BLUERING][0] = -spacing;
        dests[BLUERING][1] = top_y;
        dests[BLUERING][2] = top_z;

        dests[BLACKRING][0] = 0.0f;
        dests[BLACKRING][1] = top_y;
        dests[BLACKRING][2] = top_z;

        dests[REDRING][0] = spacing;
        dests[REDRING][1] = top_y;
        dests[REDRING][2] = top_z;

        dests[YELLOWRING][0] = (float)( -spacing / 2.0 );
        dests[YELLOWRING][1] = bottom_y;
        dests[YELLOWRING][2] = bottom_z;

        dests[GREENRING][0] = (float)( spacing / 2.0 );
        dests[GREENRING][1] = bottom_y;
        dests[GREENRING][2] = bottom_z;

        base = 2.0f; 
        height = 2.0f;
        theTorus = gl_.glGenLists(1);
        gl_.glNewList( theTorus, GL.GL_COMPILE );
            fillTorus( 0.1f, 8, 1.0f, 25 );
        gl_.glEndList();

        x = (float)XSIZE;
        y = (float)YSIZE;
        aspect = x / y;
        gl_.glEnable( GL.GL_CULL_FACE );
        gl_.glCullFace( GL.GL_BACK );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glClearDepth( 1.0f );

        gl_.glClearColor( 0.5f, 0.5f, 0.5f, 0.0f );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, light0_ambient );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, light0_diffuse );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_SPECULAR, light0_specular );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, light0_position );
        gl_.glEnable( GL.GL_LIGHT0 );

        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_LOCAL_VIEWER, lmodel_local );
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside );
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );
        gl_.glEnable( GL.GL_LIGHTING );

        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, bevel_mat_ambient );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, bevel_mat_shininess );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, bevel_mat_specular );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, bevel_mat_diffuse );

        gl_.glColorMaterial( GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE );
        gl_.glEnable( GL.GL_COLOR_MATERIAL );
        gl_.glShadeModel( GL.GL_SMOOTH );
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 45f, 1.33f, 0.1f, 100.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Returns a GL pipeline for this GLEventListener */
    public GL getGL() {
        return gl_;
      }

    /** Re-initializes the scene */
    private void reInitialize() {
        float deviation;

        deviation = myRand() / 2;
        deviation *= deviation;
        for ( int i = 0 ; i < RINGS ; i++ ) {
            offsets[i][0] = myRand();
            offsets[i][1] = myRand();
            offsets[i][2] = myRand();

            angs[i] = 260.0f * myRand();

            rotAxis[i][0] = myRand();
            rotAxis[i][1] = myRand();
            rotAxis[i][2] = myRand();

            iters[i] = (int)( deviation * myRand() + 60.0f );
          }
      }

    /** Generates a torus */
    private void fillTorus( float rc, int numc, float rt, int numt ) {
        int i, j, k;
        double s, t;
        double x, y, z;
        double pi, twopi;

        pi = Math.PI;
        twopi = 2 * pi;
 
        for (i = 0; i < numc; i++) {
            gl_.glBegin( GL.GL_QUAD_STRIP );
            for (j = 0; j <= numt; j++) {
                for (k = 1; k >= 0; k--) {
                    s = (i + k) % numc + 0.5;
                    t = j % numt;
   
                    x = Math.cos(t*twopi/numt) * Math.cos(s*twopi/numc);
                    y = Math.sin(t*twopi/numt) * Math.cos(s*twopi/numc);
                    z = Math.sin(s*twopi/numc);
                    gl_.glNormal3f( (float)x, (float)y, (float)z );

                    x = (rt + rc * Math.cos(s*twopi/numc)) * Math.cos(t*twopi/numt);
                    y = (rt + rc * Math.cos(s*twopi/numc)) * Math.sin(t*twopi/numt);
                    z = rc * Math.sin(s*twopi/numc);
                    gl_.glVertex3f( (float)x, (float)y, (float)z );
                  }
              }
            gl_.glEnd();
          }
      }

    /** Renders the scene */
    public void display( GLDrawable component ) {
        for ( int i = 0 ; i < RINGS ; i++ ) {
            if ( iters[i] != 0 ) {
                for ( int j = 0 ; j < 3 ; j++ ) {
                    offsets[i][j] = clamp( iters[i], offsets[i][j] );
                  }
                angs[i] = clamp( iters[i], angs[i] );
                iters[i]--;
              }
          }

        gl_.glPushMatrix();
    
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
        glu_.gluLookAt( 0f, 0f, 10f, 0f, 0f, 0f, 0f, 1f, 0f );

        for ( int i = 0 ; i < RINGS ; i++ ) {
            gl_.glColor3ubv( rgbColors[i] );
            gl_.glPushMatrix();
            gl_.glTranslatef( dests[i][0]+offsets[i][0], 
                              dests[i][1]+offsets[i][1],
                              dests[i][2]+offsets[i][2]);
            gl_.glRotatef( angs[i], rotAxis[i][0], rotAxis[i][1], rotAxis[i][2] );
            gl_.glCallList( theTorus );
            gl_.glPopMatrix();
        }

        gl_.glPopMatrix();
      }

    /** Randomizer */
    private float myRand() {
        return (float)( 10.0 * ( ( (float)Math.random() * RAND_MAX ) /
                                 ( (float)RAND_MAX - 0.5 ) ) );
      }

    /** Clamps values */
    private float clamp( int itersLeft, float t ) {
        if ( itersLeft < 3 ) {
            return 0.0f;
          }
        return (float)( ( itersLeft - 2 ) * t / itersLeft );
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
            return;
          }
        if ( evt.getKeyChar() == ' ' ) {
            /** 
             * Lock the context to avoid things writing to the array we're
             * about to update, or read from it resulting in weird effects...
             */
            glc.getContext().lock();
            reInitialize();
            glc.getContext().unlock();
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
