/**
 * $Id: reflect.java.stub,v 1.15 1998/10/31 17:12:14 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * This is a Java version of Brian Paul's ``reflect'' demo bundled with the
 * Mesa OpenGL-a-like library
 *
 * $Log: reflect.java.stub,v $
 * Revision 1.15  1998/10/31 17:12:14  descarte
 * Made a few traps to stop Mesa-3.0 crashing on textured poly draws.
 *
 * Revision 1.14  1998/10/19 11:09:48  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.13  1998/10/02 12:37:17  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.12  1998/04/21 15:30:05  descarte
 * Added new demoTemplate methods. Added standalone code. Added WindowListener.
 *
 * Revision 1.11  1998/03/30 11:42:55  descarte
 * Added applet resource handling.
 *
 * Revision 1.10  1998/03/28 17:10:19  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.9  1998/03/24 17:19:00  descarte
 * Converted to new GLDrawable interface.
 *
 * Revision 1.8  1998/03/19 22:37:08  descarte
 * Added getGL() method.
 *
 * Revision 1.7  1998/03/04 00:09:11  descarte
 * Changed Texture constructor call to have the context removed since it is
 * now tucked inside the GLComponent.
 *
 * Revision 1.6  1998/02/24 17:46:04  descarte
 * Altered glc.start() call. Added setContinuousRefresh() call.
 *
 * Revision 1.5  1998/02/23 22:14:03  descarte
 * Converted demo to beta 4 GLComponent structure.
 *
 * Revision 1.4  1998/01/07 20:51:02  descarte
 * Added additional thread handling for applets. This doens't really work.
 *
 * Revision 1.3  1998/01/04 19:58:09  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.2  1998/01/03 15:43:19  descarte
 * Fixed add() method for Microsoft VM. The "Center" argument was missing.
 *
 * Revision 1.1  1998/01/01 22:28:22  descarte
 * Initial revision
 *
 * Revision 1.2  1997/11/11 13:28:12  descarte
 * Tidied up demo program, added a description and fixed a bug here and there.
 *
 * Revision 1.1  1997/11/11 00:19:39  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.mesa;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class reflect extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private GL gl_ = null;
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL components to render onto */
    private GLComponent glc = null;

    /** The dimensions of the components */
    private static final int WIDTH = 400,
                             HEIGHT = 300;

    /** Converter from degrees to radians */
    private float DEG2RAD = (float)( Math.PI / 180.0 );

    /** Variable to track thread status in applets */
    private boolean appletThreadStopped = false;

    /** The display list for the table */
    private int tableList = -1;

    /** Display lists for the objects */
    private int[] objectsList = new int[2];

    /** Rotation values */
    float xrot, yrot, spin;

    /** Position of the light source */
    float[] light_pos = { 0.0f, 20.0f, 0.0f, 1.0f };

    /** The table texture */
    private Texture tableTexture = null;
    private boolean tableTextureInitialized = false;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        reflect d = new reflect();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    /** Default constructor */
    public reflect() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Reflections" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: reflect.java.stub,v 1.15 1998/10/31 17:12:14 descarte Exp descarte $" );
      }

    /** Returns a description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the stencil buffer\n" );
        sb.append( "to render reflective surfaces using OpenGL. The auxiliary \n" );
        sb.append( "Texture class supplied with Magician is also used to \n" );
        sb.append( "texture-map the surface.\n\n" );
        sb.append( "The texture for the surface will most likely fail to appear\n" );
        sb.append( "when you run this demo since the URL for the texture map is\n" );
        sb.append( "hard-coded into the demonstation program. Simply alter the\n" );
        sb.append( "source code for the demonstration to load the file '1.gif'\n" );
        sb.append( "present in the demo classes distribution instead, or any other\n" );
        sb.append( "image you wish to use as a sample map.\n\n" );
        sb.append( "Hit the ``Escape'' key to quit this demo." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Brian Paul <brianp@ra.avid.com>" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );
        setBackground( Color.black );

        /** Create the OpenGL state machine */
        gl_ = new CoreGL();

        /** Create the OpenGL Components */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window and do layout */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );
        cap.setStencilBits( 1 );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Tell the component to generate and process events */
        glc.initialize();
        glc.start();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        /** Build the objects in the scene */
        makeTable();
        makeObjects();

        /** Initialize the starting location */
        xrot = 30.0f;
        yrot = 50.0f;
        spin = 0.0f;

        /** Setup the environment */
        gl_.glDisable( GL.GL_CULL_FACE );
        gl_.glShadeModel( GL.GL_FLAT );

        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );

        gl_.glClearColor( 0.5f, 0.5f, 0.5f, 1.0f );
        gl_.glEnable( GL.GL_NORMALIZE );
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -( (float)width / (float)height ), 
                       ( (float)width / (float)height ), 
                       -1.0, 1.0, 4.0, 300.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {

        /** Distance from the scene */
        float dist = 20.0f;

        /** Camera position */
        float eyex = 0;
        float eyey = 0;
        float eyez = 0;

        /** Move the scene about */
        spin += 2.0f;
        yrot += 3.0f;

        /** Do the OpenGL stuff */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT |
                     GL.GL_STENCIL_BUFFER_BIT );

        /** Calculate the camera location */
        eyex = (float)( dist * Math.cos( yrot * DEG2RAD ) * Math.cos( xrot * DEG2RAD ) );
        eyey = (float)( dist * Math.sin( yrot * DEG2RAD ) * Math.cos( xrot * DEG2RAD ) );
        eyez = (float)( dist * Math.sin( xrot * DEG2RAD ) );

        /** View from the top */
        gl_.glPushMatrix();
            glu_.gluLookAt( eyex, eyey, eyez, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, light_pos );

            /** Draw table into stencil planes */
            gl_.glEnable( GL.GL_STENCIL_TEST );
            gl_.glDisable( GL.GL_DEPTH_TEST );
            gl_.glStencilFunc( GL.GL_ALWAYS, 1, 0xffffffff );
            gl_.glStencilOp( GL.GL_REPLACE, GL.GL_REPLACE, GL.GL_REPLACE );
            gl_.glColorMask( (byte)GL.GL_FALSE, (byte)GL.GL_FALSE, 
                             (byte)GL.GL_FALSE, (byte)GL.GL_FALSE );
            drawTable();
            gl_.glColorMask( (byte)GL.GL_TRUE, (byte)GL.GL_TRUE, 
                             (byte)GL.GL_TRUE, (byte)GL.GL_TRUE );
            gl_.glEnable( GL.GL_DEPTH_TEST );

            /** Render view from below ( reflected viewport ) */
            /** Only draw where stencil = 1 */
            if ( eyey > 0.0f ) {
                gl_.glPushMatrix();
                    gl_.glStencilFunc( GL.GL_EQUAL, 1, 0xffffffff );
                    gl_.glStencilOp( GL.GL_KEEP, GL.GL_KEEP, GL.GL_KEEP );
                    gl_.glScalef( 1.0f, -1.0f, 1.0f );

                    /** Reposition light in reflected space */
                    gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, light_pos );

                    drawObjects( eyex, eyey, eyez );
                gl_.glPopMatrix();

                /** Restore light's original unreflected position */
                gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, light_pos );
              }

            gl_.glDisable( GL.GL_STENCIL_TEST );

            gl_.glEnable( GL.GL_BLEND );
            gl_.glBlendFunc( GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA );
            if ( tableTexture.isValid() ) {
                gl_.glEnable( GL.GL_TEXTURE_2D );
              }
            drawTable();
            if ( tableTexture.isValid() ) {
                gl_.glDisable( GL.GL_TEXTURE_2D );
              }
            gl_.glDisable( GL.GL_BLEND );

            gl_.glPushMatrix();
                drawObjects( eyex, eyey, eyez );
            gl_.glPopMatrix();
        gl_.glPopMatrix();
      }

    /** Returns the current GL pipeline for this listener */
    public GL getGL() {
        return gl_;
      }

    /** Creates the ``table'' in the scene */
    private void makeTable() {
        float[] table_mat = { 1.0f, 1.0f, 1.0f, 0.6f };
        float[] gray = { 0.4f, 0.4f, 0.4f, 1.0f };

        if ( tableTexture == null ) {
            /** Fetch the texture for the table */
            try {
                Image tableImage = 
                    Toolkit.getDefaultToolkit().getImage( new URL( "http://aberlemno/img/arcanelogo.gif" ) );
                tableTexture = 
                    new Texture( tableImage, glc, Texture.SCALE_NEAREST );
              } catch ( Exception e ) {
                e.printStackTrace();
              }
          }

        tableList = gl_.glGenLists( 1 );
        gl_.glNewList( tableList, GL.GL_COMPILE );

            if ( tableTextureInitialized == false ) {
                if ( tableTexture.isValid() ) {
                    /** Setup the texture for the table */
                    glu_.gluBuild2DMipmaps( GL.GL_TEXTURE_2D, 4, 
                                           tableTexture.getWidth(), tableTexture.getHeight(),
                                           GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, 
                                           tableTexture.getTexture() );

                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_REPEAT );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_REPEAT );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST );
                    gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST );
    
                    tableTextureInitialized = true;
                  }
              }

            /** Load table's texture */
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, table_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, table_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, gray );

            /** Draw textured square for the table */
            gl_.glPushMatrix();
                gl_.glScalef( 4.0f, 4.0f, 4.0f );
                gl_.glBegin( GL.GL_POLYGON );
                    gl_.glNormal3f( 0.0f, 1.0f, 0.0f );
                    gl_.glTexCoord2f( 0.0f, 0.0f ); gl_.glVertex3f( -1.0f, 0.0f, 1.0f );
                    gl_.glTexCoord2f( 1.0f, 0.0f ); gl_.glVertex3f( 1.0f, 0.0f, 1.0f );
                    gl_.glTexCoord2f( 1.0f, 1.0f ); gl_.glVertex3f( 1.0f, 0.0f, -1.0f );
                    gl_.glTexCoord2f( 0.0f, 1.0f ); gl_.glVertex3f( -1.0f, 0.0f, -1.0f );
                gl_.glEnd();
            gl_.glPopMatrix();

            gl_.glDisable( GL.GL_TEXTURE_2D );

        gl_.glEndList();

      }

    /** Draws the table */
    private void drawTable() {

        /** 
         * Check to see if the texture is valid and not initialized. If so,
         * make a new display list for the table with the texture
         */
        if ( tableTextureInitialized == false ) {
            if ( tableTexture.isValid() ) {
                makeTable();
              }
          }

        gl_.glCallList( tableList );
      }

    /** Creates the objects within the scene */
    private void makeObjects() {
        GLUQuadric q = new GLUQuadric();

        float[] cyan = { 0.0f, 1.0f, 1.0f, 1.0f };
        float[] green = { 0.2f, 1.0f, 0.2f, 1.0f };
        float[] black = { 0.0f, 0.0f, 0.0f, 0.0f };

        glu_.gluQuadricDrawStyle( q, GLU.GLU_FILL );
        glu_.gluQuadricNormals( q, GLU.GLU_SMOOTH );

        objectsList[0] = gl_.glGenLists( 1 );
        gl_.glNewList( objectsList[0], GL.GL_COMPILE );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, cyan );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, black );
            glu_.gluCylinder( q, 0.5f, 0.5f, 1.0f, 15, 10 );
        gl_.glEndList();

        objectsList[1] = gl_.glGenLists( 1 );
        gl_.glNewList( objectsList[1], GL.GL_COMPILE );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, green );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, black );
            glu_.gluCylinder( q, 1.5f, 0.0f, 2.5f, 15, 10 );
        gl_.glEndList();
      }

    /** Renders the objects */
    private void drawObjects( float eyex, float eyey, float eyez ) {
        gl_.glPushMatrix();
            gl_.glTranslatef( 1.0f, 1.5f, 0.0f );
            gl_.glRotatef( spin, 1.0f, 0.5f, 0.0f );
            gl_.glRotatef( 0.5f * spin, 0.0f, 0.5f, 1.0f );
            gl_.glCallList( objectsList[0] );
        gl_.glPopMatrix();

        gl_.glPushMatrix();
            gl_.glTranslatef( -1.0f, (float)( 0.85 + 3.0 * Math.abs( Math.cos( 0.01 * spin ) ) ), 0.0f );
            gl_.glRotatef( 0.5f * spin, 0.0f, 0.5f, 1.0f );
            gl_.glRotatef( spin, 1.0f, 0.5f, 0.0f );
            gl_.glScalef( 0.5f, 0.5f, 0.5f );
            gl_.glCallList( objectsList[1] );
        gl_.glPopMatrix();
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
