/* Copyright (c) Mark J. Kilgard, 1996. */

/* This program is freely distributable without licensing fees 
   and is provided without guarantee or warrantee expressed or 
   implied. This program is -not- in the public domain. */

/**
 * $Id: molecule.java,v 1.3 1998/10/19 11:15:03 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Definition of a molecule viewed by the molview application
 *
 * $Log: molecule.java,v $
 * Revision 1.3  1998/10/19 11:15:03  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.2  1998/04/22 08:39:12  descarte
 * Added accessor methods.
 *
 * Revision 1.1  1997/12/01 01:13:39  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.molview;

import java.awt.*;
import java.util.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class molecule {

    /** The OpenGL state machine */
    CoreGL gl_ = new CoreGL();
    CoreGLU glu_ = new CoreGLU();

    /** OpenGL rendering context */
    GLContext context = new GLContext();

    /** Number of atoms in this molecule */
    private int natoms = -1;

    /** Types of atoms are stored in this Vector */
    private Vector types = new Vector();

    /** Instances of atoms are stored in this Vector */
    private Vector instances = new Vector();

    /** Bounding box minimums */
    float xmin = 0,
          ymin = 0,
          zmin = 0;

    /** Bounding box dimensions */
    float xsize = 0,
          ysize = 0,
          zsize = 0;

    /** Dimension */
    float maxdim = 0;

    /** Constructor */
    public molecule() {
      }

    /** Returns the number of atoms in this molecule */
    public int getNumAtoms() {
        return natoms;
      }

    /** Adds an atom type */
    public void addAtomType( atomType type ) {
        System.err.println( type.toString() );
        types.addElement( type );
      }

    /** Adds an atom instance */
    public void addAtomInstance( atomInstance instance ) {
        System.err.println( instance.toString() );

        /** Locate the atom type for this instance and link it in */
        boolean foundType = false;
        for ( Enumeration enum = types.elements() ; enum.hasMoreElements() ; ) {
            atomType type = (atomType)enum.nextElement();
            if ( foundType == false ) { 
                if ( type.name.equals( instance.name ) ) {
                    instance.setType( type );
                    foundType = true;
                  }
              }
          }

        if ( foundType ) {
            instances.addElement( instance );
            natoms++;
          } else {
            System.err.println( "Couldn't locate atom type for instance" );
          }
      }

    /** Returns a given atom instance */
    public atomInstance getAtomInstance( int index ) {
        return (atomInstance)instances.elementAt( index );
      }

    /** Calculate the extents of this molecule */
    public void calculateExtents() {
        float xmin, xmax, ymin, ymax, zmin, zmax;
        float radius;

        Enumeration enum = instances.elements();
        atomInstance inst = (atomInstance)enum.nextElement();
        radius = inst.getType().getRadius();
        xmin = inst.x - radius;
        xmax = inst.x + radius;
        ymin = inst.y - radius;
        ymax = inst.y + radius;
        zmin = inst.z - radius;
        zmax = inst.z + radius;

        for ( ; enum.hasMoreElements() ; ) {

            inst = (atomInstance)enum.nextElement();
            radius = inst.getType().getRadius();
            xmin = MIN( inst.x - radius, xmin );
            xmax = MAX( inst.x + radius, xmax );
            ymin = MIN( inst.y - radius, ymin );
            ymax = MAX( inst.y + radius, ymax );
            zmin = MIN( inst.z - radius, zmin );
            zmax = MAX( inst.z + radius, zmax );
          }
        this.xmin = xmin;
        this.ymin = ymin;
        this.zmin = zmin;
        this.xsize = xmax - xmin;
        this.ysize = ymax - ymin;
        this.zsize = zmax - zmin;
        this.maxdim = MAX( MAX( this.xsize, this.ysize ), this.zsize );
      }

    /** Calculates maximum of two values */
    private float MAX( float a, float b ) {
        return ( ( a > b ) ? a : b );
      }

    /** Calculates minimum of two values */
    private float MIN( float a, float b ) {
        return ( ( a < b ) ? a : b );
      }

    /** Renders the atoms within this molecule */
    public void renderAtoms( int sphereVersion ) {
        float radius;
        int name;
        float SCALE_FACTOR = 4.0f;

        gl_.glEnable( GL.GL_COLOR_MATERIAL );
        gl_.glScalef( SCALE_FACTOR / maxdim,
                      SCALE_FACTOR / maxdim,
                      SCALE_FACTOR / maxdim );
/*        System.err.println( "( " + -( xmin + xsize / 2 ) + ", " +
                            -( ymin + ysize / 2 ) + ", " +
                            -( zmin + zsize / 2 ) + " )" ); */
        gl_.glTranslatef( -( xmin + xsize / 2 ),
                          -( ymin + ysize / 2 ),
                          -( zmin + zsize / 2 ) );

        name = 0;
        for ( Enumeration enum = instances.elements() ; enum.hasMoreElements() ; ) {
            atomInstance atom = (atomInstance)enum.nextElement();

            gl_.glLoadName( name );
            gl_.glPushMatrix();
                gl_.glTranslatef( atom.x, atom.y, atom.z );
                radius = atom.getType().getRadius();
                gl_.glScalef( radius, radius, radius );
                gl_.glColor3fv( atom.getType().getColour() );
                gl_.glCallList( sphereVersion );
            gl_.glPopMatrix();
            name++;
          }
        gl_.glDisable( GL.GL_COLOR_MATERIAL );
      }
  }
