/* Copyright (c) Mark J. Kilgard, 1996. */

/* This program is freely distributable without licensing fees 
   and is provided without guarantee or warrantee expressed or 
   implied. This program is -not- in the public domain. */

/**
 * $Id: molview.java,v 1.7 1998/10/19 11:15:05 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Molecule Viewer - Converted from Mark Kilgard's demo program
 *
 * $Log: molview.java,v $
 * Revision 1.7  1998/10/19 11:15:05  descarte
 * Made JDK-1.1 compliant by removing deprecated methods. Updated copyright.
 *
 * Revision 1.6  1998/10/02 12:41:57  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.5  1998/04/22 19:03:33  descarte
 * Added trap for unloaded molecule before attempting to pick.
 *
 * Revision 1.4  1998/04/22 08:49:34  descarte
 * Added description and radius field printout.
 *
 * Revision 1.2  1998/04/21 15:34:33  descarte
 * Added WindowListener.
 * Added new demoTemplate methods.
 * Added standalone code.
 *
 * Revision 1.1  1998/04/07 16:55:36  descarte
 * Initial revision
 *
 * Revision 1.1  1997/11/29 17:13:20  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.molview;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class molview extends Frame implements demoTemplate, GLEventListener, ActionListener, MouseMotionListener, MouseListener, WindowListener {

    /** The OpenGL state machine */
    CoreGL gl_ = new CoreGL();
    CoreGLU glu_ = new CoreGLU();

    /** OpenGL-aware AWT Component */
    GLComponent glc = null;

    /** Dimensions of the OpenGL component */
    private int WIDTH = 400;
    private int HEIGHT = 300;

    /** The text fields containing picked information */
    TextField atomNameField = null;
    TextField atomPositionField = null;
    TextField atomRadiusField = null;

    /** Checkboxes regulating some features */
    CheckboxMenuItem spinCheckbox = null;
    CheckboxMenuItem autoHiResCheckbox = null;
    CheckboxMenuItem hiResSpinCheckbox = null;

    /** Display list ids */
    private final int HI_RES_SPHERE = 1;
    private final int LO_RES_SPHERE = 2;

    /** The current molecule */
    molecule currentMolecule = null;

    /** Current rendering mode for the molecule */
    private int sphereVersion = HI_RES_SPHERE;

    /** Current quaternion rotation */
    private float[] curQuat = new float[4];

    /** Last quaternion rotation */
    private float[] lastQuat = new float[4];

    /** The trackball */
    private trackball ball = null;

    /** Mouse tracking variables */
    private int beginx = 0,
                beginy = 0;

    /** Signifies whether the molecule has been moved or not */
    private boolean moleculeMoved = false;

    /** The font used to display messages */
    private GLMolviewBitmapFont font = new GLMolviewBitmapFont();

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        molview d = new molview();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    /** Constructor */
    public molview() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Molecule Viewer" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: molview.java,v 1.7 1998/10/19 11:15:05 descarte Exp descarte $" );
      }

    /** Returns a description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration allows you to load molecule definitions\n" );
        sb.append( "from a datafile and rotate the molecule around by means\n" );
        sb.append( "of a ``virtual trackball''. That is, the molecule will\n" );
        sb.append( "contained within a sphere which you can ``roll around''.\n" );
        sb.append( "You can also click on various atoms within each molecule\n" );
        sb.append( "to find out which element they are comprised of. You can\n" );
        sb.append( "also tweak the settings of the program including wireframe\n" );
        sb.append( "rendering and the ability to make the molecule spin of its\n" );
        sb.append( "own volition after you have rotated in.\n\n" );
        sb.append( "You can locate some molecule datafiles within the ``molecules''\n" );
        sb.append( "sub-directory in the Magician demos/molview class directory.\n\n" );
        sb.append( "Press ``Escape'' to quit this demo." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mark Kilgard <mjk@sgi.com>" );
      }

    public void startDemo() {

        setLayout( new GridBagLayout() );

        /** Create a new frame */
        setSize( WIDTH, HEIGHT );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        constrain.constrain( this, glc, 0, 0, 1, 1 );

        /** Create the Panel containing the info fields */
        Panel infoPanel = new Panel();
        infoPanel.setLayout( new GridBagLayout() );
        constrain.constrain( infoPanel, new Label( "Atom Name:" ), 0, 0, 1, 1 );
        constrain.constrain( infoPanel, new Label( "XYZ Position:" ),
                             0, 1, 1, 1 );
        constrain.constrain( infoPanel, new Label( "Radius:" ), 0, 2, 1, 1 );

        /** Add the text fields */
        atomNameField = new TextField( "n/a", 16 );
        atomNameField.setEditable( false );
        atomPositionField = new TextField( "n/a", 16 );
        atomPositionField.setEditable( false );
        atomRadiusField = new TextField( "n/a", 16 );
        atomRadiusField.setEditable( false );
        constrain.constrain( infoPanel, atomNameField, 1, 0, 1, 1 );
        constrain.constrain( infoPanel, atomPositionField, 1, 1, 1, 1 );
        constrain.constrain( infoPanel, atomRadiusField, 1, 2, 1, 1 );

        /** Add the Panel to the Frame */
        constrain.constrain( this, infoPanel, 0, 1, 1, 1,
                             GridBagConstraints.NONE,
                             GridBagConstraints.CENTER,
                             1.0, 1.0, 0, 0, 0, 0 );

        /** Add a menubar to the frame */
        MenuBar mb = new MenuBar();
        Menu fileMenu = new Menu( "File" );
        MenuItem[] fileMenuItems = new MenuItem[2];
        String[] fileMenuItemsLabels = {
            "Open Molecule...",
            "separator",
            "Exit"
          };
        for ( int i = 0, index = 0 ; i < fileMenuItemsLabels.length ; i++ ) {
            if ( fileMenuItemsLabels[i].equals( "separator" ) ) {
                fileMenu.addSeparator();
              } else {
                fileMenuItems[index] = new MenuItem( fileMenuItemsLabels[i] );
                fileMenu.add( fileMenuItems[index] );
                fileMenuItems[index].addActionListener( this );
                index++;
              }
          }
        mb.add( fileMenu );

        /** Add the options menu */
        Menu optionsMenu = new Menu( "Options" );
        MenuItem[] optionsMenuItems = new MenuItem[2];
        optionsMenuItems[0] = new MenuItem( "Solid" );
        optionsMenuItems[1] = new MenuItem( "Wireframe" );
        optionsMenu.add( optionsMenuItems[0] );
        optionsMenuItems[0].addActionListener( this );
        optionsMenu.add( optionsMenuItems[1] );
        optionsMenuItems[1].addActionListener( this );

        optionsMenu.addSeparator();

        spinCheckbox = new CheckboxMenuItem( "Spin Momentum" );
        spinCheckbox.setState( false );
        autoHiResCheckbox = new CheckboxMenuItem( "Auto Hi-Res" );
        autoHiResCheckbox.setState( true );
        hiResSpinCheckbox = new CheckboxMenuItem( "Hi-Res Spin" );
        hiResSpinCheckbox.setState( false );

        optionsMenu.add( spinCheckbox );
        optionsMenu.add( autoHiResCheckbox );
        optionsMenu.add( hiResSpinCheckbox );

        mb.add( optionsMenu );
        setMenuBar( mb );

        pack(); show();

        /** Setup the capabilities of the OpenGL rendering glc.getContext() */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setColourBits( 24 );
        cap.setDepthBits( 12 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLComponent with the GLEventListener */
        glc.addGLEventListener( this );

        /** Register a listener for mouse drags */
        glc.addMouseMotionListener( this );

        /** Register a mouse listener */
        glc.addMouseListener( this );

        /** Register a window listener */
        this.addWindowListener( this );

        /** Initialize the GLComponent */
        glc.initialize();
      }

    /** Initializes the application */
    public void initialize( GLDrawable component ) {
        /** Create a high detail sphere */
        GLUQuadric quadObj = glu_.gluNewQuadric();
        glu_.gluQuadricDrawStyle( quadObj, GLU.GLU_FILL );
        glu_.gluQuadricOrientation( quadObj, GLU.GLU_OUTSIDE );
        glu_.gluQuadricNormals( quadObj, GLU.GLU_SMOOTH );

        gl_.glNewList( HI_RES_SPHERE, GL.GL_COMPILE );
            glu_.gluSphere( quadObj, 1.0f, 32, 32 );
        gl_.glEndList();

        /** Low-detail sphere */
        gl_.glNewList( LO_RES_SPHERE, GL.GL_COMPILE );
            glu_.gluSphere( quadObj, 1.0f, 6, 6 );
        gl_.glEndList();

        glu_.gluDeleteQuadric( quadObj );

        /** Set up some scene basics */
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glEnable( GL.GL_CULL_FACE );
        gl_.glClearColor( 0.4f, 0.4f, 0.4f, 0.0f );
        gl_.glClearDepth( 1.0f );
        gl_.glEnable( GL.GL_NORMALIZE );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Setup the scene lighting */
        float[] mat_specular = { .72f, .8f, .93f, 1.0f };
        float[] mat_diffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] mat_shininess = { 128.0f };

        float[] light_ambient = { 0.1f, 0.1f, 0.1f, 1.0f };
        float[] light_diffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] light_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] light_position = { 1.0f, 1.0f, 1.5f, 0.0f };
        float[] light0_position = { -1.0f, -1.0f, 1.5f, 0.0f };

        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_AMBIENT, light_ambient );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, light_diffuse );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, light_position );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_SPECULAR, light_specular );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, light0_position );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_SHININESS, mat_shininess );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
        gl_.glColorMaterial( GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE );
        gl_.glEnable( GL.GL_LIGHT1 );
        gl_.glEnable( GL.GL_LIGHTING );

        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glTranslatef( 0, 0, -4 );

        ball = new trackball();
        curQuat = ball.buildQuaternion( 0.0f, 0.0f, 0.0f, 0.0f );
      }

    /** Reshapes the viewport */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {

        float aspect;

        /** Reshape the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        aspect = (float)width / (float)height;
        glu_.gluPerspective( 60.0f, aspect, 1.0f, 7.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
      }

    /** Renders the scene */
    public void display( GLDrawable component ) {

        /** Clear the buffers */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        if ( currentMolecule == null ) {
            renderNoMolecule( ((GLComponent)component).getSize().width, 
                              ((GLComponent)component).getSize().height );
          } else {
            renderMolecule( currentMolecule );
          }

        /** Animate with auto-spin if selected */
        if ( glc.isRunning() ) {
            curQuat = ball.addQuats( lastQuat, curQuat );
          }
      }

    /** Renders a molecule */
    private void renderMolecule( molecule mol ) {
        /** Render the current molecule... */
        float[][] m = new float[4][4];

        gl_.glPushMatrix();
            m = ball.buildMatrix( curQuat );
            gl_.glMultMatrixf( m );
            currentMolecule.renderAtoms( sphereVersion );
        gl_.glPopMatrix();
      }

    /** Renders no molecule, but sticks some text up instead */
    private void renderNoMolecule( int width, int height ) {
        gl_.glPushMatrix();
            gl_.glLoadIdentity();
            gl_.glMatrixMode( GL.GL_PROJECTION );
            gl_.glDisable( GL.GL_LIGHTING );
            gl_.glPushMatrix();
                gl_.glLoadIdentity();
                glu_.gluOrtho2D( 0, width,
                                 0, height );
                gl_.glColor3f( 0, 0, 0 );
                gl_.glRasterPos2i( ( ( width - ( 9 * 18 ) ) / 2 ),
                                   ( height - 18 ) / 2 );
                font.drawString( "No Molecule Loaded" );
            gl_.glPopMatrix();
            gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glPopMatrix();
        gl_.glEnable( GL.GL_LIGHTING );
      }

    /** Returns an OpenGL pipeline to the GLEventListener */
    public GL getGL() {
        return gl_;
      }

    /** AWT Event handler */
    public void actionPerformed( ActionEvent evt ) {
        /** Check for ``Exit'' */
        if ( evt.getActionCommand().equals( "Exit" ) ) {
            glc.destroy();
            dispose();
            if ( callExit ) {
                System.exit( 0 );
              }
            return;
          }

        /** Check for ``Open Molecule...'' */
        if ( evt.getActionCommand().equals( "Open Molecule..." ) ) {
            FileDialog fd = 
                new FileDialog( this, "Molecule to Load", 
                                FileDialog.LOAD );
            fd.show();

            /** Should have the filename here....... */
            boolean ok =
                readMolecule( fd.getDirectory() + File.separator +
                              fd.getFile() );
            if ( !ok ) {
                /** @@Do something sensible here.... */
                System.err.println( "Bad molecule file!" );
              } else {
                /** Calculate the molecule extents */
                try {
                    currentMolecule.calculateExtents();
                  } catch ( Exception e ) {
                    currentMolecule = null;
                  }
                glc.repaint();
              }

            return;
          }

        /** Check for ``Wireframe'' */
        if ( evt.getActionCommand().equals( "Wireframe" ) ) {

            /** Make the rendering glc.getContext() current */
            glc.getContext().makeCurrent();

            /** Set the rendering mode to wireframe */
            gl_.glPolygonMode( GL.GL_FRONT_AND_BACK, GL.GL_LINE );

            /** Release the rendering glc.getContext()... */
            glc.getContext().unlock();

            /** ...and redraw the scene */
            glc.repaint();

            return;
          }

        /** Check for ``Solid'' */
        if ( evt.getActionCommand().equals( "Solid" ) ) {

            /** Make the rendering glc.getContext() current */
            glc.getContext().makeCurrent();

            /** Set the rendering mode to wireframe */
            gl_.glPolygonMode( GL.GL_FRONT_AND_BACK, GL.GL_FILL );

            /** Release the rendering glc.getContext()... */
            glc.getContext().unlock();

            /** ...and redraw the scene */
            glc.repaint();

            return;
          }
      }

    /** Mouse Listener */
    public void mouseClicked( MouseEvent evt ) {
      }

    public void mouseEntered( MouseEvent evt ) {
      }

    public void mouseExited( MouseEvent evt ) {
      }

    public void mousePressed( MouseEvent evt ) {
        beginx = evt.getX();
        beginy = evt.getY();

        /** Stop the spinning of the molecules */
        glc.stop();
        moleculeMoved = false;

        /** Do any picking stuff */
        if ( currentMolecule != null ) {
            atomInstance atom = pickScene( currentMolecule, evt.getX(), evt.getY() );

            /** Update the text fields containing the picked atom data */
            if ( atom == null ) {
                atomNameField.setText( "N/A" );
                atomPositionField.setText( "N/A" );
                atomRadiusField.setText( "N/A" );
              } else {
                atomNameField.setText( atom.getName() );
                float[] pos = atom.getPosition();
                atomPositionField.setText( "( " + pos[0] + ", " + pos[1] + ", " + pos[2] + " )" );
                atomRadiusField.setText( atom.getRadius() + "" );
              }
          }
      }

    public void mouseReleased( MouseEvent evt ) {
        if ( spinCheckbox.getState() ) {
            if ( glc.isRunning() == false && moleculeMoved ) {
                glc.start();
              } else {
                sphereVersion = HI_RES_SPHERE;
                glc.repaint();
              }
          } else {
            if ( autoHiResCheckbox.getState() ) {
                sphereVersion = HI_RES_SPHERE;
                glc.repaint();
              }
          }
      }

    /** MouseMotionListener */
    public void mouseDragged( MouseEvent evt ) {
        int width = evt.getComponent().getSize().width;
        int height = evt.getComponent().getSize().height;
        int x = evt.getX();
        int y = evt.getY();

        lastQuat =
            ball.buildQuaternion( (float)( 2.0f * beginx - width ) / (float)width,
                                  (float)( height - 2.0f * beginy ) / (float)height,
                                  (float)( 2.0f * x - width ) / (float)width,
                                  (float)( height - 2.0f * y ) / (float)height );

        if ( false ) {
          } else {
            if ( hiResSpinCheckbox.getState() ) {
                sphereVersion = HI_RES_SPHERE;
              } else {
                sphereVersion = LO_RES_SPHERE;
              }
            curQuat = ball.addQuats( lastQuat, curQuat ); 
            glc.repaint();
          }

        beginx = x;
        beginy = y;

        /** Signify that the molecule has been moved */
        moleculeMoved = true;
      }

    public void mouseMoved( MouseEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Generates the scene into the selection buffer and processes hits */
    private atomInstance pickScene( molecule mol, int x, int y ) {
        int[] selectBuffer = null;
        int[] viewport = new int[4];
        float aspect;
        int hits;

        /** Create the selection buffer */
        if ( mol.getNumAtoms() > 0 ) {
            selectBuffer = new int[4 * mol.getNumAtoms()];
          } else {
            return null;
          }

        /** Make the rendering context current */
        glc.getContext().makeCurrent();

        /** Setup the viewport and selection buffer */
        gl_.glGetIntegerv( GL.GL_VIEWPORT, viewport );
        gl_.glSelectBuffer( selectBuffer.length, selectBuffer );
        gl_.glRenderMode( GL.GL_SELECT );
        gl_.glInitNames();
        gl_.glPushName( -1 );

        /** Redraw the scene in selection mode */
        gl_.glPushMatrix();
            gl_.glMatrixMode( GL.GL_PROJECTION );
            gl_.glPushMatrix();
                gl_.glLoadIdentity();
                glu_.gluPickMatrix( x, viewport[3] - y, 1.0f, 1.0f, viewport );
                aspect = (float)viewport[2] / (float)viewport[3];
                glu_.gluPerspective( 60.0f, aspect, 1.0f, 7.0f );
                gl_.glMatrixMode( GL.GL_MODELVIEW );
                renderMolecule( mol );
                gl_.glMatrixMode( GL.GL_PROJECTION );
            gl_.glPopMatrix();
            gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glPopMatrix();
        hits = gl_.glRenderMode( GL.GL_RENDER );

        /** Unlock the rendering context */
        glc.getContext().unlock();

        return processHits( mol, hits, selectBuffer );
      }

    /** Processes the hit records generated by the selection buffer */
    private atomInstance processHits( molecule mol, int hits,
                                      int[] selectBuffer ) {

        long minz = 0xffffffff;
        long z;
        int index = 0;
        int match = 0; 
        int names = 0;
        atomInstance atom = null;

        if ( hits < 0 ) {
            System.err.println( "WARNING: select buffer overflow!" );
            return null;
          }

        if ( hits == 0 ) {
            return null;
          }

        for ( int i = 0 ; i < hits ; i++ ) {
            names = selectBuffer[index];
            index++;
            z = selectBuffer[index];
            index++;
            index++;
            if ( z <= minz ) {
                minz = z;
                match = selectBuffer[index];
              }
            index += names;
          }
        atom = atomInfo( mol, match );
        return atom;
      }

    /** Returns the correct atom from the molecule */
    private atomInstance atomInfo( molecule mol, int picked ) {
        int name = 0;
        return mol.getAtomInstance( picked );
      }

    /** Reads a molecule in from a datafile */
    private boolean readMolecule( String filename ) {

        boolean fileOK = true;

        FileReader iStream = null;
        try {
            iStream = new FileReader( filename );
          } catch ( FileNotFoundException e ) {
            return false;
          }
        StreamTokenizer st = new StreamTokenizer( iStream );

        /** Values we're going to be reading in from the file */
        String name = null;
        float radius = 0;
        float red = 0,
              green = 0,
              blue = 0;
        float x = 0,
              y = 0,
              z = 0;

        /** Setup the tokenizer */
        st.commentChar( ';' );
        st.commentChar( '#' );
        st.commentChar( '!' );
        st.wordChars( ':', ':' );
        st.wordChars( '=', '=' );
        st.whitespaceChars( ',', ',' );
        st.whitespaceChars( '(', ')' );
        st.eolIsSignificant( false );

        /** Create a new molecule */
        currentMolecule = new molecule();

        try {
            while ( st.nextToken() != StreamTokenizer.TT_EOF ) {
                switch ( st.ttype ) {
                    case StreamTokenizer.TT_WORD: {
                        name = new String( st.sval );
    
                        /** Test for either an equals or a colon */
                        st.nextToken();
                        if ( st.ttype == StreamTokenizer.TT_WORD ) {
                            if ( st.sval.equals( "=" ) ) {
                                /** Looks like an atom type...... */
                                atomType type = new atomType( name );

                                /** Read the radius */
                                st.nextToken();
                                if ( st.ttype != StreamTokenizer.TT_NUMBER ) {
                                    fileOK = false;
                                    break;
                                  }
                                type.setRadius( (float)st.nval );

                                /** Read the colour */
                                st.nextToken();
                                if ( st.ttype != StreamTokenizer.TT_NUMBER ) {
                                    fileOK = false;
                                    break;
                                  }
                                type.setRed( (float)st.nval );

                                st.nextToken();
                                if ( st.ttype != StreamTokenizer.TT_NUMBER ) {
                                    fileOK = false;
                                    break;
                                  }
                                type.setGreen( (float)st.nval );

                                st.nextToken();
                                if ( st.ttype != StreamTokenizer.TT_NUMBER ) {
                                    fileOK = false;
                                    break;
                                  }
                                type.setBlue( (float)st.nval );

                                /** Add the type to the molecule */
                                currentMolecule.addAtomType( type );
                              }
                          } else {
                            if ( name.endsWith( ":" ) ) {
                                String tmpName =
                                    name.substring( 0, name.length() - 1 );

                                /** Create a new instance */
                                atomInstance instance = new atomInstance( tmpName );

                                /** Read the location of this instance */
                                if ( st.ttype != StreamTokenizer.TT_NUMBER ) {
                                     fileOK = false;
                                     break;
                                   }
                                instance.x = (float)st.nval;

                                st.nextToken();
                                if ( st.ttype != StreamTokenizer.TT_NUMBER ) {
                                    fileOK = false;
                                    break;
                                  }
                                instance.y = (float)st.nval;

                                st.nextToken();
                                if ( st.ttype != StreamTokenizer.TT_NUMBER ) {
                                    fileOK = false;
                                    break;
                                  }
                                instance.z = (float)st.nval;

                                /** Add the instance into the molecule */
                                currentMolecule.addAtomInstance( instance );
                              } else {
                                fileOK = false;
                                break;
                              }
                          }
    
                        break;
                      }
                    default: {
                        break;
                      }
                  }
              }
          } catch ( IOException e ) {
            e.printStackTrace();
          }

        return fileOK;
      }
  }
