/**
 * $Id: mtpaperplane.java.stub,v 1.13 1998/10/19 00:53:12 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * $Log: mtpaperplane.java.stub,v $
 * Revision 1.13  1998/10/19 00:53:12  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.12  1998/10/02 11:18:09  descarte
 * Changed GLComponentListener to GLEventListener.
 * Altered reshape() arguments
 *
 * Revision 1.11  1998/04/21 10:55:30  descarte
 * Added main() handling
 * Added WindowListener handling.
 *
 * Revision 1.10  1998/03/30 00:47:47  descarte
 * Commented out debugging.
 *
 * Revision 1.9  1998/03/30 00:41:58  descarte
 * Added applet resource handling.
 *
 * Revision 1.8  1998/03/28 12:36:32  descarte
 * Converted to GLComponentFactory
 * Upgraded to Java 1.1 listener. Dropped AWT 1.0 support.
 *
 * Revision 1.7  1998/03/24 15:16:54  descarte
 * Changed GLEventListener prototypes.
 *
 * Revision 1.6  1998/03/16 10:35:49  descarte
 * Converted from old beta 4 syntax to new beta 4 syntax.
 *
 * Revision 1.5  1998/02/24 17:30:57  descarte
 * Removed some dead code.
 * Changed the GLComponent constructor to the default one and added
 * setContinuousRefresh() call instead.
 *
 * Revision 1.4  1998/01/07 20:45:43  descarte
 * Removed errant closing brace.
 *
 * Revision 1.3  1998/01/07 20:35:16  descarte
 * Added applet stopping and starting code. This is *very* flaky.
 *
 * Revision 1.2  1998/01/01 16:17:10  descarte
 * Added GLComponent argument to all GLEventListener interface methods.
 *
 * Revision 1.1  1998/01/01 15:01:18  descarte
 * Initial revision
 *
 * Revision 1.7  1997/09/02 22:24:43  descarte
 * Added GLContextException traps.
 *
 * Revision 1.6  1997/09/01 22:22:35  descarte
 * Added synchronization lockObject.
 *
 * Revision 1.5  1997/08/31 21:23:58  descarte
 * Lightened thread priority to MIN_PRIORITY instead of NORM_PRIORITY. This
 * seems to have cured the race condition within the native code.
 *
 * Revision 1.4  1997/08/31 11:03:35  descarte
 * Added 'destroyContext()' call.
 *
 * Revision 1.3  1997/08/25 21:29:37  descarte
 * Altered demo into demoTemplate structure.
 *
 * Revision 1.2  1997/08/24 23:54:23  descarte
 * Added demoTemplate versioning information.
 *
 * Revision 1.1  1997/08/04 19:14:21  descarte
 * Initial revision
 *
 * Revision 1.1  1997/07/17 21:50:30  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class mtpaperplaneapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    private static final int MAXPLANES = 20;
    private int currentPlane = 0;

    /** Width of the viewport */
    private static final int WIDTH = 200;

    /** Height of the viewport */
    private static final int HEIGHT = 200;

    /** The OpenGL state machine */
    GL gl_ = null;

    /** The CoreGL Component */
    GLComponent glc = null;

    /** Frame rate measurer */
    FrameRateComponent frc = new FrameRateComponent();

    /** The array of planes */
    plane[] planes = null;

    private CoreGL coregl_ = new CoreGL();
    private TraceGL tracer_ = new TraceGL();
    private ProfileGL profiler_ = new ProfileGL();

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Paperplanes" );
      }

    /** Returns the version information for this demonstration */
    public String getVersion() {
        return new String( "$Id: mtpaperplane.java.stub,v 1.13 1998/10/19 00:53:12 descarte Exp descarte $" );
      }

    /** Returns the description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration spins three paperplanes around the sky in\n" );
        sb.append( "figures of 8. The height of the planes alters over time, and\n" );
        sb.append( "each plane has a random colour and speed. This demonstration\n" );
        sb.append( "is multi-threaded, with the animation being driven by a separate\n" );
        sb.append( "thread. This does not suffer slow down, unlike it's single-\n" );
        sb.append( "threaded counterpart demo.\n\n" );
        sb.append( "To destroy this demo at any point, hit the ``Escape'' key\n" );
        sb.append( "within the window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mark J. Kilgard <mjk@sgi.com>" );
      }

    public void startDemo() {
        /** 
         * Setup the layout manager. BorderLayout ensures that
         * resizing the Frame will resize the GLComponent
         */
        setLayout( new BorderLayout() );

        /** Set the rendering pipelines up */
        tracer_.setParent( coregl_ );
        profiler_.setParent( coregl_ );
        profiler_.setMode( ProfileGL.SUMMARY );
        gl_ = (GL)coregl_;

        /** Create the CoreGL component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add it to the Frame */
        add( "Center", glc );
        /**#layout */

        /** Setup the capabilities for the context */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Set the frame rate to display averages only */
        frc.setSummaryOnly( true );

        /** Register the GLEventListener */
        glc.addGLEventListener( this );

        /** Register the keyboard listener with the GLComponent */
        glc.addKeyListener( this );

        /** Register a window listener */
        /**#windowListener */

        /** Startup the component processing events */
        glc.initialize();
        glc.start();
      }

    public void initialize( GLDrawable component ) {
        /** Set the z-buffer clear depth */
        gl_.glClearDepth( 1.0 );

        /** Set the z-buffer clear colour to black */
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );

        /** Create the array of planes */
        planes = new plane[MAXPLANES];

        /** Now, add three planes to begin with */
        addPlane();
        addPlane();
        addPlane();
      }

    public void reshape( GLDrawable component, int x, int y, 
                         int width, int height ) {
        /** Define the view frustum */
        gl_.glViewport( glc, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0, 1.0, -1.0, 1.0, 1.0, 20 );

        /** Get ready to specify some objects in the scene! */
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Adds a plane to the scene */
    private void addPlane() {

        planes[currentPlane] = new plane();
        double randomNumber2 = Math.random();
        randomNumber2 *= 10;
        int randomNumber = (int)( randomNumber2 % 6 );
        switch ( randomNumber ) {
            case 0: {
                planes[currentPlane].setColour( 1.0f, 0.0f, 0.0f );
                break;
              }
            case 1: {
                planes[currentPlane].setColour( 1.0f, 1.0f, 1.0f );
                break;
              }
            case 2: {
                planes[currentPlane].setColour( 0.0f, 1.0f, 0.0f );
                break;
              }
            case 3: {
                planes[currentPlane].setColour( 1.0f, 0.0f, 1.0f );
                break;
              }
            case 4: {
                planes[currentPlane].setColour( 1.0f, 1.0f, 0.0f );
                break;
              }
            case 5: {
                planes[currentPlane].setColour( 0.0f, 1.0f, 1.0f );
                break;
              }
          }
    
        /** Set the speed of each plane now */
        planes[currentPlane].setSpeed( (float)( ( Math.random() * 20 ) * 0.001 + 0.02 ) );
        if ( ( (int)( Math.random() * 10 ) & 0x01 ) == 1 ) {
            float speed = planes[currentPlane].getSpeed();
            planes[currentPlane].setSpeed( (float)( speed *= -1 ) );
          }
    
        /** Set the angle of the plane */
        planes[currentPlane].setTheta( (float)( ( Math.random() * 256 ) * 0.1111 ) );
    
        /** Update the planes location and orientation */
        planes[currentPlane].tick();
    
        currentPlane++;
      }

    /** Display callback */
    public void display( GLDrawable component ) {

        /** Move all the planes */
        for ( int i = 0 ; i < currentPlane ; i++ ) {
            planes[i].tick();
          }
    
        /** Start timing the rendering */
        frc.startSample();

        /** Clear the z-buffer */
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT );

        /** And disable depth-testing just now */
        gl_.glDisable( GL.GL_DEPTH_TEST );

        /**
         * Make sure we're smooth-shading just now. We want to
         * do this since we want to blend the colours of the
         * background polygon 
         */
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Draw the graded black -> blue polygon first */
        gl_.glBegin( GL.GL_POLYGON );
            gl_.glColor3f( 0f, 0f, 0f );
            gl_.glVertex3f( -20f, 20f, -19f );
            gl_.glVertex3f( 20f, 20f, -19f );
            gl_.glColor3f( 0f, 0f, 1f );
            gl_.glVertex3f( 20f, -20f, -19f );
            gl_.glVertex3f( -20f, -20f, -19f );
        gl_.glEnd();

        /** Enable depth-testing... */
        gl_.glEnable( GL.GL_DEPTH_TEST );
    
        /** And now flat-shade stuff */
        gl_.glShadeModel( GL.GL_FLAT );
    
        /** Draw each plane......... */
        for ( int i = 0 ; i < currentPlane ; i++ ) {
            gl_.glPushMatrix();
            gl_.glTranslatef( planes[i].getX(), 
                             planes[i].getY(), 
                             planes[i].getZ() );
            gl_.glRotatef( (float)290.0, 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( planes[i].getAngle(), 0.0f, 0.0f, 1.0f );
            gl_.glScalef( (float)( 1.0 / 3.0 ), 
                         (float)( 1.0 / 4.0 ), 
                         (float)( 1.0 / 4.0 ) );
            gl_.glTranslatef( 0.0f, -4.0f, -1.5f );

            gl_.glBegin( GL.GL_TRIANGLE_STRIP );
                gl_.glVertex3f(-7.0f, 0.0f, 2.0f);
                gl_.glVertex3f(-1.0f, 0.0f, 3.0f);

                float red = planes[i].getRed();
                float green = planes[i].getGreen();
                float blue = planes[i].getBlue();

                gl_.glColor3f( (float)red, (float)green, (float)blue );
                gl_.glVertex3f(-1.0f, 7.0f, 3.0f);

                /** Left side */
                gl_.glColor3f( (float)( 0.6 * red ), 
                              (float)( 0.6 * green ), 
                              (float)( 0.6 * blue ) );
                gl_.glVertex3f(0.0f, 0.0f, 0.0f);
                gl_.glVertex3f(0.0f, 8.0f, 0.0f);

                /** Right side */
                gl_.glVertex3f(1.0f, 0.0f, 3.0f);
                gl_.glVertex3f(1.0f, 7.0f, 3.0f);

                /** Final tip of right wing */
                gl_.glColor3f( (float)red, (float)green, (float)blue);
                gl_.glVertex3f(7.0f, 0.0f, 2.0f);
            gl_.glEnd();
            gl_.glPopMatrix();
          }

        /** Stop timing the rendering pipeline */
        frc.stopSample();
      }

    /** Returns a valid GL pipeline to GLEventListener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** If we're profiling, dump the totals...... */
            if ( gl_ instanceof ProfileGL ) {
                ((ProfileGL)gl_).displayStatistics();
              }

            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
