/**
 * $Id: multiView.java.stub,v 1.11 1998/10/31 11:14:40 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * This is a version of 'movelight' from the Red Book examples, but uses 
 * multiple GLComponents to which a single GLContext is rendered onto
 * in separate threads.
 *
 * $Log: multiView.java.stub,v $
 * Revision 1.11  1998/10/31 11:14:40  descarte
 * Split the display list IDs for each component.
 *
 * Revision 1.10  1998/10/19 00:53:13  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.9  1998/10/02 11:18:21  descarte
 * Changed GLComponentListener to GLEventListener.
 * Altered reshape() arguments
 *
 * Revision 1.8  1998/03/30 00:48:00  descarte
 * Added applet resource handling.
 *
 * Revision 1.7  1998/03/26 00:46:51  descarte
 * Removed GLComponent abstract class hack.
 *
 * Revision 1.6  1998/03/24 15:21:13  descarte
 * Changed component listener method prototypes to new GLDrawable syntax.
 *
 * Revision 1.5  1998/03/19 21:53:43  descarte
 * Added getGL() method.
 *
 * Revision 1.4  1998/03/16 10:36:12  descarte
 * Converted from old beta 4 syntax to new beta 4 syntax.
 *
 * Revision 1.3  1998/02/23 17:44:13  descarte
 * Converted across to new GLComponent structure.
 *
 * Revision 1.2  1998/01/07 20:35:34  descarte
 * Added applet stopping and starting code. This is *very* flaky.
 *
 * Revision 1.1  1998/01/01 16:12:32  descarte
 * Initial revision
 *
 * Revision 1.1  1997/09/08 14:13:41  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class multiViewapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private GL gl_ = null;
    private GLU glu_ = null;

    private CoreGL coregl_ = new CoreGL();
    private CoreGLU coreglu_ = new CoreGLU();

    private TraceGL tracegl_ = new TraceGL( coregl_ );
    private TraceGLU traceglu_ = new TraceGLU( coreglu_ );

    /** The OpenGL components to render onto */
    private GLComponent frontComponent = null;
    private GLComponent planComponent = null;
    private GLComponent sideComponent = null;

    /** The dimensions of the components */
    private static final int WIDTH = 200,
                             HEIGHT = 200;

    /** The amount of spin */
    private int spin = 0;

    /** Indicates whether the thread is paused or not */
    private boolean appletThreadStopped = false;

    /** The display list ID for the torus */
    private int frontTorusList = -1,
                planTorusList = -1,
                sideTorusList = -1;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
        gl_ = coregl_;
        glu_ = coreglu_;
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Multiple Views" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: multiView.java.stub,v 1.11 1998/10/31 11:14:40 descarte Exp descarte $" );
      }

    /** Returns a description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of Magician's ability\n" );
        sb.append( "to easily render multiple OpenGL rendering contexts onto\n" );
        sb.append( "multiple GLComponents. In this case, we are rendering a\n" );
        sb.append( "torus with a mobile light source, driven by a standard\n" );
        sb.append( "Java Thread object. There are 3 separate GLComponents in\n" );
        sb.append( "the frame, each rendering the scene from a different angle.\n\n" );
        sb.append( "The position of the light is signified by the wireframe\n" );
        sb.append( "cube, rendered with the CoreGLU method wireCube().\n\n" );
        sb.append( "A similar example, on which this demo is based, can be\n" );
        sb.append( "found in the ``Red Book'' demos, entitled ``Moving Light''.\n\n" );
        sb.append( "Hit the ``Escape'' key to quit this demo." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH * 3;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Alligator Descartes <descarte@arcana.co.uk>" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new GridBagLayout() );
        setBackground( Color.black );

        /** Create the OpenGL Components */
        frontComponent = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        planComponent = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        sideComponent = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        constrain.constrain( this, frontComponent, 0, 0, 1, 1,
                             GridBagConstraints.BOTH,
                             GridBagConstraints.CENTER,
                             1.0f, 1.0f, 0, 0, 0, 0 );
        constrain.constrain( this, planComponent, 1, 0, 1, 1,
                             GridBagConstraints.BOTH,
                             GridBagConstraints.CENTER,
                             1.0f, 1.0f, 0, 0, 0, 0 );
        constrain.constrain( this, sideComponent, 2, 0, 1, 1,
                             GridBagConstraints.BOTH,
                             GridBagConstraints.CENTER,
                             1.0f, 1.0f, 0, 0, 0, 0 );

        /** Some information labels */
        Label frontLabel = new Label( "Front View" );
        frontLabel.setForeground( Color.white );
        frontLabel.setBackground( Color.black );

        Label planLabel = new Label( "Plan View" );
        planLabel.setForeground( Color.white );
        planLabel.setBackground( Color.black );

        Label sideLabel = new Label( "Side View" );
        sideLabel.setForeground( Color.white );
        sideLabel.setBackground( Color.black );

        /** Add the labels */
        constrain.constrain( this, frontLabel, 0, 1, 1, 1,
                             GridBagConstraints.NONE,
                             GridBagConstraints.CENTER,
                             0, 0, 0, 0, 0, 0 );
        constrain.constrain( this, planLabel, 1, 1, 1, 1,
                             GridBagConstraints.NONE,
                             GridBagConstraints.CENTER,
                             0, 0, 0, 0, 0, 0 );
        constrain.constrain( this, sideLabel, 2, 1, 1, 1,
                             GridBagConstraints.NONE,
                             GridBagConstraints.CENTER,
                             0, 0, 0, 0, 0, 0 );

        /** Layout the GUI */
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities frontCap = frontComponent.getContext().getCapabilities();
        frontCap.setDepthBits( 12 );
        frontCap.setPixelType( GLCapabilities.RGBA );
        frontCap.setColourBits( 24 );
        frontCap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** The plan view component capabilities */
        GLCapabilities planCap = planComponent.getContext().getCapabilities();
        planCap.setDepthBits( 12 );
        planCap.setPixelType( GLCapabilities.RGBA );
        planCap.setColourBits( 24 );
        planCap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** The side-view */
        GLCapabilities sideCap = sideComponent.getContext().getCapabilities();
        sideCap.setDepthBits( 12 );
        sideCap.setPixelType( GLCapabilities.RGBA );
        sideCap.setColourBits( 24 );
        sideCap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListeners */
        frontComponent.addGLEventListener( this );
        sideComponent.addGLEventListener( this );
        planComponent.addGLEventListener( this );

        /** Register the key listener */
        frontComponent.addKeyListener( this );
        sideComponent.addKeyListener( this );
        planComponent.addKeyListener( this );

        /**#windowListener */

        /** Startup each component */
        frontComponent.initialize();
        frontComponent.start();

        sideComponent.initialize();
        sideComponent.start();

        planComponent.initialize();
        planComponent.start();
      }

    /** Standard initialization entry point */
    public void initialize( GLDrawable component ) {
        /** Initialize the contexts */
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
        gl_.glShadeModel( GL.GL_SMOOTH );

        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_DEPTH_TEST );

        /** Create the torus display list */
        if ( component == frontComponent ) {
            gl_.glNewList( frontTorusList, GL.GL_COMPILE );
                shapes.solidTorus( 0.275, 0.85, 8, 15 );
          } else {
            if ( component == planComponent ) {
                gl_.glNewList( planTorusList, GL.GL_COMPILE );
                    shapes.solidTorus( 0.275, 0.85, 8, 15 );
              } else {
                gl_.glNewList( sideTorusList, GL.GL_COMPILE );
                    shapes.solidTorus( 0.275, 0.85, 8, 15 );
              }
          }

        gl_.glEndList();
      }

    /** Handles GLComponent resizing */
    public void reshape( GLDrawable component, int x, int y,
                         int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 40.0, (float)width / (float)height, 1.0, 20.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glTranslatef( 0.0f, 0.0f, -5.0f );
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {
        /** Do the OpenGL stuff */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        /** Rotate depending on the component */
        if ( component == sideComponent ) {
            gl_.glPushMatrix();
                gl_.glRotatef( 90.0f, 0.0f, 1.0f, 0.0f );
                drawScene( frontTorusList );
            gl_.glPopMatrix();
          } else {
            if ( component == planComponent ) {
                gl_.glPushMatrix();
                    gl_.glRotatef( 90.0f, 1.0f, 0.0f, 0.0f );
                    drawScene( planTorusList );
                gl_.glPopMatrix();
              } else {
                drawScene( sideTorusList );
              }
          }

        /** Increment the spin factor */
        spin++;
      }

    /** Returns a valid GL pipeline to the GLEventListener */
    public GL getGL() {
        return gl_;
      }

    /** Renders the torus and light */
    private void drawScene( int dispList ) {
        /** Position of the light */
        float[] position = { 0.0f, 0.0f, 1.5f, 1.0f };

        /** Render the scene */
        gl_.glPushMatrix();
            gl_.glPushMatrix();
                  gl_.glRotated( (double)spin, 1.0, 0.0, 0.0 );
                  gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position );
         
                  gl_.glTranslated( 0.0, 0.0, 1.5 );
                  gl_.glDisable( GL.GL_LIGHTING );
                  gl_.glColor3f( 0.0f, 1.0f, 1.0f );
                  shapes.wireCube( 0.1 );
                  gl_.glEnable( GL.GL_LIGHTING );
             gl_.glPopMatrix();

             gl_.glCallList( dispList );
        gl_.glPopMatrix();
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        System.err.println( "keyPressed: " + evt );
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            frontComponent.destroy();
            planComponent.destroy();
            sideComponent.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        frontComponent.destroy();
        sideComponent.destroy();
        planComponent.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( frontComponent.isRunning() ) {
            frontComponent.resume();
          }
        if ( sideComponent.isRunning() ) {
            sideComponent.resume();
          }
        if ( planComponent.isRunning() ) {
            planComponent.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( frontComponent.isRunning() ) {
            frontComponent.suspend();
          }
        if ( sideComponent.isRunning() ) {
            sideComponent.suspend();
          }
        if ( planComponent.isRunning() ) {
            planComponent.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( frontComponent.isRunning() ) {
            frontComponent.suspend();
          }
        if ( planComponent.isRunning() ) {
            planComponent.suspend();
          }
        if ( sideComponent.isRunning() ) {
            sideComponent.suspend();
          }
      }

    /** Applet only */
    public void start() {
        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( frontComponent.isRunning() ) {
            frontComponent.resume();
          }
        if ( planComponent.isRunning() ) {
            planComponent.resume();
          }
        if ( sideComponent.isRunning() ) {
            sideComponent.resume();
          }
      }

    /** Applet only */
    public void destroy() {
        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        frontComponent.destroy();
        planComponent.destroy();
        sideComponent.destroy();
      }
  }
