/**
 * $Id: objViewer.java.stub,v 1.6 1998/10/19 00:53:14 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Test program that demonstrates loading in *.obj files and wiggling them
 * around
 *
 * $Log: objViewer.java.stub,v $
 * Revision 1.6  1998/10/19 00:53:14  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.5  1998/10/18 22:01:06  descarte
 * Removed deprecated API problems.
 *
 * Revision 1.4  1998/10/02 11:33:20  descarte
 * Changed GLComponentListener to GLEventListener
 * Altered arguments to reshape() method
 *
 * Revision 1.3  1998/06/01 11:30:43  descarte
 * Temporarily removed the amphora model. It was pretty crap.
 *
 * Revision 1.2  1998/05/28 16:38:21  descarte
 * Added explicit context locking and unlocking in actionPerformed() callback.
 * This was making new object loads fail under Win32.
 *
 * Revision 1.1  1998/05/14 20:00:41  descarte
 * Initial revision
 *
 * Revision 1.1  1997/09/02 22:29:25  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;
import com.hermetica.vecmath.*;

public class objViewerapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener, ActionListener, MouseListener, MouseMotionListener {

    /** OpenGL pipelines */
    private GL gl_ = null;
    private CoreGL coregl_ = new CoreGL();

    /** The dimensions of the GLComponent */
    private final int WIDTH = 200,
                      HEIGHT = 200;

    /** The OpenGL-aware AWT component we'll render into */
    private GLComponent glc = null;

    /** The arrays to contain the vertices, normals and colours */
    private static final int MAX_VERTICES = 30000;
    private float[][] vertices = new float[MAX_VERTICES][3];
    private float[][] normals = new float[MAX_VERTICES][3];
    private float[][] colours = new float[MAX_VERTICES][3];

    private int numVertices = 0;

    /** 
     * Location of the camera. This is originally based on the location and
     * size of the bounding box
     */
    private float xloc = 0,
                  yloc = 0,
                  zloc = 0;

    /** Motion increments. These are based on the bounding box of the object */
    private float xinc = 1,
                  yinc = 1,
                  zinc = 1;

    /** Amount of 'segments' the bounding box is split into */
    private static final int MOTIONDIV = 20;

    /** The bounding box size and centre */
    private float[] bboxcentre = new float[3];
    private float[] bboxsize = new float[3];
    private float bboxDiagonalLength = 200;

    /** The current quaternion expressing the rotation of the object */
    private float[] curQuat = new float[4];

    /** The trackball used to spin the object about */
    private trackball ball = new trackball();

    /** Mouse tracking variables used to calculate the rotation of the object */
    private int beginx = 0,
                beginy = 0;

    /** An array of 10 display lists that can be used */
    private int objectListBase = -1;
    private int listModifier = 0;

    /** Arrays containing names and URLs of loadable objects */
    private String[] objectNames = {
        "Volkswagen Beetle",
/*        "Amphora", */
        "Jet",
        "TIE Fighter"
      };

    private String[] objectURLs = {
        "http://www.arcana.co.uk/products/magician/demos/tri/vw.tri",
/*        "http://www.arcana.co.uk/products/magician/demos/tri/amphora.tri", */
        "http://www.arcana.co.uk/products/magician/demos/tri/acjet.tri",
        "http://www.arcana.co.uk/products/magician/demos/tri/tiefighter.tri"
      };

    private float[][] objectLocations = new float[10][4];

    private boolean[] objectsGenerated = {
        false, false, false, false, false, false, false, false, false, false
      };

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        gl_ = coregl_;
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Triangle File Object Viewer" );
      }

    public String getVersion() {
        return new String( "$Id: objViewer.java.stub,v 1.6 1998/10/19 00:53:14 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration allows you to load various triangle\n" );
        sb.append( "meshes over the Internet via the World Wide Web and spin\n" );
        sb.append( "them about in the window by clicking and dragging the mouse.\n" );
        sb.append( "If you do not have access to the Internet, you can change the\n" );
        sb.append( "demo program's source code to load the files from your hard disk.\n" );
        sb.append( "You can quit this demo by clicking on the window close button\n" );
        sb.append( "or hitting the ``Escape'' key when the mouse is on the viewer.\n" );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }
    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Alligator Descartes <descarte@arcana.co.uk>" );
      }

    public void startDemo() {
        /** Set the layout manager for this applet */
        setLayout( new GridBagLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        constrain.constrain( this, glc, 0, 0, 1, 1,
                             GridBagConstraints.BOTH,
                             GridBagConstraints.CENTER,
                             1, 1, 0, 0, 0, 0 );

        /** Add a listbox to stick URLs into */
        java.awt.List objectList = new List( 10 );
        for ( int i = 0 ; i < objectNames.length ; i++ ) {
            objectList.addItem( objectNames[i] );
          }
        objectList.addActionListener( this );
        constrain.constrain( this, objectList, 1, 0, 1, 1 );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the key listener to the component */
        glc.addKeyListener( this );

        /** Add the window listener */
        /**#windowListener */

        /** Add the mouse and motion listeners */
        glc.addMouseListener( this );
        glc.addMouseMotionListener( this );

        /** Add the listener and initialize the component */
        glc.addGLEventListener( this );
        glc.initialize();
      }

    /** Initializes stuff */
    public void initialize( GLDrawable component ) {
        gl_.glClearColor( 0.0f, 0.0f, 1.0f, 1.0f );
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glDisable( GL.GL_CULL_FACE );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Setup the lighting */
        float lightPosition[] = { 3f, 3f, 0f, 1.0f };
        float lightColour[] = { 1f, 1f, 1f, 1f };

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightPosition );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, lightColour );

        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_LIGHTING );

        /** Reserve some display lists */
        objectListBase = gl_.glGenLists( 10 );

        /** Load up the first model in the lists */
        loadModel( 0 );

        /** Setup the trackball */
        curQuat = ball.buildQuaternion( 0.0f, 0.0f, 0.0f, 0.0f );
      }


    /** Handles reshaping the viewport */
    public void reshape( GLDrawable component, int x, int y, 
                         int width, int height ) {            
        gl_.glViewport( component, x, y, 200, 200 );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0, 1.0, -1.0, 1.0, 1.0, bboxDiagonalLength );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Renders the model */
    public void display( GLDrawable component ) {
        /** Matrix expressing the quaternion rotation of the object */
        float[][] m = new float[4][4];

        /** Clear the frame and depth buffers */
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        /** Render the object */
        gl_.glPushMatrix();
            gl_.glTranslatef( xloc, yloc, zloc );
//            System.err.println( "Camera at: " + xloc + ", " + yloc + ", " + zloc );

//            gl_.glPushMatrix();
                m = ball.buildMatrix( curQuat );
                gl_.glMultMatrixf( m );
                gl_.glCallList( objectListBase + listModifier );
//            gl_.glPopMatrix();
        gl_.glPopMatrix();
      }

    /** Returns an OpenGL pipeline to the GLEventListener */
    public GL getGL() {
        return gl_;
      }

    /** Loads a model */
    private int loadModel( int index ) {
        int numVertices = -1;

        System.err.println( "Loading " + objectNames[index] );

        if ( objectsGenerated[index] == false ) {
            System.err.println( "\tlist is not already generated..." );

            /** Read some triangle data in */
            try {
                numVertices =
                    triReader.readTriFilef( new URL( objectURLs[index] ), 
                                            vertices, normals, colours );
              } catch ( Exception e ) {
                System.err.println( "Exception caught in readTriFilef: " +
                                    e.toString() );
                e.printStackTrace();
                return -1;
              }
            System.err.println( "Read " + numVertices + " vertices from file" );
    
            /** Build a display list for this object */
            gl_.glNewList( objectListBase + index, GL.GL_COMPILE );
                for ( int i = 0 ; i < numVertices ; i+=3 ) {
                    gl_.glBegin( GL.GL_TRIANGLES );
                        gl_.glNormal3fv( normals[i] );
                        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, colours[i] );
                        gl_.glVertex3fv( vertices[i] );
                        gl_.glVertex3fv( vertices[i + 1] );
                        gl_.glVertex3fv( vertices[i + 2] );
                    gl_.glEnd();
                  }
            gl_.glEndList();

            /** Indicate that the list is now generated */
            objectsGenerated[index] = true;

            /** Calculate the bounding box of the model */
            float maxx = 0, maxy = 0, maxz = 0;
            float minx = 0, miny = 0, minz = 0;
            for ( int i = 0 ; i < numVertices ; i++ ) {
                if ( vertices[i][0] < minx ) minx = vertices[i][0];
                if ( vertices[i][0] > maxx ) maxx = vertices[i][0];
                if ( vertices[i][1] < miny ) miny = vertices[i][1];
                if ( vertices[i][1] > maxy ) maxy = vertices[i][1];
                if ( vertices[i][2] < minz ) minz = vertices[i][2];
                if ( vertices[i][2] > maxz ) maxz = vertices[i][2];
              }
            System.err.println( "Bounding box: ( " + 
                                minx + ", " + miny + ", " + minz + " ) -> ( " +
                                maxx + ", " + maxy + ", " + maxz + " )" );
    
            bboxcentre[0] = ( maxx - ( maxx - minx ) / 2 );
            bboxcentre[1] = ( maxy - ( maxy - miny ) / 2 );
            bboxcentre[2] = ( maxz - ( maxz - minz ) / 2 );
    
            System.err.println( "BoundingBox centre: ( " +
                                bboxcentre[0] + ", " +
                                bboxcentre[1] + ", " +
                                bboxcentre[2] + " )" );
    
            bboxsize[0] = bboxcentre[0] * 2;
            bboxsize[1] = bboxcentre[1] * 2;
            bboxsize[2] = bboxcentre[2] * 2;
    
            System.err.println( "BoundingBox size: ( " +
                                bboxsize[0] + ", " +
                                bboxsize[1] + ", " +
                                bboxsize[2] + " )" );
    
            xloc = -bboxcentre[0];
            yloc = -bboxcentre[1];

            /** 
             * The z distance should be the diagonal distance between a corner
             * and the centre
             */
            Vec3f z1 = new Vec3f( minx, miny, minz );
            Vec3f z2 = new Vec3f( bboxcentre[0], bboxcentre[1], bboxcentre[2] );
            z2.add( z1 );
            zloc = -z2.length();
            bboxDiagonalLength = Math.abs( zloc ) * 2;

            objectLocations[index][0] = xloc;
            objectLocations[index][1] = yloc;
            objectLocations[index][2] = zloc;
            objectLocations[index][3] = bboxDiagonalLength;
          } else {
            xloc = objectLocations[index][0];
            yloc = objectLocations[index][1];
            zloc = objectLocations[index][2];
            bboxDiagonalLength = objectLocations[index][3];
          }
    
        /** Update the list modifier to point at the new object */
        listModifier = index;

        return numVertices;
      }

    /** Handles action events */
    public void actionPerformed( ActionEvent evt ) {
        if ( evt.getSource() instanceof java.awt.List ) {
            /** Make a context current */
            glc.getContext().makeCurrent();

            /** Load the selected model */
            loadModel( ((java.awt.List)evt.getSource()).getSelectedIndex() );

            /** Reshape */
            reshape( glc, 0, 0, glc.getSize().width, glc.getSize().height );

            /** Unlock the context */
            glc.getContext().unlock();

            /** And repaint */
            glc.repaint();
          }
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** MouseListener methods */
    /** Mouse Listener */
    public void mouseClicked( MouseEvent evt ) {
      }

    public void mouseEntered( MouseEvent evt ) {
      }

    public void mouseExited( MouseEvent evt ) {
      }

    public void mousePressed( MouseEvent evt ) {
        beginx = evt.getX();
        beginy = evt.getY();
      }

    public void mouseReleased( MouseEvent evt ) {
      }

    /** MouseMotionListener methods */
    public void mouseDragged( MouseEvent evt ) {
        int width = evt.getComponent().getSize().width;
        int height = evt.getComponent().getSize().height;
        int x = evt.getX();
        int y = evt.getY();

        float[] lastQuat =
            ball.buildQuaternion( (float)( 2.0f * beginx - width ) / (float)width,
                                  (float)( height - 2.0f * beginy ) / (float)height,
                                  (float)( 2.0f * x - width ) / (float)width,
                                  (float)( height - 2.0f * y ) / (float)height );
        curQuat = ball.addQuats( lastQuat, curQuat ); 

        ((GLComponent)evt.getComponent()).repaint();

        beginx = x;
        beginy = y;
      }

    public void mouseMoved( MouseEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
    
    /** Handles events */
/*    public boolean handleEvent( Event evt ) {
        switch ( evt.id ) {
            case Event.KEY_PRESS: {
                switch ( evt.key ) {
                    case 27: {
                        System.exit( 0 );
                        return true;
                      }
                    case 'h': {
                        xloc += xinc;
                        display();
                        return true;
                      }
                    case 'l': {
                        xloc -= xinc;
                        display();
                        return true;
                      }
                    case 'j': {
                        zloc += zinc;
                        display();
                        return true;
                      }
                    case 'k': {
                        zloc -= zinc;
                        display();
                        return true;
                      }
                    case 'u': {
                        yloc += yinc;
                        display();
                        return true;
                      }

                    case 'i': {
                        yloc -= yinc;
                        display();
                        return true;
                      }
                  }
              }
          }
        return false;
      }
      */
  }
