/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: aargb.java,v 1.2 1997/09/09 14:18:51 descarte Exp descarte $
 *
 * Copyright (c)1997 Alligator Descartes <descarte@hermetica.com>
 *
 * 'aargb' from the Red Book
 *
 * $Log: aargb.java,v $
 * Revision 1.2  1997/09/09 14:18:51  descarte
 * Added copyright notice.
 *
 * Revision 1.1  1997/08/16 09:34:37  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.awt.*;
import com.hermetica.magician.*;

public class aargb extends Frame {

    /** Version information */
    public static final String VERSION = new String( "$Id: aargb.java,v 1.2 1997/09/09 14:18:51 descarte Exp descarte $" );

    /** The OpenGL state machine */
    GL gl_ = new GL();
    private GLU glu_ = new GLU();

    /** The OpenGL rendering context */
    private GLContext context = new GLContext();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 200,
                             HEIGHT = 200;

    /** Angle of rotation of the two lines */
    private float rotAngle = 0;

    public aargb() {

        /** Set the layout of the frame */
        setLayout( new FlowLayout() );

        glc = new GLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( glc );
        pack();
        show();

        /** Setup the context capabilities */
        GLCapabilities cap = context.getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setRedBits( 1 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Assign the context to the component */
        glc.setContext( context );

        /** Initialize the application */
        initialize();
      }

    /** Initialization stuff */
    private void initialize() {
        try {

            /** Make the context current */
            context.makeCurrent( glc );

            /** Get some information on LINE_WIDTH_GRANULARITY */
            float[] values = new float[2];
            gl_.glGetFloatv( GL.GL_LINE_WIDTH_GRANULARITY, values );
            System.err.println( "GL_LINE_WIDTH_GRANULARITY value is " +
                                values[0] );
            gl_.glGetFloatv( GL.GL_LINE_WIDTH_RANGE, values );
            System.err.println( "GL_LINE_WIDTH_RANGE values are: " +
                                values[0] + ", " + values[1] );

            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_FLAT );

            /** Setup the viewport */
            gl_.glViewport( 0, 0, WIDTH, HEIGHT );
            gl_.glMatrixMode( GL.GL_PROJECTION );
            gl_.glLoadIdentity();
            glu_.gluOrtho2D( -1.0, 1.0, -1.0 * ( HEIGHT / WIDTH ), 
                            1.0 * ( HEIGHT / WIDTH ) );

            gl_.glEnable( GL.GL_LINE_SMOOTH );
            gl_.glEnable( GL.GL_BLEND );
            gl_.glBlendFunc( GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA );
            gl_.glHint( GL.GL_LINE_SMOOTH_HINT, GL.GL_NICEST );
            gl_.glLineWidth( 1.5f );

            /** Render the scene */
            display();

            /** Render the screen */
            context.swapBuffers( glc );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          } catch ( GLContextException e2 ) {
            e2.printStackTrace();
          }
      }

    /** Draws a single line */
    private void display() {
        try {
            /** Render two diagonal lines */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT );

            gl_.glColor3f( 0.0f, 1.0f, 0.0f );
            gl_.glPushMatrix();
            gl_.glRotatef( -rotAngle, 0.0f, 0.0f, 0.1f );
            gl_.glBegin( GL.GL_LINES );
                gl_.glVertex2f( -0.5f, 0.5f );
                gl_.glVertex2f( 0.5f, -0.5f );
            gl_.glEnd();
            gl_.glPopMatrix();

            gl_.glColor3f( 0.0f, 0.0f, 1.0f );
            gl_.glPushMatrix();
            gl_.glRotatef( rotAngle, 0.0f, 0.0f, 0.1f );
            gl_.glBegin( GL.GL_LINES );
                gl_.glVertex2f( 0.5f, 0.5f );
                gl_.glVertex2f( -0.5f, -0.5f );
            gl_.glEnd();
            gl_.glPopMatrix();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          } catch ( GLContextException e2 ) {
            e2.printStackTrace();
          }
      }

    /** AWT event handler */
    public boolean handleEvent( Event evt ) {
        switch ( evt.id ) {
            case Event.KEY_PRESS:
                switch ( evt.key ) {
                    case 27:
                        System.exit( 0 );
                    case 'r':
                    case 'R':
                        rotAngle += 20;
                        if ( rotAngle >= 360 ) {
                            rotAngle = 0;
                          }
                        display();
                        return true;
                  }
          }
        return false;
      }
  }
