/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: accpersp.java.stub,v 1.13 1998/10/19 12:16:22 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'accpersp' from the Red Book
 *
 * $Log: accpersp.java.stub,v $
 * Revision 1.13  1998/10/19 12:16:22  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.12  1998/10/02 12:56:28  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/06/07 13:58:18  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.10  1998/03/30 11:47:45  descarte
 * Added applet resource handling.
 *
 * Revision 1.9  1998/03/29 16:25:25  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.8  1998/03/24 22:56:40  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.7  1998/03/02 18:06:41  descarte
 * Converted demo across to beta 4 GLComponent syntax.
 *
 * Revision 1.6  1998/01/04 19:58:20  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.5  1998/01/03 16:43:14  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.4  1998/01/01 17:15:52  descarte
 * Converted demo across to using GLUTtonish GLEventListener interface.
 *
 * Revision 1.3  1997/09/09 14:19:52  descarte
 * Added copyright notice.
 *
 * Revision 1.2  1997/09/09 01:07:20  descarte
 * Added descriptive text.
 *
 * Revision 1.1  1997/09/09 01:06:14  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class accpersp extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 250,
                             HEIGHT = 250;

    /** Number of cumulative renders we'll do */
    private final int ACCUM_SIZE = 8;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        accpersp d = new accpersp();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public accpersp() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Accumulation Perspective" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: accpersp.java.stub,v 1.13 1998/10/19 12:16:22 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the OpenGL\n" );
        sb.append( "accumulation buffer to antialias an entire scene. Notice\n" );
        sb.append( "the distinct lack of jagged edges on all the objects.\n\n" );
        sb.append( "Hit ``Escape'' to quit this demo." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );
        cap.setAccumRedBits( 4 );
        cap.setAccumGreenBits( 4 );
        cap.setAccumBlueBits( 4 );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Tell the GLComponent to process events */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {

        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
        gl_.glClearAccum( 0.0f, 0.0f, 0.0f, 0.0f );
        gl_.glShadeModel( GL.GL_FLAT );

        /** Specify the lighting for the scene */
        float[] mat_ambient = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] mat_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] light_position = { 0.0f, 0.0f, 10.0f, 1.0f };
        float[] lm_ambient = { 0.2f, 0.2f, 0.2f, 1.0f };

        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_ambient );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
        gl_.glMaterialf( GL.GL_FRONT, GL.GL_SHININESS, 50.0f );

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, light_position );
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, lm_ambient );

        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
/*        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        if ( width <= height ) {
            gl_.glOrtho( -5.0, 5.0, -5.0 * HEIGHT / WIDTH,
                        5.0 * HEIGHT / WIDTH,
                        -5.0, 5.0 );
          } else {
          }
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity(); */
      }

    /** Draws the scene */
    private synchronized void displayObjects() throws OpenGLException {

        float[] torus_diffuse = { 0.7f, 0.7f, 0.0f, 1.0f };
        float[] cube_diffuse = { 0.0f, 0.7f, 0.7f, 1.0f };
        float[] sphere_diffuse = { 0.7f, 0.0f, 0.7f, 1.0f };
        float[] octa_diffuse = { 0.7f, 0.4f, 0.4f, 1.0f };

        /** Do the OpenGL stuff */
        gl_.glPushMatrix();
            gl_.glTranslatef( 0.0f, 0.0f, -5.0f );
            gl_.glRotatef( 30.0f, 1.0f, 0.0f, 0.0f );
            gl_.glPushMatrix();
                gl_.glTranslatef( -0.80f, 0.35f, 0.0f );
                gl_.glRotatef( 100.0f, 1.0f, 0.0f, 0.0f );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, torus_diffuse );
                shapes.solidTorus( 0.275, 0.85, 16, 16 );
            gl_.glPopMatrix();

            gl_.glPushMatrix();
                gl_.glTranslatef( -0.75f, -0.50f, 0.0f );
                gl_.glRotatef( 45.0f, 0.0f, 0.0f, 1.0f );
                gl_.glRotatef( 45.0f, 1.0f, 0.0f, 0.0f );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, cube_diffuse );
                shapes.solidCube( 1.5 );
            gl_.glPopMatrix();

            gl_.glPushMatrix();
                gl_.glTranslatef( 0.75f, 0.60f, 0.0f );
                gl_.glRotatef( 30.0f, 1.0f, 0.0f, 0.0f );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, sphere_diffuse );
                shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

            gl_.glPushMatrix();
                gl_.glTranslatef( 0.70f, -0.90f, 0.25f );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, octa_diffuse );
                shapes.solidOctahedron();
            gl_.glPopMatrix();
        gl_.glPopMatrix();
      }

    /** Renders the scene with jittering */
    public void display( GLDrawable component ) {
        int[] viewport = new int[4];
        int jitter = 0;

        /** Render the scene */
        gl_.glGetIntegerv( GL.GL_VIEWPORT, viewport );

        gl_.glClear( GL.GL_ACCUM_BUFFER_BIT );

        for ( jitter = 0 ; jitter < ACCUM_SIZE ; jitter++ ) {
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
            accum.accPerspective( 50.0, (double)( viewport[2] / viewport[3] ),
                                  1.0, 15.0, accum.j8[jitter][0], 
                                  accum.j8[jitter][1],
                                  0.0, 0.0, 1.0 );
            displayObjects();
            gl_.glAccum( GL.GL_ACCUM, (float)( 1.0 / ACCUM_SIZE ) );
          }
        gl_.glAccum( GL.GL_RETURN, 1.0f );
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
