/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: alpha3D.java.stub,v 1.14 1998/10/19 12:16:23 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'alpha3D' from the Red Book
 *
 * $Log: alpha3D.java.stub,v $
 * Revision 1.14  1998/10/19 12:16:23  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.13  1998/10/02 12:56:30  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.12  1998/06/07 13:58:18  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.11  1998/03/30 11:47:52  descarte
 * Added applet resource handling.
 *
 * Revision 1.10  1998/03/29 14:52:16  descarte
 * Upgraded to Java-1.1
 * Upgraded to GLComponentFactory
 *
 * Revision 1.9  1998/03/24 22:56:44  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.8  1998/01/04 19:58:22  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.7  1998/01/03 16:43:21  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.6  1998/01/01 21:15:29  descarte
 * Converted demo to GLEventListener interface.
 *
 * Revision 1.5  1997/09/09 14:21:46  descarte
 * Added copyright information.
 *
 * Revision 1.4  1997/09/03 00:15:30  descarte
 * Added synchronization between animation thread and display method.
 *
 * Revision 1.3  1997/09/02 23:30:52  descarte
 * Added GLContextException traps.
 *
 * Revision 1.2  1997/08/25 23:11:27  descarte
 * Moved demo into demoTemplate structure.
 *
 * Revision 1.1  1997/08/18 22:13:14  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class alpha3D extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 500;

    /** Stuff */
    private static final float MAXZ = 8.0f;
    private static final float MINZ = -8.0f;
    private static final float ZINC = 0.1f;

    private float solidZ = MAXZ;
    private float transparentZ = MINZ;
    private int sphereList = -1,
                cubeList = -1;

    private float cubeSpin = 0;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        alpha3D d = new alpha3D();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public alpha3D() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "3D Alpha Blending" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: alpha3D.java.stub,v 1.14 1998/10/19 12:16:23 descarte Exp descarte $" );
      }

    /** Returns the description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration shows the effect of 3-dimensional alpha\n" );
        sb.append( "blending. This demo moves the cube forward until it passes\n" );
        sb.append( "in front of the sphere, however, since the cube is partially\n" );
        sb.append( "transparent ( due to a low alpha value ), the sphere is still\n" );
        sb.append( "visible through the cube, albeit in a ``ghostly'' manner.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    a,A - Starts the animation running\n" );
        sb.append( "    r,R - Resets the animation to the beginning\n" );
        sb.append( "    ESC - Quits the demonstration." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register the keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Startup the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glEnable( GL.GL_BLEND );
            gl_.glBlendFunc( GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA );

            /** Initialize the objects in the scene */
            float[] mat_specular = { 1.0f, 1.0f, 1.0f, 0.15f };
            float[] mat_shininess = { 100.0f };
            float[] position = { 0.5f, 0.5f, 1.0f, 0.0f };

            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, mat_shininess );
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position );

            gl_.glEnable( GL.GL_LIGHTING );
            gl_.glEnable( GL.GL_LIGHT0 );
            gl_.glEnable( GL.GL_DEPTH_TEST );

            sphereList = gl_.glGenLists( 1 );
            gl_.glNewList( sphereList, GL.GL_COMPILE );
                shapes.solidSphere( 0.4, 16, 16 );
            gl_.glEndList();

            cubeList = gl_.glGenLists( 1 );
            gl_.glNewList( cubeList, GL.GL_COMPILE );
                shapes.solidCube( 0.6 );
            gl_.glEndList();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          } catch ( GLContextException e2 ) {
            e2.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        if ( width <= height ) {
            gl_.glOrtho( -1.5, 1.5, -1.5 * (float)height / (float)width,
                        1.5 * (float)height / (float)width, -10.0, 10.0 );
          } else {
            gl_.glOrtho( -1.5 * (float)width / (float)height,
                         1.5 * (float)width / (float)height,
                         -1.5, 1.5, -10.0, 10.0 );
          }
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {
        try {
            float[] mat_solid = { 0/75f, 0.75f, 0.0f, 1.0f };
            float[] mat_zero = { 0.0f, 0.0f, 0.0f, 0.0f };
            float[] mat_transparent = { 0.0f, 0.8f, 0.8f, 0.6f };
            float[] mat_emission = { 0.0f, 0.3f, 0.3f, 0.6f };

            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

            gl_.glPushMatrix();
                gl_.glTranslatef( -0.15f, -0.15f, solidZ );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, mat_zero );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_solid );
                gl_.glCallList( sphereList );
            gl_.glPopMatrix();

            gl_.glPushMatrix();
                gl_.glTranslatef( 0.15f, 0.15f, transparentZ );
                gl_.glRotatef( 15.0f, 1.0f, 1.0f, 0.0f );
                gl_.glRotatef( 30.0f, 0.0f, 1.0f, 0.0f );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, mat_emission );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_transparent );
                gl_.glEnable( GL.GL_BLEND );
                gl_.glDepthMask( (byte)GL.GL_FALSE );
                gl_.glBlendFunc( GL.GL_SRC_ALPHA, GL.GL_ONE );
                gl_.glRotatef( cubeSpin, 1, 1, 1 );
                gl_.glCallList( cubeList );
                gl_.glDepthMask( (byte)GL.GL_TRUE );
                gl_.glDisable( GL.GL_BLEND );
            gl_.glPopMatrix();

            /** Update the scene animation parameters */
            if ( solidZ > MINZ || transparentZ < MAXZ ) {
                solidZ -= ZINC;
                transparentZ += ZINC;
                cubeSpin++;
              }
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
        if ( evt.getKeyChar() == 'a' ||
             evt.getKeyChar() == 'A' ) {
            /** Halt the animation if it's currently running */
            if ( glc.isRunning() ) {
                glc.stop();
              }

            /** Reset the animation parameters */
            solidZ = MAXZ;
            transparentZ = MINZ;
            cubeSpin = 0;

            /** Restart the component */ 
            glc.restart();

            return;
          }
        if ( evt.getKeyChar() == 'r' ||
             evt.getKeyChar() == 'R' ) {
            /** Reset the animation parameters */
            solidZ = MAXZ;
            transparentZ = MINZ;

            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
