/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: bezcurve.java.stub,v 1.14 1998/10/19 12:16:24 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'bezcurve' from the Red Book
 *
 * $Log: bezcurve.java.stub,v $
 * Revision 1.14  1998/10/19 12:16:24  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.13  1998/10/02 12:56:31  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.12  1998/06/07 13:58:18  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.11  1998/03/30 11:47:53  descarte
 * Added applet resource handling.
 *
 * Revision 1.10  1998/03/29 16:15:52  descarte
 * Upgraded to Java 1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.9  1998/03/24 22:56:45  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.8  1998/01/04 19:58:23  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.7  1998/01/03 16:43:26  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.6  1998/01/01 16:44:13  descarte
 * Converted demo across to using GLUTtonish GLEventListener interface.
 *
 * Revision 1.5  1997/09/09 14:22:04  descarte
 * Added copyright information.
 *
 * Revision 1.4  1997/09/08 22:07:56  descarte
 * Pulled Map1f declarations into line with the original example style of
 * two-dimensional arrays and the fixed NURBS methods in GL.java.
 *
 * Revision 1.3  1997/09/03 00:02:59  descarte
 * Added GLContextException traps.
 *
 * Revision 1.2  1997/08/26 00:41:16  descarte
 * Moved demo to demoTemplate structure.
 *
 * Revision 1.1  1997/08/19 23:05:36  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class bezcurve extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 500;

    /** Control points for the bezier surface */
    float[][] ctrlpoints = {
        { -4.0f, -4.0f, 0.0f }, 
        { -2.0f, 4.0f, 0.0f },
        { 2.0f, -4.0f, 0.0f }, 
        { 4.0f, 4.0f, 0.0f }
      };

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        bezcurve d = new bezcurve();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public bezcurve() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Bezier Curves" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: bezcurve.java.stub,v 1.14 1998/10/19 12:16:24 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the NURBS\n" );
        sb.append( "evaluator functions defined in OpenGL. This example\n" );
        sb.append( "uses a 1-dimensional evaluator function to draw a\n" );
        sb.append( "Bezier curve constrained by the yellow control points.\n\n" );
        sb.append( "Hit ``Escape'' to stop this demonstration." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener interface */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
        gl_.glShadeModel( GL.GL_FLAT );
        gl_.glMap1f( GL.GL_MAP1_VERTEX_3, 0.0f, 1.0f, 3, 4,
                    ctrlpoints );
        gl_.glEnable( GL.GL_MAP1_VERTEX_3 );
      }

    /** Reshape the viewport */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        if ( width <= height ) {
            gl_.glOrtho( -5.0, 5.0, -5.0 * (float)height / (float)width,
                        5.0 * (float)height / (float)width,
                        -5.0, 5.0 );
          } else {
            gl_.glOrtho( -5.0, 5.0, -5.0 * (float)width / (float)height,
                        5.0 * (float)width / (float)height,
                        -5.0, 5.0 );
          }
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }


    /** Draws the scene */
    public void display( GLDrawable component ) {
        /** Do the OpenGL stuff */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT );
        gl_.glColor3f( 1.0f, 1.0f, 1.0f );

        /** Render the curve */
        gl_.glBegin( GL.GL_LINE_STRIP );
            for ( int i = 0 ; i <= 30 ; i++ ) {
                gl_.glEvalCoord1f( (float)i / 30 );
              }
        gl_.glEnd();

        /** Display the control points as dots */
        gl_.glPointSize( 5.0f );
        gl_.glColor3f( 1.0f, 1.0f, 0.0f );

        gl_.glBegin( GL.GL_POINTS );
            for ( int i = 0 ; i < 4 ; i++ ) {
                gl_.glVertex3f( ctrlpoints[i][0],
                               ctrlpoints[i][1],
                               ctrlpoints[i][2] );
              }
        gl_.glEnd();
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
