/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: checker.java.stub,v 1.15 1998/10/19 12:16:28 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'checker' from the Red Book
 *
 * $Log: checker.java.stub,v $
 * Revision 1.15  1998/10/19 12:16:28  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.14  1998/10/02 12:56:35  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.13  1998/06/07 13:58:19  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.12  1998/03/30 11:47:57  descarte
 * Added applet resource handling.
 *
 * Revision 1.11  1998/03/29 15:52:13  descarte
 * Upgraded to Java 1.1 listeners.
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.10  1998/03/24 22:56:49  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.9  1998/01/04 19:58:26  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.8  1998/01/03 16:43:31  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.7  1998/01/01 21:33:12  descarte
 * Converted demo across to GLEventListener interface.
 *
 * Revision 1.6  1997/09/09 14:23:06  descarte
 * Added copyright information.
 *
 * Revision 1.5  1997/09/02 23:44:10  descarte
 * Added GLContextException traps.
 *
 * Revision 1.4  1997/08/26 00:11:32  descarte
 * Added key-mapping for resetting the texture in the description.
 *
 * Revision 1.3  1997/08/26 00:10:27  descarte
 * Moved demo to demoTemplate structure.
 *
 * Revision 1.2  1997/08/17 10:43:35  descarte
 * Added routines for clamping and repeating the texture, in addition to
 * the standard 'checker' routines.
 *
 * Revision 1.1  1997/08/17 09:40:11  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class checker extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 250,
                             HEIGHT = 250;

    /** The checkerboard data */
    private int checkImageWidth = 64,
                checkImageHeight = 64;
    private byte[] checkImage = new byte[checkImageWidth * checkImageHeight * 3];

    /** Texture IDs */
    int[] texName = new int[1];

    /** Texture coordinates */
    private float textureFactor = 1.0f;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        checker d = new checker();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public checker() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Checkerboards!" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: checker.java.stub,v 1.15 1998/10/19 12:16:28 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates some basic texture mapping\n" );
        sb.append( "capabilities provided by OpenGL. The initial texture is\n" );
        sb.append( "mapped exactly onto the underlying polygon, however, we may\n" );
        sb.append( "also repeat or clamp the texture.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    r,R - Repeats the texture\n" );
        sb.append( "    c,C - Clamps the texture\n" );
        sb.append( "    SPACE - Resets the texture\n" );
        sb.append( "    ESC - Quits the demonstration" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the glc.getContext().capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            /** Some misc setup */
            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_FLAT );
            gl_.glEnable( GL.GL_DEPTH_TEST );
            makeCheckImage();
            gl_.glPixelStorei( GL.GL_UNPACK_ALIGNMENT, 1 );

            /** Create a new texture */
            gl_.glGenTextures( 1, texName );
            gl_.glBindTexture( GL.GL_TEXTURE_2D, texName[0] );
            gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S,
                                GL.GL_REPEAT );
            gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T,
                                GL.GL_REPEAT );
            gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER,
                                GL.GL_NEAREST );
            gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER,
                                GL.GL_NEAREST );
            gl_.glTexImage2D( GL.GL_TEXTURE_2D, 0, GL.GL_RGB, 
                             checkImageWidth, checkImageHeight,
                             0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE,
                             checkImage );
            gl_.glHint( GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 60, ( WIDTH / HEIGHT ), 1, 30 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glTranslatef( 0, 0, -3.6f );
      }

    /** Redraws the screen */
    public void display( GLDrawable component ) {
        try {
            /** Render a checker */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

            gl_.glEnable( GL.GL_TEXTURE_2D );
            gl_.glTexEnvf( GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE,
                          GL.GL_DECAL );
            gl_.glBindTexture( GL.GL_TEXTURE_2D, texName[0] );
            gl_.glBegin( GL.GL_QUADS );
                gl_.glTexCoord2f( 0.0f, 0.0f ); gl_.glVertex3f( -2.0f, -1.0f, 0.0f );
                gl_.glTexCoord2f( 0.0f, textureFactor ); gl_.glVertex3f( -2.0f, 1.0f, 0.0f );
                gl_.glTexCoord2f( textureFactor, textureFactor ); gl_.glVertex3f( 0.0f, 1.0f, 0.0f );
                gl_.glTexCoord2f( textureFactor, 0.0f ); gl_.glVertex3f( 0.0f, -1.0f, 0.0f );

                gl_.glTexCoord2f( 0.0f, 0.0f ); gl_.glVertex3f( 1.0f, -1.0f, 0.0f );
                gl_.glTexCoord2f( 0.0f, textureFactor ); gl_.glVertex3f( 1.0f, 1.0f, 0.0f );
                gl_.glTexCoord2f( textureFactor, textureFactor ); gl_.glVertex3f( 2.41421f, 1.0f, -1.41421f );
                gl_.glTexCoord2f( textureFactor, 0.0f ); gl_.glVertex3f( 2.41421f, -1.0f, -1.41421f );
            gl_.glEnd();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Makes a checkerboard pattern */
    private void makeCheckImage() {

        int c;
        byte[][][] tmpCheckImage = new byte[checkImageWidth][checkImageHeight][3];

        int index = 0;
        for ( int i = 0 ; i < checkImageHeight ; i++ ) {
            for ( int j = 0 ; j < checkImageWidth ; j++ ) {
                boolean tmp = ( ( ( ( i & 0x8 ) == 0 ) ^ ( ( j & 0x8 ) ) == 0 ) );
                if ( tmp ) {
                    c = 1;
                  } else {
                    c = 0;
                  }
                c *= 255;
                checkImage[index] = (byte)c;
                checkImage[index + 1] = (byte)c;
                checkImage[index + 2] = (byte)c;
                index += 3;
              }
          }
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

            return;
          }
        if ( evt.getKeyChar() == ' ' ) {
            System.err.println( "Resetting texture factor to 1.0" );
            textureFactor = 1.0f;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'r' ||
             evt.getKeyChar() == 'R' ) {
            System.err.println( "Using repeating texture" );
            textureFactor = 3.0f;

            /** Make the glc.getContext().current */
            glc.getContext().makeCurrent();

            /** Reset the texture parameters */
            gl_.glTexParameteri( GL.GL_TEXTURE_2D,
                                GL.GL_TEXTURE_WRAP_S,
                                GL.GL_REPEAT );

            /** Unlock the glc.getContext().for repaint */
            glc.getContext().unlock();

            /** Render the scene */
            glc.repaint();

            return;
          }
        if ( evt.getKeyChar() == 'c' ||
             evt.getKeyChar() == 'C' ) {
            System.err.println( "Using clamped texture" );
            textureFactor = 3.0f;

            /** Make the glc.getContext().current */
            glc.getContext().makeCurrent();

            /** Reset the texture parameters */
            gl_.glTexParameteri( GL.GL_TEXTURE_2D,
                                GL.GL_TEXTURE_WRAP_S,
                                GL.GL_CLAMP );

            /** Unlock the glc.getContext().for repaint */
            glc.getContext().unlock();

            /** Render the scene */
            glc.repaint();

            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
