/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: dof.java.stub,v 1.12 1998/10/19 12:16:31 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'dof' from the Red Book
 *
 * $Log: dof.java.stub,v $
 * Revision 1.12  1998/10/19 12:16:31  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.11  1998/10/02 12:56:39  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.10  1998/06/07 13:58:19  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.9  1998/03/30 11:48:01  descarte
 * Added applet resource handling.
 *
 * Revision 1.8  1998/03/29 16:31:15  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.7  1998/03/24 22:56:52  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.6  1998/01/04 19:58:30  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.5  1998/01/03 16:43:37  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.4  1998/01/01 21:56:37  descarte
 * Small bug fix.
 *
 * Revision 1.2  1997/09/09 14:24:22  descarte
 * Added copyright information.
 *
 * Revision 1.1  1997/09/09 08:28:57  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;
import com.hermetica.util3d.*;

public class dof extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 400,
                             HEIGHT = 400;

    /** Display list index for a teapot */
    private int teapotList = -1;

    /** Number of jittered scenes we'll render to compose the final image */
    private static final int ACCUM_SIZE = 8;

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Depth Of Field" );
      }

    /** Returns some version information about this demo */
    public String getVersion() {
        return new String( "$Id: dof.java.stub,v 1.12 1998/10/19 12:16:31 descarte Exp descarte $" );
      }

    /** Returns a meaningful description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration renders 5 teapots generated\n" );
        sb.append( "with the solidTeapot() method in the shapes class, which\n" );
        sb.append( "in turn uses the underlying NURBS handling present in \n" );
        sb.append( "OpenGL. The teapots are placed into the distance, which\n" );
        sb.append( "allows us, using the accumulation buffer, to simulate\n" );
        sb.append( "photographic focus, or a ``depth of field'' effect. In\n" );
        sb.append( "this example, the focus depth is positioned at the depth\n" );
        sb.append( "of the second teapot from the left, the gold one, which\n" );
        sb.append( "remains in focus, when the others are variably out of focus\n" );
        sb.append( "depending on their depth.\n\n" );
        sb.append( "Please give this demo a little time to run. It renders\n" );
        sb.append( "the scene 8 times into the accumulation buffers before\n" );
        sb.append( "displaying anything.\n\n" );
        sb.append( "Hit the ``Escape'' key to exit this demonstration." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        dof d = new dof();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public dof() {
        /**#start */
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );
        cap.setAccumRedBits( 4 );
        cap.setAccumGreenBits( 4 );
        cap.setAccumBlueBits( 4 );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            gl_.glClearColor( 0.0f, 0.1f, 0.1f, 0.0f );
            gl_.glClearAccum( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_SMOOTH );

            /** Some basic display code to render the scene */
            float[] ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
            float[] diffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] specular = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] position = { 0.0f, 3.0f, 3.0f, 0.0f };
            float[] lmodel_ambient = { 0.2f, 0.2f, 0.2f, 1.0f };
            float[] local_view = { 0.0f };

            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, ambient );
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse );
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position );
            gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );
            gl_.glLightModelfv( GL.GL_LIGHT_MODEL_LOCAL_VIEWER, local_view );

            gl_.glFrontFace( GL.GL_CW );
            gl_.glEnable( GL.GL_LIGHTING );
            gl_.glEnable( GL.GL_LIGHT0 );
            gl_.glEnable( GL.GL_DEPTH_TEST );
            gl_.glEnable( GL.GL_AUTO_NORMAL );
            gl_.glEnable( GL.GL_NORMALIZE );

            /** Make a teapot display list */
            teapotList = gl_.glGenLists( 1 );
            gl_.glNewList( teapotList, GL.GL_COMPILE );
                shapes.solidTeapot( 0.5 );
            gl_.glEndList();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
/**        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glOrtho( 0.0, 16.0, 0.0, 16.0 * (float)height / (float)width,
                    -10.0, 10.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW ); */
      }

    /** Renders a single teapot */
    private void renderTeapot( float x, float y, float z, 
                               float ambr, float ambg, float ambb, 
                               float difr, float difg, float difb,
                               float specr, float specg, float specb,
                               float shine ) throws OpenGLException {

        float[] mat = new float[4];

        gl_.glPushMatrix();
        gl_.glTranslatef( x, y, z );

        mat[0] = ambr; mat[1] = ambg; mat[2] = ambb; mat[3] = 1.0f;
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat );

        mat[0] = difr; mat[1] = difg; mat[2] = difb;
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat);

        mat[0] = specr; mat[1] = specg; mat[2] = specb;
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat);

        gl_.glMaterialf( GL.GL_FRONT, GL.GL_SHININESS, (float)( shine * 128.0 ) );

        gl_.glCallList( teapotList );

        gl_.glPopMatrix();
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {

        int[] viewport = new int[4];

        try {
            /** Fetch the viewport size */
            gl_.glGetIntegerv( GL.GL_VIEWPORT, viewport );

            /** Do the OpenGL stuff */
            gl_.glClear( GL.GL_ACCUM_BUFFER_BIT );

            for ( int jitter = 0 ; jitter < ACCUM_SIZE ; jitter++ ) {
                gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

                accum.accPerspective( 45.0, (double)( viewport[2] / viewport[3] ),
                                      1.0, 15.0, 0.0, 0.0, 
                                      0.33 * accum.j8[jitter][0],
                                      0.33 * accum.j8[jitter][1],
                                      5.0 );

                /** Render 5 teapots: Ruby, gold, silver, emerald, and cyan */
                renderTeapot( -1.1f, -0.5f, -4.5f, 0.1745f, 0.01175f,
                              0.01175f, 0.61424f, 0.04136f, 0.04136f,
                              0.727811f, 0.626959f, 0.626959f, 0.6f );
                renderTeapot( -0.5f, -0.5f, -5.0f, 0.24725f, 0.1995f, 
                              0.0745f, 0.75164f, 0.60648f, 0.22648f, 
                              0.628281f, 0.555802f, 0.366065f, 0.4f );
                renderTeapot( 0.2f, -0.5f, -5.5f, 0.19225f, 0.19225f, 
                              0.19225f, 0.50754f, 0.50754f, 0.50754f, 
                              0.508273f, 0.508273f, 0.508273f, 0.4f );
                renderTeapot( 1.0f, -0.5f, -6.0f, 0.0215f, 0.1745f, 0.0215f, 
                              0.07568f, 0.61424f, 0.07568f, 0.633f, 
                              0.727811f, 0.633f, 0.6f );
                renderTeapot( 1.8f, -0.5f, -6.5f, 0.0f, 0.1f, 0.06f, 0.0f, 
                              0.50980392f, 0.50980392f, 0.50196078f, 
                              0.50196078f, 0.50196078f, 0.25f );

                gl_.glAccum( GL.GL_ACCUM, (float)( 0.125 ) );
              }
            gl_.glAccum( GL.GL_RETURN, 1.0f );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
