/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: feedback.java.stub,v 1.13 1998/10/19 12:16:33 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'feedback' from the Red Book
 *
 * $Log: feedback.java.stub,v $
 * Revision 1.13  1998/10/19 12:16:33  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.12  1998/10/02 12:56:42  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/06/07 13:58:20  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.10  1998/03/30 11:48:04  descarte
 * Added applet resource handling.
 *
 * Revision 1.9  1998/03/29 16:12:40  descarte
 * Upgraded to java 1.1 listeners
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.8  1998/03/24 22:56:56  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.7  1998/01/04 19:59:52  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.6  1998/01/03 16:43:39  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.5  1998/01/01 21:39:42  descarte
 * Converted across to GLEventListener interface.
 *
 * Revision 1.4  1997/09/09 14:25:23  descarte
 * Added copyright information.
 *
 * Revision 1.3  1997/09/02 23:57:01  descarte
 * Added GLContextException traps.
 *
 * Revision 1.2  1997/08/26 00:35:43  descarte
 * Moved demo to demoTemplate structure.
 *
 * Revision 1.1  1997/08/17 15:14:46  descarte
 * Initial revision
 *
 * Revision 1.1  1997/08/15 19:28:58  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class feedbackapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 100,
                             HEIGHT = 100;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Feedback Buffer" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: feedback.java.stub,v 1.13 1998/10/19 12:16:33 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration exercises the ``feedback buffer'' in OpenGL\n" );
        sb.append( "which, when rendered to, keeps track of which primitives are\n" );
        sb.append( "in the rendering pipeline. We can then query the feedback buffer\n" );
        sb.append( "to ``replay'' the OpenGL operations that have been executed.\n\n" );
        sb.append( "Hit ``Escape'' to quit this demonstration." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_FLAT );
            gl_.glEnable( GL.GL_LIGHTING );
            gl_.glEnable( GL.GL_LIGHT0 );

          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
      }

    /** Draws the geometry aspects of the demo */
    private void drawGeometry( int mode ) throws OpenGLException {
        gl_.glBegin( GL.GL_LINE_STRIP );
            gl_.glNormal3f( 0.0f, 0.0f, 1.0f );
            gl_.glVertex3f( 30.0f, 30.0f, 0.0f );
            gl_.glVertex3f( 50.0f, 60.0f, 0.0f );
            gl_.glVertex3f( 70.0f, 40.0f, 0.0f );
        gl_.glEnd();

        if ( mode == GL.GL_FEEDBACK ) {
            gl_.glPassThrough( 1.0f );
          }

        /** This point will be clipped */
        gl_.glBegin( GL.GL_POINTS );
            gl_.glVertex3f( -100.0f, -100.0f, -100.0f );
        gl_.glEnd();

        if ( mode == GL.GL_FEEDBACK ) {
            gl_.glPassThrough( 2.0f );
          }

        gl_.glBegin( GL.GL_POINTS );
            gl_.glNormal3f( 0.0f, 0.0f, 1.0f );
            gl_.glVertex3f( 50.0f, 50.0f, 0.0f );
        gl_.glEnd();
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Prints a 3d colour vertex information */
    private int print3DColourVertex( int size, int count, float[] buffer ) {
        System.err.print( "  " );
        for ( int i = 0 ; i < 7 ; i++ ) {
            System.err.print( buffer[size - count] + " " );
            count--;
          }
        System.err.println();
        return count;
      }

    /** Dumps the feedback buffer */
    private void printBuffer( int size, float[] buffer ) {
        int count;
        float token;

        count = size;
        while ( count >= 0 ) {
            token = buffer[size - count];
            count--;

            if ( token == GL.GL_PASS_THROUGH_TOKEN ) {
                System.err.println( "GL_PASS_THROUGH_TOKEN" );
                System.err.println( "    " + buffer[size - count] );
                count--;
              } else {
                if ( token == GL.GL_POINT_TOKEN ) {
                    System.err.println( "GL_POINT_TOKEN" );
                    count = print3DColourVertex( size, count, buffer );
                  } else {
                    if ( token == GL.GL_LINE_TOKEN ) {
                        System.err.println( "GL_LINE_TOKEN" );
                        count = print3DColourVertex( size, count, buffer );
                        count = print3DColourVertex( size, count, buffer );
                      } else {
                        if ( token == GL.GL_LINE_RESET_TOKEN ) {
                            System.err.println( "GL_LINE_RESET_TOKEN" );
                            count = print3DColourVertex( size, count, buffer );
                            count = print3DColourVertex( size, count, buffer );
                          }
                      }
                  }
              }
          }
      }

    /** Displays the screen */
    public void display( GLDrawable component ) {

        float[] feedBuffer = new float[1024];
        int size;

        try {
            /** Some basic display code to render the scene */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT );
            gl_.glColor3f( 1.0f, 1.0f, 1.0f );

            /** Draw stuff */
            gl_.glMatrixMode( GL.GL_PROJECTION );
            gl_.glLoadIdentity();
            gl_.glOrtho( 0.0, 100.0, 0.0, 100.0, 0.0, 1.0 );

            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT );
            drawGeometry( GL.GL_RENDER );

            gl_.glFeedbackBuffer( 1024, GL.GL_3D_COLOR, feedBuffer );
            gl_.glRenderMode( GL.GL_FEEDBACK );
            drawGeometry( GL.GL_FEEDBACK );

            size = gl_.glRenderMode( GL.GL_RENDER );
            printBuffer( size, feedBuffer );

          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
