/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: fog.java.stub,v 1.13 1998/10/19 12:16:34 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'fog' from the Red Book
 *
 * $Log: fog.java.stub,v $
 * Revision 1.13  1998/10/19 12:16:34  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.12  1998/10/02 12:56:43  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/06/07 13:58:20  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.10  1998/03/30 11:48:05  descarte
 * Added applet resource handling.
 *
 * Revision 1.9  1998/03/29 14:55:26  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.8  1998/03/24 22:56:57  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.7  1998/01/04 19:58:50  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.6  1998/01/03 16:43:40  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.5  1998/01/01 19:00:05  descarte
 * Converted demo across to GLUTtonish GLEventListener interface.
 *
 * Revision 1.4  1997/09/09 14:25:43  descarte
 * Added copyright information.
 *
 * Revision 1.3  1997/09/02 23:32:59  descarte
 * Added GLContextException traps.
 *
 * Revision 1.2  1997/08/25 23:23:06  descarte
 * Moved demo into demoTemplate structure.
 *
 * Revision 1.1  1997/08/18 21:15:15  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class fog extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 500;

    /** The type of fogging we want to do */
    private int fogMode = GL.GL_EXP;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        fog d = new fog();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public fog() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Fog" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: fog.java.stub,v 1.13 1998/10/19 12:16:34 descarte Exp descarte $" );
      }

    /** Returns the description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration exercises OpenGL's fogging capabilities.\n" );
        sb.append( "Five spheres are rendered at increasing distances which produces\n" );
        sb.append( "differing effects depending on which fog calculation rule is\n" );
        sb.append( "used, ie, GL_LINEAR ( useful for depth-cueing and ``hazes'' ),\n" );
        sb.append( "GL_EXP ( useful for ``smoky'' effects ) and GL_EXP2 ( pea-souper! )\n" );
        sb.append( "You may toggle between these fog rules.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    f,F - Toggles between fog modes cyclically\n" );
        sb.append( "    ESC - Quits the demonstration" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            gl_.glClearColor( 0.5f, 0.5f, 0.5f, 1.0f );
            gl_.glShadeModel( GL.GL_SMOOTH );

            /** Some basic display code to render the scene */
            float[] position = { 0.5f, 0.5f, 3.0f, 0.0f };

            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position );

            gl_.glEnable( GL.GL_LIGHTING );
            gl_.glEnable( GL.GL_LIGHT0 );
            gl_.glEnable( GL.GL_DEPTH_TEST );

            float[] mat = { 0.1745f, 0.01175f, 0.01175f };

            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat );
            mat[0] = 0.61424f; mat[1] = 0.04136f; mat[2] = 0.04136f;
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat );
            mat[0] = 0.727811f; mat[1] = 0.626959f; mat[2] = 0.626959f;
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat );
            gl_.glMaterialf( GL.GL_FRONT, GL.GL_SHININESS, (float)( 0.6 * 128.0 ) );

            gl_.glEnable( GL.GL_FOG );

            float[] fogColour = { 0.5f, 0.5f, 0.5f, 1.0f };

            gl_.glFogi( GL.GL_FOG_MODE, fogMode );
            gl_.glFogfv( GL.GL_FOG_COLOR, fogColour );
            gl_.glFogf( GL.GL_FOG_DENSITY, 0.35f );
            gl_.glHint( GL.GL_FOG_HINT, GL.GL_DONT_CARE );
            gl_.glFogf( GL.GL_FOG_START, 1.0f );
            gl_.glFogf( GL.GL_FOG_END, 5.0f );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles resizing the viewport */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        if ( width <= height ) {
            gl_.glOrtho( -2.5, 2.5, -2.5 * (float)height / (float)width,
                        2.5 * (float)height / (float)width, -10.0, 10.0 );
          } else {
            gl_.glOrtho( -2.5 * (float)width / (float)height,
                         2.5 * (float)width / (float)height,
                         -2.5, 2.5, -10.0, 10.0 );
          }
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Draws a sphere */
    private void renderSphere( float x, float y, float z ) throws OpenGLException {
        gl_.glPushMatrix();
        gl_.glTranslatef( x, y, z );
        shapes.solidSphere( 0.4, 16, 16 );
        gl_.glPopMatrix();
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {
        try {
            /** Render 5 spheres at different distances */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

            renderSphere( -2.0f, -0.5f, -1.0f );
            renderSphere( -1.0f, -0.5f, -2.0f );
            renderSphere( 0.0f, -0.5f, -3.0f );
            renderSphere( 1.0f, -0.5f, -4.0f );
            renderSphere( 2.0f, -0.5f, -5.0f );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
        if ( evt.getKeyChar() == 'f' ||
             evt.getKeyChar() == 'F' ) {
            if ( fogMode == GL.GL_EXP ) {
                fogMode = GL.GL_EXP2;
                System.err.println( "Fog mode is GL_EXP2" );
              } else {
                if ( fogMode == GL.GL_EXP2 ) {
                    fogMode = GL.GL_LINEAR;
                    System.err.println( "Fog mode is GL_LINEAR" );
                  } else {
                    if ( fogMode == GL.GL_LINEAR ) {
                        fogMode = GL.GL_EXP;
                        System.err.println( "Fog mode is GL_EXP" );
                      }
                  }
              }
            try {
                /** Update the fog type */
                glc.getContext().makeCurrent();
                gl_.glFogi( GL.GL_FOG_MODE, fogMode );
                glc.getContext().unlock();

                /** Redraw the scene */
                glc.repaint();
              } catch ( OpenGLException e ) {
                e.printStackTrace();
              }
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
