/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: image.java.stub,v 1.5 1998/01/01 21:28:26 descarte Exp descarte $
 *
 * Copyright (c)1997 Alligator Descartes <descarte@hermetica.com>
 *
 * 'image' from the Red Book
 *
 * $Log: image.java.stub,v $
 * Revision 1.5  1998/01/01 21:28:26  descarte
 * Converted demo to GLComponentListener interface.
 *
 * Revision 1.4  1997/09/09 14:25:57  descarte
 * Added copyright information.
 *
 * Revision 1.3  1997/09/02 23:42:17  descarte
 * Added GLContextException traps.
 *
 * Revision 1.2  1997/08/26 00:06:05  descarte
 * Moved demo to demoTemplate structure.
 *
 * Revision 1.1  1997/08/16 16:47:12  descarte
 * Initial revision
 *
 * Revision 1.1  1997/08/16 16:17:31  descarte
 * Initial revision
 *
 * Revision 1.1  1997/08/16 15:12:48  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class image extends Frame implements demoTemplate, GLComponentListener {

    /** The OpenGL state machine */
    private GL gl_ = new GL();
    private GLU glu_ = new GLU();

    /** The OpenGL rendering context */
    private GLContext context = new GLContext();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 250,
                             HEIGHT = 250;

    /** The checkerboard data */
    private int checkImageWidth = 64,
                checkImageHeight = 64;
    private byte[] checkImage = new byte[checkImageWidth * checkImageHeight * 3];

    /** The zoom factor of copied images */
    private float zoomFactor = 1.0f;

    public image() {
        /**#start */
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: image.java.stub,v 1.5 1998/01/01 21:28:26 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the image manipulation\n" );
        sb.append( "functions available in OpenGL. Initially, a checkerboard pattern\n" );
        sb.append( "is displayed in the bottom-left corner of the viewport. By \n" );
        sb.append( "clicking and dragging the mouse, you can plop more checkerboards\n" );
        sb.append( "onto the viewport. You can also increase and decrease the zoom\n" );
        sb.append( "factor of the image before plopping.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    r,R - Reset zoom factor to 1.0\n" );
        sb.append( "    z - Increase zoom factor by 0.5\n" );
        sb.append( "    Z - Decrease zoom factor by 0.5\n" );
        sb.append( "    ESC - Quit demonstration" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = new GLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = context.getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setRedBits( 1 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Assign the context to the component */
        glc.setContext( context );

        /** Register the GLComponentListener with the GLComponent */
        glc.addGLComponentListener( this );
      }

    /** Initialization stuff */
    public void initialize( GLComponent component ) {
        try {
            /** Some misc setup */
            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_FLAT );
            makeCheckImage();
            gl_.glPixelStorei( GL.GL_UNPACK_ALIGNMENT, 1 );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLComponent component, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, 0, 0, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluOrtho2D( 0.0, width, 0, height );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
      }

    /** Redraws the screen */
    public void display( GLComponent component ) {
        try {
            /** Render a image */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT );

            gl_.glColor3f( 1.0f, 1.0f, 1.0f );
            gl_.glRasterPos2i( 0, 0 );
            gl_.glDrawPixels( checkImageWidth, checkImageHeight, GL.GL_RGB,
                             GL.GL_UNSIGNED_BYTE, checkImage );

          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Makes a checkerboard pattern */
    private void makeCheckImage() {

        int c;
        byte[][][] tmpCheckImage = new byte[checkImageWidth][checkImageHeight][3];

        int index = 0;
        for ( int i = 0 ; i < checkImageHeight ; i++ ) {
            for ( int j = 0 ; j < checkImageWidth ; j++ ) {
                boolean tmp = ( ( ( ( i & 0x8 ) == 0 ) ^ ( ( j & 0x8 ) ) == 0 ) );
                if ( tmp ) {
                    c = 1;
                  } else {
                    c = 0;
                  }
                c *= 255;
                checkImage[index] = (byte)c;
                checkImage[index + 1] = (byte)c;
                checkImage[index + 2] = (byte)c;
                index += 3;
              }
          }
      }

    /** AWT event handler */
    public boolean handleEvent( Event evt ) {
        switch ( evt.id ) {
            case Event.KEY_PRESS:
                switch ( evt.key ) {
                    case 27:
                        try {
                            context.destroyContext( glc );
                          } catch ( GLContextException e ) {
                            e.printStackTrace();
                          }
                        dispose();
                        return true;
                    case 'r':
                    case 'R':
                        zoomFactor = 1.0f;
                        try {
                            /** Re-render the scene */
                            glc.repaint();
                          } catch ( GLContextException e ) {
                            e.printStackTrace();
                          }
                        System.err.println( "zoomFactor reset to 1.0" );
                        return true;
                    case 'z':
                        zoomFactor += 0.5f;
                        if ( zoomFactor >= 3.0f ) {
                            zoomFactor = 3.0f;
                          }
                        System.err.println( "zoomFactor is now " + zoomFactor );
                        return true;
                    case 'Z':
                        zoomFactor -= 0.5;
                        if ( zoomFactor <= 0.5 ) {
                            zoomFactor = 0.5f;
                          }
                        System.err.println( "zoomFactor is now " + zoomFactor );
                        return true;
                  }
            case Event.MOUSE_DRAG:

                int screeny = ( HEIGHT - ( evt.y - HEIGHT ) - evt.y );

                try {
                    /** Make the GL context current */
                    context.makeCurrent( glc );

                    /** Draw the new rectangle */
                    gl_.glRasterPos2i( evt.x, screeny );
                    gl_.glPixelZoom( zoomFactor, zoomFactor );
                    gl_.glCopyPixels( 0, 0, checkImageWidth, checkImageHeight,
                                     GL.GL_COLOR );
                    gl_.glPixelZoom( 1.0f, 1.0f );

                    /** Unlock the context for repaint() */
                    context.unlock();

                    /** Re-render the scene */
                    glc.repaint();
                  } catch ( OpenGLException e ) {
                    e.printStackTrace();
                  } catch ( GLContextException e2 ) {
                    e2.printStackTrace();
                  }

                return true;
          }
        return false;
      }
  }
