/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: light.java,v 1.4 1997/09/09 14:26:15 descarte Exp descarte $
 *
 * Copyright (c)1997 Alligator Descartes <descarte@hermetica.com>
 *
 * 'light' from the Red Book
 *
 * $Log: light.java,v $
 * Revision 1.4  1997/09/09 14:26:15  descarte
 * Added copyright information.
 *
 * Revision 1.3  1997/09/02 23:19:56  descarte
 * Added GLContextException traps.
 *
 * Revision 1.2  1997/08/25 22:30:00  descarte
 * Moved demo into demoTemplate structure.
 *
 * Revision 1.1  1997/08/18 19:55:09  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class lightapp extends Applet implements demoTemplate {

    /** The OpenGL state machine */
    private GL gl_ = new GL();

    /** The OpenGL rendering context */
    private GLContext context = new GLContext();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 500;

    /** Some AWT checkboxes */
    Checkbox sphereCheckbox = null,
             octahedronCheckbox = null,
             teapotCheckbox = null,
             icosahedronCheckbox = null,
             cubeCheckbox = null;

    public void init() {
        startDemo();
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: light.java,v 1.4 1997/09/09 14:26:15 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration draws a solid sphere using the solidSphere()\n" );
        sb.append( "method defined in the GLU class and applies lighting to the\n" );
        sb.append( "using OpenGL's lighting model.\n\n" );
        sb.append( "Hit ``Escape'' to finish this demo." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new FlowLayout() );

        glc = new GLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( glc );

        Panel choicePanel = new Panel();
        CheckboxGroup group = new CheckboxGroup();

        sphereCheckbox = new Checkbox( "Sphere", group, true );
        octahedronCheckbox = new Checkbox( "Octahedron", group, false );
        teapotCheckbox = new Checkbox( "Teapot", group, false );
        icosahedronCheckbox = new Checkbox( "Icosahedron", group, false );
        cubeCheckbox = new Checkbox( "Cube", group, false );

        choicePanel.add( sphereCheckbox );
        choicePanel.add( octahedronCheckbox );
        choicePanel.add( teapotCheckbox );
        choicePanel.add( icosahedronCheckbox );
        choicePanel.add( cubeCheckbox );

        add( choicePanel );

        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = context.getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setRedBits( 1 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Assign the context to the component */
        glc.setContext( context );

        /** Initialize the application */
        initialize();

        /** Display the scene */
        display();
      }

    /** Initialization stuff */
    private void initialize() {
        try {
            /** Make the context current */
            context.makeCurrent( glc );

            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_SMOOTH );

            /** Setup the viewport */
            gl_.glViewport( 0, 0, WIDTH, HEIGHT );
            gl_.glMatrixMode( GL.GL_PROJECTION );
            gl_.glLoadIdentity();
            gl_.glOrtho( -1.5, 1.5, -1.5 * ( HEIGHT / WIDTH ), 
                        1.5 * ( HEIGHT / WIDTH ), -10.0, 10.0 );
            gl_.glMatrixMode( GL.GL_MODELVIEW );
            gl_.glLoadIdentity();

            /** Some basic display code to render the scene */
            float[] mat_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] mat_shininess = { 50.0f };
            float[] light_position = { 1.0f, 1.0f, 1.0f, 0.0f };

            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, mat_shininess );
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, light_position );

            gl_.glEnable( GL.GL_LIGHTING );
            gl_.glEnable( GL.GL_LIGHT0 );
            gl_.glEnable( GL.GL_DEPTH_TEST );

            /** Release the OpenGL context */
            context.unlock();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          } catch ( GLContextException e2 ) {
            e2.printStackTrace();
          }
      }

    /** Draws the scene */
    private void display() {
        try {
            /** Make the context current */
            context.makeCurrent( glc );

            /** Do the OpenGL stuff */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
            if ( sphereCheckbox.getState() ) {
                shapes.solidSphere( 1.0, 20, 16 );
              } else {
                if ( octahedronCheckbox.getState() ) {
                    shapes.solidOctahedron();
                  } else {
                    if ( teapotCheckbox.getState() ) {
                        shapes.solidTeapot( 1.0 );
                      } else {
                        if ( icosahedronCheckbox.getState() ) {
                            shapes.solidIcosahedron();
                          } else {
                            if ( cubeCheckbox.getState() ) {
                                gl_.glPushMatrix();
                                gl_.glRotatef( 30.0f, 1.0f, 1.0f, 1.0f );
                                shapes.solidCube( 1.0 );
                                gl_.glPopMatrix();
                              }
                          }
                      }
                  }
              }

            /** Flush the screen */
            context.swapBuffers( glc );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          } catch ( GLContextException e2 ) {
            e2.printStackTrace();
          }
      }

    /** AWT event handler */
    public boolean handleEvent( Event evt ) {
        switch ( evt.id ) {
            case Event.KEY_PRESS:
                switch ( evt.key ) {
                    case 27:
                        try {
                            context.destroyContext( glc );
                          } catch ( GLContextException e ) {
                            e.printStackTrace();
                          }
                        /**#dispose */
                        return true;
                  }
             case Event.ACTION_EVENT:
                display();
                return true;
          }
        return false;
      }
  }
