/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: lines.java.stub,v 1.13 1998/10/19 12:16:37 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'lines' from the Red Book
 *
 * $Log: lines.java.stub,v $
 * Revision 1.13  1998/10/19 12:16:37  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.12  1998/10/02 12:56:46  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/06/07 13:58:20  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.10  1998/03/30 11:48:08  descarte
 * Added applet resource handling.
 *
 * Revision 1.9  1998/03/28 18:02:04  descarte
 * Upgraded to Java 1.1
 * Upgraded to GLComponentFactory
 *
 * Revision 1.8  1998/03/24 22:56:59  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.7  1998/01/04 19:58:57  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.6  1998/01/03 16:43:43  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.5  1998/01/01 20:48:17  descarte
 * Converted demo to GLEventListener interface.
 *
 * Revision 1.4  1997/09/09 14:26:31  descarte
 * Added copyright information.
 *
 * Revision 1.3  1997/09/02 23:02:29  descarte
 * Added GLContextException traps.
 *
 * Revision 1.2  1997/08/25 21:53:16  descarte
 * Moved demo into demoTemplate structure.
 *
 * Revision 1.1  1997/08/15 19:28:58  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class linesapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 400,
                             HEIGHT = 150;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Line Stippling" );
      }

    /** Returns the version information about this demo */
    public String getVersion() {
        return new String( "$Id: lines.java.stub,v 1.13 1998/10/19 12:16:37 descarte Exp descarte $" );
      }

    /** Returns the description about this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the variety of effects that\n" );
        sb.append( "the line drawing code inherent in OpenGL can be used for\n" );
        sb.append( "including varying line thicknesses and line stippling.\n\n" );
        sb.append( "Hit ``Escape'' to dispose of this demonstration." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {
        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register the keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_FLAT );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Draw the scene */
    public void display( GLDrawable component ) {
        /** Some basic display code to render the scene */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT );
        gl_.glColor3f( 1.0f, 1.0f, 1.0f );

        gl_.glEnable( GL.GL_LINE_STIPPLE );

        /** 1st Row, each with a different stipple */
        gl_.glLineStipple( 1, (short)0x0101 );       /** Dotted */
        drawOneLine( 50.0f, 125.0f, 150.0f, 125.0f );
        gl_.glLineStipple( 1, (short)0x00FF );       /** Dashed */ 
        drawOneLine( 150.0f, 125.0f, 250.0f, 125.0f );
        gl_.glLineStipple( 1, (short)0x1C47 );       /** Dash/Dot/Dash */
        drawOneLine( 250.0f, 125.0f, 350.0f, 125.0f );

        /** 2nd row, 3 wide lines, each with a different stipple */
        gl_.glLineWidth( 5.0f );
        gl_.glLineStipple( 1, (short)0x0101 );       /** Dotted */
        drawOneLine( 50.0f, 100.0f, 150.0f, 100.0f );
        gl_.glLineStipple( 1, (short)0x00FF );       /** Dashed */
        drawOneLine( 150.0f, 100.0f, 250.0f, 100.0f );
        gl_.glLineStipple( 1, (short)0x1C47 );       /** Dash/Dot/Dash */
        drawOneLine( 250.0f, 100.0f, 350.0f, 100.0f );
        gl_.glLineWidth( 1.0f );

        /** 
         * 3rd row. 6 lines, with dash/dot/dash stipple as part of
         * connected line strip
         */
        gl_.glLineStipple( 1, (short)0x1C47 );
        gl_.glBegin( GL.GL_LINE_STRIP );
            for ( int i = 0 ; i < 7 ; i++ ) {
                gl_.glVertex2f( (float)( 50.0 + i * 50.0 ), 75.0f );
              }
        gl_.glEnd();

        /** 4th row. 6 independent lines with same stipple */
        for ( int i = 0 ; i < 6 ; i++ ) {
            drawOneLine( (float)( 50.0 + ( i * 50 ) ), 50.0f,
                         (float)( 50.0 + ( ( i + 1 ) * 50 ) ), 50.0f );
          }

        /** 
         * 5th row. 1 line, with dash/dot/dash stipple and a 
         * stipple repeat factor of 5
         */
        gl_.glLineStipple( 5, (short)0x1C47 );       /** Dash/Dot/Dash */
        drawOneLine( 50.0f, 25.0f, 350.0f, 25.0f );

        /** Disable stippling */
        gl_.glDisable( GL.GL_LINE_STIPPLE );
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluOrtho2D( 0.0, (double)width, 0.0, (double)height );
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Draws a single line */
    private void drawOneLine( float x1, float y1, float x2, float y2 ) throws OpenGLException {
        gl_.glBegin( GL.GL_LINES );
            gl_.glVertex2f( x1, y1 );
            gl_.glVertex2f( x2, y2 );
        gl_.glEnd();
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
