/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: material.java.stub,v 1.13 1998/10/19 12:16:38 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'material' from the Red Book
 *
 * $Log: material.java.stub,v $
 * Revision 1.13  1998/10/19 12:16:38  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.12  1998/10/02 12:56:48  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/06/07 13:58:20  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.10  1998/03/30 11:48:10  descarte
 * Added applet resource handling.
 *
 * Revision 1.9  1998/03/29 14:37:38  descarte
 * Upgraded to Java-1.1 listener
 * Upgraded to GLComponentFactory
 *
 * Revision 1.8  1998/03/24 22:57:01  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.7  1998/01/04 19:59:01  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.6  1998/01/03 16:43:45  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.5  1998/01/01 18:41:28  descarte
 * Converted demo across to GLUTtonish GLEventListener interface.
 *
 * Revision 1.4  1997/09/09 14:27:05  descarte
 * Added copyright information.
 *
 * Revision 1.3  1997/09/02 23:23:18  descarte
 * Added GLContextException traps.
 *
 * Revision 1.2  1997/08/25 22:57:33  descarte
 * Moved demo into demoTemplate structure.
 *
 * Revision 1.1  1997/08/18 20:29:45  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class materialapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 600,
                             HEIGHT = 450;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Materials" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: material.java.stub,v 1.13 1998/10/19 12:16:38 descarte Exp descarte $" );
      }

    /** Returns the description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the vast array of material\n" );
        sb.append( "that OpenGL can render easily. Here, we render 12 spheres\n" );
        sb.append( "each with a different material property.\n\n" );
        sb.append( "Hit ``Escape'' to terminate this demonstration." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            gl_.glClearColor( 0.0f, 0.1f, 0.1f, 0.0f );
            gl_.glShadeModel( GL.GL_SMOOTH );

            /** Some basic display code to render the scene */
            float[] ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
            float[] diffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] specular = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] position = { 0.0f, 3.0f, 2.0f, 0.0f };
            float[] lmodel_ambient = { 0.4f, 0.4f, 0.4f, 1.0f };
            float[] local_view = { 0.0f };

            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, ambient );
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse );
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position );
            gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );
            gl_.glLightModelfv( GL.GL_LIGHT_MODEL_LOCAL_VIEWER, local_view );

            gl_.glEnable( GL.GL_LIGHTING );
            gl_.glEnable( GL.GL_LIGHT0 );
            gl_.glEnable( GL.GL_DEPTH_TEST );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        if ( width <= ( height * 2 ) ) {
            gl_.glOrtho( -6.0, 6.0, -3.0 * ( (float)height * 2 ) / width,
                        3.0 * ( (float)height * 2 ) / (float)width,
                        -10.0, 10.0 );
          } else {
            gl_.glOrtho( -6.0 * (float)width / ( (float)height * 2 ),
                         6.0 * (float)width / ( (float)height * 2 ),
                         -3.0, 3.0, -10.0, 10.0 );
          }
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {
        try {
            /** Do the OpenGL stuff */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

            float[] no_mat = { 0.0f, 0.0f, 0.0f, 1.0f };
            float[] mat_ambient = { 0.7f, 0.7f, 0.7f, 1.0f };
            float[] mat_ambient_color = { 0.8f, 0.8f, 0.2f, 1.0f };
            float[] mat_diffuse = { 0.1f, 0.5f, 0.8f, 1.0f };
            float[] mat_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] no_shininess = { 0.0f };
            float[] low_shininess = { 5.0f };
            float[] high_shininess = { 100.0f };
            float[] mat_emission = {0.3f, 0.2f, 0.2f, 0.0f };

            /**
             * draw sphere in first row, first column
             *  diffuse reflection only; no ambient or specular  
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( -3.75f, 3.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, no_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, no_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, no_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, no_mat );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

            /*  draw sphere in first row, second column
             *  diffuse and specular reflection; low shininess; no ambient
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( -1.25f, 3.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, no_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, low_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, no_mat );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

            /*  draw sphere in first row, third column
             *  diffuse and specular reflection; high shininess; no ambient
             */
            gl_.glPushMatrix();
            gl_.glTranslatef ( 1.25f, 3.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, no_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, high_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, no_mat );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

            /*  draw sphere in first row, fourth column
             *  diffuse reflection; emission; no ambient or specular reflection
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( 3.75f, 3.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, no_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, no_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, no_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, mat_emission );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

            /*  draw sphere in second row, first column
             *  ambient and diffuse reflection; no specular  
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( -3.75f, 0.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_ambient );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, no_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, no_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, no_mat );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();
            
            /*  draw sphere in second row, second column
             *  ambient, diffuse and specular reflection; low shininess
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( -1.25f, 0.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_ambient );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, low_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, no_mat );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

            /*  draw sphere in second row, third column
             *  ambient, diffuse and specular reflection; high shininess
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( 1.25f, 0.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_ambient );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, high_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, no_mat );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

            /*  draw sphere in second row, fourth column
             *  ambient and diffuse reflection; emission; no specular
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( 3.75f, 0.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_ambient );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, no_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, no_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, mat_emission );
            shapes.solidSphere( 1.0, 16, 16);
            gl_.glPopMatrix();
                     
            /*  draw sphere in third row, first column
             *  colored ambient and diffuse reflection; no specular  
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( -3.75f, -3.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_ambient_color );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, no_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, no_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, no_mat );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

            /*  draw sphere in third row, second column
             *  colored ambient, diffuse and specular reflection; low shininess
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( -1.25f, -3.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_ambient_color );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, low_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, no_mat );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

            /*  draw sphere in third row, third column
             *  colored ambient, diffuse and specular reflection; high shininess
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( 1.25f, -3.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_ambient_color );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, high_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, no_mat );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

            /*  draw sphere in third row, fourth column
             *  colored ambient and diffuse reflection; emission; no specular
             */
            gl_.glPushMatrix();
            gl_.glTranslatef( 3.75f, -3.0f, 0.0f );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat_ambient_color );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, no_mat );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, no_shininess );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_EMISSION, mat_emission );
            shapes.solidSphere( 1.0, 16, 16 );
            gl_.glPopMatrix();

          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
