/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: mipmap.java.stub,v 1.12 1998/10/19 12:16:39 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'mipmap' from the Red Book
 *
 * $Log: mipmap.java.stub,v $
 * Revision 1.12  1998/10/19 12:16:39  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.11  1998/10/02 12:56:49  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.10  1998/06/07 13:58:20  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.9  1998/03/30 11:48:11  descarte
 * Added applet resource handling.
 *
 * Revision 1.8  1998/03/29 16:04:51  descarte
 * Upgraded to Java 1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.7  1998/03/24 22:57:02  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.6  1998/01/04 19:59:03  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.5  1998/01/03 16:43:46  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.4  1998/01/01 18:13:55  descarte
 * Converted demo to GLUTtonish GLEventListener interface.
 *
 * Revision 1.3  1997/09/09 14:27:22  descarte
 * Added copyright information.
 *
 * Revision 1.2  1997/08/26 00:20:12  descarte
 * Moved demo to demoTemplate structure.
 *
 * Revision 1.1  1997/08/17 10:19:11  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class mipmapapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 500;

    /** The mipmapboard data */
    private byte[] mipmapImage32 = new byte[32 * 32 * 4];
    private byte[] mipmapImage16 = new byte[16 * 16 * 4];
    private byte[] mipmapImage8 = new byte[8 * 8 * 4];
    private byte[] mipmapImage4 = new byte[4 * 4 * 4];
    private byte[] mipmapImage2 = new byte[2 * 2 * 4];
    private byte[] mipmapImage1 = new byte[1 * 1 * 4];

    /** Texture IDs */
    int[] texName = new int[1];

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Mipmapping" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: mipmap.java.stub,v 1.12 1998/10/19 12:16:39 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of ``mipmapping''\n" );
        sb.append( "in OpenGL. Mipmapping is a way of creating pre-scaled, or\n" );
        sb.append( "pre-filtered textures at various levels of detail. Therefore\n" );
        sb.append( "when a larger or smaller texture is needed, a mipmap texture\n" );
        sb.append( "can be substituted in quickly instead of having to calculate\n" );
        sb.append( "the new texel on the fly. This example illustrates mipmapping\n" );
        sb.append( "at various distances by mapping a solid colour texture map onto\n" );
        sb.append( "a polygon stretching off into the distance. Each colour band\n" );
        sb.append( "indicates a different mipmap.\n\n" );
        sb.append( "Hit ``Escape'' to terminate this demo." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            /** Some misc setup */
            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_FLAT );
            gl_.glEnable( GL.GL_DEPTH_TEST );

            gl_.glTranslatef( 0.0f, 0.0f, -3.6f );
            makeImages();
            gl_.glPixelStorei( GL.GL_UNPACK_ALIGNMENT, 1 );

            /** Create a new texture */
            gl_.glGenTextures( 1, texName );
            gl_.glBindTexture( GL.GL_TEXTURE_2D, texName[0] );
            gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S,
                                GL.GL_REPEAT );
            gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T,
                                GL.GL_REPEAT );
            gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER,
                                GL.GL_NEAREST );
            gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER,
                                GL.GL_NEAREST_MIPMAP_NEAREST );
            gl_.glTexImage2D( GL.GL_TEXTURE_2D, 0, GL.GL_RGBA, 
                             32, 32, 0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE,
                             mipmapImage32 );
            gl_.glTexImage2D( GL.GL_TEXTURE_2D, 1, GL.GL_RGBA, 
                             16, 16, 0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE,
                             mipmapImage16 );
            gl_.glTexImage2D( GL.GL_TEXTURE_2D, 2, GL.GL_RGBA, 
                             8, 8, 0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE,
                             mipmapImage8 );
            gl_.glTexImage2D( GL.GL_TEXTURE_2D, 3, GL.GL_RGBA, 
                             4, 4, 0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE,
                             mipmapImage4 );
            gl_.glTexImage2D( GL.GL_TEXTURE_2D, 4, GL.GL_RGBA, 
                             2, 2, 0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE,
                             mipmapImage2 );
            gl_.glTexImage2D( GL.GL_TEXTURE_2D, 5, GL.GL_RGBA, 
                             1, 1, 0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE,
                             mipmapImage1 );
            gl_.glTexEnvf( GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE,
                          GL.GL_DECAL );
            gl_.glEnable( GL.GL_TEXTURE_2D );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 60, (float)width / (float)height, 1, 30000 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Redraws the screen */
    public void display( GLDrawable component ) {
        try {
            /** Render a mipmap */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

            gl_.glBindTexture( GL.GL_TEXTURE_2D, texName[0] );
            gl_.glBegin( GL.GL_QUADS );
                gl_.glTexCoord2f( 0.0f, 0.0f ); gl_.glVertex3f( -2.0f, -1.0f, 0.0f );
                gl_.glTexCoord2f( 0.0f, 8.0f ); gl_.glVertex3f( -2.0f, 1.0f, 0.0f );
                gl_.glTexCoord2f( 8.0f, 8.0f ); gl_.glVertex3f( 2000.0f, 1.0f, -6000.0f );
                gl_.glTexCoord2f( 8.0f, 0.0f ); gl_.glVertex3f( 2000.0f, -1.0f, -6000.0f );
            gl_.glEnd();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Makes the mipmaps */
    private void makeImages() {

        int c;

        int index = 0;
        for ( int i = 0 ; i < 32 ; i++ ) {
            for ( int j = 0 ; j < 32 ; j++ ) {
                mipmapImage32[index] = (byte)255;
                mipmapImage32[index + 1] = (byte)255;
                mipmapImage32[index + 2] = (byte)0;
                mipmapImage32[index + 3] = (byte)255;
                index += 4;
              }
          }

        index = 0;
        for ( int i = 0 ; i < 16 ; i++ ) {
            for ( int j = 0 ; j < 16 ; j++ ) {
                mipmapImage16[index] = (byte)255;
                mipmapImage16[index + 1] = (byte)0;
                mipmapImage16[index + 2] = (byte)255;
                mipmapImage16[index + 3] = (byte)255;
                index += 4;
              }
          }
        
        index = 0;
        for ( int i = 0 ; i < 8 ; i++ ) {
            for ( int j = 0 ; j < 8 ; j++ ) {
                mipmapImage8[index] = (byte)255;
                mipmapImage8[index + 1] = (byte)0;
                mipmapImage8[index + 2] = (byte)0;
                mipmapImage8[index + 3] = (byte)255;
                index += 4;
              }
          }

        index = 0;
        for ( int i = 0 ; i < 4 ; i++ ) {
            for ( int j = 0 ; j < 4 ; j++ ) {
                mipmapImage4[index] = (byte)0;
                mipmapImage4[index + 1] = (byte)255;
                mipmapImage4[index + 2] = (byte)0;
                mipmapImage4[index + 3] = (byte)255;
                index += 4;
              }
          }
        
        index = 0;
        for ( int i = 0 ; i < 2 ; i++ ) {
            for ( int j = 0 ; j < 2 ; j++ ) {
                mipmapImage2[index] = (byte)0;
                mipmapImage2[index + 1] = (byte)0;
                mipmapImage2[index + 2] = (byte)255;
                mipmapImage2[index + 3] = (byte)255;
                index += 4;
              }
          }

        index = 0;
        mipmapImage1[index] = (byte)255;
        mipmapImage1[index + 1] = (byte)255;
        mipmapImage1[index + 2] = (byte)255;
        mipmapImage1[index + 3] = (byte)255;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
