/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: polyoff.java.stub,v 1.14 1998/10/19 12:16:43 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'polyoff' from the Red Book
 *
 * $Log: polyoff.java.stub,v $
 * Revision 1.14  1998/10/19 12:16:43  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.13  1998/10/02 12:56:54  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.12  1998/06/07 13:58:21  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.11  1998/03/30 11:48:16  descarte
 * Added applet resource handling.
 *
 * Revision 1.10  1998/03/29 15:40:31  descarte
 * Upgraded to Java 1.1 listeners
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.9  1998/03/24 22:57:07  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.8  1998/01/04 19:59:14  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.7  1998/01/03 16:43:50  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.6  1998/01/01 17:36:15  descarte
 * Converted demo across to GLUTtonish GLEventListener interface.
 *
 * Revision 1.5  1997/09/09 14:28:49  descarte
 * Added copyright information.
 *
 * Revision 1.4  1997/09/02 23:34:42  descarte
 * Added GLContextException traps.
 *
 * Revision 1.3  1997/08/25 23:53:50  descarte
 * Fixed description bug.
 *
 * Revision 1.2  1997/08/25 23:46:37  descarte
 * Moved demo to demoTemplate structure.
 *
 * Revision 1.1  1997/08/18 22:42:19  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class polyoffapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 500;

    /** Stuff */
    int list = -1;
    int spinx = 0,
        spiny = 0;
    float tdist = 0.0f;
    float polyFactor = 1.0f;
    float polyUnits = 1.0f;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Polygon Offset" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: polyoff.java.stub,v 1.14 1998/10/19 12:16:43 descarte Exp descarte $" );
      }

    /** Returns the description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates a ratified OpenGL extension\n" );
        sb.append( "known as ``Polygon Offset'', which allows us to modify the\n" );
        sb.append( "the depth information for polygons. In this example, we are\n" );
        sb.append( "rendering a solid sphere with normal depth information, then\n" );
        sb.append( "rendering a wireframe sphere on top of it. Without polygon\n" );
        sb.append( "offset, the wireframe may, or may not, be clearly visible\n" );
        sb.append( "but would almost vertainly exhibit an artifact known as\n" );
        sb.append( "``stitching''. With polygon offset, we can alter the depth\n" );
        sb.append( "information of the wireframe sphere to make it appear completely\n" );
        sb.append( "``outside'' the solid sphere.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    t,T - Move towards and back from the sphere\n" );
        sb.append( "    u,U - Increase and decrease polyunits\n" );
        sb.append( "    f,F - Increase and decrease polyfactor\n" );
        sb.append( "    ESC - Quits this demonstration" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {
        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Some basic display code to render the scene */
        float[] light_ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
        float[] light_diffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] light_specular= { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] light_position = { 1.0f, 1.0f, 1.0f, 0.0f };
        float[] global_ambient = { 0.2f, 0.2f, 0.2f, 1.0f };

        list = gl_.glGenLists( 1 );
        gl_.glNewList( list, GL.GL_COMPILE );
            shapes.solidSphere( 1.0, 20, 12 );
        gl_.glEndList();

        gl_.glEnable( GL.GL_DEPTH_TEST );

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, light_ambient );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, light_diffuse );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_SPECULAR, light_specular );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, light_position );

        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, global_ambient );
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 45.0, (float)width / (float)height, 1.0, 10.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        glu_.gluLookAt( 0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 );
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {
        /** Setup the various lighting and material properties */
        float[] mat_ambient = { 0.8f, 0.8f, 0.8f, 1.0f };
        float[] mat_diffuse = { 1.0f, 0.0f, 0.5f, 1.0f };
        float[] mat_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] gray = { 0.8f, 0.8f, 0.8f, 1.0f };
        float[] black = { 0.0f, 0.0f, 0.0f, 1.0f };

        /** Do the OpenGL stuff */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
        gl_.glPushMatrix();
        gl_.glTranslatef( 0.0f, 0.0f, (float)tdist );
        gl_.glRotatef( (float)spinx, 1.0f, 0.0f, 0.0f );
        gl_.glRotatef( (float)spiny, 0.0f, 1.0f, 0.0f );

        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, gray );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, black );
        gl_.glMaterialf( GL.GL_FRONT, GL.GL_SHININESS, 0.0f );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_POLYGON_OFFSET_FILL );
        gl_.glPolygonOffset( polyFactor, polyUnits );
        gl_.glCallList( list );
        gl_.glDisable( GL.GL_POLYGON_OFFSET_FILL );

        gl_.glDisable( GL.GL_LIGHTING );
        gl_.glDisable( GL.GL_LIGHT0 );
        gl_.glColor3f( 1.0f, 1.0f, 1.0f );
        gl_.glPolygonMode( GL.GL_FRONT_AND_BACK, GL.GL_LINE );
        gl_.glCallList( list );
        gl_.glPolygonMode( GL.GL_FRONT_AND_BACK, GL.GL_FILL );

        gl_.glPopMatrix();
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
        if ( evt.getKeyChar() == 't' ) {
            if ( tdist < 4.0f ) {
                tdist += 0.5f;
              }
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'T' ) {
            if ( tdist > -5.0f ) {
                tdist -= 0.5f;
              }
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'F' ) {
            polyFactor += 0.1f;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'f' ) {
            polyFactor -= 0.1f;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'U' ) {
            polyUnits += 1.0f;
            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'u' ) {
            polyUnits -= 1.0f;
            glc.repaint();
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
