/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: scene.java.stub,v 1.11 1998/10/19 12:16:46 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'scene' from the Red Book
 *
 * $Log: scene.java.stub,v $
 * Revision 1.11  1998/10/19 12:16:46  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.10  1998/10/02 12:56:58  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.9  1998/06/07 13:58:21  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.8  1998/03/30 11:48:19  descarte
 * Added applet resource handling.
 *
 * Revision 1.7  1998/03/29 16:33:29  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory]
 *
 * Revision 1.6  1998/03/24 22:57:10  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.5  1998/01/04 19:59:18  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.4  1998/01/03 16:43:54  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.3  1998/01/01 22:01:07  descarte
 * Converted demo across to GLEventListener interface.
 *
 * Revision 1.2  1997/09/09 14:30:33  descarte
 * Added copyright information.
 *
 * Revision 1.1  1997/09/09 09:20:33  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;
import com.hermetica.util3d.*;

public class sceneapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 500;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Ambiently Lit Scene" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: scene.java.stub,v 1.11 1998/10/19 12:16:46 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the OpenGL\n" );
        sb.append( "lighting model. The objects are drawn with a grey\n" );
        sb.append( "material and are illuminated with a single light source.\n\n" );
        sb.append( "Hit ``Escape'' to quit this demo." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glClearAccum( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_SMOOTH );

            /** Specify the lighting for the scene */
            float[] light_ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
            float[] light_diffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] light_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] light_position = { 1.0f, 1.0f, 1.0f, 0.0f };

            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, light_ambient );
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, light_diffuse );
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_SPECULAR, light_specular );
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, light_position );

            gl_.glEnable( GL.GL_DEPTH_TEST );
            gl_.glEnable( GL.GL_LIGHTING );
            gl_.glEnable( GL.GL_LIGHT0 );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        if ( width <= height ) {
            gl_.glOrtho( -2.5, 2.5, -2.5 * (float)height / (float)width,
                        2.5 * (float)height / (float)width,
                        -10.0, 10.0 );
          } else {
            gl_.glOrtho( -2.5 * (float)width / (float)height,
                         2.5 * (float)width / (float)height,
                         -2.5, 2.5, -10.0, 10.0 );
          }
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {
        try {
            /** Do the OpenGL stuff */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

            gl_.glPushMatrix();
                gl_.glRotatef( 20.0f, 1.0f, 0.0f, 0.0f );
                gl_.glPushMatrix();
                    gl_.glTranslatef( -0.75f, 0.5f, 0.0f );
                    gl_.glRotatef( 90.0f, 1.0f, 0.0f, 0.0f );
                    shapes.solidTorus( 0.275, 0.85, 15, 15 );
                gl_.glPopMatrix();
    
                gl_.glPushMatrix();
                    gl_.glTranslatef( -0.75f, -0.50f, 0.0f );
                    gl_.glRotatef( 270.0f, 1.0f, 0.0f, 0.0f );
                    shapes.solidCone( 1.0, 2.0, 15, 15 );
                gl_.glPopMatrix();
    
                gl_.glPushMatrix();
                    gl_.glTranslatef( 0.75f, 0.0f, -1.0f );
                    shapes.solidSphere( 1.0, 15, 15 );
                gl_.glPopMatrix();
            gl_.glPopMatrix();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
