/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: select.java.stub,v 1.13 1998/10/19 12:16:47 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'select' from the Red Book
 *
 * $Log: select.java.stub,v $
 * Revision 1.13  1998/10/19 12:16:47  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.12  1998/10/02 12:56:59  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/06/07 13:58:22  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.10  1998/03/30 11:48:21  descarte
 * Added applet resource handling.
 *
 * Revision 1.9  1998/03/29 16:44:38  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.8  1998/03/24 22:57:11  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.7  1998/03/11 15:41:23  descarte
 * Corrected demo. There was actually nothing apparently wrong with Magician's
 * handling of selection buffers, simply that additional horrendous type-casting
 * was required to get the correct z-values since they are scaled into the
 * range of 2^32 - 1. Since Java doesn't have unsigned variables, this needs
 * to be twos-complemented then divided through. Blech.
 * There are precision problems with this, notably that 0 comes out at
 * -4.656613E-10 which is pretty close to zero for all intents and
 * purposes!
 *
 * Revision 1.6  1998/01/04 19:59:20  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.5  1998/01/03 16:43:55  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.4  1997/09/09 14:30:49  descarte
 * Added copyright information.
 *
 * Revision 1.3  1997/09/02 23:53:47  descarte
 * Added GLContextException traps.
 *
 * Revision 1.2  1997/08/26 00:30:18  descarte
 * Moved demo to demoTemplate structure.
 *
 * Revision 1.1  1997/08/17 11:38:42  descarte
 * Initial revision
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class selectapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 200,
                             HEIGHT = 200;

    /** The size of the selection buffer */
    private static final int BUFSIZE = 512;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Selection Buffer" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: select.java.stub,v 1.13 1998/10/19 12:16:47 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration exercises the ``selection buffer'' present\n" );
        sb.append( "in OpenGL. We firstly render the geometry to the selection buffer\n" );
        sb.append( "which can then be queried to see which items of geometry intersect\n" );
        sb.append( "with others. The results are then displayed on System.err\n\n" );
        sb.append( "The ``Escape'' key will quit this demonstration." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Initialize the GLComponent */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_FLAT );
            gl_.glEnable( GL.GL_DEPTH_TEST );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles resizing the viewport */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 40, (float)4 / (float)3, 1, 100 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        glu_.gluLookAt( 7.5f, 7.5f, 12.5f, 2.5f, 2.5f, 
                       -5.0f, 0.0f, 1.0f, 0.0f );
      }

    /** Displays the scene */
    public void display( GLDrawable component ) {
        try {
            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
            drawScene();
            selectObjects();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Draws the scene */
    private void drawScene() throws OpenGLException {

        /** Reset the view parameters */
        reshape( glc, 0, 0, glc.getSize().width, glc.getSize().height );

        /** Draw the scene */
        gl_.glColor3f( 0.0f, 1.0f, 0.0f );
        drawTriangle( 2.0f, 2.0f, 3.0f, 2.0f, 2.5f, 3.0f, -5.0f );
        gl_.glColor3f( 1.0f, 0.0f, 0.0f );
        drawTriangle( 2.0f, 7.0f, 3.0f, 7.0f, 2.5f, 8.0f, -5.0f );
        gl_.glColor3f( 1.0f, 1.0f, 0.0f );
        drawTriangle( 2.0f, 2.0f, 3.0f, 2.0f, 2.5f, 3.0f, 0.0f );
        drawTriangle( 2.0f, 2.0f, 3.0f, 2.0f, 2.5f, 3.0f, -10.0f );
        drawViewVolume( 0.0f, 5.0f, 0.0f, 5.0f, 0.0f, 10.0f );
      }

    /** Selects the objects within the scene */
    private void selectObjects() throws OpenGLException {

        int[] selectBuf = new int[BUFSIZE];
        int hits = 0;

        gl_.glSelectBuffer( BUFSIZE, selectBuf );
        gl_.glRenderMode( GL.GL_SELECT );

        gl_.glInitNames();
        gl_.glPushName( 0 );

        gl_.glPushMatrix();
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glOrtho( 0.0f, 5.0f, 0.0f, 5.0f, 0.0f, 10.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glLoadName( 1 );
        drawTriangle( 2.0f, 2.0f, 3.0f, 2.0f, 2.5f, 3.0f, -5.0f );
        gl_.glLoadName( 2 );
        drawTriangle( 2.0f, 7.0f, 3.0f, 7.0f, 2.5f, 8.0f, -5.0f );
        gl_.glLoadName( 3 );
        drawTriangle( 2.0f, 2.0f, 3.0f, 2.0f, 2.5f, 3.0f, 0.0f );
        drawTriangle( 2.0f, 2.0f, 3.0f, 2.0f, 2.5f, 3.0f, -10.0f );
        gl_.glPopMatrix();

        hits = gl_.glRenderMode( GL.GL_RENDER );
        processHits( hits, selectBuf );
      }

    /** Processes the hits from the selection process */
    private void processHits( int hits, int[] buffer ) {

        int index = 0;
        long names = 0;

        System.err.println( "Hits = " + hits );

        for ( int i = 0 ; i < hits ; i++ ) {
            System.err.println( "\tnumber of names for hit = " +
                                buffer[index] );
            names = buffer[index];
            index++;
            System.err.println( "\t\tz1 is " + 
                                (float)( (float)~buffer[index] / (float)0x7fffffffl ) );
            index++;
            System.err.println( "\t\tz2 is " + 
                                (float)( (float)~buffer[index] / (float)0x7fffffffl ) );
            index++;
            System.err.print( "\t\t\tthe name is " );
            for ( int j = 0 ; j < names ; j++ ) {
                System.err.print( buffer[index] + " " );
                index++;
              }
            System.err.println();
          }
      }

    /** Draws a triangle */
    private void drawTriangle( float x1, float y1, float x2,
                               float y2, float x3, float y3,
                               float z ) throws OpenGLException {
        gl_.glBegin( GL.GL_TRIANGLES );
            gl_.glVertex3f( x1, y1, z );
            gl_.glVertex3f( x2, y2, z );
            gl_.glVertex3f( x3, y3, z );
        gl_.glEnd();
      }

    /** Draws the view volume */
    private void drawViewVolume( float x1, float x2, float y1, float y2,
                                 float z1, float z2 ) throws OpenGLException {

        gl_.glColor3f( 1.0f, 1.0f, 1.0f );
        gl_.glBegin( GL.GL_LINE_LOOP );
            gl_.glVertex3f( x1, y1, -z1 );
            gl_.glVertex3f( x2, y1, -z1 );
            gl_.glVertex3f( x2, y2, -z1 );
            gl_.glVertex3f( x1, y2, -z1 );
        gl_.glEnd();

        gl_.glBegin( GL.GL_LINE_LOOP );
            gl_.glVertex3f( x1, y1, -z2 );
            gl_.glVertex3f( x2, y1, -z2 );
            gl_.glVertex3f( x2, y2, -z2 );
            gl_.glVertex3f( x1, y2, -z2 );
        gl_.glEnd();

        gl_.glBegin( GL.GL_LINES );
            gl_.glVertex3f( x1, y1, -z1 );
            gl_.glVertex3f( x1, y1, -z2 );
            gl_.glVertex3f( x1, y2, -z1 );
            gl_.glVertex3f( x1, y2, -z2 );
            gl_.glVertex3f( x2, y1, -z1 );
            gl_.glVertex3f( x2, y1, -z2 );
            gl_.glVertex3f( x2, y2, -z1 );
            gl_.glVertex3f( x2, y2, -z2 );
        gl_.glEnd();
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
