/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: stencil.java.stub,v 1.14 1998/10/19 12:16:49 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'stencil' from the Red Book
 *
 * $Log: stencil.java.stub,v $
 * Revision 1.14  1998/10/19 12:16:49  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.13  1998/10/02 12:57:01  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.12  1998/06/07 13:58:22  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.11  1998/03/30 11:48:23  descarte
 * Added applet resource handling.
 *
 * Revision 1.10  1998/03/29 16:09:32  descarte
 * Upgraded to java 1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.9  1998/03/24 22:57:13  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.8  1998/01/04 19:59:22  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.7  1998/01/03 16:43:57  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.6  1998/01/01 17:21:49  descarte
 * Converted demo across to using GLUTtonish GLEventListener interface.
 *
 * Revision 1.5  1997/11/02 18:44:00  descarte
 * Altered demo to new abstracted CoreGL classes.
 * Fixed bug with this demo that had stencil buffer initialization code in the
 * wrong place. Stencilling now works entirely as per spec.
 *
 * Revision 1.4  1997/09/09 14:31:16  descarte
 * Added copyright information.
 *
 * Revision 1.3  1997/09/02 23:53:36  descarte
 * Added GLContextException traps.
 *
 * Revision 1.2  1997/08/26 00:26:39  descarte
 * Moved demo to demoTemplate structure.
 *
 * Revision 1.1  1997/08/19 21:04:23  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class stencilapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 400,
                             HEIGHT = 400;

    /** Some definitions */
    private static final int YELLOWMAT = 1;
    private static final int BLUEMAT = 2;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Stencilling" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: stencil.java.stub,v 1.14 1998/10/19 12:16:49 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the stencil buffer\n" );
        sb.append( "in OpenGL. The stencil buffer acts as a mask against which\n" );
        sb.append( "things are rendered, for example, oddly shaped viewports, such\n" );
        sb.append( "as a windscreen, or binoculars, can be rendered using a stencil\n" );
        sb.append( "mask in the stencil buffer. This example renders two tori outwith\n" );
        sb.append( "the stencil buffer, and a blue tori within it. A black diamond \n" );
        sb.append( "shape should be cut out of the two yellow tori, with the blue\n" );
        sb.append( "torus sitting in the middle of the diamond.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    ESC - Quit the demonstration" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );
        cap.setStencilBits( 1 );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        /**#windowListener */

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {

//            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
//            gl_.glShadeModel( GL.GL_FLAT );

        /** Some basic display code to render the scene */
        float[] yellow_diffuse = { 0.7f, 0.7f, 0.0f, 1.0f };
        float[] yellow_specular = { 1.0f, 1.0f, 1.0f, 1.0f };

        float[] blue_diffuse = { 0.1f, 0.1f, 0.7f, 1.0f };
        float[] blue_specular = { 0.1f, 1.0f, 1.0f, 1.0f };

        float[] position_one = { 1.0f, 1.0f, 1.0f, 0.0f };

        gl_.glNewList( YELLOWMAT, GL.GL_COMPILE );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, yellow_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, yellow_diffuse );
            gl_.glMaterialf( GL.GL_FRONT, GL.GL_SHININESS, 64.0f );
        gl_.glEndList();

        gl_.glNewList( BLUEMAT, GL.GL_COMPILE );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, blue_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, blue_diffuse );
            gl_.glMaterialf( GL.GL_FRONT, GL.GL_SHININESS, 45.0f );
        gl_.glEndList();

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position_one );

        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glShadeModel( GL.GL_SMOOTH );

        gl_.glClearStencil( 0x0 );
        gl_.glEnable( GL.GL_STENCIL_TEST );
      }

    /** 
     * Handle resizing the viewport. This also needs to recreate the stencil
     * buffer
     */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        if ( width <= height ) {
            glu_.gluOrtho2D( -3.0, 3.0, -3.0 * (float)height / (float)width,
                            3.0 * (float)height / (float)width );
          } else {
            glu_.gluOrtho2D( -3.0 * (float)width / (float)height,
                             3.0 * (float)width / (float)height, -3.0, 3.0 );
          }
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();

        gl_.glClear( GL.GL_STENCIL_BUFFER_BIT );
        gl_.glStencilFunc( GL.GL_ALWAYS, 0x1, 0x1 );
        gl_.glStencilOp( GL.GL_REPLACE, GL.GL_REPLACE, GL.GL_REPLACE );
        gl_.glBegin( GL.GL_QUADS );
            gl_.glVertex2f( -1.0f, 0.0f );
            gl_.glVertex2f( 0.0f, 1.0f );
            gl_.glVertex2f( 1.0f, 0.0f );
            gl_.glVertex2f( 0.0f, -1.0f );
        gl_.glEnd();

        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 45.0, (float)width / (float)height, 3.0, 7.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glTranslatef( 0.0f, 0.0f, -5.0f );
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        /** Draw the blue sphere where the stencil is 1 */
        gl_.glStencilFunc( GL.GL_EQUAL, 0x1, 0x1 );
        gl_.glStencilOp( GL.GL_KEEP, GL.GL_KEEP, GL.GL_KEEP );
        gl_.glCallList( BLUEMAT );
        shapes.solidSphere( 0.5, 15, 15 );

        /** Draw the tori where the stencil is not 1 */
        gl_.glStencilFunc( GL.GL_NOTEQUAL, 0x1, 0x1 );
        gl_.glPushMatrix();
        gl_.glRotatef( 45.0f, 0.0f, 0.0f, 1.0f );
        gl_.glRotatef( 45.0f, 0.0f, 1.0f, 0.0f );
        gl_.glCallList( YELLOWMAT );
        shapes.solidTorus( 0.275, 0.85, 15, 15 );
        gl_.glPushMatrix();
            gl_.glRotatef( 90.0f, 1.0f, 0.0f, 0.0f );
            shapes.solidTorus( 0.275, 0.85, 15, 15 );
        gl_.glPopMatrix();

        gl_.glPopMatrix();
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
