/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: surface.java.stub,v 1.11 1998/10/19 12:16:51 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'surface' from the Red Book
 *
 * $Log: surface.java.stub,v $
 * Revision 1.11  1998/10/19 12:16:51  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.10  1998/10/02 12:57:03  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.9  1998/06/07 13:58:22  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.8  1998/03/30 11:48:25  descarte
 * Added applet resource handling.
 *
 * Revision 1.7  1998/03/29 16:35:56  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.6  1998/03/24 22:57:15  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.5  1998/01/04 19:59:25  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.4  1998/01/03 16:43:59  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.3  1998/01/01 22:05:43  descarte
 * Converted demo across to GLEventListener interface.
 *
 * Revision 1.2  1997/11/04 21:03:21  descarte
 * Added vertex drawing toggle plus a z-buffer fix.
 *
 * Revision 1.1  1997/11/04 20:30:11  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class surface extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 500;

    /** Control points for the bezier surface */
    float[][][] ctlpoints = new float[4][4][3];

    /** NURBS renderer object */
    GLUNurbs theNurb = glu_.gluNewNurbsRenderer();

    /** Flag to indicate whether or not we render the vertices */
    private boolean showPoints = false;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        surface d = new surface();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public surface() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "NURBS Surfaces" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: surface.java.stub,v 1.11 1998/10/19 12:16:51 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the NURBS\n" );
        sb.append( "routines present within the CoreGLU library to draw complex\n" );
        sb.append( "surfaces easily within OpenGL. This has the bonus of being\n" );
        sb.append( "a ``true'' surface, and not a surface approximated with lots\n" );
        sb.append( "of polygons.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    c/C - Toggle vertices display\n    ESC - Exit\n" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            /** Setup the scene */
            float[] mat_diffuse = { 0.7f, 0.7f, 0.7f, 1.0f };
            float[] mat_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] mat_shininess = { 100.0f };

            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_SMOOTH );

            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat_specular );
            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, mat_shininess );

            gl_.glEnable( GL.GL_LIGHTING );
            gl_.glEnable( GL.GL_LIGHT0 );
            gl_.glEnable( GL.GL_DEPTH_TEST );
            gl_.glEnable( GL.GL_AUTO_NORMAL );
            gl_.glEnable( GL.GL_NORMALIZE );

            /** Initialize the surface */
            init_surface();

            /** Setup the NURBS */
            theNurb.nurbsProperty( GLU.GLU_SAMPLING_TOLERANCE, 25.0f );
            theNurb.nurbsProperty( GLU.GLU_DISPLAY_MODE, GLU.GLU_FILL );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 45.0, (float)width / (float)height, 3.0, 8.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
        gl_.glTranslatef( 0.0f, 0.0f, -5.0f );
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {

        float knots[] = { 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f, 1.0f };

        try {
            /** Do the OpenGL stuff */
           gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

            gl_.glPushMatrix();
                gl_.glRotatef( 330.0f, 1.0f, 0.0f, 0.0f );
                gl_.glScalef( 0.5f, 0.5f, 0.5f );

                theNurb.beginSurface();
                theNurb.nurbsSurface( 8, knots, 8, knots,
                                      4 * 3, 3, ctlpoints,
                                      4, 4, GL.GL_MAP2_VERTEX_3 );
                theNurb.endSurface();

                /** Render the vertices, if desired */
                if ( showPoints ) {
                    gl_.glPointSize( 5.0f );
                    gl_.glDisable( GL.GL_LIGHTING );
                    gl_.glColor3f( 1.0f, 1.0f, 1.0f );
                    gl_.glBegin( GL.GL_POINTS );
                        for ( int i = 0 ; i < 4 ; i++ ) {
                            for ( int j = 0 ; j < 4 ; j++ ) {
                                gl_.glVertex3f( ctlpoints[i][j][0],
                                                ctlpoints[i][j][1],
                                                ctlpoints[i][j][2] );
                              }
                          }
                    gl_.glEnd();
                    gl_.glEnable( GL.GL_LIGHTING );
                  }

            gl_.glPopMatrix();
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Initializes the surface */
    private void init_surface() {

        for ( int u = 0 ; u < 4 ; u++ ) {
            for ( int v = 0 ; v < 4 ; v++ ) {
                ctlpoints[u][v][0] = (float)( 2.0 * ( u - 1.5 ) );
                ctlpoints[u][v][1] = (float)( 2.0 * ( v - 1.5 ) );

                if ( ( u == 1 || u == 2 ) && ( v == 1 || v == 2 ) ) {
                    ctlpoints[u][v][2] = 3.0f;
                  } else {
                    ctlpoints[u][v][2] = -3.0f;
                  }
              }
          }
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

            return;
          }
        if ( evt.getKeyChar() == 'c' ||
             evt.getKeyChar() == 'C' ) {
            if ( showPoints ) {
                showPoints = false;
              } else {
                showPoints = true;
              }
            glc.repaint();
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
