/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: teapots.java.stub,v 1.13 1998/10/19 12:16:52 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'teapots' from the Red Book
 *
 * $Log: teapots.java.stub,v $
 * Revision 1.13  1998/10/19 12:16:52  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.12  1998/10/02 12:57:05  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/06/07 13:58:22  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.10  1998/03/30 11:48:26  descarte
 * Added applet resource handling.
 *
 * Revision 1.9  1998/03/29 14:43:59  descarte
 * Upgraded to Java-1.1 listeners.
 * Upgraded to GLComponentFactory.
 *
 * Revision 1.8  1998/03/24 22:57:16  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.7  1998/01/04 19:59:26  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.6  1998/01/03 16:44:00  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.5  1998/01/01 17:59:20  descarte
 * Converted demo across to GLUTtonish GLEventListener interface.
 *
 * Revision 1.4  1997/09/09 14:31:42  descarte
 * Added copyright information.
 *
 * Revision 1.3  1997/09/08 22:44:30  descarte
 * Brought into line with demoTemplate structure and corrected for new NURBS
 * handling.
 *
 * Revision 1.2  1997/09/02 23:27:13  descarte
 * Added GLContextException traps.
 *
 * Revision 1.1  1997/08/18 23:56:57  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class teapots extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 600;

    /** Display list index for a teapot */
    private int teapotList = -1;

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Teapots" );
      }

    /** Returns some version information about this demo */
    public String getVersion() {
        return new String( "$Id: teapots.java.stub,v 1.13 1998/10/19 12:16:52 descarte Exp descarte $" );
      }

    /** Returns a meaningful description of this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration renders an array of teapots generated\n" );
        sb.append( "with the solidTeapot() method in the shapes class, which\n" );
        sb.append( "in turn uses the underlying NURBS handling present in \n" );
        sb.append( "OpenGL. The various materials used are, from left to right\n" );
        sb.append( "top to bottom:\n\n" );
        sb.append( "    Emerald       Brass       Black           Black\n" );
        sb.append( "    Jade          Bronze      Cyan            Cyan\n" );
        sb.append( "    Obsidian      Chrome      Green           Green\n" );
        sb.append( "    Pearl         Copper      Red             Red\n" );
        sb.append( "    Ruby          Gold        White           White\n" );
        sb.append( "    Turquoise     Silver      Yellow Plastic  Yellow Rubber\n" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        teapots d = new teapots();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public teapots() {
        /**#start */
      }

    public void startDemo() {
        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        gl_.glClearColor( 0.0f, 0.1f, 0.1f, 0.0f );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Some basic display code to render the scene */
        float[] ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
        float[] diffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] specular = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] position = { 0.0f, 3.0f, 3.0f, 0.0f };
        float[] lmodel_ambient = { 0.2f, 0.2f, 0.2f, 1.0f };
        float[] local_view = { 0.0f };

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, ambient );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, position );
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );
        gl_.glLightModelfv( GL.GL_LIGHT_MODEL_LOCAL_VIEWER, local_view );

        gl_.glFrontFace( GL.GL_CW );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glEnable( GL.GL_AUTO_NORMAL );
        gl_.glEnable( GL.GL_NORMALIZE );

        /** Make a teapot display list */
        teapotList = gl_.glGenLists( 1 );
        gl_.glNewList( teapotList, GL.GL_COMPILE );
            shapes.solidTeapot( 1.0 );
        gl_.glEndList();
      }

    /** Handles resizing of the viewport */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glOrtho( 0.0, 16.0, 0.0, 16.0 * HEIGHT / WIDTH,
                    -10.0, 10.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
      }

    /** Renders a single teapot */
    private void renderTeapot( float x, float y, float ambr, float ambg,
                               float ambb, float difr, float difg, float difb,
                               float specr, float specg, float specb,
                               float shine ) throws OpenGLException {

        float[] mat = new float[4];

        gl_.glPushMatrix();
        gl_.glTranslatef( x, y, 0.0f );

        mat[0] = ambr; mat[1] = ambg; mat[2] = ambb; mat[3] = 1.0f;
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, mat );

        mat[0] = difr; mat[1] = difg; mat[2] = difb;
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, mat);

        mat[0] = specr; mat[1] = specg; mat[2] = specb;
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, mat);

        gl_.glMaterialf( GL.GL_FRONT, GL.GL_SHININESS, (float)( shine * 128.0 ) );

        gl_.glCallList( teapotList );

        gl_.glPopMatrix();
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {
        /** Do the OpenGL stuff */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        /** Render a lot of teapots! */
        renderTeapot( 2.0f, 17.0f, 0.0215f, 0.1745f, 0.0215f,
                      0.07568f, 0.61424f, 0.07568f, 0.633f, 
                      0.727811f, 0.633f, 0.6f );
        renderTeapot( 2.0f, 14.0f, 0.135f, 0.2225f, 0.1575f,
                      0.54f, 0.89f, 0.63f, 0.316228f, 0.316228f, 0.316228f, 0.1f );
        renderTeapot( 2.0f, 11.0f, 0.05375f, 0.05f, 0.06625f,
                      0.18275f, 0.17f, 0.22525f, 0.332741f, 0.328634f, 0.346435f, 0.3f );
        renderTeapot( 2.0f, 8.0f, 0.25f, 0.20725f, 0.20725f,
                      1f, 0.829f, 0.829f, 0.296648f, 0.296648f, 0.296648f, 0.088f );
        renderTeapot( 2.0f, 5.0f, 0.1745f, 0.01175f, 0.01175f,
                      0.61424f, 0.04136f, 0.04136f, 0.727811f, 0.626959f, 0.626959f, 0.6f );
        renderTeapot( 2.0f, 2.0f, 0.1f, 0.18725f, 0.1745f,
                      0.396f, 0.74151f, 0.69102f, 0.297254f, 0.30829f, 0.306678f, 0.1f );
        renderTeapot( 6.0f, 17.0f, 0.329412f, 0.223529f, 0.027451f,
                      0.780392f, 0.568627f, 0.113725f, 0.992157f, 0.941176f, 0.807843f,
                      0.21794872f );
        renderTeapot( 6.0f, 14.0f, 0.2125f, 0.1275f, 0.054f,
                      0.714f, 0.4284f, 0.18144f, 0.393548f, 0.271906f, 0.166721f, 0.2f );
        renderTeapot( 6.0f, 11.0f, 0.25f, 0.25f, 0.25f,
                      0.4f, 0.4f, 0.4f, 0.774597f, 0.774597f, 0.774597f, 0.6f );
        renderTeapot( 6.0f, 8.0f, 0.19125f, 0.0735f, 0.0225f,
                      0.7038f, 0.27048f, 0.0828f, 0.256777f, 0.137622f, 0.086014f, 0.1f );
        renderTeapot( 6.0f, 5.0f, 0.24725f, 0.1995f, 0.0745f,
                      0.75164f, 0.60648f, 0.22648f, 0.628281f, 0.555802f, 0.366065f, 0.4f );
        renderTeapot( 6.0f, 2.0f, 0.19225f, 0.19225f, 0.19225f,
                      0.50754f, 0.50754f, 0.50754f, 0.508273f, 0.508273f, 0.508273f, 0.4f );
        renderTeapot( 10.0f, 17.0f, 0.0f, 0.0f, 0.0f, 0.01f, 0.01f, 0.01f,
                      0.50f, 0.50f, 0.50f, .25f );
        renderTeapot( 10.0f, 14.0f, 0.0f, 0.1f, 0.06f, 0.0f, 0.50980392f, 0.50980392f,
                      0.50196078f, 0.50196078f, 0.50196078f, .25f );
        renderTeapot( 10.0f, 11.0f, 0.0f, 0.0f, 0.0f,
                      0.1f, 0.35f, 0.1f, 0.45f, 0.55f, 0.45f, .25f );
        renderTeapot( 10.0f, 8.0f, 0.0f, 0.0f, 0.0f, 0.5f, 0.0f, 0.0f,
                      0.7f, 0.6f, 0.6f, .25f );
        renderTeapot( 10.0f, 5.0f, 0.0f, 0.0f, 0.0f, 0.55f, 0.55f, 0.55f,
                      0.70f, 0.70f, 0.70f, .25f );
        renderTeapot( 10.0f, 2.0f, 0.0f, 0.0f, 0.0f, 0.5f, 0.5f, 0.0f,
                      0.60f, 0.60f, 0.50f, .25f );
        renderTeapot( 14.0f, 17.0f, 0.02f, 0.02f, 0.02f, 0.01f, 0.01f, 0.01f,
                      0.4f, 0.4f, 0.4f, .078125f );
        renderTeapot( 14.0f, 14.0f, 0.0f, 0.05f, 0.05f, 0.4f, 0.5f, 0.5f,
                      0.04f, 0.7f, 0.7f, .078125f );
        renderTeapot( 14.0f, 11.0f, 0.0f, 0.05f, 0.0f, 0.4f, 0.5f, 0.4f,
                      0.04f, 0.7f, 0.04f, .078125f );
        renderTeapot( 14.0f, 8.0f, 0.05f, 0.0f, 0.0f, 0.5f, 0.4f, 0.4f,
                      0.7f, 0.04f, 0.04f, .078125f );
        renderTeapot( 14.0f, 5.0f, 0.05f, 0.05f, 0.05f, 0.5f, 0.5f, 0.5f,
                      0.7f, 0.7f, 0.7f, .078125f );
        renderTeapot( 14.0f, 2.0f, 0.05f, 0.05f, 0.0f, 0.5f, 0.5f, 0.4f,
                      0.7f, 0.7f, 0.04f, .078125f );
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
