/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: texgen.java.stub,v 1.11 1998/10/19 12:16:54 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'texgen' from the Red Book
 *
 * $Log: texgen.java.stub,v $
 * Revision 1.11  1998/10/19 12:16:54  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.10  1998/10/02 12:57:07  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.9  1998/06/07 13:58:22  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.8  1998/03/30 11:48:27  descarte
 * Added applet resource handling.
 *
 * Revision 1.7  1998/03/29 15:57:19  descarte
 * Upgraded to Java-1.1 listeners
 * Upgraded to GLComponentFactory
 *
 * Revision 1.6  1998/03/24 22:57:17  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.5  1998/01/04 19:59:28  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.4  1998/01/03 16:44:02  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.3  1998/01/01 17:47:25  descarte
 * Converted demo across to GLUTtonish GLEventListener interface.
 *
 * Revision 1.2  1997/09/09 14:32:07  descarte
 * Added copyright information.
 *
 * Revision 1.1  1997/09/08 23:59:57  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class texgen extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 256,
                             HEIGHT = 256;

    /** The texgenboard data */
    private int stripeImageWidth = 32;
    private byte[] stripeImage = new byte[4 * stripeImageWidth];

    /** Texture IDs */
    int[] texName = new int[1];

    /** Texture coordinate generation planes */
    float[] xequalzero = { 1.0f, 0.0f, 0.0f, 0.0f };
    float[] slanted = { 1.0f, 1.0f, 1.0f, 0.0f };
    float[] currentCoeff = xequalzero;
    int currentPlane = GL.GL_OBJECT_PLANE;
    int currentGenMode = GL.GL_OBJECT_LINEAR;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        texgen d = new texgen();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public texgen() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Contour Mapping" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: texgen.java.stub,v 1.11 1998/10/19 12:16:54 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates some sophisticated texture mapping\n" );
        sb.append( "capabilities provided by OpenGL. We generate a stripe texture\n" );
        sb.append( "that is then mapped onto the teapot using automatic texture\n" );
        sb.append( "coordinate generation using various methods to simulate contour\n" );
        sb.append( "lines on the object.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    e,E - Generates ``eye linear'' texture coordinates\n" );
        sb.append( "    o,O - Generates ``object linear'' texture coordinates\n" );
        sb.append( "    s,S - ``Slants'' the texture coordinates\n" );
        sb.append( "    x,X - ``Unslants'' the texture coordinates\n" );
        sb.append( "    ESC - Quits the demonstration" );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the glc.getContext().capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        /** Some misc setup */
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
        gl_.glShadeModel( GL.GL_SMOOTH );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        makeStripeImage();
        gl_.glPixelStorei( GL.GL_UNPACK_ALIGNMENT, 1 );

        /** Create a new texture */
        gl_.glGenTextures( 1, texName );
        gl_.glBindTexture( GL.GL_TEXTURE_1D, texName[0] );
        gl_.glTexParameteri( GL.GL_TEXTURE_1D, GL.GL_TEXTURE_WRAP_S,
                            GL.GL_REPEAT );
        gl_.glTexParameteri( GL.GL_TEXTURE_1D, GL.GL_TEXTURE_MAG_FILTER,
                            GL.GL_LINEAR );
        gl_.glTexParameteri( GL.GL_TEXTURE_1D, GL.GL_TEXTURE_MIN_FILTER,
                            GL.GL_LINEAR );
        gl_.glTexImage1D( GL.GL_TEXTURE_1D, 0, GL.GL_RGBA,
                         stripeImageWidth, 0, GL.GL_RGBA,
                         GL.GL_UNSIGNED_BYTE,
                         stripeImage );

        gl_.glTexEnvf( GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, 
                      GL.GL_MODULATE );
        gl_.glTexGeni( GL.GL_S, GL.GL_TEXTURE_GEN_MODE, currentGenMode );
        gl_.glTexGenfv( GL.GL_S, currentPlane, currentCoeff );

        gl_.glEnable( GL.GL_TEXTURE_GEN_S );
        gl_.glEnable( GL.GL_TEXTURE_1D );
        gl_.glEnable( GL.GL_CULL_FACE );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_AUTO_NORMAL );
        gl_.glEnable( GL.GL_NORMALIZE );
        gl_.glFrontFace( GL.GL_CW );
        gl_.glCullFace( GL.GL_BACK );
        gl_.glMaterialf( GL.GL_FRONT, GL.GL_SHININESS, 64.0f );
      }

    /** Handles resizing the GLComponent */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        if ( width <= height ) {
            gl_.glOrtho( -3.5, 3.5, 
                        -3.5 * (float)height / (float)width,
                        3.5 * (float)height / (float)width,
                        -3.5, 3.5 );
          } else {
            gl_.glOrtho( -3.5 * (float)width / (float)height,
                         3.5 * (float)width / (float)height,
                         -3.5, 3.5, -3.5, 3.5 );
          }
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Redraws the screen */
    public void display( GLDrawable component ) {
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        gl_.glPushMatrix();
            gl_.glRotatef( 45.0f, 0.0f, 0.0f, 1.0f );
            gl_.glBindTexture( GL.GL_TEXTURE_1D, texName[0] );
            shapes.solidTeapot( 2.0f );
        gl_.glPopMatrix();
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Makes a texgenboard pattern */
    private void makeStripeImage() {
        for ( int j = 0 ; j < stripeImageWidth ; j++ ) {
            stripeImage[4 * j] = (byte)( ( j <= 4 ) ? 255 : 0 );
            stripeImage[4 * j + 1] = (byte)( ( j > 4 ) ? 255 : 0 );
            stripeImage[4 * j + 2] = (byte)( 0 );
            stripeImage[4 * j + 3] = (byte)( 255 );
          }
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

            return;
          }
        if ( evt.getKeyChar() == 'e' ||
             evt.getKeyChar() == 'E' ) {

            currentGenMode = GL.GL_EYE_LINEAR;
            currentPlane = GL.GL_EYE_PLANE;

            glc.getContext().makeCurrent();

            gl_.glTexGeni( GL.GL_S, GL.GL_TEXTURE_GEN_MODE,
                          currentGenMode );
            gl_.glTexGenfv( GL.GL_S, currentPlane, currentCoeff );

            glc.getContext().unlock();

            glc.repaint();
            return;
          }
        if ( evt.getKeyChar() == 'o' ||
             evt.getKeyChar() == 'O' ) {
            currentGenMode = GL.GL_OBJECT_LINEAR;
            currentPlane = GL.GL_OBJECT_PLANE;
            glc.getContext().makeCurrent();

            gl_.glTexGeni( GL.GL_S, GL.GL_TEXTURE_GEN_MODE,
                          currentGenMode );
            gl_.glTexGenfv( GL.GL_S, currentPlane, currentCoeff );

            glc.getContext().unlock();

            glc.repaint();

            return;
          }
        if ( evt.getKeyChar() == 's' ||
             evt.getKeyChar() == 'S' ) {
            currentCoeff = slanted;
            glc.getContext().makeCurrent();

            gl_.glTexGenfv( GL.GL_S, currentPlane,
                           currentCoeff );

            glc.getContext().unlock();

            glc.repaint();

            return;
          }
        if ( evt.getKeyChar() == 'x' ||
             evt.getKeyChar() == 'X' ) {
            currentCoeff = xequalzero;
            glc.getContext().makeCurrent();

            gl_.glTexGenfv( GL.GL_S, currentPlane,
                           currentCoeff );

            glc.getContext().unlock();
                            
            glc.repaint();
            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
