/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: texturesurf.java.stub,v 1.12 1998/10/19 12:16:56 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'texturesurf' from the Red Book
 *
 * $Log: texturesurf.java.stub,v $
 * Revision 1.12  1998/10/19 12:16:56  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.11  1998/10/02 12:57:09  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.10  1998/06/07 13:58:23  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.9  1998/03/30 11:48:29  descarte
 * Added applet resource handling.
 *
 * Revision 1.8  1998/03/29 16:23:32  descarte
 * Upgraded Java-1.1 listeners
 * Upgraded GLComponentFactory
 *
 * Revision 1.7  1998/03/24 22:57:19  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.6  1998/01/04 19:59:31  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.5  1998/01/03 16:44:04  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.4  1998/01/01 18:05:23  descarte
 * Converted demo to GLUTtonish GLEventListener interface.
 *
 * Revision 1.3  1997/09/09 14:32:37  descarte
 * Added copyright information.
 *
 * Revision 1.2  1997/09/08 23:12:49  descarte
 * Fixed descriptive text.
 *
 * Revision 1.1  1997/09/08 23:11:05  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;

public class texturesurf extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 500;

    /** Control points for the bezier surface */
    float[][][] ctrlpoints = {
        { 
            { -1.5f, -1.5f, 4.0f }, 
            { -0.5f, -1.5f, 2.0f },
            { 0.5f, -1.5f, -1.0f },
            { 1.5f, -1.5f, 2.0f }
          },
        {
            { -1.5f, -0.5f, 1.0f },
            { -0.5f, -0.5f, 3.0f }, 
            { 0.5f, -0.5f, 0.0f },
            { 1.5f, -0.5f, -1.0f }
          },
        {
            { -1.5f, 0.5f, 4.0f },
            { -0.5f, 0.5f, 0.0f },
            { 0.5f, 0.5f, 3.0f },
            { 1.5f, 0.5f, 4.0f }
          },
        {
            { -1.5f, 1.5f, -2.0f },
            { -0.5f, 1.5f, -2.0f },
            { 0.5f, 1.5f, 0.0f },
            { 1.5f, 1.5f, -1.0f }
          }
      };

    float[][][] texpts = {
        {
            { 0.0f, 0.0f },
            { 0.0f, 1.0f }
          },
        {
            { 1.0f, 0.0f },
            { 1.0f, 1.0f }
          }
      };

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        texturesurf d = new texturesurf();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public texturesurf() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Textured Bezier Surface" );
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: texturesurf.java.stub,v 1.12 1998/10/19 12:16:56 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the NURBS functions\n" );
        sb.append( "defined within OpenGL for drawing Bezier surfaces. A two-\n" );
        sb.append( "dimensional evaluator function is being used in this demo\n" );
        sb.append( "to draw a surface using the defined control points.\n" );
        sb.append( "Furthermore, a sneakier evaluator method, glEvalMesh2() is\n" );
        sb.append( "being used to generate the vertices and texture coordinates.\n\n" );
        sb.append( "Hit ``Escape'' to quit this demo." );
        return new String( sb );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register a keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Build the surface */
        gl_.glMap2f( GL.GL_MAP2_VERTEX_3, 0, 1, 3, 4, 0, 1, 12, 4,
                    ctrlpoints );
        gl_.glMap2f( GL.GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2,
                    texpts );
        gl_.glEnable( GL.GL_MAP2_VERTEX_3 );
        gl_.glEnable( GL.GL_MAP2_TEXTURE_COORD_2 );
        gl_.glMapGrid2f( 20, 0.0f, 1.0f, 20, 0.0f, 1.0f );

        /** Initialize the texture */
        initTexture();

        gl_.glEnable( GL.GL_DEPTH_TEST );
      }

    /** Handles resizing the viewport */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        if ( width <= height ) {
            gl_.glOrtho( -4.0, 4.0, -4.0 * (float)height / (float)width,
                        4.0 * (float)height / (float)width,
                        -4.0, 4.0 );
          } else {
            gl_.glOrtho( -4.0 * (float)width / (float)height,
                         4.0 * (float)width / (float)height,
                         -4.0, 4.0, -4.0, 4.0 );
          }
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Initializes the texturing in the scene */
    private void initTexture() {

        byte[] image = new byte[3 * 64 * 64];
        float ti, tj;

        /** Build the texture */
        for ( int i = 0 ; i < 64 ; i++ ) {
            ti = (float)( 2.0 * Math.PI * i / 64 );
            for ( int j = 0 ; j < 64 ; j++ ) {
                tj = (float)( 2.0 * Math.PI * j / 64 );

                image[3 * ( 64 * i + j )] = (byte)( 127 * ( 1.0 + Math.sin( ti ) ) );
                image[3 * ( 64 * i + j ) + 1] = (byte)( 127 * ( 1.0 + Math.cos( 2 * tj ) ) );
                image[3 * ( 64 * i + j ) + 2] = (byte)( 127 * ( 1.0 + Math.cos( ti + tj ) ) );
              }
          }

        /** Setup the texturing within OpenGL */
        gl_.glTexEnvf( GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL );
        gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_REPEAT );
        gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_REPEAT );
        gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST );
        gl_.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST );
        gl_.glTexImage2D( GL.GL_TEXTURE_2D, 0, GL.GL_RGB, 64, 64, 0,
                     GL.GL_RGB, GL.GL_UNSIGNED_BYTE, image );
        gl_.glEnable( GL.GL_TEXTURE_2D );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glShadeModel( GL.GL_FLAT );
      }

    /** Draws the scene */
    public void display( GLDrawable component ) {
        /** Do the OpenGL stuff */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
        gl_.glColor3f( 1.0f, 1.0f, 1.0f );

        gl_.glPushMatrix();
        gl_.glRotatef( 85.0f, 1.0f, 1.0f, 1.0f );
        gl_.glEvalMesh2( GL.GL_FILL, 0, 20, 0, 20 );
        gl_.glPopMatrix();
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

            return;
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
