/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/**
 * $Id: varray.java.stub,v 1.13 1998/10/19 12:16:58 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * 'varray' from the Red Book
 *
 * $Log: varray.java.stub,v $
 * Revision 1.13  1998/10/19 12:16:58  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.12  1998/10/02 12:57:12  descarte
 * Changed GLComponentListener to GLEventListener. Changed reshape() arguments.
 *
 * Revision 1.11  1998/06/12 18:35:41  descarte
 * Updated demo for 1.0 build and testing.
 *
 * Revision 1.10  1998/06/07 13:58:23  descarte
 * Converted from COM.hermetica.* to com.hermetica.* hierarchy.
 *
 * Revision 1.9  1998/03/30 11:48:32  descarte
 * Added applet resource handling.
 *
 * Revision 1.8  1998/03/24 22:57:22  descarte
 * Converted from beta 3 to beta 4. Converted to GLDrawable interface.
 *
 * Revision 1.7  1998/01/04 19:59:34  descarte
 * Converted to new pipeline class names, ie, AbstractGL -> GL; GL -> CoreGL.
 *
 * Revision 1.6  1998/01/03 16:44:06  descarte
 * Corrected 'add()' method to have explicit placement argument for MSVM port.
 *
 * Revision 1.5  1997/09/09 14:33:26  descarte
 * Added copyright information.
 *
 * Revision 1.4  1997/09/03 00:04:35  descarte
 * Added GLContextException traps.
 *
 * Revision 1.3  1997/08/27 23:40:42  descarte
 * Added left-click toggle for changing between interleaved and non-interleaved
 * rendering.
 *
 * Revision 1.2  1997/08/26 00:46:52  descarte
 * Moved demo to demoTemplate structure.
 *
 * Revision 1.1  1997/08/24 14:54:10  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos.redbook;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class varrayapp extends Applet implements demoTemplate, GLEventListener, MouseListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private GL gl_ = null;
    private GLU glu_ = null;

    private CoreGL coregl_ = new CoreGL();
    private CoreGLU coreglu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 350,
                             HEIGHT = 350;

    /** Some variables */
    private static final int POINTER = 1;
    private static final int INTERLEAVED = 2;

    private static final int DRAWARRAY = 1;
    private static final int ARRAYELEMENT = 2;
    private static final int DRAWELEMENTS = 3;

    private static int setupMethod = POINTER;
    private static int derefMethod = DRAWARRAY;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        startDemo();
        gl_ = coregl_;
        glu_ = coreglu_;
      }

    /** Returns the version information for this demo */
    public String getVersion() {
        return new String( "$Id: varray.java.stub,v 1.13 1998/10/19 12:16:58 descarte Exp descarte $" );
      }

    /** Returns a description for this demo */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration demonstrates the use of vertex arrays\n" );
        sb.append( "for rapidly drawing objects, colour, texture coordinates and\n" );
        sb.append( "so on at high speed.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    Click - Toggles interleaved vertex array rendering\n" );
        sb.append( "    CTRL Click - Cycles through vertex drawing styles\n" );
        sb.append( "    ESC - Quits demonstration" );
        return new String( sb );
      }

    /** Exit handling */
    public void setExit( boolean mode ) {
        callExit = mode;
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Vertex Arrays" );
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register a GLEventListener */
        glc.addGLEventListener( this );

        /** Add the keyboard, mouse and window listeners */
        glc.addKeyListener( this );
        /**#windowListener */
        glc.addMouseListener( this );

        /** Tell the GLComponent to start processing */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {

        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Setup some vertex array pointers */
        setupPointers();
      }

    /** Sets up vertex array pointers */
    private void setupPointers() {

        int[] vertices = {
            25, 25,
            100, 325,
            175, 25,
            175, 325,
            250, 25,
            325, 325
          };

        float[] colours = {
            1.0f, 0.2f, 0.2f,
            0.2f, 0.2f, 1.0f,
            0.8f, 1.0f, 0.2f,
            0.75f, 0.75f, 0.75f,
            0.35f, 0.35f, 0.35f,
            0.5f, 0.5f, 0.5f
          };

        gl_.glEnableClientState( GL.GL_VERTEX_ARRAY );
        gl_.glEnableClientState( GL.GL_COLOR_ARRAY );

        gl_.glVertexPointer( 2, GL.GL_INT, 0, vertices );
        gl_.glColorPointer( 3, GL.GL_FLOAT, 0, colours );
      }

    /** Sets up the interleaved arrays */
    private void setupInterleave() {
        float[] intertwined = {
            1.0f, 0.2f, 1.0f, 100.0f, 100.0f, 0.0f,
            1.0f, 0.2f, 0.2f, 0.0f, 200.0f, 0.0f,
            1.0f, 1.0f, 0.2f, 100.0f, 300.0f, 0.0f,
            0.2f, 1.0f, 0.2f, 200.0f, 300.0f, 0.0f,
            0.2f, 1.0f, 1.0f, 300.0f, 200.0f, 0.0f,
            0.2f, 0.2f, 1.0f, 200.0f, 100.0f, 0.0f
          };

        gl_.glInterleavedArrays( GL.GL_C3F_V3F, 0, intertwined );
      }

    /** Handles viewport reshaping */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluOrtho2D( 0.0, width, 0.0, height );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }


    /** Draws the scene */
    public void display( GLDrawable component ) {

        gl_.glClear( GL.GL_COLOR_BUFFER_BIT );

        if ( derefMethod == DRAWARRAY ) {
            gl_.glDrawArrays( GL.GL_TRIANGLES, 0, 6 );
          } else {
            if ( derefMethod == ARRAYELEMENT ) {
                gl_.glBegin( GL.GL_TRIANGLES );
                    gl_.glArrayElement( 2 );
                    gl_.glArrayElement( 3 );
                    gl_.glArrayElement( 5 );
                gl_.glEnd();
              } else {
                if ( derefMethod == DRAWELEMENTS ) {
                    int[] indices = { 0, 1, 3, 4 };

                    gl_.glDrawElements( GL.GL_POLYGON, 4, 
                                       GL.GL_UNSIGNED_INT, indices );
                  }
              }
          }
      }

    /** Returns the OpenGL pipeline to the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Mouse listeners */
    public void mouseClicked( MouseEvent evt ) {
        if ( evt.isControlDown() ) {
            if ( derefMethod == DRAWARRAY ) {
                derefMethod = ARRAYELEMENT;
              } else {
                if ( derefMethod == ARRAYELEMENT ) {
                    derefMethod = DRAWELEMENTS;
                  } else {
                    if ( derefMethod == DRAWELEMENTS ) {
                        derefMethod = DRAWARRAY;
                      }
                  }
              }
          } else {
            /** Left mouse click handler */
            if ( setupMethod == POINTER ) {
                setupMethod = INTERLEAVED;
                glc.getContext().makeCurrent();
                setupInterleave();
                glc.getContext().unlock();
              } else {
                if ( setupMethod == INTERLEAVED ) {
                    setupMethod = POINTER;
                    glc.getContext().makeCurrent();
                    setupPointers();
                    glc.getContext().unlock();
                  }
              }
          }
        glc.repaint();
      }

    public void mouseEntered( MouseEvent evt ) {
      }

    public void mouseExited( MouseEvent evt ) {
      }

    public void mousePressed( MouseEvent evt ) {
      }

    public void mouseReleased( MouseEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
