/**
 * $Id: sharedContext.java.stub,v 1.9 1998/10/19 00:53:15 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Demonstration that allows two components to share a single display list.
 * This is essentially testGL with two components. The first component only
 * creates the display list, but both execute it.
 *
 * $Log: sharedContext.java.stub,v $
 * Revision 1.9  1998/10/19 00:53:15  descarte
 * .Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.8  1998/10/02 11:22:20  descarte
 * Changed GLComponentListener to GLEventListener
 * Altered arguments to reshape() methods
 *
 * Revision 1.7  1998/06/08 17:26:19  descarte
 * Changed width from 200 to 400 for correct applet display.
 *
 * Revision 1.6  1998/03/30 00:55:42  descarte
 * Commented out debugging statements.
 *
 * Revision 1.5  1998/03/30 00:51:09  descarte
 * Added applet resource handling.
 *
 * Revision 1.4  1998/03/28 13:38:29  descarte
 * Upgraded to use GLComponentFactory
 * Upgraded to JDK-1.1 listener interfaces.
 *
 * Revision 1.3  1998/03/26 00:47:15  descarte
 *  Removed GLComponent abstract class hack.
 *
 * Revision 1.2  1998/03/16 10:36:29  descarte
 * Converted from old beta 4 syntax to new beta 4 syntax.
 *
 * Revision 1.1  1998/03/15 16:13:00  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class sharedContext extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent parentComponent = null;
    GLComponent childComponent = null;

    /** The dimensions of the demo */
    private int WIDTH = 200,
                HEIGHT = 200;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        sharedContext d = new sharedContext();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public sharedContext() {
        gl_ = coregl_;
        /**#start */
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Shared Contexts" );
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getVersion() {
        return new String( "$Id: sharedContext.java.stub,v 1.9 1998/10/19 00:53:15 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration draws a single polygon the size of the window\n" );
        sb.append( "shaded black to blue. The polygon is actually created as a single\n" );
        sb.append( "display list in the left-hand GLComponent. However, due to the\n" );
        sb.append( "ability to share display lists, the second component on the\n" );
        sb.append( "right-hand can also execute the display list.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH * 2;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Alligator Descartes <descarte@arcana.co.uk>" );
      }

    public void startDemo() {
        setLayout( new GridLayout( 1, 2 ) );

        /** Create the OpenGL Component */
        parentComponent = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );
        childComponent = GLComponentFactory.createGLComponent( parentComponent, WIDTH, HEIGHT );

        /** Setup the GUI */
        add( parentComponent );
        add( childComponent );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = parentComponent.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        parentComponent.initialize();
        parentComponent.addGLEventListener( this );

        /** Register the keyboard listeners */
        parentComponent.addKeyListener( this );
        childComponent.addKeyListener( this );

        this.addWindowListener( this );

        /** Setup the second component */
        childComponent.getContext().setCapabilities( cap );
        childComponent.addGLEventListener( this );
        childComponent.initialize();
      }

    public void initialize( GLDrawable component ) {
        /** Only create the display list in the first component */
        if ( component == parentComponent ) {
            gl_.glNewList( 100, GL.GL_COMPILE );
                gl_.glDisable( GL.GL_DEPTH_TEST );
                gl_.glShadeModel( GL.GL_SMOOTH );
                gl_.glBegin( GL.GL_POLYGON );
                    gl_.glColor3f( 0.0f, 0.0f, 0.0f );
                    gl_.glVertex3f( -1000, 1000, -999 );
                    gl_.glVertex3f( 1000, 1000, -999 );
                    gl_.glColor3f( 0.0f, 0.0f, 1.0f );
                    gl_.glVertex3f( 1000, -1000, -999 );
                    gl_.glVertex3f( -1000, -1000, -999 );
                gl_.glEnd();
            gl_.glEndList();
          }
      }

    /** Resizes the viewport */
    public void reshape( GLDrawable component, int x, int y,
                         int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0, 1.0, -1.0, 1.0, 1.0, 1000 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }
        
    public void display( GLDrawable component ) {
        gl_.glClearColor( 0.0f, 0.0f, 1.0f, 1.0f );
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        gl_.glEnable( GL.GL_DEPTH_TEST );

        gl_.glCallList( 100 );
      }

    /** Returns a GL pipeline for GLEventListener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            parentComponent.destroy();
            childComponent.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        parentComponent.destroy();
        childComponent.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( parentComponent.isRunning() ) {
            parentComponent.resume();
          }
        if ( childComponent.isRunning() ) {
            childComponent.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( parentComponent.isRunning() ) {
            parentComponent.suspend();
          }
        if ( childComponent.isRunning() ) {
            childComponent.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( parentComponent.isRunning() ) {
            parentComponent.suspend();
          }
        if ( childComponent.isRunning() ) {
            childComponent.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( parentComponent.isRunning() ) {
            parentComponent.resume();
          }
        if ( childComponent.isRunning() ) {
            childComponent.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        parentComponent.destroy();
        childComponent.destroy();
      }
  }
