/**
 * $Id: solarSystem.java.stub,v 1.6 1998/10/19 00:53:19 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Demonstrates picking moving objects
 *
 * $Log: solarSystem.java.stub,v $
 * Revision 1.6  1998/10/19 00:53:19  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.5  1998/10/02 11:23:51  descarte
 * Changed GLComponentListener to GLEventListener
 * Altered arguments to reshape() method
 *
 * Revision 1.4  1998/03/30 15:50:29  descarte
 * Removed static variable declarations. This caused multiple solar system
 * demos to report selection data!
 *
 * Revision 1.3  1998/03/30 00:55:24  descarte
 * Added applet resource handling.
 *
 * Revision 1.2  1998/03/28 13:49:16  descarte
 * Upgraded to GLComponentFactory
 * Upgraded to JDK-1.1 listeners.
 *
 * Revision 1.1  1998/03/27 01:50:33  descarte
 * Initial revision
 *
 * Revision 1.1  1998/03/24 16:55:21  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.magician.fonts.*;
import com.hermetica.util3d.*;

public class solarSystemapp extends Applet implements demoTemplate, GLEventListener, KeyListener, MouseMotionListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    /** The GLU pipeline */
    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** The display list for Mars */
    public int marsList = 0;

    /** The display list for Sol */
    public int solList = 1;

    /** The display list for Terra */
    public int terraList = 2;

    /** The display list for Luna */
    public int lunaList = 3;

    /** The currently picked planet and the previously picked planet */
    public int currentPlanet = 0;
    private int lastPlanet = 0;

    /** The rotation of Mars */
    public float marsRotation = 0.0f;

    /** The rotation of Terra */
    public float terraRotation = 0.0f;

    /** The rotation of Luna */
    public float lunaRotation = 0.0f;

    GLTimesRoman24BitmapFont timesRoman24 =
        new GLTimesRoman24BitmapFont();

    /** A quadric creation object */
    private GLUQuadric quadric;

    /** The select buffer */
    private int SELECT_BUFFER_SIZE = 32;
    private int[] selectBuffer = new int[SELECT_BUFFER_SIZE];

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        gl_ = coregl_;
        glu_ = coreglu_;
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    public String getName() {
        return new String( "Solar System" );
      }

    public String getVersion() {
        return new String( "$Id: solarSystem.java.stub,v 1.6 1998/10/19 00:53:19 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demo uses the OpenGL selection mechanisms to\n" );
        sb.append( "allow you to highlight different planets in a small\n" );
        sb.append( "( highly inaccurate! ) solar system model. Moving the\n" );
        sb.append( "mouse over each planet will display its name.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Alligator Descartes <descarte@arcana.co.uk>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the listener with the component */
        glc.addGLEventListener( this );

        /** Register a keyboard and a mouse listener */
        glc.addKeyListener( this );
        glc.addMouseMotionListener( this );
        /**#windowListener */

        /** Start the component processing events */
        glc.initialize();
        glc.start();
      }

    /** Perform some basic setup of the scene */
    public void initialize( GLDrawable component ) {

        /** Create the display lists */
        quadric = glu_.gluNewQuadric();
        glu_.gluQuadricDrawStyle( quadric, GLU.GLU_FILL );
        glu_.gluQuadricNormals( quadric, GLU.GLU_SMOOTH );
        glu_.gluQuadricOrientation( quadric, GLU.GLU_OUTSIDE );

        /** Create a display list of a red sphere for Mars */
        marsList = gl_.glGenLists( 1 );
        gl_.glNewList( marsList, GL.GL_COMPILE );
            gl_.glColor3f( 1.0f, 0.0f, 0.0f );
            shapes.solidSphere( 1, 16, 16 );
        gl_.glEndList();

        /** Create a display list of a red sphere for Sol */
        solList = gl_.glGenLists( 1 );
        gl_.glNewList( solList, GL.GL_COMPILE );
            gl_.glColor3f( 1.0f, 1.0f, 0.0f );
            shapes.solidSphere( 3, 16, 16 );
        gl_.glEndList();

        /** Create a display list of a sphere for Terra */
        terraList = gl_.glGenLists( 1 );
        gl_.glNewList( terraList, GL.GL_COMPILE );
            gl_.glColor3f( 0.0f, 0.5f, 0.75f );
            shapes.solidSphere( 0.8, 16, 16 );
        gl_.glEndList();

        /** Create a display list of a white sphere for Luna */
        lunaList = gl_.glGenLists( 1 );
        gl_.glNewList( lunaList, GL.GL_COMPILE );
            gl_.glColor3f( 1.0f, 1.0f, 1.0f );
            shapes.solidSphere( 0.25, 16, 16 );
        gl_.glEndList();

        /** Setup OpenGL lighting */
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_COLOR_MATERIAL );

        /** Set the clear colour for the framebuffer */
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );

        /** Enable depth testing and back-face culling */
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glEnable( GL.GL_CULL_FACE );
      }

    /** Resizes the viewport when the window is stretched */
    public void reshape( GLDrawable component, int x, int y,
                         int width, int height ) {

        float[] lightPosition = { 1.0f, 1.0f, 1.0f, 0.0f };

        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 60.0f, (float)height / (float)width, 1.0f, 128.0f );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightPosition );

        gl_.glTranslatef( 0.0f, 0.0f, -20.0f );
        gl_.glRotatef( 15.0f, 0.0f, 1.0f, 0.0f );
        gl_.glRotatef( 15.0f, 1.0f, 0.0f, 0.0f );
      }
        
    /** Renders the scene */
    public void display( GLDrawable component ) {

        /** Clear the frame buffer and depth buffer */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        /** Draw the scene */
        gl_.glPushMatrix();
            /** Draw Mars */
            gl_.glPushMatrix();
                gl_.glRotatef( marsRotation, 0.0f, 1.0f, 0.0f );
                gl_.glTranslatef( 0.0f, 0.0f, 10.0f );
                gl_.glCallList( marsList );
            gl_.glPopMatrix();

            /** Draw Sol */
            gl_.glPushMatrix();
                gl_.glCallList( solList );
            gl_.glPopMatrix();

            /** Draw Earth system */
            gl_.glPushMatrix();
                gl_.glRotatef( terraRotation, 0.0f, 1.0f, 0.0f );
                gl_.glTranslatef( 0.0f, 0.0f, 7.0f );
                gl_.glCallList( terraList );

                /** Draw Luna */
                gl_.glPushMatrix();
                    gl_.glRotatef( lunaRotation, 1.0f, 0.0f, 0.0f );
                    gl_.glRotatef( lunaRotation, 0.0f, 1.0f, 0.0f );
                    gl_.glRotatef( lunaRotation, 0.0f, 0.0f, 1.0f );
                    gl_.glTranslatef( 1.0f, 1.0f, 1.0f );
                    gl_.glCallList( lunaList );
                gl_.glPopMatrix();
            gl_.glPopMatrix();
        gl_.glPopMatrix();

        /** If a hit occurred on a planet, setup the text printing stuff */
        if ( currentPlanet != 0 ) {
            /** Disable lighting and depth-buffering. We don't want 'em */
            gl_.glDisable( GL.GL_DEPTH_TEST );
            gl_.glDisable( GL.GL_LIGHTING );

            /** Set the text colour to white */
            gl_.glColor3f( 1.0f, 1.0f, 1.0f );

            /** Extract the bottom-left corner from the current viewport size */
            double[] modelViewMatrix = new double[16];
            double[] projectionMatrix = new double[16];
            int[] viewport = new int[4];
            double[] win = new double[4];

            gl_.glGetIntegerv( GL.GL_VIEWPORT, viewport );
            gl_.glGetDoublev( GL.GL_MODELVIEW_MATRIX, modelViewMatrix );
            gl_.glGetDoublev( GL.GL_PROJECTION_MATRIX, projectionMatrix );
            glu_.gluUnProject( 10, 10, 1, modelViewMatrix, projectionMatrix,
                               viewport, win );

            /** Set the position to draw the string at */
            gl_.glRasterPos3d( win[0], win[1], win[2] );
          }

        /** 
         * Check to see if a selection hit has occurred and if so,
         * display the planet name
         */
        switch ( currentPlanet ) {
            case 1: {
                /** Draw the string */
                timesRoman24.drawString( "Mars" );
                break;
              }
            case 2: {
                /** Draw the string */
                timesRoman24.drawString( "Sol" );
                break;
              }
            case 3: {
                /** Draw the string */
                timesRoman24.drawString( "Terra" );
                break;
              }
            case 4: {
                /** Draw the string */
                timesRoman24.drawString( "Luna" );
                break;
              }
          }
        if ( currentPlanet != 0 ) {
            /** Re-enable lighting and depth-buffering */
            gl_.glEnable( GL.GL_LIGHTING );
            gl_.glEnable( GL.GL_DEPTH_TEST );
          }

        /** Rotate Mars */
        marsRotation++; 
        if ( marsRotation >= 360.0f ) {
            marsRotation -= 360.0f;
          }

        /** Rotate Terra */
        terraRotation += 2;
        if ( terraRotation >= 360.0f ) {
            terraRotation -= 360.0f;
          }

        /** Rotate Luna */
        lunaRotation += 5;
        if ( lunaRotation >= 360.0f ) {
            lunaRotation -= 360.0f;
          }
      }

    /** Handles picking */
    private int pick( int x, int y ) {
        int picked = 0;
        int hits = 0;

        /** The viewport dimensions */
        int[] viewport = new int[4];

        /** Setup the select buffer */
        gl_.glSelectBuffer( SELECT_BUFFER_SIZE, selectBuffer );

        /** Get the current viewport parameters */
        gl_.glGetIntegerv( GL.GL_VIEWPORT, viewport );

        /** Set the render mode to selection */
        gl_.glRenderMode( GL.GL_SELECT );
        gl_.glInitNames();
        gl_.glPushName( 0 );

        /** Setup a picking matrix and render into selection buffer */
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glPushMatrix();
            gl_.glLoadIdentity();
            glu_.gluPickMatrix( x, viewport[3] - y, 5.0f, 5.0f, viewport );
            glu_.gluPerspective( 60.0f, 
                                 (float)viewport[3] / (float)viewport[2],
                                 1.0f, 128.0f );
            gl_.glMatrixMode( GL.GL_MODELVIEW );
            gl_.glLoadIdentity();
            gl_.glTranslatef( 0.0f, 0.0f, -20.0f );
            gl_.glRotatef( 15.0f, 0.0f, 1.0f, 0.0f );
            gl_.glRotatef( 15.0f, 1.0f, 0.0f, 0.0f );

            gl_.glPushMatrix();
                /** Load the ``Mars'' name and draw it */
                gl_.glPushMatrix();
                    gl_.glRotatef( marsRotation, 0.0f, 1.0f, 0.0f );
                    gl_.glTranslatef( 0.0f, 0.0f, 10.0f );
                    gl_.glLoadName( 1 );
                    gl_.glCallList( marsList );
                gl_.glPopMatrix();

                /** Load the ``Sun'' name and draw it */
                gl_.glPushMatrix();
                    gl_.glLoadName( 2 );
                    gl_.glCallList( solList );
                gl_.glPopMatrix();

                /** Draw Earth system */
                gl_.glPushMatrix();
                    gl_.glLoadName( 3 );
                    gl_.glRotatef( terraRotation, 0.0f, 1.0f, 0.0f );
                    gl_.glTranslatef( 0.0f, 0.0f, 7.0f );
                    gl_.glCallList( terraList );

                    /** Draw Luna */
                    gl_.glPushMatrix();
                        gl_.glLoadName( 4 );
                        gl_.glRotatef( lunaRotation, 1.0f, 0.0f, 0.0f );
                        gl_.glRotatef( lunaRotation, 0.0f, 1.0f, 0.0f );
                        gl_.glRotatef( lunaRotation, 0.0f, 0.0f, 1.0f );
                        gl_.glTranslatef( 1.0f, 1.0f, 1.0f );
                        gl_.glCallList( lunaList );
                    gl_.glPopMatrix();
                gl_.glPopMatrix();
            gl_.glPopMatrix();

            gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glPopMatrix();
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glFlush();

        hits = gl_.glRenderMode( GL.GL_RENDER );
//        System.err.println( "Hits: " + hits );

        picked = 0;
        int index = 0;
        int depth = -1;
        for ( int i = 0 ; i < hits ; i++ ) {
            int save = GL.GL_FALSE;
            int numNames = selectBuffer[index];
//            System.err.println( "numNames: " + numNames );
            index++;

            if ( selectBuffer[index] <= depth ) {
                depth = selectBuffer[index];
                save = GL.GL_TRUE;
              }
            index++;

            if ( selectBuffer[index] / (float)0x7fffffffl <= depth ) {
                depth = selectBuffer[index];
                save = GL.GL_TRUE;
              }
            index++;

            if ( save == GL.GL_TRUE ) {
                picked = selectBuffer[index];
              }

//            System.err.println( "name: " + selectBuffer[index] );
            index += numNames;
          }
        return picked;
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** Handles mouse motion events */
    /** Handles mouse drag events */
    public void mouseDragged( MouseEvent evt ) {
      }

    /** Handles mouse motion events */
    public void mouseMoved( MouseEvent evt ) {
        /** Make the rendering context current */
        glc.getContext().makeCurrent();

        currentPlanet = pick( evt.getX(), evt.getY() );

        /** Unlock the rendering context */
        glc.getContext().unlock();

//        System.err.println( "pick(): " + currentPlanet );
        if ( ( currentPlanet != lastPlanet ) ) {
            repaint();
          }
        lastPlanet = currentPlanet;

        glc.getContext().setTraceStatus( false );
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
