/**
 * $Id: tentacle.java.stub,v 1.2 1998/10/19 00:53:20 descarte Exp descarte $
 *
 * Copyright (c)1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Magician demo program that draws a wiggling tentacle
 *
 * $Log: tentacle.java.stub,v $
 * Revision 1.2  1998/10/19 00:53:20  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.1  1998/07/09 09:26:07  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.magician.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class tentacleapp extends Applet implements demoTemplate, GLEventListener, KeyListener, WindowListener, MouseListener, MouseMotionListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** Angle of rotation. Used in animation */
    private float angle = 0.0f;

    /** Scale amount for each tentacle segment */
    private float scaleAmt = 0.9f;

    /** Base proportions for each tentacle segment */
    float[] baseSegmentSize = { 2.0f, 0.5f, 1.0f };

    /** Number of segments per tentacle */
    private int maxLevel = 12;

    /** The number of tentacles */
    private int numTentacles = 6;

    /** The diffuse tentacle colour */
    private float[] matTentacleDiffuse = { 1.0f, 0.0f, 1.0f, 1.0f };
    private float[] matSphereDiffuse = { 0.0f, 0.0f, 0.8f, 1.0f };

    /** Bend and curl values */
    private float bend = 0,
                  curl = 0;

    /** Mouse tracking variables */
    private int beginx = 0,
                beginy = 0;

    /** The trackball */
    private trackball ball = new trackball();

    /** Current quaternion rotation */
    private float[] curQuat = new float[4];

    /** Last quaternion rotation */
    private float[] lastQuat = new float[4];

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        gl_ = coregl_;
        glu_ = coreglu_;
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Tentacle" );
      }

    public String getVersion() {
        return new String( "$Id: tentacle.java.stub,v 1.2 1998/10/19 00:53:20 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration program generates a swirling and flexing\n" );
        sb.append( "creature with tentacles. You can increase the number of\n" );
        sb.append( "tentacles the creature has by hitting the 'i' key. Similarly,\n" );
        sb.append( "you can decrease the number of tentacles by hitting 'd'.\n" );
        sb.append( "The beastie can also be spun about by clicking and dragging\n" );
        sb.append( "the mouse.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Bill Thibault <tebo@csuhayward.edu>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the key listener to the component */
        glc.addKeyListener( this );

        /** Add the mouse listeners */
        glc.addMouseListener( this );
        glc.addMouseMotionListener( this );

        /** Add the window listener */
        /**#windowListener */

        /** Add the listener and initialize the component */
        glc.addGLEventListener( this );
        glc.initialize();
        glc.start();
      }

    /** Gets called exactly once when GLComponent.initialize() is invoked */
    public void initialize( GLDrawable component ) {
        /** Set the clear colour for the framebuffer */
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Setup some materials */
        float[] matSpecular = { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] matAmbient = { 0.2f, 0.2f, 0.2f, 1.0f };
        float[] matShininess = { 50.0f };

        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SPECULAR, matSpecular );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, matTentacleDiffuse );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_AMBIENT, matAmbient );
        gl_.glMaterialfv( GL.GL_FRONT, GL.GL_SHININESS, matShininess );

        /** Setup the scene lighting */
        float[] lightPosition = { 1.0f, 1.0f, 1.0f };
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightPosition );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, matTentacleDiffuse );

        /** Enable some states */
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glEnable( GL.GL_NORMALIZE );

        /** Initialize the trackball quaternion */
        curQuat = ball.buildQuaternion( 0.0f, 0.0f, 0.0f, 0.0f );
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y,
                         int width, int height ) {
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        glu_.gluPerspective( 65.0f, (float)width / (float)height, 1.0,  50.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Renders the scene */
    public void display( GLDrawable component ) {
        /** Clear the framebuffer */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );

        /** Draw the scene */
        float[][] m = new float[4][4];
        bend = 30 * (float)Math.sin( 7 * ( ( 2.0f * Math.PI ) / 360.0f ) * ( angle % 360 ) );
        curl = 30 * (float)Math.sin( 3 * ( ( 2.0f * Math.PI ) / 360.0f ) * ( angle % 360 ) );

        gl_.glPushMatrix();
            gl_.glLoadIdentity();
            gl_.glTranslatef( 0.0f, 0.0f, -25.0f );
            m = ball.buildMatrix( curQuat );
            gl_.glMultMatrixf( m );

            gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, matTentacleDiffuse );
            shapes.solidSphere( 2.0f, 6, 6 );
            for ( int j = 0 ; j < (int)( (float)numTentacles / 6 ) ; j++ ) {
                gl_.glPushMatrix();
                gl_.glRotatef( (float)j * ( 90.0f / ( (float)numTentacles / 6.0f ) ),
                               1.0f, 1.0f, 1.0f );
                for ( int i = 0 ; i < numTentacles ; i++ ) {
                    gl_.glPushMatrix();
                        switch ( i % 6 ) {
                            case 0: {
                                break;
                              }
                            case 1: {
                                gl_.glRotatef( -180, 0, 0, 1 );
                                break;
                              }
                            case 2: {
                                gl_.glRotatef( 90, 0, 0, 1 );
                                break;
                              }
                            case 3: {
                                gl_.glRotatef( -90, 0, 0, 1 );
                                break;
                              }
                            case 4: {
                                gl_.glRotatef( -90, 0, 1, 0 );
                                break;
                              }
                            case 5: {
                                gl_.glRotatef( 90, 0, 1, 0 );
                                break;
                              }
                            default: {
                                break;
                              }
                          }
                        whip( i, maxLevel );
                    gl_.glPopMatrix();
                  }
                gl_.glPopMatrix();
              }
        gl_.glPopMatrix();

        /** Update the angle */
        angle++;
      }

    /** Draws a tentacle */
    private void whip( int id, int level ) {
        float[] segsize = new float[3];
        float[] lightPos = new float[3];

        /** Compute the size of this segment. As ``level'' decreases, so does size */
        for ( int i = 0 ; i < 3 ; i++ ) {
            segsize[i] = baseSegmentSize[i] * (float)Math.pow( scaleAmt, maxLevel - level );
          }

        if ( level > 0 ) {
            gl_.glTranslatef( segsize[0] / ( 2.0f * scaleAmt ), 0, 0 );
            gl_.glRotatef( (float)curl, 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( (float)bend, 0.0f, 0.0f, 1.0f );
            gl_.glTranslatef( segsize[0] / 2.0f, 0.0f, 0.0f );
            gl_.glPushMatrix();
                gl_.glScalef( segsize[0], segsize[1], segsize[2] );
                gl_.glMaterialfv( GL.GL_FRONT, GL.GL_DIFFUSE, matTentacleDiffuse );
                shapes.solidCube( 1.0f );
            gl_.glPopMatrix();

            whip( id, level - 1 );
          } else {
            lightPos[0] = segsize[0];
            gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightPos );
          }
      }

    public GL getGL() {
        return coregl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }

        /** Increase the number of tentacles by 6 */
        if ( evt.getKeyChar() == 'i' ||
             evt.getKeyChar() == 'I' ) {
            numTentacles += 6;
            if ( numTentacles > 60 ) {
                numTentacles = 60;
              }
          }

        /** Decrease the number of tentacles by 6 */
        if ( evt.getKeyChar() == 'd' ||
             evt.getKeyChar() == 'D' ) {
            numTentacles -= 6;
            if ( numTentacles < 6 ) {
                numTentacles = 6;
              }
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Mouse Listener */
    public void mouseClicked( MouseEvent evt ) {
      }

    public void mouseEntered( MouseEvent evt ) {
      }

    public void mouseExited( MouseEvent evt ) {
      }

    public void mousePressed( MouseEvent evt ) {
        beginx = evt.getX();
        beginy = evt.getY();
      }

    public void mouseReleased( MouseEvent evt ) {
      }

    /** MouseMotionListener */
    public void mouseDragged( MouseEvent evt ) {
        int width = evt.getComponent().getSize().width;
        int height = evt.getComponent().getSize().height;
        int x = evt.getX();
        int y = evt.getY();

        lastQuat =
            ball.buildQuaternion( (float)( 2.0f * beginx - width ) / (float)width,
                                  (float)( height - 2.0f * beginy ) / (float)height,
                                  (float)( 2.0f * x - width ) / (float)width,
                                  (float)( height - 2.0f * y ) / (float)height );

        curQuat = ball.addQuats( lastQuat, curQuat ); 

        beginx = x;
        beginy = y;

        glc.repaint();
      }

    public void mouseMoved( MouseEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
