/**
 * $Id: testGL.java.stub,v 1.21 1998/10/19 00:53:22 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd. <http://www.arcana.co.uk>
 *
 * Test program that demonstrates the OpenGL/Java link
 *
 * $Log: testGL.java.stub,v $
 * Revision 1.21  1998/10/19 00:53:22  descarte
 * Made JDK-1.1 compliant. Updated copyright information.
 *
 * Revision 1.20  1998/10/02 11:17:47  descarte
 * Changed GLComponentListener to GLEventListener.
 * Altered reshape() arguments
 *
 * Revision 1.19  1998/06/01 11:30:27  descarte
 * Updated this to be more efficient. Added extra comments as well.
 *
 * Revision 1.18  1998/03/30 00:47:34  descarte
 * Commented out debugging.
 *
 * Revision 1.17  1998/03/30 00:41:45  descarte
 * Added applet resource handling.
 *
 * Revision 1.16  1998/03/28 12:36:09  descarte
 * Converted to GLComponentFactory
 * Upgraded to Java 1.1 listeners. Dropped AWT 1.0 support.
 *
 * Revision 1.15  1998/03/24 15:16:40  descarte
 * Changed GLEventListener prototypes.
 *
 * Revision 1.14  1998/03/22 20:44:05  descarte
 * Converted to new GLComponentFactory syntax.
 *
 * Revision 1.13  1998/03/19 20:12:42  descarte
 * Added getGL() GLEventListener methods.
 *
 * Revision 1.12  1998/03/16 10:36:03  descarte
 * Converted from old beta 4 syntax to new beta 4 syntax.
 *
 * Revision 1.10  1998/01/03 15:16:20  descarte
 * Swapped arguments of the Container add method. They were wrong. God knows
 * why the Sun compiler let that through!
 *
 * Revision 1.9  1998/01/01 16:15:26  descarte
 * Added GLComponent arguments to each GLEventListener interface method.
 *
 * Revision 1.8  1997/12/31 22:27:13  descarte
 * Converted demo across to implementing GLEventListener methods. The code
 * looks a lot more GLUT-like now.
 *
 * Revision 1.7  1997/11/28 13:53:42  descarte
 * Moved demo into auto-generating demo structure.
 * Added author attribution.
 *
 * Revision 1.5  1997/09/02 22:17:17  descarte
 * Added GLContextException traps.
 *
 * Revision 1.4  1997/08/31 11:01:59  descarte
 * Added 'destroyContext()' call
 *
 * Revision 1.3  1997/08/25 21:24:31  descarte
 * Added correct demoTemplate interface methods and made test a subclass of
 * Frame.
 * Added handleEvent() to remove the demo.
 *
 * Revision 1.2  1997/08/24 23:47:42  descarte
 * Added description.
 *
 * Revision 1.1  1997/08/24 23:37:26  descarte
 * Initial revision
 *
 */

package com.hermetica.magician.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;

public class testGL extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {
//public class testGL extends GLFullScreenFrame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 200,
                HEIGHT = 200;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        testGL d = new testGL();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public testGL() {
        gl_ = coregl_;
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Simple OpenGL Test" );
      }

    public String getVersion() {
        return new String( "$Id: testGL.java.stub,v 1.21 1998/10/19 00:53:22 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration draws a single polygon the size of the window\n" );
        sb.append( "shaded black to blue, illustrating the smooth shading capabilities\n" );
        sb.append( "of OpenGL. It is primarily used to check Magician is functional.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Alligator Descartes <descarte@arcana.co.uk>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the key listener to the component */
        glc.addKeyListener( this );

        /** Add the window listener */
        this.addWindowListener( this );

        /** Add the listener and initialize the component */
        glc.addGLEventListener( this );
        glc.initialize();
      }

    /** Gets called exactly once when GLComponent.initialize() is invoked */
    public void initialize( GLDrawable component ) {
        /** Create a new display list */
        gl_.glNewList( 100, GL.GL_COMPILE );
            gl_.glDisable( GL.GL_DEPTH_TEST );
            gl_.glShadeModel( GL.GL_SMOOTH );
            gl_.glBegin( GL.GL_POLYGON );
                gl_.glColor3f( 0.0f, 0.0f, 0.0f );
                gl_.glVertex3f( -1000, 1000, -999 );
                gl_.glVertex3f( 1000, 1000, -999 );
                gl_.glColor3f( 0.0f, 0.0f, 1.0f );
                gl_.glVertex3f( 1000, -1000, -999 );
                gl_.glVertex3f( -1000, -1000, -999 );
            gl_.glEnd();
        gl_.glEndList();

        /** Set the clear colour for the framebuffer */
        gl_.glClearColor( 0.0f, 0.0f, 1.0f, 1.0f );

        /** Disable depth-buffering. We don't use it here */
        gl_.glDisable( GL.GL_DEPTH_TEST );
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        gl_.glViewport( component, 0, 0, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0, 1.0, -1.0, 1.0, 1.0, 1000 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }

    /** Renders the scene */
    public void display( GLDrawable component ) {
        /** Clear the framebuffer */
        gl_.glClear( GL.GL_COLOR_BUFFER_BIT );

        /** Call the display list to draw the background */
        gl_.glCallList( 100 );
      }

    public GL getGL() {
        return coregl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
